begin transaction;
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/caja','57','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/engrampa','23','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/eom','27','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/libmatekbd','13','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/libmatemixer','10','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/libmateweather','98','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/marco','22','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/mate-backgrounds','11','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/mate-common','2','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/mate-control-center','41','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/mate-desktop','13','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/mate-icon-theme','77','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/mate-media','7','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/mate-menus','7','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/mate-notification-daemon','11','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/mate-panel','38','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/mate-polkit','8','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/mate-power-manager','17','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/mate-screensaver','14','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/mate-session-manager','15','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/mate-settings-daemon','25','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/mate-system-monitor','25','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/mate-terminal','14','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/mate-themes','20','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/docutils','130','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/gksu','5','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/glade','40','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/gssdp','6','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/gtk-engines','17','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/gtksourceview3','32','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/gupnp','10','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/libgksu','7','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/libgtop','14','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/libgxps','9','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/libpeas','19','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/libunique','16','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/libunique3','9','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/libwnck3','12','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/murrine','8','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/pangox-compat','5','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/rarian','9','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/vala','41','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/vte3','21','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/yelp-tools','4','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/yelp-xsl','4','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/zenity','13','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('extra/atril','62','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('extra/caja-actions','53','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('extra/caja-dropbox','5','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('extra/caja-extensions','21','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('extra/mate-applets','53','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('extra/mate-calc','14','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('extra/mate-icon-theme-faenza','153','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('extra/mate-sensors-applet','14','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('extra/mate-utils','35','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('extra/mozo','4','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('extra/pluma','51','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('testing/libindicator','5','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('testing/mate-indicator-applet','6','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('testing/mate-user-share','9','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('testing/numix-icon-theme','143','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('testing/numix-icon-theme-circle','22','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('testing/numix-icon-theme-shine','5','7000.28','~');
commit;
