begin transaction;
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('BeautifulSoup','3','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accountsservice','11','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('alacarte','4','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('autoconf-archive','6','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('caribou','15','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('cinnamon','115','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('cinnamon-control-center','56','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('cinnamon-desktop','22','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('cinnamon-menus','11','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('cinnamon-screensaver','18','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('cinnamon-session','20','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('cinnamon-settings-daemon','44','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('cinnamon-themes','4','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('cinnamon-translations','18','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('cjs','37','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('clutter','47','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('clutter-gtk','11','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('cogl','54','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('colord','29','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('cracklib','14','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('dbus-python3','7','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gksu','6','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gnome-common','3','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gnome-menus','7','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('json-glib','7','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('krb5','52','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libgee','15','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libgksu','7','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libgnomekbd','11','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libgtop','13','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libgusb','5','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('lxml','48','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('metacity','31','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('mint-y-icons','82','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('mint-y-theme','5','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('mozjs','281','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('muffin','53','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('nemo','57','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('pam','22','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('pangox-compat','4','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('pexpect','3','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('polib','1','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ptyprocess','1','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('py3cairo','3','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('pygobject3-python3','12','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python-pam','2','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python3','102','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('setproctitle','2','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('speex','9','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('vala','45','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('xapps','14','7000.28','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('zenity','11','7000.28','~');
commit;
