begin transaction;
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/caja','67','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/engrampa','27','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/eom','33','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/libmatekbd','16','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/libmatemixer','12','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/libmateweather','118','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/marco','26','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/mate-backgrounds','12','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/mate-common','2','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/mate-control-center','50','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/mate-desktop','17','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/mate-icon-theme','95','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/mate-media','9','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/mate-menus','8','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/mate-notification-daemon','13','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/mate-panel','45','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/mate-polkit','9','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/mate-power-manager','20','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/mate-screensaver','17','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/mate-session-manager','17','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/mate-settings-daemon','31','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/mate-system-monitor','30','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/mate-terminal','18','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('base/mate-themes','23','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/docutils','16','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/gksu','7','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/glade','47','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/gssdp','7','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/gtk-engines','21','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/gtksourceview3','39','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/gupnp','12','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/libgksu','9','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/libgtop','17','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/libgxps','11','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/libpeas','23','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/libunique','21','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/libunique3','11','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/libwnck3','16','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/murrine','8','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/pangox-compat','6','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/rarian','10','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/vala','51','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/vte3','29','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/yelp-tools','4','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/yelp-xsl','5','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('deps/zenity','17','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('extra/atril','77','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('extra/caja-actions','64','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('extra/caja-dropbox','7','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('extra/caja-extensions','26','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('extra/mate-applets','65','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('extra/mate-calc','18','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('extra/mate-icon-theme-faenza','185','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('extra/mate-sensors-applet','17','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('extra/mate-utils','42','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('extra/mozo','5','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('extra/pluma','63','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('testing/libindicator','7','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('testing/mate-indicator-applet','7','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('testing/mate-user-share','11','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('testing/numix-icon-theme','176','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('testing/numix-icon-theme-circle','27','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('testing/numix-icon-theme-shine','5','7000.27','~');
commit;
