begin transaction;
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('BeautifulSoup','3','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accountsservice','10','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('alacarte','5','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('autoconf-archive','6','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('caribou','17','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('cinnamon','125','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('cinnamon-control-center','64','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('cinnamon-desktop','28','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('cinnamon-menus','12','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('cinnamon-screensaver','22','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('cinnamon-session','25','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('cinnamon-settings-daemon','51','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('cinnamon-themes','4','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('cinnamon-translations','18','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('cjs','43','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('clutter','55','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('clutter-gtk','13','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('cogl','66','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('colord','36','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('cracklib','16','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('dbus-python3','8','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gksu','6','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gnome-common','3','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gnome-menus','8','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('json-glib','10','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('krb5','62','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libgee','18','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libgksu','8','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libgnomekbd','14','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libgtop','15','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libgusb','6','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('lxml','57','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('metacity','37','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('mint-y-icons','104','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('mint-y-theme','5','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('mozjs','282','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('muffin','63','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('nemo','73','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('pam','28','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('pangox-compat','5','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('pexpect','3','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('polib','2','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ptyprocess','1','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('py3cairo','4','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('pygobject3-python3','14','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python-pam','1','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python3','123','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('setproctitle','2','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('speex','12','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('vala','54','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('xapps','16','7000.27','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('zenity','13','7000.27','~');
commit;
