begin transaction;
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/CAPS_coevolution','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/Data2FCS','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/EMBASSY','159','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/EMBOSS','341','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/FCSalyzer','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/Gblocks','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/GeoGebra','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/HMMER','76','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ImageJ','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/Kst','192','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/Mnemosyne','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/PhyML','66','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/PhyTime','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/PyAMG','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/QtiPlot','423','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/R','301','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/WordNet','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/Xyce','1682','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/abntex','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/aenig4','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/aline','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/amd','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/amplicon','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/amplifx','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/antiprism','84','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/archaeopteryx','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/aris','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/arka','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/armadillo','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/arpack','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/arpack-ng','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/artha','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/avl','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/avogadro','381','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bedtools','56','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/biblesync','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bibletime','245','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bibus','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bowtie','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bowtie2','84','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bpp-core','161','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bpp-phyl','511','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bpp-phyl-omics','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bpp-popgen','47','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bpp-seq','147','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bpp-seq-omics','93','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bppsuite','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/btf','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bwa','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/calc','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/calcoo','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/camd','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cap3','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cblas','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ccolamd','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/celestia','62','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/chemtool','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cholmod','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cistrome-CEAS','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cistrome-MACS','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cistrome-conductGO','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cistrome-extra','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cistrome-mdseqpos','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cld2','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/clustalo','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/clustalw','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/clustalx','39','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/colamd','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/convertall','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/copasi','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/coq','473','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/csparse','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cufflinks','135','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cutadapt','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cxsparse','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/drawtiming','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ds9','251','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/easy_spice','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/edsim51di','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/engauge','240','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ent','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/equalx','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ess','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/evopedia_qt','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/exonerate','282','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/extrema','250','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/fann','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/fastqc','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/fet','447','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/finchtv','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/fityk','108','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/flann','104','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/flashqard','55','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/free42','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/freechartgeany','78','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/g3data','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/galculator','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gaw3','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gcalctool','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gchemutils','158','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gcompris','164','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/geda-gaf','69','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gelemental','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/geneconv','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/genius','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/genometools','114','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/genpak','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/geomview','89','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gerbv','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ggobi','91','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ghemical','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/glpk','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gnucap','69','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gp2fasta','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gpredict','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/grace','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gresistor','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gretl','184','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/grpn','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gspiceui','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gtkwave','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gtypist','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gwyddion','200','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/iTest','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ij-plugins-LOCI','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ij-plugins-UCSD','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/io_lib','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/irsim','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/isoelectric-point','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/itex2mml','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/jalview','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/jpicedt','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/jugglinglab','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/karma','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/kile','123','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/klavaro','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/klu','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/knime','65','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ktikz','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/kurso','68','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/labplot2','111','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ladr','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/lalnview','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/lapack95','44','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/lapacke','249','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ldl','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/libghemical','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/libqalculate','60','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/link-grammar','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/loggerpro','47','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/lpsolve','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/mafft','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/magic','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/mathics','112','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/mathomatic','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/matio','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/maxima','85','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/mendeleydesktop','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/metis','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/mkDSSP','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/mpmath','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/mrbayes','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/muscle','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ncbi-blast','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ngspice','174','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/octave','1896','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/paml','42','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pamlX','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/papi','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/parmetis','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/patsy','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pcb','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/perlprimer','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pianobooster','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/plasmidomics','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/primer3','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pspp','102','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pulseview','162','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pyfits','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/qalculate-gtk','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/qhull','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/qrupdate','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/qtoctave','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/qucs','322','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/rbio','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/reduce-algebra','471','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/root','1004','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/sage','8289','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/samtools','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/scidavis','252','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/scilab','109','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/scipy','500','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/scipy3','500','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/seaview','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/sequence-analysis','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/sigrok-cli','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/siril','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/slatec','207','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/smath-studio','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/snpEff','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/solfege','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/speedcrunch','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/spiceopus','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/spidey','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/spqr','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/squizz','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/staden','90','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/stellarium','404','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/suitesparse','54','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/suitesparseconfig','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/sundials','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/sword','62','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/sword-data-kjv','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/sympy','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/t_coffee','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/tabix','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/tilem','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/tilem2','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/tkgate','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/topdraw','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/tophat','93','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/treeview','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/tuxmath','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/tuxtype2','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ucsc-blat','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ugene','3023','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/umfpack','42','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/units','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/vcftools','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/verilog','88','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/veusz','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/wehi-cellsymphony','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/wehi-deid','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/wehi-hackit','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/wehi-weasel','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/wmcalc','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/wxMaxima','75','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/wxmacmolplt','56','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/xcircuit','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/xephem','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/xfoil','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/xiphos','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/xoscope','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/zotero','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/Synkron','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/easystroke','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/eflite','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/espeak','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/espeakup','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/flite','109','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/jpilot','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/pastebinit','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/ruvim','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/shorturl','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/speech-dispatcher','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/svkbd','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/unclutter','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/unclutter-xfixes','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/wgetpaste','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/xdotool','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/xsel','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/HamsiManager','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/MusicMixer','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/Pd-extended','792','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/SAM','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/SuperCollider','318','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/TiMidity++','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/aacgain','57','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/aacplusenc','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/abcde','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/accuraterip-checksum','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/acousticbrainz-client','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/acousticbrainz-gui','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/acousticbrainz-music-extractor','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/aeolus','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/aften','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/aj-snapshot','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/alsa-plugin-ffmpeg','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/alsa-plugin-jack','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/alsa-tools','59','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/alsacap','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/alsaequal','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/alsamixergui','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/alsamodularsynth','57','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/amSynth','45','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/amrnb','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/amrwb','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ardour','1335','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ario','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/asunder','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/atunes','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/audacious-aac','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/audacious-cue','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/audacious-ffmpeg','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/audacity','554','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/audenc','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/bristol','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/butt','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/calf','51','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/calf-ladspa','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/caps','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/cd-discid','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/celt','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/celt051','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/chuck','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/clam','264','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/clam_annotator','150','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/clam_voice2midi','81','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/clementine','733','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/clthreads','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/clxclient','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/clyrics','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/cmt','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/cmus','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/connie','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/creox4','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/crip','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/cuberok','60','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/darkice','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/dcaenc','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/dcd','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/deadbeef','103','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/decibel-audio-player','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/distrho-mini-series','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/distrho-ports','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/drumkv1','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/dssi-vst','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/easymp3gain','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/easytag','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/eawpats','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ecasound','90','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/eyeD3','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/fIcy','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/faac','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/faad2','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/fabla','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/fantasia','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/faust','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ff8dls','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/flac123','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/flacon','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/flake','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/fluid-soundfont','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/fluidsynth','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/fluidsynth-dssi','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/fmit','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/fmmidi','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/foo-yc20','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/freac','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/freepats','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/fst','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ghostess','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gimmix','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gmpc','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gmpc-plugins','48','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gmusicbrowser','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gnomad2','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gnump3d','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gogglesmm','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/google-musicmanager','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gqradio','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/grip2','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gtklick','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/guitarix','361','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gvolwheel','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/herrie','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/horgand','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/hydrogen','126','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/icecast','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ices','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ices-cc','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/id3','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/id3tool','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/id3v2','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/invada_studio_plugins_lv2','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/iripdb','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jaaa','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jack1','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jack-keyboard','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jack-rack','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jack-tools','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jack2','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jack_capture','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jack_delay','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jackmeter','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jalv','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jamin','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/japa','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jkmeter','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jnoise','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jokosher','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jvmetro','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/kanola','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/kid3','81','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/klick','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/kradio','69','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ladspa_sdk','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/lash','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/lastfm','81','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/libbs2b','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/libebur128','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/lingot','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/listener','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/llcon','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/lmms','187','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/lsmi','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/luppp','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/lv2','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/lv2-mda-metapiano','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/lv2dynparam1','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/lv2file','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/lxmusic','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mac','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/match-vamp-plugin','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mda-lv2','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/meterbridge','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mhwaveedit','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mididings','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/midillo','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/milkytracker','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/millennium-plugins','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/miniAudicle','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/minimodem','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mixxx','853','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mkcue','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mktoc','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mp3blaster','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mp3check','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mp3gain','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mp3splt','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mp3splt-gtk','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mp3val','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mp3wrap','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mp4tools','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mpc','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mpd','153','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mpdscribble','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mppenc','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mt-daapd','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/multimidicast','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/muse','180','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/museeks','39','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/musepack-tools','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/nas','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ncmpc','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ncmpcpp','179','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/nekobee','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/nero-aac','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ngjackspa','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/nnls-chroma','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/non','76','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ntk','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ocp','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ogmtools','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/opencore-amr','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/opus','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/opus-tools','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/opusfile','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/oss','42','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/paman','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/paprefs','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pasystray','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/patchage','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pavumeter','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pd','44','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pd_mrpeach','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/phat','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pianobar','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pithos','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/playmp3list','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pmidi','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/podget','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pogo','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ponymix','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/portmidi','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/puddletag','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pulseaudio-ctl','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pyalsaaudio','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/python-audiotools','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qastools','207','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qjackctl','35','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qm-vamp-plugins','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qmidiarp','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qmidinet','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qmidiroute','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qmmp','172','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qmmp-plugin-pack','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qsynth','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qtractor','80','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/quodlibet','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rakarrack','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rebmp','49','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/retrovol','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rev-plugins','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rezound','92','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rhythmbox','139','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rosegarden','332','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rplay','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rubberband','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rubyripper','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rumor','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/samplv1','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/sayonara-player','315','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/sbagen','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/schismtracker','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/seq24','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/sfarkxtc','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/sfc','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/shell-fm','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/shntool','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/shorten','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/sidplayfp','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/snd','63','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/sonata','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/sonic-visualiser','406','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/sooperlooper','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/soundconverter','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/soundkonverter','111','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/soxr','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/spandsp','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/specimen','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/speex','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/stops','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/streamripper','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/streamtuner','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/sunvox','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/swami','67','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/swh-plugins','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/swh-plugins-lv2','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/synthv1','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/tagtool','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/tap_plugins','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/tapiir','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/transcribe','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/tta','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ttaenc','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/tuxguitar','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/twolame','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/vamp-plugin-sdk','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/vcf_plugins','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/vimpc','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/vmpk','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/vocoder','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/vocoder-ladspa','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/volti','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/volume.app','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/volumeicon','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/volwheel','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/vorbisgain','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/waon','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/wavbreaker','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/whipper','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/whysynth','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/wmusic','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xcfa','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xforge','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmms-ctl','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmms-jack','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmms-ladspa','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmms-mp4','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmms-scrobbler','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmms-shn','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmms-wma','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmms2','40','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmp','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xsynth-dssi','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xwax','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/yoshimi','50','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/zita-ajbridge','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/zita-dpl1','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/zita-rev1','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/zynaddsubfx','70','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/zynjacku','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('business/AssociationSubscribersManager','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('business/beancount','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('business/ledger','232','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('business/lemonpos','52','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('business/openerp-client','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('business/openerp-server','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('business/reckon','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('business/skrooge','136','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('business/stansoft','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('business/tryton','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('business/trytond','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/2bwm','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/9menu','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/Alphacube-GTK','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/AlsaMixer.app','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/Ater-MPlayer-skin','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ClipboardViewer','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ComixCursors','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/CurseTheWeather','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/LX-Colors-Revival','268','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/LX-Colors-themes','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/Pulse-Glass','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/PyPanel','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/QtCurve-Gtk2','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/QtCurve-KDE4','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/Vanilla-DMZ','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/Window_Manager_Scripts','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/XDecorations','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/Xfce-Theme-Manager','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/adwaita-qt','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/afterstep','76','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/alarm','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/alltray','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ambiance-radiance-flat-colors-gtk-theme','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/anki','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/arandr','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/arc-firefox-theme','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/arc-icon-theme','67','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/arc-theme','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/arc-theme-Red','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/areao43-icon-theme','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/avant-window-navigator','73','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/aweather','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/awemenugen','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/awesome','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/awesome-appmenu','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/awesome-extra','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/awn-extras','79','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/awoken-icon-theme','572','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/bashish','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/bashrun','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/bashrun2','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/bmpanel2','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/boje-themes','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/bspwm','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/bunsen-themes','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/caffeine-ng','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cairo-compmgr','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cairo-dock','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cairo-dock-plugins','117','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/catfish','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cathexis-suite-themes','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cbatticon','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ccsm','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cellwriter','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ceti-2-theme','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/clearlooks-classic-themes','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/clearlooks-phenix-theme','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/clipit','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cmatrix','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/compiz-boxmenu','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/compton','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/compton-conf','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/conky-manager','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/copyq','56','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cryptkeeper','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/crystal','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/crystalcursors','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ctwm','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cwm','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cwm-openbsd','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cwp','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/dark-aurora-gtk-theme','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/devilspie','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/disper','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/dmenu','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/dmenu2','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/dunst','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/dwm','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/dzen2','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/e16','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/echinus','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ede','76','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ee','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/elementary-xfce','129','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/emerald','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/emerald-themes','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/enlightenment','257','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/evilwm','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/faba-icon-theme','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/faba-mono-icons','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/faenza-cupertino','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/faenza-icon-theme','535','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/faenza-xfce','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/faience-icon-theme','188','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/fbmenugen','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/fbpanel','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/fbrowse-tray','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ffmpegthumbnailer','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/flatpak','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/florence','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/fusion-icon','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/fvwm-crystal','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/fvwm95','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gis-weather','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/glabels','44','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gmrun','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gmusicbrowser-art','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-menus','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gntp-send','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gr1d','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/grits','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/grun','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gsimplecal','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtk-chtheme','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtk-engines','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtk-engines-unico','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtk-kde4','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtk-nodoka-engine','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtk-theme-Albatross','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtk-theme-Blackbird','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtk-theme-Bluebird','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtk-theme-Greybird','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtk-theme-Orion','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtk1-industrial-engine','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtk1-theme-switch','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtkhash','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gxkb','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/herbstluftwm','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/homerun','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/hsetroot','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/i3','56','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/i3blocks','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/i3lock','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/i3minator','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/i3pystatus','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/i3situation','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/i3status','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/icewm','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/icewm-themes','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/idesk','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ideviceinstaller','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/imgur-screenshot','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/insync-thunar','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ion','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/j4-dmenu-desktop','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/jwm','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/kbdd','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/kcometen4','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/kde-gtk-config','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/kde44-oxygen-molecule','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/keynav','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/kfaenza-icon-theme','208','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/kover','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/kwalletcli','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lemonbar','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/light','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lumina','619','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/luna-wallpapers','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lxappearance','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lxappearance-obconf','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lxde-common','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lxlauncher','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lxmed','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lxmenu-data','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lxpanel','64','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/macopix','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/maitreya','110','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/matrixgl','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mcwm','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/meliae-icon-theme','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/menumaker','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/menutray','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mhsetroot','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mimi','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/moka-icon-theme','290','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/moka-minimal','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/moka-themes','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mousepad','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/murrine','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/murrine-themes','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/musca','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/myGtkMenu','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/neofetch','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/netwmpager','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/neutral','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nitrogen','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/notification-daemon','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/notion','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/numix-blue-theme','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/numix-folders','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/numix-gtk-theme','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/numix-icon-theme','229','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/numix-icon-theme-circle','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/numix-icon-theme-square','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/numix-ocean-theme','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/numix-pack','132','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/numix-sx-theme','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/obapps','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/obbrowser','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/obconf','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/obkey','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/obmenu','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/obmenu-generator','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/obsession','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/obshutdown','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/openbox','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/openbox-menu','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/openbox-simple-theme','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/openbox-themes','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/oranchelo-icon-theme','63','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/orchis-gtk-theme','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/paper-gtk-theme','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/paper-icon-theme','459','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/parcellite','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/pclock','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/pdfsam','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/peksystray','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/pekwm','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/perwindowlayoutd','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/pidgin-guifications','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/pidgin-libnotify','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/pidgin-window_merge','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/plank','70','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/plasma-adjustable-clock','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/plasma-applet-daisy','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/plasma-eyasdp','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/plasma-geek-clock','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/plasma-runner-browsefirefoxbookmarks','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/plasma-widget-menubar','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/plasmaxplanet','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/polar-cursor-themes','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/protozoa-cursor-themes','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/pspshrink','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ptbatterysystemtray','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ptwit','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/py3status','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/pyful','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/python-specbar','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/pytyle','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/q4wine','96','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/qlipper','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/qmmp-skins','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/qt-recordmydesktop','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/qt5ct','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/qtile','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/qtpass','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/quadkonsole4','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/quickaccess','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/quickswitch-i3','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ratpoison','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/razercfg','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/razorqt','143','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/recoll','78','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/recorditnow','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/redshift','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/rodent','67','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/rofi','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/root-tail','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/rootactions_servicemenu','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/rsl','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/rss-glx','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/rss-guard','184','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/s1kls','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/sawfish','47','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/sawfish-pager','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/sawfish-session-dialog','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/screenfetch','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/screenkey','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/screensaver-icon','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/seafile-gui','85','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/searchmonkey','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/sgsearch','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/simon','297','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/siva-flat-themes','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/skb','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/skippy-xd','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/skulpture','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/slack-wallpapers','76','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/slackware-xdg-menu','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/slackware-xdm-theme','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/slock','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/smaragd','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/spectrwm','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ssr','79','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/stalonetray','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/stoken','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/subtle','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/sun','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/superkey-launch','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/sxhkd','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/sync-engine','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/synce-kpm','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/thunar-archive-plugin','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/thunar-dropbox','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/thunar-media-tags-plugin','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/thunar-sendto-clamtk','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/thunar-thumbnailers','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/thunar-vcs-plugin','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/tint2','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/trayer-srg','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/tty-clock','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/uwm','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/variety','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/vbox-runner','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/velox','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/vim-qt','47','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/vivacious-colors-gtk-icon-theme','467','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/vivacious-colors-gtk-theme','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/vtwm','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/vwm','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wally','64','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wbar','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmCalClock','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmMoonClock','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmSun','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmSunMoon','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmakerconf','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmbackground','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmbiff','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmblob','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmcliphist','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmclock','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmclockmon','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmctrl','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmcube','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmdate','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmdl','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmdrawer','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmfire','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmii','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wminfo','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmmixer','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmmon','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmname','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmnet','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmphoto+','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmpinboard','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmpower','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmsm.app','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmswallow','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmsystemtray','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmtime','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmtimer','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmtop','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmudmount','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmweather','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmweather+','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmwebcam','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmxss','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/x-tile','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xdgmenumaker','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xdgurl','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xdo','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xed','75','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-battery-plugin','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-cpufreq-plugin','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-cpugraph-plugin','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-datetime-plugin','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-dict','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-diskperf-plugin','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-embed-plugin','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-eyes-plugin','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-fsguard-plugin','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-genmon-plugin','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-kbdleds-plugin','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-mailwatch-plugin','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-mount-plugin','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-mpc-plugin','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-netload-plugin','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-notes-plugin','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-places-plugin','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-quicklauncher-plugin','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-sensors-plugin','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-smartbookmark-plugin','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-timer-plugin','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-verve-plugin','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-volumed-pulse','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-wavelan-plugin','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-whiskermenu-plugin','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-windowck-plugin','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-xkb-plugin','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-xmms-plugin','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xipper','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xkblayout-state','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xlunch','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xmms-skins-almond','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xmms-skins-ater','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xmms-status-plugin','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xmobar','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xmonad','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xonclock','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xpenguins','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xsession','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xtitle','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xtrascreenhacks','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xtrlock','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xvkbd','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xwarppointer','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xxkb','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/yad','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/yawp','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/zenity','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/zuki-themes','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/4th','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Arduino','1178','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Bottleneck','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/CImg','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/CVSTools','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Catch','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ChezScheme','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Coin','185','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Editra','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/FreeBASIC','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/GitEye','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Hex2bin','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/J-Link','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/JSONObject','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/KKEdit','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/LLgen','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/MySQL-python','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Naked','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/OpenCobolIDE','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ParadisEO','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Pivy','261','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/PythonToolkit','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/QStlink','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/SDL2_gfx','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/SQLAlchemy','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/SoQt','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Sphinx','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/THE','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/WebStorm','62','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Xdialog','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/acpica','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/afl','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ahven','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/alembic','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/alex','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/alloy','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/amd-app-sdk','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/android-studio','159','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/android-tools','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/anjuta','275','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/antlr2','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/apache-ant','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/apache-ivy','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/apache-jmeter','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/apache-log4cxx','98','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/apache-maven','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/apitrace','148','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/arduino','47','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/argouml','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/arm-binutils','183','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/as31','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/asem51','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/asl','44','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/asm6809','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/astyle','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/atasm','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/atom','1024','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/autogen','46','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/avarice','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/avr-binutils','91','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/avr-gcc','801','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/avr-gdb','141','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/avr8-burn-o-mat','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/avr8-gnu-toolchain','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/avra','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/avrdude','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bacon','132','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bakefile','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bas','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bas55','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bashdb','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bats','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bfg','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bigloo','172','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/binwalk','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/blassic','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bluefish','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bmake','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/boost-di','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bpython','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/brackets','78','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bsdiff','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bvi','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/byacc','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bzr','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/camlp4','68','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/camlp5','55','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cargo','398','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cargo-vendor','252','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/casperjs','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cc-tool','50','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cc65','45','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ccl','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cgdb','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cgit','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/check','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/checkbashisms','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/chibi-scheme','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/chicken','188','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cim','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cmocka','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/coccinelle','45','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/codeblocks','246','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/codelite','1401','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/colm','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/colordiff','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/complexity','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/composer','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/conan','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/couchdb','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cpan2tgz','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cppcheck','73','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cppcms','120','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cppdb','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cppunit','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cream','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/crosstool-ng','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cryptoki','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cunit','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/curlpp','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cvsd','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cvsps','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cxxtest','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cxxtools','143','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cycler','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dao','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/darcs','253','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dart','138','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dbeaver','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dejagnu','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/devhelp','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/device-tree-compiler','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dfu-programmer','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dhex','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/diakonos','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/diffoscope','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/diffuse','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dkms','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dmd','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/docutils','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dpkg','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dreampie','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dwarf','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/eagle','54','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ebe','55','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ecl','144','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/eclim','45','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/eclipse-cpp','46','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/eclipse-java','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/eclipse-jee','73','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/edb-debugger','54','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/egypt','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/electron','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/elixir','55','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/energia','91','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/enki','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/envytools','57','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/epydoc','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/eric','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/erlang-otp','216','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ex-vi','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/extra-cmake-modules','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/f2c','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/facile','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/flawfinder','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/fossil','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/fpc','122','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/fpc-source','40','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/freetds','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ftjam','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ftnchek','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/fuel','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/g3log','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gambas3','692','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gambit','171','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gaphor','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gauche','129','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gcclegacy494','2197','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/geany','91','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/geany-plugins','118','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gecode','383','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gedit','78','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gedit-plugins','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/generatorrunner','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gengetopt','46','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gforth','55','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gisto','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/git-cola','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/git-lfs','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gitstats','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/glade','69','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/glm','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/global','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gnats','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gnome-common','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gnome-inform7','76','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gnulib','35','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gnuradio','2509','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gnustep-base','57','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gnustep-make','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gnustep-startup','135','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gob2','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/golang-googlecode-gonet','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/golang-googlecode-gotools','67','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/google-appengine-gosdk','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/google-appengine-pysdk','35','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/google-go-lang','205','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gplcver','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gprolog','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gpsim','96','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gputils','120','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gr-osmosdr','60','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gradle','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/grails','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/groovy','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gsoap','92','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gtest','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gtkdialog','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gtkextra','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/guile-json','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/guile1.8','63','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/hackrf','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/happy','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/henplus','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/hexedit','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/highlight','52','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/hte','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/hub','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/hugs98','202','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/icecream','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/icemon','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/icon','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/icoutils','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/idea','126','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/idlex','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ign-math2','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ignition-math','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/inform','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ioncube-loader','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jam','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jasspa','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/javacc','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jdk','80','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jeex','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jsmin','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/julia','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/julius','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter-ipykernel','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter-ipyleaflet','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter-ipywidgets','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter-nbconvert','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter-nbdime','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter-nbformat','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter-notebook','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter-qtconsole','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter-widgetsnbextension','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter_client','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter_console','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter_core','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jython','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kaaedit','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kconfig-frontends','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kcov','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kdesvn','87','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kdiff3','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kelbt','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kforth','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kicad','2040','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kicad-doc','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kicad-library','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kiwi','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/komodo-edit','88','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/komodo-ide','92','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kompozer','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kotlin','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/latrace','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/lazarus','192','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/lcov','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ldns','72','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/le','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/leiningen','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/leo','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/leveldb','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/libb2','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/libbitcoin','104','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/libcutl','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/libodb','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/libodb-boost','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/libodb-mysql','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/libodb-pgsql','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/libodb-qt','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/libodb-sqlite','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/libretro-samples','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/libxbgi','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/libxdg-basedir','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/lighttable','157','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/likepython','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/liquibase','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/lmdb','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/logisim','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/love','82','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/love-legacy072','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/love-legacy080','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ltrace','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/lua','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/luajit','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/luarocks','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/lwtools','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/maude','137','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mcrl2','1082','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mcu8051ide','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/medit','51','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/meld','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/meld3','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/memchan','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/menhir','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/meson','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mg-openbsd','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ming','72','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mit-scheme','45','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mkchroot','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mlton','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mm-common','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mockpp','39','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/monkey-studio','160','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mono','1366','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mono-addins','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mono-basic','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mono-debugger','50','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mosml','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mphidflash','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mspgcc-legacy','333','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mstch','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mtasc','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mtd-utils','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mutagen','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mysql-pinba','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mysql-workbench','1601','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nant','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ne','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nedit','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/neovim','74','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/netbeans','121','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/newlib-headers','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/newlisp','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nexus','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nickle','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ninja','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ninja-ide','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nodejs','1268','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nose','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/noweb','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/npm2tgz','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nqp','234','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nsis','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nuget','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/numpy','158','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/numpy-legacy','160','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/numpy-legacy3','156','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/numpy3','163','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nvi','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nwjs','44','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/obcpl','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ocaml','177','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ocamlbuild','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/odb','163','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ola','433','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/omake','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/omniORB','151','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/omniORBpy','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/oniguruma','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/opencl-headers','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/opencvs','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/opendbx','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/openjdk','2029','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/openjdk6','1958','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/openjdk8','2015','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/openocd','40','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/osc','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/osmo-sdr','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/p4','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/p4d','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pandas','303','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/parrot','73','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pasdoc','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/patchutils','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pcc','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pd-psql','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pforth','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/phantomjs','1571','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/php-apcu','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/php-pinba','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/phpDocumentor','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pingendo','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pkgconf','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pkgdiff','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/poedit','66','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/polyml','42','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/portaudio','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/power-architect','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/premake','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/processing','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pstack-expect-gdb','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/psycopg2','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pudb','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pycharm','92','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pyclewn','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pycscope','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pyenchant','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pylint','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pyside-tools','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/python3-cycler','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pyvim','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/qbs','246','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/qconf','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/qt-creator','3228','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/qt-creator2','1001','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/quilt','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/racket','1616','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/radare2','122','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ragel','84','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rakudo','412','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rapidsvn','47','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rarian','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/re2c','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/reckless','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/referenceassemblies-pcl','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/regexxer','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/regina-rexx','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/robomongo','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rstudio-desktop','653','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ruff','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rust','2271','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/s51dude','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sauce','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sbcl','809','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sbt','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/scala','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/schroot','88','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/scite','49','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/scratch','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sdcc','206','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/setuptools-git','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/shc','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/shed','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/shiboken','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/simavr','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/simulavr','124','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/slibtool','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sloccount','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/smalltalk','86','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/smartgit','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/smartsvn','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/snack','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/snappy','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sonarqube','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/source-highlight','70','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sourcenav','204','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/spdlog','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sphinx_rtd_theme','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/spice-protocol','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/spin','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/spl','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/splint','52','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/spyder','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sqlcl','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sqldeveloper','74','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sqlitebrowser','58','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sqliteman','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/squeak-vm','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/squirrel-sql','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/src-vcs','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/srcML','227','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/srecord','64','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/staruml','39','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/stgit','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/stlink','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sublime_text','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/svn-all-fast-export','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/swfmill','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/swi-prolog','117','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tcc','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tcludp','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tclvfs','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tea','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/teensy_loader_cli','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/texi2html','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/textadept','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/thunderpad','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tidy-html5','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tig','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tiled-qt','89','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tkcvs','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tkdiff','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tkdnd','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tklib','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tora','234','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/trac','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/trf','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tstoolbox','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tyrutils-ericw','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/u-boot-tools','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ucblogo','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ucpp','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/udis86','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/uemacs','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/uncrustify','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/unifdef','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/unittest-cpp','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/universal-ctags','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/urjtag','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/uz80as','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/vala','52','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/vala-12','47','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/valgrind','109','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/valkyrie','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/viewvc','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/vile','35','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/vispatch','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/vstudio','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/watchman','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/wheel','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/wxHexEditor','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xa','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xdebug','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xemacs','77','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xmlcopyeditor','51','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xmlstarlet','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xsp','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xtruss','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xvile','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xxdiff','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zeal','35','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zed','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zile','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zope.interface','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zope.sqlalchemy','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zope.testing','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/0ad','834','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/0ad-data','376','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/3D-Fasteroids','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/4digits','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/4do-libretro','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/7kaa','60','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/BeastieWorker','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/BeastieWorker-levels','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/Chatbot-Eliza','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/Craft-libretro','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/FlightGear','956','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/FlightGear-data','555','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/Genesis-Plus-GX','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/KAdventure','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/KoboDeluxe','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/OpenJK','240','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/OpenXcom','271','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/PyLotRO','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/Pyfa','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/QuickNES-Core','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/RetroArch','59','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/SLADE','270','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/Scorched3D','246','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/Sokoban','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/SpaceZero','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/Tempus-Irae','44','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/UrbanTerror','337','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/abe','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/abuse','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ace','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/adl','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/adom','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/advancemame','446','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/advancescan','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ags','79','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/aklabeth','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/alephone','132','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/alienarena','244','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/alienwave','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/an','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/angband','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/antimicro','82','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ardentryst','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/armagetronad','71','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/arnold-cpc','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/arx-libertatis','205','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/asc','476','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/asciijump','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/assaultcube','49','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/assaultcube-reloaded','52','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/asteroidsinfinity','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/astromenace','84','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/atanks','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/atari++','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/atari800','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/atari800_roms','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/barrage','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bastet','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/beetle-bsnes-libretro','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/beetle-gba-libretro','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/beetle-lynx-libretro','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/beetle-ngp-libretro','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/beetle-pce-fast-libretro','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/beetle-pcfx-libretro','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/beetle-psx-libretro','42','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/beetle-supergrafx-libretro','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/beetle-vb-libretro','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/beetle-wswan-libretro','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/berusky','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/berusky-data','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/berusky2','44','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/berusky2-data','125','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bitfighter','146','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/blastem-libretro','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/blobby2','50','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/blobwars','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bloodcm','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bloodfrontier','144','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/blueMSX-libretro','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bluemoon','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bnes-libretro','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bomberclone','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/boswars','134','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/braincurses','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/brainparty','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/brickout','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/briquolo','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/brutalchess','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bsnes-libretro','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bsnes-mercury','51','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bstone','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/burgerspace','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bzflag','262','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/caph','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ceferino','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cgames','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cgmadness','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cgoban','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/chessx','231','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/chexquest3','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/chocolate-doom','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/chocolate_duke3D','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/chroma','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/chromium-bsu','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/circuslinux','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ckmame','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/clonekeen','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/colem','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/commandergenius','284','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/connectagram','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/conquest','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cosmic_assault','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cosmosmash','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cowsay','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cpat','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/crack-attack','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/crimson','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/crispy-doom','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/crossfire-client','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cubosphere','119','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/curseofwar','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cutemaze','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cuyo','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cyphesis','96','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/d1x-rebirth','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/d2x-rebirth','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/darkplaces','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/defendguin','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/desmume','69','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dhewm3','152','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dice','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/digger','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/domination','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/doom_shareware_data','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/doomretro','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/doomsday','1036','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/doomseeker','111','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dopewars','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dreamchess','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dreamchess-music','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ds-models','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dunelegacy','172','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dungeon','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dustrac','136','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ecwolf','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/edgar','73','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/eduke32','53','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/eduke32_hires_pack','57','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/eduke32_shareware_data','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/egoboo','107','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/einstein','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ember','1716','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/empire','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/enigma','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/etlegacy','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/etlegacy-data','54','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/eureka','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/extremetuxracer','54','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/exult','141','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/f1spirit','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fbalpha','152','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fceux','68','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ffgo','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fgo','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fgrun','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fifengine','324','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fillets-ng','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fillets-ng-data','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/firestorm','100','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/firestorm-opensim','69','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fizmo','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/flare','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/flare-game','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fmsx-libretro','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/foobillardplus','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fortune-ASR','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fortune-dharma','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fortune-dune','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fortune-futurama','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fortune_doctor_who','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fortune_firefly','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/freeblocks','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/freeciv','133','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/freedoom','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/freedroid','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/freedroidrpg','82','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/freeminer','552','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/freeorion','1081','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/frogatto','378','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/frotz','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/frozen-bubble','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fs2open','331','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fuse-emulator','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/galaxyv2','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gambatte-libretro','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/game_1945','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/garden','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gargoyle','83','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gemrb','60','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gens-gs','65','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ggmud','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gl-117','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/glPortal','90','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/glbsp','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/glestae','341','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/glob2','194','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/glxosd','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gnonograms','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gnubg','63','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gnugo','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gnuski','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/golly','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/goonies','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gottet','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gpsp-libretro','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/greed','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gtkballs','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gtklife','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gw-libretro','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gzdoom','269','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gzdoom-legacy','273','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/hatari','125','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/hatari_tos_roms','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/haxima-nazghul','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/hedgewars','115','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/heretic_shareware_data','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/hex-a-hop','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/hexalate','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/hexen_demo_data','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/hexglass','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/higan','124','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/hlsdk','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/hnefatafl','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ho1','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/icebreaker','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ifm','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/innoextract','42','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/instead','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ioquake3','80','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jackhammer','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jag','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/javacpc','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jezzball-kazzmir','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jfsw','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jfsw_demo_data','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jfsw_hires_pack','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jin','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jzintv','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/kardsgt','74','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/kildclient','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/knights','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/koules','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/larn','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/lbreakout2','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/lgogdownloader','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/libretro-2048','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/libretro-desmume','44','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/libretro-fceumm','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/libretro-handy','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/libretro-lutro','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/libretro-mgba','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/libretro-ppsspp','136','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/libretro-prboom','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/libretro-reicast','177','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/libretro-vecx','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/libretro-yabause','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/lightyears','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/lincity-ng','69','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/liquidwar6','76','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/lmarbles','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/lmpc','46','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/loemu','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ltris','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/lutris','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/m64py','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/madbomber','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/maelstrom','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mame','3220','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mame2000-libretro','81','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mame2010-libretro','513','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mame2014-libretro','1264','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mangband','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/marathon-data','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/marathon-eternal-data','131','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/marathon-evil-data','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/marathon-infinity-data','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/marathon-red-data','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/marathon-rubicon-data','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/marathon2-data','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mari0','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mars','61','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/meandmyshadow','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mednafen','303','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/megaglest','474','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/megamario','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/meteor-libretro','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mfe','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mgba','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/micropolis','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/minetest','187','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/minetest_game','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mininim','53','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/miscom','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mog','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/monkey-bubble','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/moon-buggy','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/moria','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mrrescue','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mupen64plus','64','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mupen64plus-libretro','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/myman','153','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nFlood','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nSnake','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/naev','82','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nestopia','128','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nestopia-libretro','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nethack','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/neverball','141','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nevernoid','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nexuiz','168','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ninvaders','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/njam','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nlarn','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/noteye','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/notpacman','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nottetris2','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/npush','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nsudoku','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nsuds','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nuvie','39','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nxengine-libretro','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/o2em','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/oblige','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/oblige-legacy4','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/odamex','84','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/oolite','142','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/open-adventure','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openarena','83','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openclonk','524','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/opendune','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openjazz','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openlierox','230','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openmortal','99','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openmsx','349','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openmsx-catapult','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openmsx-debugger','54','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openmw','554','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/opensonic','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openspades','138','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openttd','124','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/opentyrian','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openyahtzee','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ostrichriders','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pacman-arena','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pangzero','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/parallel-n64','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pasang-emas','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pcsx-rearmed','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pcsx2','739','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pcsxr','45','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/peg-e','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/penguin-command','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pengupop','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pentagram','99','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/picodrive-libretro','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pinball','51','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pingus','179','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pioneer','469','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/playonlinux','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pokerth','284','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pom1','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ppsspp','296','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/prboom','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/prboom-plus','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/prosystem-libretro','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pushover','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/puzzles','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pybik','643','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pychess','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pysolfc','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/qmc2','389','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/qstat','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/qtsixa','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/quake3_shareware_data','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/quake_shareware_data','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/quakeforge','118','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/qxw','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/qzdl','39','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/redeclipse','324','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/rejoystick','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/residualvm','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/retux','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/rezerwar','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/roadfighter','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/robotfindskitten','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/roccat-tools','154','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/rocksndiamonds','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/rogue','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/rott','48','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/sauerbraten','212','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/scid','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/scid_vs_pc','35','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/scribble','66','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/scummvm','301','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/scummvm-libretro','534','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/sdlpop','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/simplesok','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/simsu','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/singularity','65','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/skulltag','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/sl','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/smashbattle','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/smc','112','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/smokinguns','84','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/sms_sdl','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/snes9x','70','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/snes9x-libretro','54','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/snes9x2002','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/snes9x2005','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/snes9x2010','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/sokowiz','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/solarus','148','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/solarus-quest-editor','128','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/solarwolf','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/speed-dreams','522','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/spring','1844','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/starfighter','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/steam','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/steem','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/stella','203','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/stella-libretro','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/stockfish','45','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/stone_soup','834','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/stormbaancoureur','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/stratagus','70','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/stuntrally','669','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/sumwars','131','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/supertux','193','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/supertuxkart','597','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/syobonaction','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/taipan','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/talkfilters','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tanglet','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/teeworlds','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/term2048','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tintin','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tome','132','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tome-ah','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tome-sx','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tong','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/torcs','241','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/trackballs','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/transfusion','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/trigger-rally','50','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/triplea','69','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tt','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tty-solitaire','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tuxfootball','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tuxkart','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tuxnes','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tworld','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/typespeed','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/typhoon_2001','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tyrquake-libretro','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ucr','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ufoai','337','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/uligo','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/unknown-horizons','91','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/unnethack','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/unvanquished','138','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/uqm','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/uqm_3domusic','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/uqm_remixes','48','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/uqm_voice','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ut2004','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vba-next','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vbam','60','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vbam-libretro','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vcmi','982','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vcmi-core','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vcmi-resolutions','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vcmi-wog','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vectoroids','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vera','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/virtualjaguar-libretro','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vitetris','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vms-empire','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/voxelands','152','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/wargus','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/warmux','91','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/warsow','274','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/warzone2100','117','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/wesnoth','1340','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/widelands','972','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/wmquake','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/wolf4sdl','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/wordwarvi','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/worldofgoo','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/worldofpadman','204','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xarchon','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xcowsay','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xcpc','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xmahjongg','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xmoto','98','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xonotic','281','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xpilot-ng','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xroar','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xskat','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xspacewarp','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xu4','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xye','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/yabause','63','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/yae','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/yahtzee','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/yamagi-quake2','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/yar','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/yetris','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/z26','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zaz','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zdbsp','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zdoom','106','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zelda-roth-se','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zennode','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/znake','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zod-engine','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zoom','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zork','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zsdx','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zsnes','171','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zsxd','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ztools','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ztrack','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/CreateCloudMap','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/Fiona','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/GMT','141','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/Geohash','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/OWSLib','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/SFCGAL','333','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/Shapely','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/TauDEM','96','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/basemap','63','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/cartopy','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/cligj','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/foxtrotgps','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/gdal','520','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/geogig','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/geogig-py','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/geographiclib-python','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/geojson','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/geopy','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/geos','93','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/geoserver','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/geoserver-control-flow','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/geoserver-css','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/geoserver-javascript','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/geoserver-oracle','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/geoserver-pyramid','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/geoserver-python','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/geoserver-wps','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/gmapcatcher','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/google-earth','52','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/gpsbabel','153','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/gpsd','45','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/grass','373','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/josm','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/laszip','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/libLAS','100','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/libgeotiff','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/librasterlite','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/libspatialindex','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/libspatialite','74','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/mapnik','1661','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/merkaartor','114','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/mobac','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/navit','65','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/opencpn','372','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/opengrads','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/openorienteering-mapper','226','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/osgEarth','688','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/osm-gps-map','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/osm2pgrouting','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/osm2pgsql','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/pdal','421','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/pgrouting','91','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/pgsql-ogr-fdw','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/pointcloud','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/polyline','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/postgis','64','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/proj','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/pyproj','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/pyshp','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/pyspatialite','92','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/python-mapnik','198','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/pytrainer','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/qgis','1824','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/qlandkartegt','263','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/rasterio','59','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/readosm','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/saga-gis','1179','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/spatialite-tools','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/spatialite_gui','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/tangogps','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/udig','40','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/viking','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/xplanet','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/Alchemy','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/Blender','902','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/CairoSVG','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/FreeCAD','1908','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/GPS','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/GraphicsMagick','63','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/HotShots','50','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/LDView','64','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/LDraw_data','69','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/OpenCASCADE','2615','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/Pixie','90','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/PyMOL','103','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/Shellpic','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/SweetHome3D','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/VariCAD','46','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/XnViewMP','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/advancecomp','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/aewan','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/asciiquarium','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/autotrace','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/baires','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/bdfedit','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/blender','93','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/brlcad','1115','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/converseen','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/darktable','211','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/dblatex','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/dcraw','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/dia','161','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/digikam','814','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/discwrapper','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/djview4','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/draftsight','62','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/dvisvgm','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/enblend-enfuse','149','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/entangle','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/evolus-pencil','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/exact-image','35','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/exif','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/exiftags','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fbida','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fbpic','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fbv','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/feh','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fig2sxd','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fim','40','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/findimagedupes','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/flpsed','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fontforge','192','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fontmatrix','77','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fotowall','50','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fotoxx','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/freewrl','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/frescobaldi','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fritzing','335','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fyre','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/g3dviewer','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gallery','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gbdfed','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gifsicle','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gimagereader','48','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gimp-feca_hdr-plugin','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gimp-lqr-plugin','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gimp-plugin-bimp','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gimp-wideangle-plugin','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gliv','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gmic','387','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gnofract4d','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gnome-themes-extras','132','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gnome-web-photo','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/goocanvas','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gource','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gpaint','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gpicview','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gpscorrelate','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/grafx2','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/graph-easy','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/graphite2','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/graphviz','120','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gthumb','211','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gtkam','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/hugin','456','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/iccprofiles','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/imgmin','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/inkscape','1146','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/jUploadr','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/jhead','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/jp2a','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/jpeg2ps','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/jpegoptim','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/k3d','2326','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/kamerka','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/kim4','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/kphotoalbum','281','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/kuickshow','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/lcdtest','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/lddot','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/ldglite','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/leocad','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/leocad_pieces','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/librecad','597','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/lilypond','166','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/luminance-hdr','168','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/maim','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/makehuman','47','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mapcrafter','88','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mcomix','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/meh','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/meshlab','251','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/metapixel','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mftrace','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mirage','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mitsuba','685','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mitsuba-blender','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mlbrot','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mozjpeg','35','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mtpaint','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mypaint','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/nomacs','109','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/nvidia-cg-toolkit','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/openclipart','336','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/opencollada','392','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/opencolorio','91','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/openimageio','347','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/openscad','236','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/optipng','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/oyranos','65','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pdf2png','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pdf2svg','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pencil3','97','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/phatch','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pho','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/photivo','205','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/photoprint','67','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/photoqt','48','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/phototonic','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pinta','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/plotutils','72','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pngcrush','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pngquant','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/potrace','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/povray','258','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/psftools','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pyformex','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pygifme','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pygraphviz','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/qcad','1082','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/qcomicbook','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/qelectrotech','405','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/qiv','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/qrencode','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/raw-thumbnailer','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/rawstudio','68','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/rawtherapee','406','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/resynthesizer','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/ristretto','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/scantailor','315','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/scour','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/screencloud','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/screengrab','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/scrot','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/separate+','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/shotwell','92','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/shutter','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/sng','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/svp','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/sxiv','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/teighafileconverter','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/teighaviewer','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/tesseract','119','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/textext','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/tgif-QPL','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/tintii','62','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/tkpng','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/triangulizor','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/ttyload','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/tuxpaint','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/tuxpaint-stamps','40','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/ufraw','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/uniconvertor','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/unpaper','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/urbanlightscape','57','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/viewnior','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/vips','86','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/vsxu','249','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/vuescan','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/wavelet-denoise','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/wayland-egl','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/wayland-protocols','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/whyteboard','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xbmbrowser','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xcalib','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xdot','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xli','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xmedcon','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xpe','69','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xzgv','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/yacreader','304','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/yafaray','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/yafaray-blender','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/yagf','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/yed','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/yesplz','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/zbar','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/zgv','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/7plus','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/aldo','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/aprx','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/baudline','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/chirp','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/cwdaemon','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/cwstation','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/demorse','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/direwolf','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/dxcc','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/ebook2cw','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/ebook2cwgui','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/fldigi','72','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/gqrx-sdr','123','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/gridloc','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/grig','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/gsmlib','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/hamlib','59','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/klog','40','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/linlogbook','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/linpsk','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/nec2c','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/psk31lx','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/qradiopredict','35','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/qrq','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/rtl-sdr','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/soundmodem','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/svxlink','81','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/svxlink-sounds-en_US-heather-16k','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/tlf','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/tqsl','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/twhamqth','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/unixcw','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/wxapt','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/xanalyser','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/xdemorse','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/xdx','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/xfhell','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/xgridloc','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/xlog','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/xnec2c','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/xpsk31','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/xwxapt','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/ghc','2899','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-Crypto','44','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-Diff','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-FindBin','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-GLURaw','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-GLUT','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-HTTP','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-HUnit','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-JuicyPixels','134','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-ObjectName','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-OpenGL','86','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-OpenGLRaw','298','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-QuickCheck','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-SHA','46','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-ShellCheck','314','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-StateVar','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-Tensor','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-X11','57','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-X11-xft','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-aeson','73','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-ansi-terminal','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-ansi-wl-pprint','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-asn1-encoding','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-asn1-parse','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-asn1-types','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-async','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-attoparsec','53','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-base-compat','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-base16-bytestring','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-base64-bytestring','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-blaze-builder','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-blaze-html','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-blaze-markup','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-bmp','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-byteable','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cabal-install','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-case-insensitive','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cereal','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cgi','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cipher-aes','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cipher-des','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cipher-rc4','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-clock','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cmark','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cmdlib','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-colour','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-composition','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-conduit','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-connection','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cookie','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-crypto-cipher-types','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-crypto-numbers','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-crypto-pubkey','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-crypto-pubkey-types','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-crypto-random','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cryptohash','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cryptonite','94','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-curl','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-data-default','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-data-default-class','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-data-default-instances-base','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-data-default-instances-containers','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-data-default-instances-dlist','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-data-default-instances-old-locale','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-data-ordlist','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-dataenc','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-deepseq-generics','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-digest','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-dlist','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-enclosed-exceptions','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-env-locale','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-exceptions','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-extensible-exceptions','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-fail','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-fgl','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-filemanip','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-fixed','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-foundation','95','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-ghc-paths','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-gloss','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-gloss-rendering','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-graphviz','162','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-haddock-library','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-half','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hashable','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hashed-storage','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-haskell-src','45','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-highlighting-kate','150','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hinotify','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hostname','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hourglass','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hslogger','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hslua','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-html','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-http-client','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-http-client-tls','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-http-types','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hxt','104','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hxt-charproperties','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hxt-regex-xmlschema','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hxt-unicode','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-idna','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-json','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-lcs','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-lifted-base','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-memory','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-mime-types','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-mmap','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-mmorph','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-monad-control','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-monads-tf','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-mtl','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-multipart','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-network','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-network-uri','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-old-locale','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-old-time','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-pandoc-types','78','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-parallel','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-parsec','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-pem','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-polyparse','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-primitive','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-publicsuffixlist','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-punycode','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-random','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-regex-applicative','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-regex-base','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-regex-compat','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-regex-compat-tdfa','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-regex-pcre-builtin','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-regex-posix','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-regex-tdfa','114','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-resourcet','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-sandi','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-scientific','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-securemem','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-semigroups','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-setlocale','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-shellish','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-socks','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-split','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-stm','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-streaming-commons','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-strict','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-stringprep','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-stringsearch','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-syb','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-tagged','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-tagsoup','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-tar','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-temporary','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-test-framework','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-test-framework-hunit','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-test-framework-quickcheck2','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-texmath','114','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-text','81','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-text-icu','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-tf-random','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-tls','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-transformers-base','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-transformers-compat','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-unix-compat','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-unordered-containers','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-utf8-string','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-vector','102','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-void','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-wl-pprint-text','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-x509','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-x509-store','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-x509-system','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-x509-validation','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-xml','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-yaml','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-zip-archive','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-zlib','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/hscolour','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/xmonad-contrib','184','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Atlas-C++','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/BeautifulSoup','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/BeautifulSoup4','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Botan','318','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/CCfits','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/CEGUI','121','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/CEGUI0.7','115','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/CGAL','40','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/CherryPy','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/CoinMP','222','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/CondConfigParser','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ConfigParser','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Crypt-SSLeay','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/DateTime','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/DevIL','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/DirectFB','97','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ETL','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/FormEncode','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/FreeImage','69','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/GLee','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/GeoIP-Python','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Geraldo','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/GitPython','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/GtkD','543','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/HawkNL','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Hypodermic','76','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/IMAPClient','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Impacket','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Ipopt','69','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Jinja2','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Kivy','110','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/LucenePlusPlus','462','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/MyGUI','296','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Nevow','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ORBit2','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/OpenAL','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/OpenBLAS','302','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/OpenCSG','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/OpenSceneGraph','1115','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/PDCurses','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/PDFlib-Lite','62','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/PrettyTable','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/PyODE','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/PyOpenGL','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/PyQt5','1148','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/PyQwt','63','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/PySoundCard','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/PyXML','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/PyYAML','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Pykka','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/QDarkStyle','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/QUSB','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/QtUsb','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/RxCpp','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SDL2','46','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SDL2_image','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SDL2_mixer','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SDL2_net','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SDL2_ttf','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SDL_Pango','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SDL_gfx','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SDL_sound','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SFGUI','74','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SFML','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SQLObject','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SimGear','361','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SlowAES','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Smarty','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/VTK','3216','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/VitaMTP','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/WebOb','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ZConfig','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ZODB3','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/activemq-cpp','399','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/adodb','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/afflib','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/agg','86','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/alglib','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/allegro','77','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/allegro4','55','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/alure','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ant-contrib','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/apiextractor','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/appmenu-qt','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/appstream-glib','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/aqbanking','98','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/argon2','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/argtable','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/asio','107','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/assimp','198','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/async','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/atlas','13361','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/aubio','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/audiere','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/audioread','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/avbin','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/avr-libc','122','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/backports-ssl-match-hostname','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/bamf','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/barry','114','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/batinfo','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/bcdoc','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/belle-sip','62','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/blas','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/boolstuff','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/botocore','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/buddy','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/bullet','249','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/bwidget','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/bzrtp','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/c++-gtk-utils','62','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/c-ares','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cal3d','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/capnproto','65','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cbase','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cchardet','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ccrtp','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cdk','45','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cffi','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cfitsio','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/chromaprint','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/clalsadrv','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cln','102','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/collada-dom','237','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/compface','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/compiz-bcop','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/compiz-plugins-experimental','51','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/compiz-plugins-extra','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/compiz-plugins-main','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/compizconfig-python','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/configobj','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/confuse','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/console_bridge','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cpp-netlib','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cpprestsdk','637','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cracklib','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/crossguid','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cryptopp','45','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ctemplate','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cuneiform','89','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/curses_ex','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/daq','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/darcsver','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/db46','84','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/dbus-cpp','46','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/dbus-sharp','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/dbus-sharp-glib','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/dcadec','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/dietlibc','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/discogs-client','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/dlib','138','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/dotconf','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/double-conversion','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/dropbox-python','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/edelib','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/efivar','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/efl','716','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/enca','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/enum34','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/eris','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/evernote-python','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/exempi','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/exiftool','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/facebook-python','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fcgi','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fcl','161','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ffms2','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fifechan','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/flatbuffers','46','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/flatzebra','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/flickcurl','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fltk','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fmodapi','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fox-toolkit','110','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fox-toolkit-devel','229','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fpconst','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/freealut','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/freexl','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/frei0r','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ftgl','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ftplib','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/futures','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fuzzylite','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gallerize','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ganv','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gaphas','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gavl','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gazebo','5719','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gcab','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gconfmm','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gdata','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gdl','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/genshi','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/geshi','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gevent','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gf2x','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gflags','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/giblib','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/girara','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gitdb','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gitdb2','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gl2ps','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/glfw','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/glfw3','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/glog','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gloox','104','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gmtk','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gnet','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gnome-media','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gnome-python2-gconf','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gnome-sharp','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gnome-vfsmm','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gnonlin','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/goffice','89','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/goffice0.8','91','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/google-fruit','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gperftools','57','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/grantlee-qt5','70','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/greenlet','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/grx','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gsm','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gsnmp','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gssdp','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gstreamer-editing-services','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gstreamermm','110','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtk-aurora-engine','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtk-qt-engine','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtk-sharp','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtkdatabox','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtkglarea','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtkglext','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtkglextmm','45','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtkhotkey','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtkhtml','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtkimageview','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtksourceview','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtksourceview3','47','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtkspell3','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gts','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/guichan','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gupnp','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/guppy','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gwenhywfar','46','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hdf','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hdf5','128','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hidapi','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hinnant-date','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hiredis','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hoedown','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hpx','811','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/html5lib','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/htmlcxx','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/http-parser','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hug','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hurry-filesize','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hyphen','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/id3lib','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ignition-math2','46','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ignition-transport','59','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/iksemel','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/imageloader','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/imlib','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/imlib2','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/imlib2_loaders','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/iniparser','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ipaddress','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/irrlicht','105','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/isodate','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/itcl','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/jai','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/jai-imageio','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/jansson','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/jbig2dec','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/jbig2enc','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/jbigkit','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/jcal','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/jreen','45','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/json-glib','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/jsoncpp','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/kaadbg','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/kashmir','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lablgtk','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lame','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lapack','194','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lapack-atlas','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lastfmlib','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lensfun','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/leptonica','87','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/levmar','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lgi','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libAfterImage','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libEMF','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libaacs','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libaaio','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libaal','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libabw','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libalkimia','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libantlr3c','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libaosd','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libasr','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libass','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libast','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libaudclient','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libavc1394','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libb64','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libbde','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libbdplus','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libbluray','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libbonobo','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libbonoboui','44','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libbsd','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libburn','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libc++','57','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcacard','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcangjie','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libccd','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcdaudio','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcdr','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcec','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libchewing','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcitadel','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libclaw','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcli','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcmis','127','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcoap','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcompizconfig','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libconfig','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcpuid','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcryptui','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcsv','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcue','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcuefile','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdaemon','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdbh2','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdbi','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdbi-drivers','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdbus-c++','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdbusmenu','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdc1394','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdca','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdesktop-agnostic','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdivecomputer','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdmapsharing','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdmtx','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdockapp','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdsk','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdv','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdvbcsa','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdvbpsi','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdvdcss','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdynamite','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libe-book','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libearth','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libeatmydata','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libebml','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libechonest','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libedit','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libee','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libeot','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libesedb','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libesmtp','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libestr','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libetonyek','187','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libetpan','52','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libev','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libevhtp','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libevt','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libevtx','39','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libewf','42','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libexif-gtk','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libexttextcat','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfaketime','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfastjson','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfdk-aac','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libffado','110','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfilezilla','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfilteraudio','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfixposix','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfm','50','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfm-extra','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfprint','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfreehand','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfsntfs','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libftdi','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libftdi1','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfvde','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfwnt','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfwsi','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libg15','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libg15render','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libg3d','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgadu','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgaiagraphics','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgaminggear','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgcrypt15','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgda','159','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgdiplus','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgee','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgee1','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgexiv2','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgig','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgit2','105','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgit2-glib','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgksu','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libglademm','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libglpng','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgltf','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgme','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnome','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnomecanvas','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnomecanvasmm','44','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnomecups','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnomemm','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnomeprint','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnomeprintui','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnomeui','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnomeuimm','73','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgraphqlparser','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgringotts','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgsasl','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgtop','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgusb','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libharu','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libhid','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libhoard','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libhtp','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libicns','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libidn2','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libiec61883','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libilbc','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libinfinity','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libinput','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libinstpatch','40','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libiptcdata','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libircclient','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libisoburn','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libisofs','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libixion','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libixp','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libjson-rpc-cpp','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libkate','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libkface','40','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libkgeomap4','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libkml','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libkqueue','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libkvkontakte','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblangtag','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblightgrep','80','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblinebreak','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblnk','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblo','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblogging','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblqr','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblrdf','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblscp','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmaa','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmatroska','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmediainfo','175','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmemcached','162','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmicrohttpd','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmikmod','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmimedir','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmimic','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libminizip','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmirage','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmms','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmodbus','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmodplug','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmp3splt','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmp4v2','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmpd','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmpdclient','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmpeg2','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmrss','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmseed','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmsiecf','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmspack','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmspub','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmusicbrainz','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmusicbrainz3','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmwaw','197','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmygpo-qt','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libnet','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libnfs','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libnice','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libnids','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libnova','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libnxml','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liboauth','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libofa','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libofx','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liboglappth','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libolecf','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liboop','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libopenraw','45','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libopenshot','152','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libopenshot-audio','56','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libopusenc','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liborcus','179','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liborigin','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libosinfo','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libosmo-dsp','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libotf','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libotr','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libowfat','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libp11','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpagemaker','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpano13','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpaper','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpar2','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpeas','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpff','40','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpgf','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpng-legacy12','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpqxx','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpseudo','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpst','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpwquality','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libqcow','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libqinfinity','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libqsqlpsql','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libquicktime','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libquvi','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libquvi-scripts','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libqxt','84','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libr3','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librapi2','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librcd','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libreadline-java','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libregf','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librelp','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librep','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libreplaygain','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librevisa','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librfm5','53','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librra','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librsync','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librtfcomp','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libscca','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libscrypt','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsearpc','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libseccomp','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libserialport','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libshout','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsidplay2','81','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsidplayfp','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsieve','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsigc++-legacy12','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsigrok','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsigrokdecode','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsigscan','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libskk','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libslack','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsmbios','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsmdev','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsmf','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsmi','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsmraw','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsodium','46','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libspectrum','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libstatgrab','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libstrophe','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsuinput','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsunpinyin','57','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsynce','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtar','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtecla','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtelnet','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtermkey','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libticables2','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libticalcs2','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libticonv','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtifiles2','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtomcrypt','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtommath','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtorrent','57','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtorrent-rasterbar','269','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtsm','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtubo0','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtunepimp','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtxc_dxtn','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libu2f-host','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libuchardet','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libucil','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libunibreak','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libunicap','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libunicapgtk','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libunique','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libunique3','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libunwind','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libupnp','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liburcu','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libuv','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvdpau-va-gl','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libversion','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvhdi','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvidcap','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libviper','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvirt','272','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvirt-glib','47','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvirt-python','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvmdk','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvmime','394','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvorbisidec','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvshadow','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvslvm','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvterm','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libwacom','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libwebp','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libwebsocketpp','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libwebsockets','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libwfut','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libwhisker2','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libwnck3','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libwps','39','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libx86emu','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libxfcegui4','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libxkbcommon','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libxmi','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libxml++','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libxmp','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libxnvctrl','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libyubikey','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libzdb','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libzen','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libzookeeper','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libzrtpcpp','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lilv','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/linphone-srtp','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/live555','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lksctp-tools','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lockdev','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/log4c','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/log4cplus','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/log4cpp','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/log4net','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/log4shib','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/logilab-astng','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/logilab-common','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lua-MessagePack','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lua-cmsgpack','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lua-filesystem','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lua-lpeg','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lua-md5','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lua-mpack','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lua-zlib','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/luacrypto','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/luadbi','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/luaevent','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/luaexpat','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/luasec','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/luasocket','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lunatic-python','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lvtk','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lxqt-build-tools','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/manubulon-snmp-plugins','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mariadb_client','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mathgl','89','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/matplotlib','57','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mbedtls','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mdds','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/meanwhile','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/menu-cache','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mercator','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mini18n','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/miniupnpc','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mongo-c-driver','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mongo-cxx-driver','213','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mowitz','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mpir','45','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/msgpack-c','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/msx264','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/muParser','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mujs','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/munkres','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/musicbrainzngs','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mxml','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mygpoclient','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mysql++','56','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mysql-connector-c++','133','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mysql-connector-python','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mysqltcl','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/nacl','93','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/netcdf','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/nextaw','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/nini','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/nltk','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/node-xoauth2','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/npth','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/nspluginwrapper','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ntl','523','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/nx-libs','187','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/oath-toolkit','57','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ocaml-batteries','40','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ocaml-bisect','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ocaml-camomile','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ocaml-findlib','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ocaml-ounit','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ode','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ogre','635','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ois','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/openbabel','190','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/opencv','1104','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/opendht','66','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/opensaml','86','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ordereddict','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/p4api','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pangox-compat','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pcre2','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pexpect','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pgplot','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pgtcl-ng','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/php-imagick','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/php-memcache','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/php-memcached','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/php-mssql','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/php-oci8','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/php-pdo_dblib','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/php-pgsql','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/php-redis','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/php-ssh2','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/php-stomp','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/physfs','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pidgin-mra','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pies','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pies2overrides','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pjproject-ring','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pjsip','49','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/platform','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/plib','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/plplot','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/plumbum','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/plyr','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/png++','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/poco','436','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/podofo','75','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/polarssl','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/polyclipping','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/poppler-qt5','128','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/process-cpp','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/progressbar','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/properties-cpp','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/protobuf-c','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/psqlodbc','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pthsem','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ptlib','59','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ptypes','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pugixml','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/purple-plugin_pack','55','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pyPEG2','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pyacoustid','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pycparser','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pyechonest','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pyftpdlib','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pyjf3','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pykdtree','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pylast','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pylibpcap','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pymdstat','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pymediainfo','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pyqode.core','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pysendfile','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pyte','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python-efl','387','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python-gnupg','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python-librtmp','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python-ntlm','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python-xlrd','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python3-PyQt5','1134','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python3-PyYAML','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python3-dvdvideo','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python3-pillow','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qmltermwidget','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qoauth','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qt-assistant-compat','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qt3','595','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qt5','13375','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qt5-webkit','2917','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qtermwidget','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qtmimetypes','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/quazip','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/quesoglc','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qutepart','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qwt','77','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qwt5','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qwtplot3d','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qwtpolar','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rabbitmq-c','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rapidjson','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/raptor','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rarfile','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rep-gtk','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/reportlab','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rest','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/restbed','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rhino','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ring-daemon','195','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ring-lrc','151','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rlog','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rocksdb','647','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rpyc','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rrdtool','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rtorrent-python','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rudiments','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rxtx','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/sdformat','100','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/secp256k1','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/serd','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/setproctitle','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/shairplay','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/simbody','617','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/skstream','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/slib','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/slv2','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/smmap','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/smmap2','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/soil','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/sord','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/soundtouch','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/sparsehash','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/sqlpp11','53','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/sqlpp11-connector-mysql','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/sqlpp11-connector-sqlite3','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/squish','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/sratom','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/stfl','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/stk','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/suil','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/swfdec','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/t4k_common','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tDOM','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tamu_anova','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tbb','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tcl-inotify','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tclap','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tcllib','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tff','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tinyxml','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tinyxml2','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tkimg','59','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tls','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tlsh','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tmdb3','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tolua++','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/toro','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/totem-pl-parser','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/trader','45','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tre','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/trilinos','594','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tsocks','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ucl','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ucommon','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/unarr','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/unibilium','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/unixODBC','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/upscaledb','274','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/uptime','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/urdfdom','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/urdfdom_headers','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/uriparser','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/utfcpp','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/uthash','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/uuid','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/varconf','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/vdpau-video','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/vid.stab','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/vigra','363','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/virglrenderer','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/vsqlite++','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/vte3','52','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/vte3-ng','52','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wayland','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wcslib','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/web.py','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/webkit2gtk','9587','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/webkitgtk','5553','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/webkitgtk3','7016','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wfmath','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wjelement','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ws4py','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wv','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wvstreams','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wxGTK','134','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wxGTK3','337','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wxPython','347','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wxPython3','881','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wxSQLite3','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wxX11','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wxsvg','68','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xalan','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xalan-c','177','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xapian-bindings','51','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xawplus','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xblas','99','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xcb-util-xrm','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xerces','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xerces-c','114','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xf86-input-libinput','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xforms','48','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xine-plugin','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xlt','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xml-security-c','42','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xmlgen','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xmlrpc-c','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xmlsec','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xmltooling','77','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xmpppy','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xulrunner','2535','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xview','144','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xylib','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/yajl','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/yaml-cpp','66','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/yaml-cpp0.3','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/yaz','42','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zbase32','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zc.lockfile','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zdaemon','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zeromq','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zfec','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zita-alsa-pcmi','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zita-convolver','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zita-resampler','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zope.component','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zope.configuration','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zope.event','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zope.hookable','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zope.i18nmessageid','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zope.location','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zope.proxy','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zope.schema','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zope.security','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zsi','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zziplib','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/BISP','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/Colormake','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/KeePass','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/KeePassHttp','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/QCMA','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/abduco','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/abgx360','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/abgx360gui','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ardesia','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ascii','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/asr-manpages','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/asterixInspector','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/bbrun','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/bchunk','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/bdf2psf','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/bdfresize','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/beep','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/biosdisk','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/birthday','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/bodr','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/boncuk','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/boxes','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/brewtarget','104','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/bsdmainutils','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/bsdsfv','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ccze','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/cflow','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/chemical-mime-data','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/chm2pdf','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/chntpw','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ciso','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/cksfv','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/clawsker','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/conkyforecast','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/cups-windows','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/cwiid','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/dbview','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/dcc','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/deb2tgz','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/discount','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/dmg2img','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/doclifter','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/dos2unix','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/drg2sbg','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/dvtm','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/dwdiff','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/esniper','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fbcat','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fbdump','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fcitx','69','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fcitx-configtool','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fcitx-mozc','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fcitx-qt5','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fcrackzip','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/figlet','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/figlet-contributed','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/figlet-international','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/flashrom','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/floating-IPS','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fribid','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/funny-manpages','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/g15composer','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/g15daemon','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/g15stats','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/gcolor2','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/gcp','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/glogg','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/gnome-mime-data','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/gourmet','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/gr-iqbal','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/grabc','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/gramps','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/grc','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/gxmessage','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/hashkill','74','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/heimdall','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/heyu','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/hoorex','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/html2ps','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ibus','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ibus-anthy','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ibus-chewing','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ibus-hangul','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ibus-pinyin','64','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ibus-qt','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ibus-skk','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ibus-sunpinyin','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ibus-unikey','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/igal2','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/iscan-firmware','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/iscan-proprietary-drivers','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/jmri','35','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/kasumi','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/kcm-fcitx','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/kde_cdemu','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/keychain','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/krename','44','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/kronometer','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/lbdb','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/libydpdict','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/linux_logo','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/lppf','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ltunify','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/mdic','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/megatunix','57','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/moedict-desktop.app','73','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/moreutils','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/mosquitto','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/myclippings','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/nfoview','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/nixnote','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/no-more-secrets','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/nvclock','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/opencc','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ophcrack','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/owfs','56','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/paperkey','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/par','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/par2cmdline','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/par2cmdline-tbb','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/pinfo','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/po4a','54','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/portecle','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/projectM','46','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/protobuf','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/protobuf3','152','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/qtkeychain','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/qtspell','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ramdefrag','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/rarcrack','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/readesm','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/recode','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/rlwrap','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/rsibreak','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/rsnapshot','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/samdump2','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/sbo-templates','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/scim-chewing','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/sdcv','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/seahorse-plugins','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/sigrok-firmware-fx2lafw','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/slop','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/snow','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/solunar','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/spotlighter','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ssss','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/stardict','113','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/stardict-tools','40','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/subsurface','646','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/sutils','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/tal','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/tegaki-zinnia-japanese','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/texscythe','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/toilet','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/torrentinfo','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/txt2tags','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/uni2ascii','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/utimer','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/vdpauinfo','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/wcd','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/weather','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/wkhtmltopdf','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/wmbday','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xapian-omega','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xbanish','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xca','65','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xcape','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xclip','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xdelta','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xdelta3','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xvnkb','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xwinwrap','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/yapet','42','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/yara','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ydpdict','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ykpers','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/yubikey-personalization-gui','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/zinnia','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/zyGrib','212','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/AtomicParsley','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/DivFix++','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/ExMplayer','91','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/Gem','266','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/JSampler','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/MP3Diags','95','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/MediathekView','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/Mopidy','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/MuseScore','1007','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/OpenLP','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/OpenPHT','2520','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/QMPlay2','149','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/SimpleSysexxer','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/Yolo','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/a2jmidid','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/aces-dev','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/aces_container','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/acetoneiso','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/aegisub','320','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/anyremote','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/apetag','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/audiopreview','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/avidemux','574','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/bdtools','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/beets','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/bino','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/bombono-dvd','346','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/cantata','152','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/castnow','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/ccxstream','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/cfourcc','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/cinelerra','306','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/clamz','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/coriander','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/cowbell','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/cuetools','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/devedeng','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/dirac','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/dsmidiwifi','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/dssi','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/dvbsnoop','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/dvd-slideshow','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/dvdauthor','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/dvdbackup','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/dvdrip','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/dvdstyler','125','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/dvgrab','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/emoc','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/exaile','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/farsight2','35','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/farstream','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/fdkaac','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/ffmpeg','297','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/ffmpeg2theora','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/filebot','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/flashplayer-plugin','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/flowblade','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/flvtool2','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/freshplayerplugin','56','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gaupol','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gecko-mediaplayer','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/genpuid','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/get_iplayer','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gigedit','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gimp-gap','244','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/glyr','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gmidimonitor','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gmtp','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gnash','655','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gnome-mplayer','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gnupod','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/google-talkplugin','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gpac','132','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gpodder','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gst-libav','329','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gst-plugins-bad','149','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gst-plugins-ugly','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gst0-ffmpeg','126','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gst0-plugins-bad','128','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gst0-plugins-ugly','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gtick','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gtk-recordmydesktop','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gtkpod','155','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/guvcview','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/hexter','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/huludesktop','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/iat','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/imagination','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/ivtv-utils','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/jlscp','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/kaffeine','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/kaffeine-mozilla','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/kdenlive','134','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/l-smash','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/lightspark','312','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/linuxsampler','46','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/lircaile','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/lives','219','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/lsdvd','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/makemkv','47','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mediainfo','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mediatomb','51','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mimms','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/minidlna','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/minitube','70','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/miro','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mjpegtools','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mkclean','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mkvalidator','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mkvtoolnix','349','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mlt','92','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mozplugger','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mp3fm','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mpgtx','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mplayer-codecs32','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mpv','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mtpfs','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/munt','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/musique','69','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mythplugins','120','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mythtv','611','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/nightingale','44','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/nrg2iso','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/nted','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/obs-studio','91','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/oggconvert','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/oggvideotools','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/openshot','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/parole','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/pepperflash-plugin','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/picard','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/picard-plugins','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/pitivi','61','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/playerctl','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/plex-home-theater','1712','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/plexmediaserver','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/pngnq-s9','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/podcastparser','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/popcorntime','71','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/pragha','49','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/pyradio','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/qsampler','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/radiotray','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/ripit','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/ripperX','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/rosa-media-player','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/rtmpdump','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/schroedinger','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/serviio','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/sfnt2woff','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/shotcut','180','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/simpleburn','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/sinthgunt','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/smpeg','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/smplayer','83','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/smtube','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/spek','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/spotify','75','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/stills2dv','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/subtitlecomposer','46','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/subtitleeditor','736','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/subtitleripper','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/tabu_audio_player','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/telepathy-farstream','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/telepathy-glib','142','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/telepathy-qt','192','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/tivodecode','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/tomahawk','317','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/totem','79','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/tovid','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/transcode','53','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/tstools','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/tvheadend','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/tvtime','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/ucview','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vcdimager','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/videocut','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vitunes-mplayer','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vivaldi-codecs-ffmpeg-extra','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vkeybd','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vlc','466','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vlsub','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vobcopy','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/w_scan','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/whistle','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/winff','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/wxcam','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/x264','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/x265','51','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/xawtv','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/xjadeo','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/xmltv','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/xmms-crossfade','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/xvid4conf','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/xvidcore','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/xvst','130','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/yarock','97','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/youtube-viewer','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/zvbi','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/3proxy','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ApacheDirectoryStudio','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/BitchX','40','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/CherryMusic','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ClusterSSH','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/DarTui','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/DenyHosts','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/EarthReader-Web','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/Electrum','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/Electrum-DASH','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/FireWorks','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/Flootty','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/Free-Cinema','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/GeoIP','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/GhostInTheMail','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/LOIC','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/MEGAsync','141','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/NetworkManager-openconnect','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/NetworkManager-openvpn','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/NetworkManager-pptp','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/NetworkManager-vpnc','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/PAC','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/Pafy','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/PaleMoon','1924','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/PyRSS2Gen','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/PySocks','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/Pyrit','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/QuiteRSS','87','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/RSSOwl','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/RYMTracks','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/Signal-Desktop','46','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/SocksiPy','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/SoulseekQt','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/WireGuard','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/aMule','112','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/aggregate','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/aiccu','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/aircrack-ng','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/airpwn','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/aldryn-client','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/amap','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/amavisd-new','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/aoetools','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/argus','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/argus-clients','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/aria2','250','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/arm','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/arno-iptables-firewall','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/arora','59','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/arp-scan','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/arping','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/arpwatch','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/asciinema','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/asterisk','150','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/atftp','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/authbind','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/autossh','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/avahi','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/awscli','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/awstats','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/axel','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/b43-firmware','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/b43-fwcutter','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/barnyard2','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/base','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bip','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bird','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bitcoin','684','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bitlbee','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bitlbee-steam','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bmon','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bottle','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bozohttpd','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/broadcom-sta','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bsflite','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/btpd','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/btsync','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bwm-ng','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/c-toxcore','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cacti','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cacti-spine','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cadaver','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/canto','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cclive','51','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ccnet','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cgterm','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cherokee','66','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/choqok','88','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/chromium','16104','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/chrony','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cicb','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ckermit','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/clamav-unofficial-sigs','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/claws-mail','152','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/clipgrab','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/clive','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cmdiag','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cmst','57','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/comgt','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/connman','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/corebird','49','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/corkscrew','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/courier-authlib','74','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/courier-imap','106','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/courier-unicode','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cowpatty','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cryptcat-unix','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/csync','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ctorrent','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cyrus-imapd','69','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dansguardian','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/darkhttpd','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/darkstat','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/davmail','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dclib','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ddclient','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/deis','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/deluge','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dhcping','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dianara','53','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dillo','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dkimproxy','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dnscrypt-proxy','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dnscrypt-wrapper','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dnstop','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dnstracer','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dobbscoin','443','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/docsis','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dogecoin','330','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dokuwiki','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dovecot','166','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dovecot-pigeonhole','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/driftnet','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dropbear','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dropbox','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dsniff','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dukto','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dwb','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/easyrsa','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/efax-gtk','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/eiskaltdcpp','344','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/elemental-ircd','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/elinks','58','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/emacs-w3m','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/emailrelay','44','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/emesene','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/enet','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/etherape','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/etherpad-lite','53','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/etherpoke','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ettercap','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ettercap-NG','40','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/exim','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/fail2ban','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/fcgiwrap','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/fdm','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ferm','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/filezilla','218','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/firehol','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/firewalld','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/flexget','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/flvstreamer','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/fping','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/fping3','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/fping6','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/fqterm','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/freeradius-server','93','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/freerapid','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/freerdp','67','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/frostwire','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ftp-cloudfs','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/fwbuilder','349','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/fwknop','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gajim','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ganglia','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ganglia-web','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/geoipupdate','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/glassfish','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/glusterfs','208','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gmvault','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gns3','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/google-cli','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/googlecl','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/googler','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gophernicus','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gq','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/graphite-carbon','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/greenbone-security-assistant','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/grsync','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gtk-gnutella','191','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gtmess','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gtorrentviewer','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/guacamole-server','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gui-ufw','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/hamachi','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/haproxy','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/havp','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/heimdal','833','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/hiawatha','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/hipchat','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/hipchat2','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/hostapd','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/hostsblock','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/hping3','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/httpie','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/httping','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/httptunnel','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/httrack','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/hydra','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/hylafax','47','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/hylafax+','55','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/icecat','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/icinga2','322','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/identicurse','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ifstatus','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ii','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ike','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/imapfilter','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/imapproxy','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/insync','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/iodine','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ip2location-c','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ipcalc','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/iperf','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/iperf3','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ipscan','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ipv6calc','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ipvsadm','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/irssi_otr','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/isync','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/jabberd2','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/jboss-as','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/jetty','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/jigdo','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/jitsi','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kadu','445','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kasp_updater','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kcm_ufw','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kea','493','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/keepalived','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/keybase-client','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kfilebox','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kismet','65','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kivy-garden','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kmess','61','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/knemo','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/knock','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kobby','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/konversation','87','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/krb5','76','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kvirc','254','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kwakd','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ldapvi','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/leafnode','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/leechcraft','206','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/libcapi20','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/libdnet','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/libident','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/libmaxminddb','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/libnatpmp','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/licq','184','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/liferea','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/light-firefox','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/lighttpd','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/lighttpd2','44','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/limnoria','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/linphone','72','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/linuxdcpp','122','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/livestreamer','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/livestreamer-curses','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/lldpd','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/luakit','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/lurch','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/macchanger','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/maildrop','67','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mailman','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/malheur','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/masscan','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mdk3','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/megatools','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/memcached','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/memcached-init','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mew','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/midori','59','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/milter-greylist','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/miredo','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mk-livestatus','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mldonkey','158','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_auth_kerb','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_bw','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_chroot','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_dnssd','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_evasive','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_fcgid','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_geoip2','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_ruid2','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_wsgi','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_wsgi-metrics','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/modsecurity-apache','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/monkey','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mosaic-ck','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mosh','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/movgrab','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mpop','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mps','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mps-youtube','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mrtg','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/msmtp','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/msn-proxy','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mumble','73','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/munge','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/murmur','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/museek+','143','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mysecureshell','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nagios','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nagiosgraph','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/naim','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nbd','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nbtscan','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nbwmon','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ncrack','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/netcat','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/netdrive-connector','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nethogs','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/netperf','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/netsniff-ng','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/netstat-nat','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/netsurf','74','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/newsbeuter','46','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nextcloud-client','119','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nfdump','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nghttp2','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nginx','35','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ngircd','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ngrep','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/niceshaper','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nicotine+','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nikto','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nload','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nmh','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/noip2','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nrpe','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nsca','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nsd','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nss-mdns','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ntop','85','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ntpclient','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nxclient','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nzbget','69','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/offlineimap','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/oidentd','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/onioncat','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openconnect','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/opendchub','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/opendkim','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openfire','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openfortivpn','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openl2tp','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openldap-server','158','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openntpd','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openrdate','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openresolv','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/opensmtpd','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openssh-krb5','44','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openvas-cli','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openvas-libraries','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openvas-manager','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openvas-scanner','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openvswitch-utils','57','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/opera-developer','46','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/opera-legacy','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/otter','198','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/owncloud-client','121','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/owncloud-server','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/p0f','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/packETH','35','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/packit','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/palemoon','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pdns-recursor','49','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pdnsd','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pebrot','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pentadactyl','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/periscope','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pflogsumm','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pgbouncer','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pgl','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/phodav','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/phpmyadmin','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-authorization-blocker','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-birthday-reminder','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-extprefs','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-gfire','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-musictracker','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-nudge','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-opensteamworks','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-otr','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-privacy-please','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-sipe','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-skypeweb','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-sound','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-toobars','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-toolbar-shrink','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-visnotes','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-whatsapp','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pirate-get','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/plasma-nm-openconnect','94','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/plowshare','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/plowshare-modules','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/policyd','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/policyd2','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/polipo','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/postfix','45','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/postgrey','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pound','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pptp','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pptpd','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/privoxy','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/profanity','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/prosody','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/proxychains','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/proxymini','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/proxytunnel','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/psad','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/psi','260','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ptokax','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pubnub-curses','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pure-ftpd','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/purple-facebook','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/putty','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pyNeighborhood','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pysub-dl','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/python-onedrive','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pyzor','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/qTox','151','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/qbittorrent','154','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/qbittorrent-qt5','322','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/qtmib','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/quagga','63','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/quassel','139','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/qupzilla','345','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/quvi','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/r2e','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/r8168','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/radicale','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/radiusplugin','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/radvd','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/razor-agents','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rclone','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/reaver','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/redir','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rejik','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rekonq','58','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/remmina','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/retroshare','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rhapsody','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/roger-router','75','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rssh','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rtl8188eu','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rtorrent','74','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rubygem-showterm','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sabnzbd','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/scapy','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/seafile-client','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/seafile-server','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/seahub','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/secpanel','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/serialmail','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sftp-cloudfs','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/shibboleth-sp','156','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/shorewall','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/shorewall-core','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/shorewall6','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sic','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sickrage','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/siege','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sieve-connect','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/silc-toolkit','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sipcalc','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sipp','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sks-keyserver','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/skylable-sx','70','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/skype','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/skype4pidgin','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/slimjet','61','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/slowhttptest','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/slurm','117','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/smb4k','48','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/smcroute','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/smstools3','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/smuxi','67','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sniffjoke','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/snmpsim','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/snmptt','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/snort','72','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/socat','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/social-engineer-toolkit','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sopcast','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sopcast-player','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/spamassassin','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/spambayes','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/spawn-fcgi','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/speedometer','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/speedtest-cli','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sphinx','63','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/spice','67','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/spice-gtk','64','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sqlmap','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sqlninja','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sqlrelay','83','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/squid','293','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/squidGuard','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/squidanalyzer','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sshblock','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sshfp','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sshfs-fuse','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sshguard','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sshpass','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sshuttle','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sslh','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ssllabs-scan','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sslscan','152','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sslstrip','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sstp-client','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ssvnc','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/strongswan','69','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/surf','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/surfraw','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/suricata','66','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/swaks','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sylpheed','46','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/syncthing','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/synergy','82','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/t50','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tahoe-lafs','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tcpflow','65','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tcptunnel','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/teamviewer','35','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/telegram','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/telegram-cli','42','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/thrift','198','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/thttpd','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tightvnc','62','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tinc','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tinyproxy','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tixati','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tmate','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tnftp','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tntnet','123','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tor','92','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tor-browser','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tornado','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/torsocks','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/toxcore','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tramp','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/transgui','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/translate','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/transmission','94','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/transmission-remote-gtk','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tribler','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/trojita','272','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ttcp','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ttdnsd','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tucan','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tunctl','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/turbolift','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/twine','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/twitter','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/uTox','76','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ucspi-ssl','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ucspi-tcp','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/udns','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/udpcast','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/udpxy','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ufw','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/uget','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/uget-chrome-wrapper','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/unbound','59','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/unicornscan','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/urlgrabber','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/urlview','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/urlwatch','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/uudeview','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/uwsgi','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vacuum-im','230','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/valknut','69','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/varnish','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vblade','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vcspull','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vde2','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/verlihub','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/verm','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vimb','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vivaldi','54','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vmware-horizon-client','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vnstat','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vpnc','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vrrpd','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vuze','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/w3af','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/w3m','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wakeonlan','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wavemon','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/weathercli','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/webalizer','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/weboob','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/weechat','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/weetwit','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/weighmail','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wifi-radar','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wifite','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wildfly','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wire','74','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wireshark','786','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wmget','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wmnd','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wmnotify','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wol','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wput','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wrk','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wvdial','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/x11vnc','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/x2goclient','35','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/x2goserver','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/xdman','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/xinetd','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/xl2tpd','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/xnetload','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/xombrero','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/xtables-addons','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/yadifa','98','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/yoda','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/you-get','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/youtube-dl','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/youtube-dl-gui','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/youtube-dl-server','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/youtube-upload','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/yturl','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/zabbix_agentd','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/zabbix_proxy','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/zabbix_server','46','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/zmap','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/znc','92','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/zsync','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/CherryTree','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/JabRef','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/LanguageTool','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/LibreOffice','13827','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/MasterPDFEditor','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/NME','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/ProjectLibre','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/TaskCoach','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/Ted','112','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/abiword','820','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/abook','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/adobe-reader','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/adobe-reader-fontpacks','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/aigaion','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/aiksaurus','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/alot','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/antiword','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/apvlv','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/asymptote','127','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/basket','42','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/beaver','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/bibutils','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/bookbinder','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/briss','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/broffice.org','137','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/calcurse','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/calibre','64','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/catdoc','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/ccal','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/color-theme','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/convertlit','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/coolreader','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/cups-pdf','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/devtodo','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/dictd','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/diffpdf','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/ding','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/djvusmooth','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/docx2txt','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/elyxer','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/epdfview','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/eqonomize','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/etmtk','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/evince','132','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/fbpdf','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/fbreader','137','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/flowkeeper','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/focuswriter','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/fop','49','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/freemind','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/full-pack','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/ganttproject','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/gbgoffice','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/gcal','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/gnokii','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/gnucash','213','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/gnumeric','232','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/gocr','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/goldendict','114','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/grisbi','55','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/gummi','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hamster','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hebcal','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hevea','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/homebank','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/htmldoc','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hunspell-de','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hunspell-en','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hunspell-es','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hunspell-id','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hunspell-pl','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hyphen-id','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/impressive','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/jstock','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/juffed','59','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/kate-latex-plugin','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/kbbi-qt','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/kbgoffice','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/kbibtex','71','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/kchmviewer','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/kchmviewer-qt','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/keepassx','67','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/keepassxc','117','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/keepnote','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/kmymoney','388','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/lcal','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/leafpad','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/libreoffice','111','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/libreoffice-helppack','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/libreoffice-langpack','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/lout','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/lyx','399','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mairix','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/marave','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mathtex','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mdbtools','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mdp','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mined','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/minidjvu','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mobiperl','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/moneydance','45','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mtp2-fonts','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mu','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/multivalent-tool-pdf','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mupdf','68','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/muttprint','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/myrulib','94','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mytetra','42','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mythes','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mythes-id','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/notmuch','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/nts','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/odt2txt','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/openoffice-langpack','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/openoffice.org','69','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/org-mode','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/osmo','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pandoc','172','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/paps','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pcal','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pdf2djvu','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pdfchain','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pdfjam','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pdfpc','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pdfshuffler','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pdfstudio11','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pdfstudio12','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pdftk','98','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/ps2eps','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pstoedit','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pstotext','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pyspread','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/python-gcalcli','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/qpdfview','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/qute','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/rednotebook','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/referencer','54','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/remind','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/sc-im','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/scribus','522','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/siag','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/sigil','412','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/smoffice2016','162','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/sofastats','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/t-prot','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/task','75','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/taskd','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/taskjuggler','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/tasksh','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/teapot','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/tellico','162','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/texlive','630','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/texlive-docs','447','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/texlive-extra','135','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/texlive-fonts','223','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/texmacs','333','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/texmaker','293','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/texstudio','295','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/text2pdf','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/texworks','59','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/timetrap','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/tnote','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/tpp','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/traduisons','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/treesheets','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/trelby','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/tudu','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/unoconv','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/verbiste','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/vlna','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/vym','58','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/wammu','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/watson','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/wink','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/wps-office','139','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/wyrd','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/xmind','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/xournal','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/xpad','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/xreader','83','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/xtrkcad','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/zathura','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/zathura-cb','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/zathura-djvu','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/zathura-pdf-mupdf','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/zathura-ps','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/zim','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/Gtk2-GladeXML','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/Net-SSLeay','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/cpanminus','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/libpurple-kwallet-plugin','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/libwww-perl','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/mod_perl','66','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Algorithm-Diff','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Alien-SDL','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-AnyEvent','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-AnyEvent-HTTP','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-AnyEvent-I3','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-AppConfig','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Archive-Extract','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Archive-Zip','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Astro-SunTime','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Audio-FLAC-Header','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Authen-SASL','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-BerkeleyDB','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Bit-Vector','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CDDB-get','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CGI','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CPAN-Changes','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CPAN-Perl-Releases','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CPANPLUS','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CPANPLUS-Dist-Build','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CPANPLUS-Dist-Slackware','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CSS-Squish','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Cache-FastMmap','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Canary-Stability','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Capture-Tiny','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Carp-Clan','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-Data-Inheritable','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-Factory-Util','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-Gomor','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-Inspector','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-Load','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-Method-Modifiers','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-MethodMaker','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-ReturnValue','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-Singleton','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Compress-Bzip2','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Config-File','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Config-Find','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Config-IniFiles','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Config-Simple','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Convert-ASN1','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Convert-BinHex','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Convert-Color','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Convert-TNEF','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Convert-UU','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Convert-UUlib','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-Blowfish','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-Blowfish_PP','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-CAST5','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-CBC','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-DES','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-IDEA','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-OpenSSL-Bignum','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-OpenSSL-DSA','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-OpenSSL-EC','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-OpenSSL-ECDSA','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-OpenSSL-RSA','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-OpenSSL-Random','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-Rijndael','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Cstools','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Curses','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Curses-UI','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DBD-Pg','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DBD-SQLite','39','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DBIx-Simple','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Danga-Socket','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Data-Compare','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Data-OptList','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Data-Random','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Data-UUID','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Date-Calc','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Date-Manip','40','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DateTime','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DateTime-Format-Builder','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DateTime-Format-ISO8601','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DateTime-Format-Strptime','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DateTime-Locale','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DateTime-TimeZone','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Devel-CheckLib','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Devel-GlobalDestruction','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Devel-PatchPerl','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Devel-StackTrace','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Devel-Symdump','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Device-SerialPort','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Digest-GOST','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Digest-MD4','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Dist-CheckConflicts','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-EV','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Email-Date-Format','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Encode-Detect','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Encode-EUCJPASCII','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Encode-HanExtra','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Encode-ISO2022','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Encode-JISX0213','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Exception-Class','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Expect','63','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Exporter-Tiny','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-ExtUtils-Config','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-ExtUtils-Helpers','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-ExtUtils-Install','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-ExtUtils-InstallPaths','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-ExtUtils-ModuleMaker','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-ExtUtils-XSBuilder','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-FCGI','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Copy-Recursive','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Find-Rule','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Finder','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-HomeDir','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-ReadBackwards','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Remove','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Save-Home','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-ShareDir','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-ShareDir-Install','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Slurp','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Slurp-Tiny','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Tail','65','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Which','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-pushd','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Font-AFM','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-GD','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Geo-Coordinates-OSGB','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Geo-Coordinates-UTM','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Geo-IP','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Geography-Countries','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Glib-Object-Introspection','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Gnome2-Vte','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Gtk2-Ex-Simple-List','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Gtk2-Notify','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Gtk2-Unique','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-HTML-TableExtract','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-HTML-Tree','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-HTTP-Cache-Transparent','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-HTTP-Proxy','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-All','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-CaptureOutput','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-HTML','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-Interface','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-Multiplex','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-SessionData','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-Socket-INET6','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-Socket-Multicast','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-Socket-SSL','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-Socket-Socks','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-Socket-Timeout','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-String','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-Stty','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-Tty','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-stringy','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IP-Country','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IPC-DirQueue','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IPC-Run','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IPC-Run3','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IPC-System-Simple','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IPTables-ChainMgr','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IPTables-Parse','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Image-Info','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Image-Size','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Import-Into','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Inline','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Inline-C','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-JSON','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-JSON-XS','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-LWP-Protocol-https','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Lingua-EN-Numbers-Ordinate','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Lingua-Preferred','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-List-MoreUtils','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-List-UtilsBy','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Locale-Maketext-Lexicon','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-LockFile-Simple','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Log-Dispatch','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Log-Log4perl','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Log-Message','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Log-Message-Simple','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MIME-Charset','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MIME-Lite','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MIME-tools','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MP3-Info','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Mail-DKIM','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Mail-DomainKeys','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Mail-SPF','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Mail-SPF-Query','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MailTools','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-Base85','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-BigInt','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-BigInt-GMP','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-Complex_C','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-Complex_C-L','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-Complex_C-Q','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-Decimal64','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-GMPf','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-GMPq','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-GMPz','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-LongDouble','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-MPC','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-MPFR','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-Prime-Util-GMP','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-Round','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Module-Build','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Module-Build-Tiny','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Module-Implementation','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Module-Install','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Module-Manifest','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Module-Pluggable','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Module-Runtime','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Module-ScanDeps','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Module-Versions-Report','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Mojolicious','39','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Moo','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Mozilla-CA','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MusicBrainz-DiscID','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-ARP','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-CIDR','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-CIDR-Lite','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-DNS-Native','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-DNS-Resolver-Programmable','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-DNS-SEC','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Daemon','72','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-IPv4Addr','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-IPv6Addr','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Ident','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Jabber','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-LDAP','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-LibIDN','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Libdnet','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Netmask','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Patricia','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Pcap','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-RawIP','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-SNMP','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Server','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Telnet','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-UPnP','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Write','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-XMPP','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-NetAddr-IP','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-NetPacket','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Number-Bites-Human','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Number-Compare','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Object-Accessor','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Ogg-Vorbis-Header-PurePerl','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-PAR','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-PAR-Dist','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-PHP-Serialization','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Package-Constants','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Package-DeprecationManager','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Package-Stash','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Package-Stash-XS','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Params-Classify','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Params-Util','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Params-Validate','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Parse-RecDescent','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Parser-MGC','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Path-Class','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Pegex','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-PerlIO-Layers','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-PerlIO-via-Timeout','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Pod-Coverage','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Probe-Perl','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Proc-Daemon','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Proc-PID-File','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Proc-ProcessTable','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Proc-Simple','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Readonly','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Regexp-Common','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Regexp-IPv6','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Religion-Islam-Quran','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Role-Tiny','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-SDL','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-SNMP-Info','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-SOAP-Lite','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sidef','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Socket-GetAddrinfo','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Socket6','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sort-Naturally','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sort-Versions','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Statistics-Descriptive','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-String-PerlIdentifier','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-String-ShellQuote','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sub-Exporter','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sub-Exporter-Progressive','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sub-Install','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sub-Override','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sub-Quote','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sub-Uplevel','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sys-CPU','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sys-Hostname-Long','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sys-MemInfo','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sys-Mmap','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sys-Syscall','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Task-Weaken','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-TeX-Hyphen','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Template-Toolkit','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Term-Animation','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Term-ProgressBar','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Term-ReadLine-Gnu','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Term-UI','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Deep','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Differences','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-DistManifest','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Exception','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Fatal','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Harness','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Inter','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-LongString','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-MockModule','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-MockObject','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Most','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-NoWarnings','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Number-Delta','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Output','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-PerlTidy','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Pod','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Pod-Coverage','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Requires','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Script','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-SharedFork','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Simple','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Spec','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-TCP','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Trap','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Warnings','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Autoformat','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-CSV','90','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-CSV_XS','67','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-CharWidth','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Diff','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Glob','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Hyphen','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Iconv','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Password-Pronounceable','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Patch','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Quoted','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Reform','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Tabs+Wrap','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Tabulate','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-WrapI18N','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Tidy','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Tie-IxHash','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Tie-Simple','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Time-Out','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Time-Piece','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Time-modules','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-TimeDate','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Tk-TableMatrix','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Tree-Simple','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Try-Tiny','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Types-Serialiser','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Unicode-LineBreak','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Unicode-Map','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Unicode-Map8','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Unicode-String','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Unicode-UTF8simple','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Unix-Syslog','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-WWW-Curl','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-WebService-MusicBrainz','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-X10','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-X11-Protocol-Other','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-XML-Filter-BufferText','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-XML-LibXSLT','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-XML-SAX-Writer','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-XML-Stream','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-XML-TreePP','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-XML-Writer','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-XML-XPath','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-YAML-LibYAML','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-YAML-Syck','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-YAML-Tiny','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-ZMQ-Constants','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-cairo','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-cgi-minimal','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-class-accessor','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-class-accessor-chained','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-common-sense','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-config-general','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-data-dump','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-data-page','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-data-page-pageset','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-digest-hmac','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-digest-sha1','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-encode-locale','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-event','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-extutils-depends','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-extutils-pkgconfig','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-file-basedir','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-file-desktopentry','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-file-libmagic','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-file-listing','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-file-mimeinfo','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-file-path-expand','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-glib','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-gnome2','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-gnome2-canvas','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-gnome2-gconf','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-gnome2-vfs','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-gnome2-wnck','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-goo-canvas','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-gstreamer','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-gtk2','91','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-gtk2-imageview','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-html-form','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-html-parser','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-html-tagset','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-http-cookies','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-http-daemon','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-http-date','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-http-message','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-http-negotiate','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-http-response-encoding','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-http-server-simple','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-image-bmp','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-inc-latest','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-libintl','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-libnet','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-linux-desktop-files','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-lirc-client','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-local-lib','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-lwp-mediatypes','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-lwp-useragent-cached','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-net-dbus','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-net-dns','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-net-http','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-net-ip','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-palm-pdb','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-pango','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-rename','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-strictures','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-test-warn','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-text-aspell','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-text-unaccent','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-tk','51','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-trayicon','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-tree-dagnode','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-www-mechanize','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-www-robotrules','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-x11-protocol','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-xml-fast','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-xml-libxml','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-xml-parser-lite','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-xml-parser-lite-tree','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-xml-twig','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-yaml','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perlbrew','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/APScheduler','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Automat','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Baker','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Flask','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Flask-Babel','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Flask-Compress','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Flask-Cors','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Flask-Gravatar','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Flask-HTMLmin','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Flask-Login','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Flask-Mail','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Flask-Principal','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Flask-RESTful','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Flask-SQLAlchemy','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Flask-Security','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Flask-WTF','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Logbook','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Mako','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Markdown','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/MarkupSafe','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Parsley','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Paver','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PyAudio','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PyColorizer','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PyDispatcher','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PyGithub','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PyGreSQL','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PyInstaller','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PyRIC','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PyStemmer','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PyXB','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Pygments','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Pyro','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Pyro4','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PythonQt','178','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/RBTools','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/SOAPpy','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Scrapy','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Unidecode','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/WTForms','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/XlsxWriter','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/affine','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/alabaster','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/aniso8601','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/apsw','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/argcomplete','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/argh','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/args','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/artifacts','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/asciimatics','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/asn1crypto','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/astroid','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/attrs','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/autopep8','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/babel','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/babelfish','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/backports-abc','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/bencode','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/binplist','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/bitarray','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/bleach','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/blessings','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/blinker','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/boto','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/boto3','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/bottle-mysql','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/bsddb3','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/buildbot','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/buildbot-slave','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cairocffi','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cddb-py','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cfgparse','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/characteristic','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cheroot','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/chump','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/click','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/click-default-group','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/click-plugins','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/clint','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cocos2d','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/colorama','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/colorclass','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/colored','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/constantly','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/construct','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cov-core','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/coverage','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cryptography','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cssselect','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cssutils','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/curtsies','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cx_Freeze','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/darts.util.lru','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/dbus-python3','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/decorator','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/dfdatetime','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/dfvfs','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/dfwinreg','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/dhcpy6d','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/distorm','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/dnspython','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/docopt','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/dotty','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/dpkt','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/drawille','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/dulwich','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/dxfwrite','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/easygui','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ecdsa','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/egenix-mx-base','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/entrypoints','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/enzyme','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/etk.docking','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/euca2ools','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/extras','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/fabric','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/fasteners','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/feedgenerator','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/feedparser','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/filechunkio','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/fixtures','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/flake8','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/flask-paginate','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/flask-restplus','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/foolscap','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/frosted','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/fudge','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/funcsigs','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/functools-lru-cache','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/functools32','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/git-fame','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/githubpy','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/gmpy','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/gnome-python','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/gnome-python-desktop','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/google-api-python-client','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/google-apputils','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/gst-python','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/gst-python3','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/gst0-python','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/guess-language','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/guessit','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/gunicorn','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/hachoir-core','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/hachoir-metadata','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/hachoir-parser','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/halberd','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/helpy','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/hg-git','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/hgsubversion','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/hpack','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/html2text','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/htmlmin','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/httplib2','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/hyperlink','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/i3-py','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/idna','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/imagesize','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/incremental','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/invoke','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ipaddr-py','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ipython','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ipython_genutils','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/isort','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/itsdangerous','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/jedi','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/jellyfish','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/jmespath','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/json-py','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/jsonschema','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/kaptan','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/linecache2','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/llfuse','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/lockfile','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/lxml','60','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/mando','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/mechanize','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/minidb','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/mistune','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/mnemonic','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/mock','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/monotonic','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/monty','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/msgpack-python','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/munch','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ndg_httpsclient','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/netifaces','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/node-semver','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/npyscreen','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/numexpr','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/numpydoc','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/nxt-python','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/obspy','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/p4python','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/packaging','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pandocfilters','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/paramiko','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/parsel','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/passlib','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/path.py','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pathlib','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pattern','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pbr','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pefile','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pelican','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pendulum','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/phply','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pickleshare','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pilkit','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pillowfight','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pip','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pipdeptree','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/piprot','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pipstat','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pkginfo','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/planes','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/plaso','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pluggy','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pluginbase','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ply','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/polib','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/portend','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/poster','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/progressive','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/prompt_toolkit','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ps_mem','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/psutil','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ptpython','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ptyprocess','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/purl','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/py','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/py-cpuinfo','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/py3cairo','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/py3sensors','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/py4j','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyOpenSSL','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyPdf','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pySmartDL','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyaes','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyasn1','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyasn1-modules','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pybloomfiltermmap','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pycassa','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pycdio','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pychart','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pychecker','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pychm','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pycld2','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pycodestyle','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pycrypto','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pycryptopp','59','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pycxx','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pydblite','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pydf','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pydot','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyewmh','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyfeed','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyfiglet','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyflakes','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyfltk','70','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pygame','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyglet','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pygobject3-python3','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pygoocanvas','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pygpgme','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pygrametl','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pygsl','47','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pygtksourceview','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pygtkspell','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyicu','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyinotify','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyjwt','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyliblo','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pylirc','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pymongo','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pymysql','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pynzb','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyorbit','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyparsing','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyperclip','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pypoppler','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyqode.cobol','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyqode.qt','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyquery','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyresample','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyrfc3339','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyrsistent','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pysed','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyserial','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyside','617','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pysmssend','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pysnmp','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pystatsd','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pysvn','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pytables','50','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pytest','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pytest-cov','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pytest-runner','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-Levenshtein','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-MonthDelta','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-Safe','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-augeas','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-authres','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-autobahn','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-axolotl','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-axolotl-curve25519','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-bigfloat','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-blake2','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-cached-property','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-certifi','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-chardet','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-cheetah','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-clamd','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-cluster','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-cmdln','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-configargparse','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-d2to1','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-daemon','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-dateutil','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-demjson','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-discid','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-distro','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-distutils-extra','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-django','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-django-legacy','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-django-tagging','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-djvulibre','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-docker','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-docker-pycreds','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-dockerpty','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-editor','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-elementtree','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-elib.intl','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-esmre','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-evdev','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-faulthandler','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-fonttools','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-fusepy','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-future','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-gammu','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-gattlib','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-gflags','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-ipcalc','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-jsonrpclib','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-kerberos','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-keybinder','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-keyczar','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-keyring','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-keyutils','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-klein','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-lazy-object-proxy','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-ldap','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-lhafile','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-libarchive-c','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-libnacl','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-libsass','121','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-libtmux','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-ly','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-m2r','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-magic','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-magick','306','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-mccabe','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-memcached','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-mimeparse','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-morbid','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-mpd','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-musicbrainz2','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-nbxmpp','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-neovim','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-netaddr','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-notify2','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-oauth','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-oauth2','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-oauth2client','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-oauthlib','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-openid','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-orbited','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-parse','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-parsedatetime','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-patch','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-pbkdf2','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-pcapy','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-pdfminer','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-pmw','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-poppler-qt4','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-pydns','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-pyparted','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-pypolicyd-spf','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-pyspf','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-qrcode','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-requestbuilder','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-requests','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-requests-kerberos','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-rubymarshal','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-scandir','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-sh','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-simpy','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-slacklog','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-slip','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-stomper','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-stsci.distutils','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-suds','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-swiftclient','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-tabulate','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-tblib','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-texttable','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-tlslite','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-transaction','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-tvrage','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-twisted','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-twitter','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-txaio','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-unicodecsv','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-unicodedata2','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-uri-templates','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-urllib3','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-uuid','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-vatnumber','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-vulndb','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-webencodings','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-whisper','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-wordpress-xmlrpc','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-wrapt','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-x11_hash','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-xcffib','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-xlib','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-xlwt','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-xrandr','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-yenc','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pythondialog','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3','128','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-Cython','88','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-Pygments','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-appdirs','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-augeas','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-dateutil','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-decorator','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-defusedxml','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-dugong','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-future','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ipython','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ipython_genutils','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-jedi','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-jupyter-ipykernel','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-jupyter_client','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-jupyter_core','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-lhafile','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pexpect','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pickleshare','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-prompt_toolkit','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ptyprocess','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pycrypto','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyparsing','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyparted','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pythondialog','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyzmq','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-simplegeneric','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-sip','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-six','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-tornado','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-traitlets','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-wcwidth','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-xlib','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pytsk','57','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pytz','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pytzdata','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyudev','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyusb','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyutil','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pywebkitgtk','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyxdg','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyzmq','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/queuelib','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ramlfications','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/rebulk','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/regex','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/relatorio','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/requests-futures','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/requests-oauthlib','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/requests-toolbelt','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/requirements-detector','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/rope','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/rsa','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/rst2ansi','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/s3cmd','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/s3transfer','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/scikit-learn','153','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/secretstorage','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/selenium','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/service_identity','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/setuptools-scm','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sge-pygame','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/shutil_get_terminal_size','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/shutil_which','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/shutilwhich','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/simplegeneric','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/simplejson','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/singledispatch','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/six','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sk1libs','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/snowballstemmer','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/snuggs','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/soundcloud','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/speaklater','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sphinxcontrib-plantuml','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sphinxcontrib-websupport','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sql','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sqlalchemy-migrate','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sqlparse','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/statsmodels','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/stem','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/stevedore','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/stopit','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/subvertpy','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/suds-jurko','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/tagpy','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/tempita','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/tempora','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/termcolor','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/terminado','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/terminaltables','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/testpath','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/testscenarios','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/testtools','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/threadpool','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/thunarx-python','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/timehash','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/tinycss','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/tornado_systemd','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/tox','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/tqdm','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/traceback2','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/traitlets','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/trezor','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/trollius','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/tweepy','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/txi2p','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/txrequests','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/txsocksx','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/txtorcon','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/typing','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/tzlocal','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/unrardll','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/vcversioner','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/veryprettytable','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/vinetto','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/virtualenv','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/virtualenv-clone','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/virtualenvwrapper','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/vobject','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/w3lib','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/waitress','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/wcwidth','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/webassets','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/websocket-client','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/werkzeug','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/winpdb','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/wsgiref','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/xe','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/xopen','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/xtermcolor','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/yara-python','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/zxcvbn-python','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/chronic','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/gem2tgz','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/heroku-client','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/ncurses-ruby','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/passenger','210','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rbenv','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/ruby-build','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/ruby-faraday','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/ruby-gist','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/ruby-haste','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/ruby-json','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/ruby-multipart-post','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/ruby-native-package-installer','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-asciidoctor','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-atk','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-bundler','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-cairo','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-coderay','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-cool.io','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-fastercsv','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-fluentd','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-gdk_pixbuf2','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-glib2','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-gosu','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-gtk2','96','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-highline','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-hpricot','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-http_parser.rb','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-lolcat','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-method_source','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-msgpack','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-mustache','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-paint','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-pango','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-pkg-config','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-pry','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-rack','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-rdiscount','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-ronn','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-sass','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-sigdump','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-slop3','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-string-scrub','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-terminal-table','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-thread_safe','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-trollop','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-tzinfo','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-tzinfo-data','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-yajl-ruby','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygems-update','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/sequel','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/sqlite3','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/AnonymousPro','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/Attic','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/BasiliskII','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/CNS11643-font','85','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/Eterm','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/FontAwesome','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/GentiumPlus','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/I-Nex','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/Iosevka','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/MySQL-Sandbox','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/PySixel','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/TLP','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/TermRecord','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/Type1_to_OTF','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/USBdev','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ZoneMinder','143','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/a2tools','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ack','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/acpi','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/acpi_call','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/acpitool','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/adobe-source-code-pro-font','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/adobe-source-han-sans-fonts','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/adobe-source-sans-pro-font','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/adobe-source-serif-pro-font','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/aespipe','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/afdko','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/afio','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/afpfs-ng','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/afuse','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/agedu','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/aha','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/aide','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/alien','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/amanda','105','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/anacron','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/andromeda','221','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ansible','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/anything-sync-daemon','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/apache-activemq','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/apache-cassandra','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/apache-tomcat','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/apachetop','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/apcupsd','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/aqemu','72','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/arc','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/argyllcms','73','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/arj','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/artwiz-aleczapka','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/artwiz-cursor','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/asbt','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/aterm','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/atool','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/atop','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/augeas','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/autoarchive','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/autocutsel','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/autojump','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/avfs','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/backasa','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bacula','62','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bacula-client','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bareos','59','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bareos-client','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/barman','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bcache-tools','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/betty','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bicon','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/biew','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bin2iso','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bindfs','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bit-babbler','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/blake2','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bleachbit','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/blktrace','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/blocks','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bochs','81','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bogofilter','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bonnie++','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bootchart','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/borgbackup','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/brasero','122','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/btrbk','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bubblewrap','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bulk_extractor','78','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/burp','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/butterfly','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/byobu','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/c-icap','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cabextract','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cantarell-fonts','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/caprice32','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/capstone','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cbmbasic','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cbmfs','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ccd2iso','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ccid','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ccrypt','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cdemu-client','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cdemu-daemon','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cdrkit','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cdw','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cdwrite','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cfengine','57','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/chkrootkit','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/chrpath','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ciopfs','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/clamav','285','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/clamsmtp','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/clamtk','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cldr-emoji-annotation','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cndrvcups-capt','70','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cndrvcups-common','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cntlm','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/collectd','58','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/colord','57','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/colord-gtk','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/commons-daemon','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/conky','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/convmv','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cool-retro-term','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/coolkey','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/courier-prime','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cpmtools','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cpu-g','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cpuid','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cpuid2cpuflags','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cpulimit','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/crashplan','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/crikey','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/crmsh','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cronie','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/crunch','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/culmus','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cups-bjnp','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/curlftpfs','69','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cutecom','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cve-check-tool','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cwtex-q-fonts-TTFs','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/d-feet','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/d52','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/daa2iso','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/daemonize','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dahdi-complete','51','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dar','75','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dash','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dateutils','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/davfs2','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ddg','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ddrescue-gui','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/debootstrap','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ded','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dehydrated','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/detox','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dfc','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dfu-util','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dieharder','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/diit','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dina-font','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/disk-manager','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/diskscan','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/disksearch','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/docbook-xml','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/docker-compose','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dos33fsprogs','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dosbox','51','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dosemu','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/drbd-utils','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/driconf','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ds4drv','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dstat','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dtach','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dtrx','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dump','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/duplicity','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dvdisaster','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dymo-cups-drivers','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dynamips','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/eaglemode','69','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ecm-tools','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ecofont-ttf','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/efitools','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/eibd','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/elasticsearch','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/eltclsh','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/em','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/emelfm2','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/encfs','52','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/entr','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/epson-inkjet-printer-escpr','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/etckeeper','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/eventlog','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/evilvte','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/exfat-utils','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ext4magic','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/extract-xiso','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/extundelete','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/f2fs_tools','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/f3','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/facter','153','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fakeroot','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fasd','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fbterm','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fcron','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fdupes','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/file-roller','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/findcmd','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/findpkg','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fio','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fira-fonts','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/firejail','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/firetools','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/firmware-gobi-2000','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fish','58','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/flSynclient','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/flashcards','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/flexbackup','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/font-manager','51','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/foomatic-db','57','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/foomatic-db-engine','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/foremost','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/forkstat','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fpm2','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fr','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/franny','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/freedup','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/freefont','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/freeswitch','742','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fs-uae','166','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fs-uae-arcade','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fs-uae-launcher','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fsarchiver','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fslint','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fsv','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fsviewer','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fsvs','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fuse-emulator-utils','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fuse-exfat','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fuseiso','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fwupd','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fwupdate','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fxload','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fzf','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gammu','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gcdemu','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gconf-editor','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gdmap','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/geckodriver','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gencfs','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/genext2fs','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gentoo','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/geoclue','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/geoclue2','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gigolo','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gimp-help','1978','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gitolite','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gkrellm-volume','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gksu','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/glances','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/glyptodon','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gnome-commander','84','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gnome-disk-utility','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gnome-terminal','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gnome-vfs','44','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gnomint','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gnu-unifont','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/goaccess','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gohu-font','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/google-droid-fonts','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gpart','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gphotofs','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/graphterm','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/graveman','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/grub4dos','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gsmartcontrol','53','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gtk-vnc','82','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gtklp','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/guake','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/guix','147','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hack-font-ttf','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hal-flash','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hardinfo','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hashID','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hashcat','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/haveged','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hdapsd','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hddtemp','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hebe','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/heirloom-doctools','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/heirloom-pax','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/heirloom-sh','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hercules','142','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/herculesstudio','45','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hh','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hime','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hntool','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hplip-plugin','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hungrycat','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hwinfo','311','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hwloc','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/i2c-tools','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/i8kutils','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ibam','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/icinga-web','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/icingaweb2','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/idle3-tools','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ifuse','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/image-analyzer','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/imwheel','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/incron','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/influxdb','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/info2man','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/innotop','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/intel-microcode','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/inxi','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ioping','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/iotop','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ipmitool','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/irqbalance','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/isdct','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/isight-firmware-tools','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/isomaster','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/isomd5sum','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/iucode_tool','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/jdiskreport','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/jenkins','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/jmtpfs','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/john','89','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/jq','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/jsawk','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/jshon','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kafka','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kbfs','56','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kc','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kcm_touchpad','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kegs','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kexec-tools','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/keybase','68','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/keylaunch','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kitchen-sync','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kmscon','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/komparator4','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/krusader','80','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ksh-openbsd','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ktsuss','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kvpm','80','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kyotocabinet','68','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/laptop-mode-tools','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lbench','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lbrate','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lbzip2','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/letsencrypt','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lhasa','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/libnatspec','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lilyterm','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/linkchecker','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/linux-libertine-fonts','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/linuxconsoletools','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lirc','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lnav','42','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/localepurge','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/loggedfs','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/logkeys','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/logpp','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/logtool','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lomoco','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lshw','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lsyncd','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/luckybackup','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxdm','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxinput','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxrandr','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxsession','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxsplit','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxtask','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxterminal','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lynis','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lz4','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lziprecover','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lzop','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/macemu','47','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/macutils','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/maintboot','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/makefile2graph','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/makepasswd','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/makepp','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/makeself','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/man-db','59','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/man-pages-es','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/man-pages-fr','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/man-pages-ja','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/man-pages-ru','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mariadb-plugin-saslauthd','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mbootpack','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mcrypt','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/md5deep','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mdf2iso','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mdocml','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/memtest86','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/memtest86+','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/memuse','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mhddfs','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/microcode_ctl','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/microemulator','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/midisport-firmware','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mjpg-streamer','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mksh','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mktorrent','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mlterm','53','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/modules','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mongo-tools','42','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/monit','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/monitoring-plugins','55','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/monitorix','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/motion','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mpich','233','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mrxvt','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ms-sys','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/msitools','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/msr-tools','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mtftar','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mucommander','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/multibootusb','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/multicorn','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/multipath-tools','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/multitail','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/munin-node','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/musl','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mysql-utilities','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nagios-plugins','63','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nagvis','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nanum-coding-fonts-ttf','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nanum-fonts-ttf','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nbench','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ncdu','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/netatop','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/netdata','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/newrelic-sysmond','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nilfs-utils','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nix','82','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nmon','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nnn','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nomarch','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/noto-emoji','613','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/noto-fonts','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nsca-ng','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nss-pam-ldapd','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nss_ldap','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/numactl','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/numlockx','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nut','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nvidia-driver','75','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nvidia-firmware','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nvidia-kernel','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nvidia-legacy304-driver','52','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nvidia-legacy304-kernel','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nvidia-legacy340-driver','60','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nvidia-legacy340-kernel','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nvme-cli','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ohsnap','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/oldschool-pc-fonts','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/onerng','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/opam','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/open-vm-tools','119','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/openct','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/openmpi','575','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/openrc','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/openrc-services','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/opensc','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/openscap','105','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/openstego','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/oracle-instantclient-basic','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/oracle-instantclient-devel','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/osinfo-db','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/osinfo-db-tools','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ossec-agent','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ossec-local','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ossec-server','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ostree','35','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/otf-hermit','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/otf2bdf','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/overpass-desktop-fonts','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/overpass-webfonts','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/p7zip','91','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/packagekit','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/parallel','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/partclone','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/partimage','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/passwdqc','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/password-store','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/passwordsafe','236','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/patchelf','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/patool','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pax','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/paxctl','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pbzip2','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pce','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pcmanfm','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pcsc-lite','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pcsc-perl','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pcsc-tools','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pdfgrep','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pdksh','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pdsh','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/percona-toolkit','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/percona-xtrabackup','103','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pev','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pgadmin3','277','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pgbadger','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pgsanity','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/phoronix-test-suite','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/phpsysinfo','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/physlock','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/picocom','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pigz','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pipemeter','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/plan9port','291','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pmount','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/polyglotman','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pommed-light','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/porg','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/posixovl','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/postgresql','145','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/powerline-status','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/powernowd','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/prelink','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/preload','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/procinfo-ng','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/profile-cleaner','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/profile-sync-daemon','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/progress','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pstate-frequency','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ptop','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pv','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pwgen','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pwsafe','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pyarchey','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pycp','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pymux','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qControlCenter','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qdirstat','66','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qemu','298','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qemu-guest-agent','52','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qingy','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qingy_0.3_themepack','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qps','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qstardict','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qterminal','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qtfm','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qtgzmanager','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/quotatool','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qxkb','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rabbitmq-server','60','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ranger','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rar','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rar2fs','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rdiff-backup','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rdup','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/read-edid','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/recoverdm','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/recoverjpeg','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/redis','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/redis-py','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/refind','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/regionset','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/reiser4progs','46','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rename','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/repeat','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/reptyr','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rfdump','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rhash','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rkhunter','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rmw','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rng-tools','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rover','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rox-filer','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/roxterm','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rsyslog','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rtirq','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/run-one','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/runit','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rxvt-unicode','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/s3ql','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/safecopy','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sakura','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/salt','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/samhain','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/samsung-mfp-drivers','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sandy','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sbbdep','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sbotools','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sboui','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sbsigntools','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/scalpel','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/scanmem','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/schedtool','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/scrypt','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sdl-jstest','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sdltrs','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/seahorse','44','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/secure-delete','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sentimental-skk','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ser2net','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/set_rlimits','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/setconf','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/shake','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/signify','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slack-utils','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slackchlog','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slackroll','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slackware-grub2-theme','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slackyd','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sleepd','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sleuthkit','46','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slim','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slpkg','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/smart','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/smbldap-tools','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/smem','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/softhsm','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/solarize-12x29-psf','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/spacefm','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/speedcontrol','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/spice-vdagent','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/spkg','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/spl-solaris','52','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/splix','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sqlcipher','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sqlite2','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/squidclamav','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/srm','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ssdeep','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/st','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/statifier','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/steamos-xpad','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/stm32flash','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/stress','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/suckless-tools','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sudosh2','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/suhosin','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/supervisor','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/swatch','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/swun','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/symbola-font-ttf','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sysdig','134','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/syslog-ng','100','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/t1utils','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tabbed','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tagainijisho','179','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tamsyn-font','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/targetcli-fb','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tarsnap','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tc-play','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/termenu','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/terminator','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/terminology','60','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/termite','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/termsaver','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/testdisk','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/the_silver_searcher','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/thermald','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/thinkfan','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tiger','42','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tilda','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tini','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tinyterm','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tinyvm','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tkdvd','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tmuxp','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tokyocabinet','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tokyotyrant','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tomb','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tp_smapi','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tpc','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tpm','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/trachet','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tracker','89','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/trackfs','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/trash-cli','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/trashy','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/trmaid','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/trrntzip','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ts','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-aller','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-arphic-uming','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-bitstream-vera','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-caladea','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-carlito','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-coda','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-console-fonts','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-crosscorefonts','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-dark-courier','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-inconsolata','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-monaco','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-mononoki','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-open-sans','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-paratype-pt-fonts','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-roboto','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-ubuntu-font-family','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-zekton','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttmkfdir','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttop','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tty2gif','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttygif','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttyrec','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/twapake','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/twin','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/u3-tool','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/uae','49','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/udevil','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/udftools','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/udiskie','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ufiformat','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/uhd','267','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/uif2iso','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/un-fonts-core','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unac','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unace','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unar','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unetbootin','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unhide','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unicode-character-database','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unison','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/univga-font','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unp','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unrar','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unrtf','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unshield','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/untex','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/uptimed','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/upx','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/usbmon','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/usbredir','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/usbview','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/usermin','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vagrant','39','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vbindiff','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vcp','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/veracrypt','115','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vhba-module','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vice','154','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vifm','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vinterm','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/virt-manager','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/virt-viewer','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/virtualbox','949','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/virtualbox-addons','177','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/virtualbox-extension-pack','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/virtualbox-kernel','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/virtualbox-kernel-addons','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vlock','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/volatility','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vtcol','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wacomtablet','49','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wangfonts','65','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/watch-fs','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wdfs','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wdiff','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/webcore-fonts','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/webmin','55','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/whowatch','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wiimms-iso-tools','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wine','1380','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wine-staging','1442','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/winetricks','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/winusb','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wipe','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wmconfig','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wmfsm','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wmgtemp','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/worker','143','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wqy-microhei-font-ttf','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wterm','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/x86_energy_perf_policy','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xarchiver','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xautolock','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xautomation','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xbattmon','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xbindkeys','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xboxdrv-linux','161','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xbrightness','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xcdroast','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xchm','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xdiskusage','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xen','414','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xf86-video-qxl','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xfburn','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xfe','48','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xfsudo','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xinput_calibrator','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xjobs','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xnp2','54','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xonsh','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xosd','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xrdp','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xrestop','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xstow','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xtrs','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xwiimote','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/yakuake','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/yash','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/yeahconsole','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/yelp','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/yelp-tools','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/yelp-xsl','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/z','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zarfy','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zerofree','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zfs-on-linux','251','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zookeeper','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zopfli','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zpaq','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zstd','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zuluCrypt','124','7000','~');
commit;
