begin transaction;
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/CAFS_divergence','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/CAPS_coevolution','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/Data2FCS','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/EMBASSY','139','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/EMBOSS','308','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/FCSalyzer','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/Gblocks','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/GeoGebra','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/HMMER','64','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ImageJ','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/Kst','176','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/Mnemosyne','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/PhyML','58','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/PhyTime','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/PyAMG','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/QtiPlot','371','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/R','278','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/WordNet','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/Xyce','3009','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/abntex','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/aenig4','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/aline','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/amd','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/amplicon','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/antiprism','73','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/archaeopteryx','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/aris','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/arka','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/armadillo','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/arpack','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/arpack-ng','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/artha','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/avl','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/avogadro','338','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bedGraphToBigWig','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bedtools','48','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/biblesync','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bibletime','224','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bibus','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bowtie','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bowtie2','65','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bpp-core','138','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bpp-phyl','470','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bpp-phyl-omics','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bpp-popgen','40','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bpp-seq','127','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bpp-seq-omics','86','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bppsuite','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/btf','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bwa','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/calc','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/calcoo','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/camd','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cap3','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cblas','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ccolamd','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/celestia','54','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/chemtool','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cholmod','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cistrome-CEAS','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cistrome-MACS','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cistrome-conductGO','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cistrome-extra','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cistrome-mdseqpos','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cld2','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/clustalo','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/clustalw','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/clustalx','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/colamd','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/convertall','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/copasi','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/coq','477','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/csparse','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cufflinks','123','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cutadapt','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cxsparse','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/drawtiming','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ds9','245','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/easy_spice','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/edsim51di','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/engauge','200','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ent','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/equalx','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ess','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/evopedia_qt','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/exonerate','221','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/extrema','222','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/fann','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/fastqc','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/fet','404','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/finchtv','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/fityk','94','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/flann','101','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/flashqard','51','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/free42','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/freechartgeany','66','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/g3data','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/galculator','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gaw3','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gcalctool','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gchemutils','134','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gcompris','140','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/geda-gaf','62','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gelemental','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/geneconv','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/genius','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/genometools','96','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/genpak','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/geomview','76','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gerbv','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ggobi','78','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ghemical','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/glpk','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gnucap','58','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gp2fasta','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gpredict','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/grace','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gresistor','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gretl','154','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/grpn','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gspiceui','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gtkwave','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gtypist','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gwyddion','161','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/iTest','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ij-plugins-LOCI','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ij-plugins-UCSD','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/io_lib','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/irsim','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/isoelectric-point','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/itex2mml','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/jalview','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/jpicedt','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/jugglinglab','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/karma','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/kile','109','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/klavaro','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/klu','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/knime','56','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ktikz','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/kurso','67','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/labplot2','94','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ladr','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/lalnview','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/lapack95','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/lapacke','208','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ldl','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/libghemical','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/libqalculate','59','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/link-grammar','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/loggerpro','39','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/lpsolve','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/mafft','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/magic','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/mathics','87','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/mathomatic','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/matio','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/maxima','78','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/megacc','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/mendeleydesktop','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/metis','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/mkDSSP','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/mpmath','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/mrbayes','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/muscle','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ncbi-blast','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ncbi-blast-plus','106','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ngspice','155','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/octave','1610','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/paml','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pamlX','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/papi','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/parmetis','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/patsy','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pcb','35','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/perlprimer','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pianobooster','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/plasmidomics','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/prank-msa','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/primer3','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pspp','87','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pulseview','134','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pyfits','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/qalculate-gtk','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/qhull','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/qrupdate','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/qtoctave','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/qucs','285','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/rbio','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/reduce-algebra','390','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/root','886','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/sage','7495','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/samtools','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/scidavis','230','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/scilab','93','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/scipy','419','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/scipy3','498','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/seaview','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/sequence-analysis','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/sigrok-cli','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/siril','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/slatec','230','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/smath-studio','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/snpEff','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/solfege','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/spades','392','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/speedcrunch','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/spiceopus','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/spidey','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/spqr','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/squizz','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/staden','89','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/stellarium','391','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/suitesparse','51','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/suitesparseconfig','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/sundials','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/sword','51','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/sword-data-kjv','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/sympy','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/t_coffee','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/tabix','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/tilem','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/tilem2','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/topdraw','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/tophat','94','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/treeview','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/tuxmath','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/tuxtype2','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ucsc-blat','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ugene','2765','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/umfpack','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/units','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/vcftools','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/verilog','78','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/veusz','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/wehi-cellsymphony','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/wehi-deid','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/wehi-hackit','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/wehi-weasel','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/wmcalc','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/wxMaxima','66','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/wxmacmolplt','55','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/xcircuit','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/xephem','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/xfoil','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/xiphos','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/xoscope','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/zotero','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/Synkron','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/easystroke','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/eflite','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/espeak','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/espeakup','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/flite','108','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/jpilot','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/pastebinit','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/ruvim','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/shorturl','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/speech-dispatcher','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/svkbd','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/unclutter','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/unclutter-xfixes','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/wgetpaste','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/xdotool','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/xsel','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/HamsiManager','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/Pd-extended','658','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/SAM','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/SuperCollider','244','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/TiMidity++','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/aacgain','50','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/aacplusenc','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/abcde','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/accuraterip-checksum','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/acousticbrainz-client','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/acousticbrainz-gui','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/acousticbrainz-music-extractor','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/aeolus','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/aften','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/aj-snapshot','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/alsa-plugin-ffmpeg','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/alsa-plugin-jack','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/alsa-tools','47','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/alsacap','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/alsaequal','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/alsamixergui','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/alsamodularsynth','48','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/amSynth','35','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/amrnb','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/amrwb','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ardour','1247','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ario','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/asunder','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/atunes','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/audacious-aac','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/audacious-cue','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/audacious-ffmpeg','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/audacity','487','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/audenc','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/bristol','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/butt','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/calf','45','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/calf-ladspa','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/caps','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/cd-discid','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/celt','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/celt051','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/chuck','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/clam','222','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/clam_annotator','129','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/clam_voice2midi','70','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/clementine','578','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/clthreads','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/clxclient','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/clyrics','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/cmt','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/cmus','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/connie','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/creox4','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/crip','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/cuberok','52','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/darkice','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/dcaenc','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/dcd','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/deadbeef','82','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/decibel-audio-player','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/deezer-desktop-for-linux','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/distrho-mini-series','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/distrho-ports','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/drumkv1','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/easymp3gain','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/easytag','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/eawpats','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ecasound','76','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/eyeD3','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/fIcy','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/faac','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/faad2','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/fabla','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/fantasia','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/faust','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ff8dls','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/flac123','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/flacon','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/flake','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/fluid-soundfont','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/fluidsynth','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/fluidsynth-dssi','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/fmit','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/fmmidi','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/foo-yc20','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/freac','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/freepats','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ghostess','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gimmix','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gmpc','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gmpc-plugins','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gmusicbrowser','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gnomad2','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gnump3d','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gogglesmm','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/google-musicmanager','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gqradio','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/grip2','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gtklick','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/guitarix','303','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gvolwheel','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/herrie','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/horgand','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/hydrogen','105','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/icecast','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ices','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ices-cc','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/id3','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/id3tool','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/id3v2','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/invada_studio_plugins_lv2','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/iripdb','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jaaa','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jack1','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jack-keyboard','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jack-rack','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jack-tools','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jack2','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jack_capture','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jack_delay','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jackmeter','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jalv','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jamin','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/japa','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jkmeter','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jnoise','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jokosher','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jvmetro','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/kanola','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/kid3','70','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/klick','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/kradio','60','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ladspa_sdk','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/lash','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/lastfm','68','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/libbs2b','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/libebur128','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/lingot','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/listener','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/llcon','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/lmms','157','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/lsmi','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/luppp','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/lv2','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/lv2-mda-metapiano','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/lv2dynparam1','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/lv2file','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/lxmusic','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mac','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/match-vamp-plugin','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mda-lv2','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/meterbridge','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mhwaveedit','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mididings','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/midillo','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/milkytracker','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/millennium-plugins','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/minimodem','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mixxx','714','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mkcue','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mktoc','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mp3blaster','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mp3check','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mp3gain','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mp3splt','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mp3splt-gtk','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mp3val','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mp3wrap','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mpc','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mpd','125','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mpdscribble','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mppenc','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mt-daapd','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/muse','161','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/museeks','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/musepack-tools','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/nas','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ncmpc','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ncmpcpp','158','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/nekobee','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ngjackspa','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/nnls-chroma','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/non','61','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ntk','40','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ocp','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ogmtools','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/opencore-amr','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/opus','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/opus-tools','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/opusfile','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/oss','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/paman','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/paprefs','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pasystray','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/patchage','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pavumeter','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pd','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pd_mrpeach','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/phat','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pianobar','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pithos','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/playmp3list','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pmidi','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/podget','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pogo','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ponymix','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/portmidi','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/puddletag','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pulseaudio-ctl','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pyalsaaudio','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/python-audiotools','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qastools','171','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qjackctl','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qm-vamp-plugins','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qmidiarp','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qmidinet','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qmidiroute','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qmmp','140','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qmmp-plugin-pack','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qsynth','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qtractor','74','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/quodlibet','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rakarrack','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rebmp','40','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/retrovol','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rev-plugins','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rezound','80','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rhythmbox','121','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rosegarden','289','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rplay','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rubberband','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rubyripper','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rumor','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/samplv1','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/sayonara-player','252','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/sbagen','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/schismtracker','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/seq24','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/sfc','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/shell-fm','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/shntool','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/shorten','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/sidplayfp','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/snd','57','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/sonata','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/sonic-visualiser','348','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/sooperlooper','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/soundconverter','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/soundkonverter','95','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/soxr','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/spandsp','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/specimen','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/speex','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/stops','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/streamripper','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/streamtuner','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/sunvox','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/swami','66','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/swh-plugins','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/swh-plugins-lv2','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/synthv1','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/tagtool','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/tap_plugins','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/tapiir','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/transcribe','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/tta','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ttaenc','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/tuxguitar','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/twolame','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/vamp-plugin-sdk','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/vcf_plugins','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/vimpc','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/vmpk','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/vocoder','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/vocoder-ladspa','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/volti','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/volume.app','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/volumeicon','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/volwheel','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/vorbisgain','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/waon','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/wavbreaker','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/whipper','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/whysynth','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/wmusic','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xcfa','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xforge','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmms-ctl','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmms-jack','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmms-ladspa','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmms-mp4','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmms-scrobbler','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmms-shn','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmms-wma','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmms2','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmp','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xsynth-dssi','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xwax','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/yoshimi','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/zita-ajbridge','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/zita-dpl1','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/zita-rev1','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/zynaddsubfx','55','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/zynjacku','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('business/AssociationSubscribersManager','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('business/beancount','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('business/ledger','231','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('business/lemonpos','48','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('business/openerp-client','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('business/openerp-server','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('business/reckon','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('business/skrooge','121','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('business/stansoft','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('business/tryton','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('business/trytond','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/2bwm','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/9menu','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/Alphacube-GTK','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/AlsaMixer.app','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/Ater-MPlayer-skin','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ClipboardViewer','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ComixCursors','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/CurseTheWeather','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/LX-Colors-Revival','126','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/LX-Colors-themes','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/Pulse-Glass','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/PyPanel','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/QtCurve-Gtk2','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/QtCurve-KDE4','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/Vanilla-DMZ','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/Window_Manager_Scripts','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/XDecorations','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/Xfce-Theme-Manager','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/adwaita-qt','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/afterstep','59','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/alarm','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/alltray','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ambiance-radiance-flat-colors-gtk-theme','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/anki','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/arandr','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/arc-firefox-theme','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/arc-icon-theme','57','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/arc-theme','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/arc-theme-Red','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/areao43-icon-theme','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/avant-window-navigator','58','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/aweather','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/awemenugen','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/awesome','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/awesome-appmenu','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/awesome-extra','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/awn-extras','55','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/awoken-icon-theme','439','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/bashish','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/bashrun','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/bashrun2','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/bmpanel2','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/boje-themes','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/bspwm','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/bunsen-themes','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/caffeine-ng','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cairo-compmgr','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cairo-dock','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cairo-dock-plugins','96','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/catfish','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cathexis-suite-themes','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cbatticon','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ccsm','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cellwriter','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ceti-2-theme','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/clearlooks-classic-themes','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/clearlooks-phenix-theme','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/clipit','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cmatrix','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/compiz-boxmenu','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/compton','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/compton-conf','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/conky-manager','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/copyq','49','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cryptkeeper','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/crystal','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/crystalcursors','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ctwm','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cwm','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cwm-openbsd','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cwp','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/dark-aurora-gtk-theme','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/devilspie','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/disper','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/dmenu','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/dmenu2','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/dunst','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/dwm','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/dzen2','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/e16','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/echinus','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ede','58','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ee','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/elementary-xfce','104','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/emerald','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/emerald-themes','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/enlightenment','218','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/evilwm','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/faba-icon-theme','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/faba-mono-icons','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/faenza-cupertino','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/faenza-icon-theme','259','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/faenza-xfce','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/faience-icon-theme','88','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/fbmenugen','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/fbpanel','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/fbrowse-tray','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ffmpegthumbnailer','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/flatpak','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/florence','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/fusion-icon','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/fvwm-crystal','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/fvwm95','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gis-weather','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/glabels','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gmrun','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gmusicbrowser-art','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-menus','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gntp-send','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gr1d','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/grits','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/grun','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gsimplecal','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtk-chtheme','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtk-engines','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtk-engines-unico','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtk-kde4','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtk-nodoka-engine','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtk-theme-Albatross','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtk-theme-Blackbird','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtk-theme-Bluebird','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtk-theme-Greybird','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtk-theme-Orion','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtk1-industrial-engine','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtk1-theme-switch','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtkhash','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gxkb','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/herbstluftwm','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/homerun','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/hsetroot','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/i3','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/i3blocks','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/i3lock','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/i3minator','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/i3pystatus','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/i3situation','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/i3status','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/icewm','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/icewm-themes','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/idesk','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ideviceinstaller','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/imgur-screenshot','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/insync-thunar','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ion','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/j4-dmenu-desktop','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/jwm','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/kbdd','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/kcometen4','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/kde-gtk-config','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/kde44-oxygen-molecule','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/keynav','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/kfaenza-icon-theme','98','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/kover','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/kwalletcli','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lemonbar','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/light','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lumina','523','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/luna-wallpapers','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lxappearance','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lxappearance-obconf','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lxde-common','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lxlauncher','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lxmed','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lxmenu-data','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lxpanel','53','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/macopix','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/maitreya','97','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/matrixgl','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mcwm','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/meliae-icon-theme','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/menumaker','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/menutray','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mhsetroot','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mimi','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/moka-icon-theme','235','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/moka-minimal','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/moka-themes','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mousepad','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/murrine','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/murrine-themes','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/musca','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/myGtkMenu','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/neofetch','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/netwmpager','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/neutral','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nitrogen','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/notification-daemon','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/notion','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/numix-blue-theme','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/numix-folders','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/numix-gtk-theme','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/numix-icon-theme','186','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/numix-icon-theme-circle','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/numix-icon-theme-square','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/numix-ocean-theme','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/numix-pack','61','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/numix-sx-theme','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/obapps','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/obbrowser','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/obconf','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/obkey','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/obmenu','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/obmenu-generator','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/obsession','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/obshutdown','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/openbox','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/openbox-menu','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/openbox-simple-theme','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/openbox-themes','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/oranchelo-icon-theme','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/orchis-gtk-theme','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/paper-gtk-theme','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/paper-icon-theme','214','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/parcellite','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/pclock','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/pdfsam','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/peksystray','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/pekwm','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/perwindowlayoutd','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/pidgin-guifications','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/pidgin-libnotify','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/pidgin-window_merge','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/plank','58','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/plasma-adjustable-clock','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/plasma-applet-daisy','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/plasma-eyasdp','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/plasma-geek-clock','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/plasma-runner-browsefirefoxbookmarks','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/plasma-widget-menubar','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/plasmaxplanet','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/polar-cursor-themes','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/polybar','84','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/protozoa-cursor-themes','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/pspshrink','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ptbatterysystemtray','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ptwit','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/py3status','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/pyful','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/python-specbar','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/pytyle','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/q4wine','81','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/qlipper','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/qmmp-skins','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/qt-recordmydesktop','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/qt5ct','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/qtile','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/qtpass','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/quadkonsole4','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/quickaccess','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/quickswitch-i3','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ratpoison','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/razercfg','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/razorqt','120','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/recoll','72','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/redshift','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/rodent','53','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/rofi','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/root-tail','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/rootactions_servicemenu','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/rsl','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/rss-glx','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/rss-guard','152','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/s1kls','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/sawfish','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/sawfish-pager','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/sawfish-session-dialog','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/screenfetch','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/screenkey','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/screensaver-icon','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/seafile-gui','71','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/searchmonkey','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/sgsearch','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/simon','252','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/siva-flat-themes','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/skb','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/skippy-xd','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/skulpture','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/slack-wallpapers','70','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/slackware-xdg-menu','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/slackware-xdm-theme','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/slock','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/smaragd','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/spectrwm','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ssr','69','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/stalonetray','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/stoken','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/subtle','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/sun','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/superkey-launch','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/sxhkd','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/sync-engine','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/synce-kpm','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/thunar-archive-plugin','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/thunar-dropbox','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/thunar-media-tags-plugin','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/thunar-sendto-clamtk','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/thunar-thumbnailers','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/thunar-vcs-plugin','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/tint2','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/trayer-srg','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/tty-clock','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/uwm','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/variety','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/vbox-runner','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/velox','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/vim-qt','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/vivacious-colors-gtk-icon-theme','225','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/vivacious-colors-gtk-theme','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/vtwm','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/vwm','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wally','52','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wbar','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmCalClock','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmMoonClock','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmSun','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmSunMoon','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmakerconf','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmbackground','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmbiff','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmblob','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmcliphist','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmclock','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmclockmon','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmctrl','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmcube','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmdate','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmdl','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmdrawer','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmfire','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmii','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wminfo','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmmixer','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmmon','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmname','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmnet','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmphoto+','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmpinboard','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmpower','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmsm.app','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmswallow','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmsystemtray','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmtime','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmtimer','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmtop','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmudmount','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmweather','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmweather+','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmwebcam','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmxss','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/x-tile','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xdgmenumaker','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xdgurl','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xdo','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xed','64','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-battery-plugin','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-cpufreq-plugin','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-cpugraph-plugin','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-datetime-plugin','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-dict','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-diskperf-plugin','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-embed-plugin','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-eyes-plugin','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-fsguard-plugin','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-genmon-plugin','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-kbdleds-plugin','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-mailwatch-plugin','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-mount-plugin','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-mpc-plugin','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-netload-plugin','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-notes-plugin','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-places-plugin','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-quicklauncher-plugin','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-sensors-plugin','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-smartbookmark-plugin','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-timer-plugin','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-verve-plugin','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-volumed-pulse','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-wavelan-plugin','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-whiskermenu-plugin','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-windowck-plugin','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-xkb-plugin','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-xmms-plugin','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xipper','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xkblayout-state','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xlunch','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xmms-skins-almond','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xmms-skins-ater','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xmms-status-plugin','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xmobar','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xmonad','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xonclock','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xpenguins','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xsession','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xtitle','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xtrascreenhacks','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xtrlock','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xvkbd','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xwarppointer','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xxkb','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/yad','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/yawp','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/zenity','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/zuki-themes','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/4th','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Bottleneck','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/CImg','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/CVSTools','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Catch','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ChezScheme','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Coin','149','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Editra','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/FreeBASIC','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/GitEye','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Hex2bin','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/J-Link','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/JSONObject','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/KKEdit','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/LLgen','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/MySQL-python','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Naked','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/OpenCobolIDE','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ParadisEO','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Pivy','232','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/PythonToolkit','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/QStlink','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/SDL2_gfx','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/SQLAlchemy','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/SoQt','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Sphinx','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/THE','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/WebStorm','54','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Xdialog','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/acpica','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/afl','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ahven','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/alembic','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/alex','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/alloy','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/amd-app-sdk','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/android-studio','144','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/android-tools','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/anjuta','237','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/antlr2','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/apache-ant','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/apache-ivy','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/apache-jmeter','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/apache-log4cxx','83','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/apache-maven','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/apitrace','120','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/arduino','40','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/argouml','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/arm-binutils','161','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/as31','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/asem51','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/asl','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/asm6809','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/astyle','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/atasm','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/atom','969','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/atom-amd64','178','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/autogen','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/avarice','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/avr-binutils','74','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/avr-gcc','637','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/avr-gdb','115','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/avr8-burn-o-mat','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/avr8-gnu-toolchain','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/avra','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/avrdude','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bacon','100','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bakefile','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bas','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bas55','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bashdb','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bats','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bfg','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bigloo','143','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/binwalk','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/blassic','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bluefish','35','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bmake','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/boost-di','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bpython','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/brackets','66','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bsdiff','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bvi','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/byacc','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bzr','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/camlp4','65','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/camlp5','47','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cargo','335','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cargo-vendor','214','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/casperjs','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cc-tool','45','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cc65','39','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ccl','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cgdb','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cgit','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/check','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/checkbashisms','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/chibi-scheme','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/chicken','163','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cim','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cmocka','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/coccinelle','42','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/codeblocks','201','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/codelite','1170','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/colm','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/colordiff','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/complexity','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/composer','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/conan','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/couchdb','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cpan2tgz','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cppcheck','61','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cppcms','108','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cppdb','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cppunit','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cream','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/crosstool-ng','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cryptoki','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cudatoolkit','394','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cudnn','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cunit','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/curlpp','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cvsd','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cvsps','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cxxtest','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cxxtools','122','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cycler','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dao','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/darcs','220','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dart','109','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dbeaver','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dejagnu','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/devhelp','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/device-tree-compiler','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dfu-programmer','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dhex','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/diakonos','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/diffoscope','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/diffuse','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dkms','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dmd','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/docutils','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dpkg','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dreampie','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dwarf','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/eagle','35','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ebe','49','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ecl','113','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/eclim','39','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/eclipse-cpp','42','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/eclipse-java','35','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/eclipse-jee','72','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/edb-debugger','48','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/egypt','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/electron','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/elixir','54','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/energia','89','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/enki','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/envytools','48','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/epydoc','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/eric','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/erlang-otp','182','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ex-vi','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/extra-cmake-modules','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/f2c','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/facile','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/flawfinder','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/flow','40','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/fossil','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/fpc','205','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/fpc-source','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/freetds','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ftjam','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ftnchek','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/fuel','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/g3log','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gambas3','613','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gambit','158','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gaphor','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gauche','103','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gcclegacy494','2057','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/geany','74','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/geany-plugins','95','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gecode','320','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gedit','64','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gedit-plugins','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/generatorrunner','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gengetopt','39','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gforth','54','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gisto','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/git-cola','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/git-lfs','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gitstats','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/glade','57','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/glm','55','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/global','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gnats','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gnome-common','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gnome-inform7','64','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gnulib','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gnuradio','2033','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gnustep-base','50','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gnustep-make','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gnustep-startup','96','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gob2','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/golang-googlecode-gonet','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/golang-googlecode-gotools','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/google-appengine-gosdk','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/google-appengine-pysdk','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/google-go-lang','172','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/google-webdesigner','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gplcver','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gprolog','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gpsim','81','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gputils','95','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gr-osmosdr','53','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gradle','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/grails','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/groovy','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gsoap','79','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gtest','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gtkdialog','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gtkextra','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/guile-json','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/guile1.8','51','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/hackrf','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/happy','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/henplus','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/hexedit','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/hhvm','4801','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/highlight','44','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/hopper','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/hte','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/hub','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/hugs98','179','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/icecream','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/icemon','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/icon','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/icoutils','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/idea','103','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/idlex','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ignition-math','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/inform','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ioncube-loader','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jam','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jasspa','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/javacc','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jdk','71','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jeex','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jsmin','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/julia','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/julius','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter-ipykernel','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter-ipyleaflet','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter-ipywidgets','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter-nbconvert','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter-nbdime','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter-nbformat','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter-notebook','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter-qtconsole','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter-widgetsnbextension','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter_client','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter_console','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter_core','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jython','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kaaedit','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kconfig-frontends','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kcov','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kdesvn','74','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kdiff3','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kelbt','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kicad','1845','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kicad-doc','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kicad-library','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kiwi','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/komodo-edit','77','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/komodo-ide','81','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kotlin','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/latrace','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/lazarus','484','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/lcov','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ldns','59','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/le','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/leiningen','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/leo','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/leveldb','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/libb2','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/libbitcoin','82','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/libcutl','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/libodb','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/libodb-boost','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/libodb-mysql','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/libodb-pgsql','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/libodb-qt','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/libodb-sqlite','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/libretro-samples','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/libxbgi','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/libxdg-basedir','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/lighttable','144','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/likepython','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/liquibase','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/lmdb','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/logisim','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/love','67','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/love-legacy072','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/love-legacy080','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ltrace','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/lua','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/luajit','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/luarocks','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/lwtools','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/maude','114','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mcrl2','904','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mcu8051ide','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/medit','44','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/meld','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/meld3','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/memchan','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/menhir','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/meson','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mg-openbsd','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ming','63','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mit-scheme','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mkchroot','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mlton','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mm-common','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mockpp','35','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mongodb-compass','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/monkey-studio','145','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mono','1189','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mono-basic','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mono-debugger','47','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mosml','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mphidflash','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mspgcc-legacy','364','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mstch','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mtasc','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mtd-utils','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mutagen','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mysql-pinba','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mysql-workbench','1460','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nant','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ne','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nedit','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/neovim','66','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/netbeans','109','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/newlib-headers','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/newlisp','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nexus','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nickle','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ninja','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ninja-ide','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nodejs','1190','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nose','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/noweb','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/npm2tgz','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nqp','218','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nuget','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/numpy','138','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/numpy-legacy','126','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/numpy-legacy3','128','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/numpy3','140','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nvi','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nwjs','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/obcpl','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ocaml','172','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ocamlbuild','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/odb','135','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ola','372','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/omake','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/omniORB','132','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/omniORBpy','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/oniguruma','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/opencl-headers','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/opencvs','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/opendbx','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/openjdk','1895','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/openjdk6','1717','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/openjdk8','1878','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/openocd','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/osc','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/osmo-sdr','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/p4','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/p4d','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/p4v','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pandas','248','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/parrot','69','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pasdoc','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/patchutils','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pcc','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pd-psql','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/perf','47','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pforth','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/phantomjs','1470','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/php-apcu','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/php-pinba','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/phpDocumentor','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pingendo','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pkgconf','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pkgdiff','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/poedit','59','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/polyml','40','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/portaudio','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/power-architect','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/premake','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/processing','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pstack-expect-gdb','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/psycopg2','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pudb','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pycharm','80','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pyclewn','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pycscope','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pyenchant','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pylint','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pyside-tools','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/python3-cycler','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pyvim','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/qbs','201','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/qconf','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/qt-creator','2658','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/qt-creator2','830','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/quilt','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/racket','1676','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/radare2','109','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ragel','77','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rakudo','381','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rapidsvn','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rarian','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/re2c','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/reckless','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/referenceassemblies-pcl','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/regexxer','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/regina-rexx','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/robomongo','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rstudio-desktop','588','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ruff','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rust','2081','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/s51dude','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sauce','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sbcl','958','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sbt','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/scala','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/schroot','75','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/scite','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/scratch','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sdcc','185','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/setuptools-git','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/shc','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/shed','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/shiboken','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/simavr','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/simulavr','108','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/slibtool','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sloccount','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/smalltalk','75','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/smartgit','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/smartsvn','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/snack','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/snappy','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sonarqube','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/source-highlight','60','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sourcenav','177','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/spdlog','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sphinx_rtd_theme','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/spice-protocol','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/spin','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/spl','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/splint','44','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/spyder','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sqlcl','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sqldeveloper','73','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sqlitebrowser','50','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sqliteman','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/squeak-vm','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/squirrel-sql','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/src-vcs','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/srcML','201','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/srecord','60','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/staruml','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/stgit','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/stlink','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sublime_text','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/svn-all-fast-export','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/swfmill','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/swi-prolog','105','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tcc','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tcludp','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tclvfs','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tea','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/teensy_loader_cli','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/texi2html','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/textadept','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/thunderpad','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tidy-html5','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tig','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tiled-qt','79','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tkcvs','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tkdiff','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tkdnd','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tklib','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tora','203','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/trac','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/trf','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tstoolbox','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tyrutils-ericw','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/u-boot-tools','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ucblogo','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ucpp','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/udis86','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/uemacs','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/uncrustify','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/unifdef','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/unittest-cpp','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/universal-ctags','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/urjtag','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/uz80as','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/vala','46','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/vala-12','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/valgrind','102','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/valkyrie','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/viewvc','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/vile','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/vispatch','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/vscode','412','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/vstudio','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/watchman','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/wheel','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/wxHexEditor','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xa','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xdebug','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xemacs','69','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xmlcopyeditor','45','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xmlstarlet','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xsp','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xtruss','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xvile','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xxdiff','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zeal','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zed','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zile','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zope.interface','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zope.sqlalchemy','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zope.testing','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zulu-openjdk7','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zulu-openjdk8','35','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/0ad','821','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/0ad-data','342','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/3D-Fasteroids','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/4digits','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/4do-libretro','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/7kaa','57','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/BeastieWorker','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/BeastieWorker-levels','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/Chatbot-Eliza','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/Craft-libretro','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/FlightGear','1078','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/FlightGear-data','564','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/Genesis-Plus-GX','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/KAdventure','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/KoboDeluxe','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/OpenJK','216','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/OpenXcom','252','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/PyLotRO','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/Pyfa','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/QuickNES-Core','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/RetroArch','56','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/SLADE','250','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/Scorched3D','211','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/Sokoban','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/SpaceZero','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/Tempus-Irae','35','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/UrbanTerror','319','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/abe','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/abuse','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ace','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/adl','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/adom','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/advancemame','446','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/advancescan','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ags','68','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/aklabeth','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/alephone','109','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/alienarena','228','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/alienwave','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/an','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/angband','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/antimicro','70','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ardentryst','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/armagetronad','63','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/arnold-cpc','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/arx-libertatis','187','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/asc','394','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/asciijump','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/assaultcube','42','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/assaultcube-reloaded','44','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/asteroidsinfinity','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/astromenace','71','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/atanks','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/atari++','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/atari800','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/atari800_roms','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/barrage','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bastet','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/beetle-bsnes-libretro','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/beetle-gba-libretro','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/beetle-lynx-libretro','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/beetle-ngp-libretro','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/beetle-pce-fast-libretro','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/beetle-pcfx-libretro','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/beetle-psx-libretro','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/beetle-saturn-libretro','92','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/beetle-supergrafx-libretro','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/beetle-vb-libretro','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/beetle-wswan-libretro','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/berusky','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/berusky-data','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/berusky2','40','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/berusky2-data','97','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bitfighter','129','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/blastem-libretro','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/blobby2','44','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/blobwars','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bloodcm','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bloodfrontier','148','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/blueMSX-libretro','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bluemoon','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bnes-libretro','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bomberclone','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/boswars','124','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/braincurses','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/brainparty','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/brickout','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/briquolo','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/brutalchess','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bsnes-libretro','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bsnes-mercury','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bstone','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/burgerspace','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bzflag','237','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/caph','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ceferino','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cgames','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cgmadness','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cgoban','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/chessx','198','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/chexquest3','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/chocolate-doom','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/chroma','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/chromium-bsu','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/circuslinux','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ckmame','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/clonekeen','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/colem','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/commandergenius','262','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/connectagram','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/conquest','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cosmic_assault','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cosmosmash','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cowsay','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cpat','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/crack-attack','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/crimson','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/crispy-doom','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/crossfire-client','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cubosphere','109','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/curseofwar','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cutemaze','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cuyo','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cyphesis','88','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/d1x-rebirth','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/d2x-rebirth','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/darkplaces','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/defendguin','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/desmume','68','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dgen','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dhewm3','136','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dice','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/digger','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dolphin-emu','382','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/domination','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/doom_shareware_data','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/doomretro','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/doomsday','932','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/doomseeker','99','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dopewars','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dreamchess','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dreamchess-music','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ds-models','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dunelegacy','158','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dungeon','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dustrac','119','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ecwolf','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/edgar','61','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/eduke32','49','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/eduke32_hires_pack','51','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/eduke32_shareware_data','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/egoboo','92','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/einstein','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ember','2037','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/empire','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/enigma','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/etlegacy','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/etlegacy-data','49','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/eureka','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/extremetuxracer','40','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/exult','136','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/f1spirit','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fbalpha','143','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fceux','61','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ffgo','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fgo','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fgrun','46','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fifengine','296','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fillets-ng','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fillets-ng-data','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/firestorm','94','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fizmo','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/flare','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/flare-game','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fmsx-libretro','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/foobillardplus','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fortune-ASR','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fortune-dharma','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fortune-dune','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fortune-futurama','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fortune_doctor_who','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fortune_firefly','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/freeblocks','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/freeciv','111','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/freedoom','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/freedroid','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/freedroidrpg','72','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/freeminer','585','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/freeorion','1058','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/frogatto','349','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/frotz','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/frozen-bubble','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fs2open','298','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fuse-emulator','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/galaxyv2','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gambatte-libretro','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/game_1945','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/garden','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gargoyle','75','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gemrb','53','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ggmud','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gl-117','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/glPortal','82','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/glbsp','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/glestae','374','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/glob2','173','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/glxosd','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gnonograms','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gnubg','58','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gnugo','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gnuski','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/golly','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/goonies','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gottet','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gpsp-libretro','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/greed','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gtkballs','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gtklife','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gw-libretro','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gzdoom','226','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gzdoom-legacy','231','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/hatari','105','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/hatari_tos_roms','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/haxima-nazghul','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/hedgewars','110','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/heretic_shareware_data','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/hex-a-hop','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/hexalate','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/hexen_demo_data','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/hexglass','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/higan','134','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/hlsdk','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/hnefatafl','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ho1','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/icebreaker','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ifm','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/innoextract','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/instead','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ioquake3','66','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/iortcw','125','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jackhammer','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jag','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/javacpc','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jezzball-kazzmir','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jfsw','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jfsw_demo_data','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jfsw_hires_pack','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jin','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jzintv','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/kardsgt','62','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/kildclient','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/knights','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/koules','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/larn','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/lbreakout2','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/lgogdownloader','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/libretro-2048','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/libretro-desmume','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/libretro-fceumm','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/libretro-handy','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/libretro-lutro','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/libretro-mgba','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/libretro-ppsspp','114','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/libretro-prboom','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/libretro-reicast','145','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/libretro-vecx','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/libretro-yabause','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/lightyears','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/lincity-ng','56','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/liquidwar6','64','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/lmarbles','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/lmpc','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/loemu','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ltris','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/lutris','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/m64py','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/madbomber','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/maelstrom','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mame','2978','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mame2000-libretro','74','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mame2010-libretro','503','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mame2014-libretro','1771','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mangband','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/marathon-data','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/marathon-eternal-data','102','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/marathon-evil-data','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/marathon-infinity-data','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/marathon-red-data','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/marathon-rubicon-data','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/marathon2-data','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mari0','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mars','52','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/meandmyshadow','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mednafen','374','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/megaglest','409','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/megamario','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/meteor-libretro','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mfe','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mgba','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/micropolis','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/minetest','159','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/minetest_game','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mininim','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/miscom','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mog','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/monkey-bubble','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/moon-buggy','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/moria','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mrrescue','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mupen64plus','55','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mupen64plus-libretro','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/myman','112','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nFlood','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nSnake','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/naev','73','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nestopia','101','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nestopia-libretro','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nethack','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/neverball','93','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nevernoid','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nexuiz','153','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ninvaders','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/njam','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nlarn','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/noteye','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/notpacman','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nottetris2','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/npush','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nsudoku','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nsuds','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nuvie','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nxengine-libretro','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/o2em','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/oblige','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/oblige-legacy4','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/odamex','70','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/oolite','124','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/open-adventure','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openarena','78','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openclonk','432','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/opendune','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openjazz','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openlierox','169','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openmortal','46','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openmsx','299','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openmsx-catapult','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openmsx-debugger','40','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openmw','473','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/opensonic','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openspades','115','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openttd','108','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/opentyrian','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openyahtzee','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ostrichriders','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pacman-arena','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pangzero','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/parallel-n64','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pasang-emas','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pcsx-rearmed','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pcsxr','39','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/peg-e','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/penguin-command','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pengupop','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pentagram','85','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/picodrive-libretro','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pinball','44','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pingus','148','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pioneer','411','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pokerth','254','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pom1','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ppsspp','246','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/prboom','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/prboom-plus','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/prosystem-libretro','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pushover','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/puzzles','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pybik','478','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pychess','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pysolfc','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/qmc2','350','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/qstat','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/qtsixa','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/quake3_shareware_data','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/quake_shareware_data','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/quakeforge','96','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/qxw','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/qzdl','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/redeclipse','292','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/rejoystick','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/residualvm','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/retux','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/rezerwar','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/roadfighter','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/robotfindskitten','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/roccat-tools','132','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/rocksndiamonds','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/rogue','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/rott','39','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/sauerbraten','185','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/scid','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/scid_vs_pc','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/scribble','48','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/scummvm','292','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/scummvm-libretro','491','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/sdlpop','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/simplesok','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/simsu','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/singularity','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/skulltag','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/sl','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/smashbattle','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/smc','82','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/sms_sdl','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/snes9x','57','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/snes9x-libretro','52','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/snes9x2002','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/snes9x2005','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/snes9x2010','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/sokowiz','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/solarus','121','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/solarus-quest-editor','105','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/solarwolf','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/speed-dreams','474','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/spring','1659','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/starfighter','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/steam','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/stella','171','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/stella-libretro','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/stockfish','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/stone_soup','729','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/stormbaancoureur','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/stratagus','55','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/stuntrally','602','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/sumwars','118','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/supertux','160','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/supertuxkart','556','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/syobonaction','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/taipan','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/talkfilters','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tanglet','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/teeworlds','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/term2048','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tintin','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tome','119','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tome-ah','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tome-sx','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tong','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/torcs','201','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/trackballs','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/trigger-rally','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/triplea','63','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tt','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tty-solitaire','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tuxfootball','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tuxkart','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tworld','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/typespeed','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tyrquake-libretro','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ucr','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ufoai','322','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/uligo','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/unknown-horizons','78','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/unnethack','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/unvanquished','124','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/uqm','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/uqm_3domusic','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/uqm_remixes','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/uqm_voice','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ut2004','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vba-next','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vbam','63','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vbam-libretro','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vcmi','832','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vcmi-core','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vcmi-resolutions','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vcmi-wog','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vectoroids','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vera','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/virtualjaguar-libretro','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vitetris','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vms-empire','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/voxelands','142','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/wargus','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/warmux','79','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/warsow','243','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/warzone2100','97','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/wesnoth','1127','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/widelands','891','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/wmquake','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/wolf4sdl','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/wordwarvi','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/worldofgoo','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/worldofpadman','200','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xarchon','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xcowsay','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xcpc','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xmahjongg','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xmoto','86','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xonotic','272','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xpilot-ng','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xroar','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xskat','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xspacewarp','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xu4','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xye','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/yabause','56','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/yae','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/yahtzee','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/yamagi-quake2','35','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/yar','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/yetris','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zaz','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zdbsp','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zdoom','94','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zelda-roth-se','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zennode','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/znake','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zod-engine','35','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zoom','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zork','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zsdx','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zsxd','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ztools','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ztrack','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/CreateCloudMap','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/Fiona','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/GMT','122','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/Geohash','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/OWSLib','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/SFCGAL','303','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/Shapely','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/TauDEM','86','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/basemap','55','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/cartopy','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/cligj','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/foxtrotgps','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/gdal','416','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/geogig','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/geogig-py','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/geographiclib-python','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/geojson','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/geopy','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/geos','77','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/geoserver','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/geoserver-control-flow','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/geoserver-css','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/geoserver-javascript','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/geoserver-oracle','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/geoserver-pyramid','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/geoserver-python','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/geoserver-wps','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/gmapcatcher','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/google-earth','47','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/gpsbabel','124','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/gpsd','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/grass','297','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/josm','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/laszip','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/libLAS','87','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/libgeotiff','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/librasterlite','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/libspatialindex','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/libspatialite','54','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/mapnik','1422','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/merkaartor','98','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/mobac','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/navit','61','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/opencpn','350','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/opengrads','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/openorienteering-mapper','197','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/osgEarth','593','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/osm-gps-map','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/osm2pgrouting','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/osm2pgsql','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/pdal','380','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/pgrouting','78','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/pgsql-ogr-fdw','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/pointcloud','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/polyline','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/postgis','49','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/proj','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/pyproj','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/pyshp','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/pyspatialite','87','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/python-mapnik','165','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/pytrainer','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/qgis','1607','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/qlandkartegt','235','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/qmapshack','349','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/rasterio','47','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/readosm','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/routino','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/saga-gis','980','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/spatialite-tools','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/spatialite_gui','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/tangogps','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/udig','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/viking','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/xplanet','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/Alchemy','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/Blender','766','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/CairoSVG','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/FreeCAD','1631','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/GPS','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/GraphicsMagick','53','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/HotShots','40','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/LDView','56','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/LDraw_data','57','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/OpenCASCADE','2039','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/Pixie','76','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/PyMOL','84','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/Shellpic','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/SweetHome3D','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/VariCAD','40','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/XnViewMP','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/advancecomp','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/aewan','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/asciiquarium','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/autotrace','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/baires','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/bdfedit','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/blender','84','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/brlcad','984','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/converseen','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/darktable','177','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/dblatex','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/dcraw','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/dia','134','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/digikam','682','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/discwrapper','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/djview4','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/dvisvgm','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/enblend-enfuse','130','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/entangle','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/evolus-pencil','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/exact-image','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/exif','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/exiftags','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fbida','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fbpic','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fbv','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/feh','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fig2sxd','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fim','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/findimagedupes','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/flpsed','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fontforge','162','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fontmatrix','66','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fotowall','45','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fotoxx','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/freewrl','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/frescobaldi','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fritzing','284','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fyre','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/g3dviewer','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gallery','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gbdfed','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gifsicle','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gimagereader','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gimp-feca_hdr-plugin','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gimp-lqr-plugin','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gimp-plugin-bimp','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gimp-wideangle-plugin','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gliv','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gmic','447','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gnofract4d','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gnome-themes-extras','106','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gnome-web-photo','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/goocanvas','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gource','35','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gpaint','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gpicview','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gpscorrelate','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/grafx2','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/graph-easy','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/graphite2','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/graphviz','109','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gthumb','166','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gtkam','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/hugin','420','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/iccprofiles','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/imgmin','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/inkscape','980','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/jUploadr','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/jhead','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/jp2a','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/jpeg2ps','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/jpegoptim','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/k3d','1923','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/kamerka','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/kim4','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/kphotoalbum','257','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/kuickshow','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/lcdtest','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/lddot','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/ldglite','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/leocad','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/leocad_pieces','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/librecad','533','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/lilypond','150','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/luminance-hdr','147','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/luxrender','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/maim','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/makehuman','42','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mapcrafter','74','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mcomix','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/meh','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/meshlab','226','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/metapixel','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mftrace','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mirage','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mitsuba','594','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mitsuba-blender','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mlbrot','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mozjpeg','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mtpaint','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mypaint','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/nomacs','93','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/nvidia-cg-toolkit','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/openclipart','285','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/opencollada','359','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/opencolorio','77','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/openimageio','294','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/openscad','221','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/optipng','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/oyranos','50','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/paraview','4984','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pdf2png','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pdf2svg','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pencil3','85','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/phatch','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pho','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/photivo','181','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/photoprint','55','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/photoqt','42','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/phototonic','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/plotutils','63','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pngcrush','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pngquant','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/potrace','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/povray','224','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/psftools','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pyformex','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pygifme','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pygraphviz','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/qcad','962','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/qcomicbook','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/qelectrotech','350','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/qiv','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/qrencode','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/raw-thumbnailer','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/rawstudio','58','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/rawtherapee','357','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/resynthesizer','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/ristretto','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/scantailor','268','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/scour','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/screencloud','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/screengrab','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/scrot','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/separate+','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/shotwell','77','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/shutter','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/sng','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/svp','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/sxiv','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/tesseract','100','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/textext','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/tgif-QPL','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/tintii','55','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/tkpng','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/triangulizor','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/ttyload','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/tuxpaint','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/tuxpaint-stamps','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/ufraw','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/uniconvertor','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/unpaper','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/urbanlightscape','51','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/viewnior','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/vips','73','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/vsxu','270','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/vuescan','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/vulkansdk','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/wavelet-denoise','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/wayland-egl','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/wayland-protocols','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/whyteboard','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xbmbrowser','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xcalib','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xdot','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xli','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xmedcon','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xpe','59','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xzgv','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/yacreader','259','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/yafaray','46','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/yafaray-blender','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/yagf','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/yed','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/yesplz','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/zbar','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/zgv','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/aldo','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/aprx','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/baudline','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/chirp','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/cwdaemon','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/cwstation','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/demorse','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/direwolf','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/dxcc','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/ebook2cw','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/ebook2cwgui','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/fldigi','61','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/gqrx-sdr','109','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/gridloc','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/grig','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/gsmlib','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/hamlib','49','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/klog','35','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/linlogbook','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/linpsk','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/nec2c','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/psk31lx','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/qradiopredict','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/qrq','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/rtl-sdr','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/soundmodem','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/svxlink','70','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/svxlink-sounds-en_US-heather-16k','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/tlf','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/tqsl','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/twhamqth','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/unixcw','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/wxapt','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/xanalyser','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/xdemorse','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/xdx','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/xfhell','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/xgridloc','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/xlog','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/xnec2c','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/xpsk31','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/xwxapt','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/ghc','2917','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-Crypto','46','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-Diff','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-FindBin','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-GLURaw','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-GLUT','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-HTTP','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-HUnit','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-JuicyPixels','134','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-ObjectName','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-OpenGL','88','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-OpenGLRaw','265','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-QuickCheck','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-SHA','91','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-ShellCheck','310','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-StateVar','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-Tensor','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-X11','54','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-X11-xft','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-aeson','73','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-ansi-terminal','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-ansi-wl-pprint','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-asn1-encoding','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-asn1-parse','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-asn1-types','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-async','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-attoparsec','51','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-base-compat','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-base16-bytestring','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-base64-bytestring','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-blaze-builder','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-blaze-html','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-blaze-markup','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-bmp','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-byteable','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cabal-install','40','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-case-insensitive','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cereal','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cgi','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cipher-aes','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cipher-des','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cipher-rc4','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-clock','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cmark','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cmdlib','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-colour','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-composition','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-conduit','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-connection','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cookie','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-crypto-cipher-types','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-crypto-numbers','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-crypto-pubkey','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-crypto-pubkey-types','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-crypto-random','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cryptohash','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cryptonite','87','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-curl','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-data-default','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-data-default-class','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-data-default-instances-base','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-data-default-instances-containers','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-data-default-instances-dlist','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-data-default-instances-old-locale','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-data-ordlist','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-dataenc','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-deepseq-generics','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-digest','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-dlist','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-enclosed-exceptions','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-env-locale','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-exceptions','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-extensible-exceptions','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-fail','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-fgl','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-filemanip','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-fixed','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-foundation','90','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-ghc-paths','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-gloss','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-gloss-rendering','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-graphviz','160','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-haddock-library','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-half','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hashable','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hashed-storage','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-haskell-src','46','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-highlighting-kate','142','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hinotify','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hostname','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hourglass','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hslogger','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hslua','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-html','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-http-client','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-http-client-tls','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-http-types','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hxt','106','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hxt-charproperties','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hxt-regex-xmlschema','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hxt-unicode','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-idna','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-json','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-lcs','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-lifted-base','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-memory','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-mime-types','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-mmap','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-mmorph','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-monad-control','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-monads-tf','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-mtl','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-multipart','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-network','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-network-uri','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-old-locale','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-old-time','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-pandoc-types','75','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-parallel','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-parsec','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-pem','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-polyparse','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-primitive','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-publicsuffixlist','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-punycode','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-random','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-regex-applicative','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-regex-base','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-regex-compat','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-regex-compat-tdfa','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-regex-pcre-builtin','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-regex-posix','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-regex-tdfa','110','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-resourcet','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-sandi','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-scientific','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-securemem','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-semigroups','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-setlocale','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-shellish','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-socks','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-split','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-stm','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-streaming-commons','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-strict','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-stringprep','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-stringsearch','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-syb','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-tagged','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-tagsoup','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-tar','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-temporary','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-test-framework','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-test-framework-hunit','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-test-framework-quickcheck2','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-texmath','115','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-text','81','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-text-icu','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-tf-random','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-tls','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-transformers-base','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-transformers-compat','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-unix-compat','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-unordered-containers','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-utf8-string','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-vector','100','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-void','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-wl-pprint-text','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-x509','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-x509-store','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-x509-system','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-x509-validation','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-xml','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-yaml','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-zip-archive','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-zlib','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/hscolour','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/xmonad-contrib','176','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Atlas-C++','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/BeautifulSoup','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/BeautifulSoup4','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Botan','289','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/CCfits','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/CEGUI','128','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/CEGUI0.7','97','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/CGAL','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/CherryPy','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/CoinMP','206','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/CondConfigParser','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ConfigParser','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Crypt-SSLeay','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/DateTime','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/DevIL','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/DirectFB','85','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ETL','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/FormEncode','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/FreeImage','67','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/GLee','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/GeoIP-Python','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Geraldo','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/GitPython','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/GtkD','409','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/HawkNL','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Hypodermic','78','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/IMAPClient','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Impacket','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Ipopt','63','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Jinja2','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Kivy','94','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/LucenePlusPlus','483','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/MyGUI','268','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Nevow','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ORBit2','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/OpenAL','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/OpenBLAS','269','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/OpenCSG','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/OpenSceneGraph','949','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/PDCurses','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/PDFlib-Lite','53','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/PrettyTable','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/PyODE','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/PyOpenGL','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/PyQt5','969','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/PyQwt','56','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/PySoundCard','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/PyXML','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/PyYAML','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Pykka','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/QDarkStyle','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/QUSB','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/QtUsb','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/RxCpp','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SDL2','39','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SDL2_image','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SDL2_mixer','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SDL2_net','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SDL2_ttf','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SDL_Pango','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SDL_gfx','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SDL_sound','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SFGUI','62','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SFML','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SQLObject','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SimGear','395','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SlowAES','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Smarty','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/VTK','2746','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/VitaMTP','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/WebOb','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ZConfig','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ZODB3','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/activemq-cpp','328','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/adodb','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/afflib','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/agg','73','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/alglib','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/allegro','66','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/allegro4','45','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/alure','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ant-contrib','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/apiextractor','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/appmenu-qt','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/appstream-glib','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/aqbanking','76','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/argon2','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/argtable','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/asio','91','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/assimp','159','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/async','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/atlas','552','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/aubio','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/audiere','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/audioread','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/avbin','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/avr-libc','102','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/backports-ssl-match-hostname','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/bamf','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/barry','101','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/batinfo','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/bcdoc','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/belle-sip','56','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/blas','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/boolstuff','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/botocore','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/buddy','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/bullet','222','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/bwidget','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/bzrtp','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/c++-gtk-utils','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/c-ares','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cal3d','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/capnproto','60','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cbase','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cchardet','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ccrtp','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cdk','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cffi','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cfitsio','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/chromaprint','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/clalsadrv','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cln','92','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/collada-dom','204','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/compface','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/compiz-bcop','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/compiz-plugins-experimental','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/compiz-plugins-extra','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/compiz-plugins-main','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/compizconfig-python','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/configobj','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/confuse','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/console_bridge','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cpp-netlib','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cpprestsdk','580','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cracklib','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/crossguid','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cryptopp','44','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ctemplate','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cuneiform','81','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/curses_ex','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/daq','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/darcsver','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/db46','66','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/dbus-cpp','42','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/dcadec','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/dietlibc','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/discogs-client','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/dlib','123','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/dotconf','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/double-conversion','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/dropbox-python','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/edelib','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/efivar','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/efl','611','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/enca','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/enum34','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/eris','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/evernote-python','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/exempi','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/exiftool','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/facebook-python','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fcgi','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fcl','148','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ffms2','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fifechan','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/flashplugin-lahf-fix','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/flatbuffers','42','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/flatzebra','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/flickcurl','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fltk','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fmodapi','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fox-toolkit','88','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fox-toolkit-devel','171','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fpconst','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/freealut','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/freexl','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/frei0r','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ftgl','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ftplib','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/futures','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fuzzylite','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gallerize','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ganv','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gaphas','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gavl','50','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gazebo','6393','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gcab','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gconfmm','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gdata','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gdl','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/genshi','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/geshi','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gevent','45','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gf2x','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gflags','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/giblib','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/girara','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gitdb','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gitdb2','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gl2ps','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/glfw','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/glfw3','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/glog','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gloox','99','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gmtk','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gnet','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gnome-media','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gnome-python2-gconf','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gnome-sharp','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gnome-vfsmm','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gnonlin','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/goffice','74','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/goffice0.8','81','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/google-fruit','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gperftools','50','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/grantlee-qt5','52','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/greenlet','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/grx','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gsm','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gsnmp','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gssdp','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gstreamer-editing-services','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gstreamermm','91','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtk-aurora-engine','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtk-qt-engine','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtk-sharp','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtkdatabox','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtkglarea','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtkglext','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtkglextmm','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtkhotkey','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtkhtml','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtkimageview','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtksourceview','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtksourceview3','39','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtkspell3','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gts','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/guichan','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gupnp','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/guppy','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gwenhywfar','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hdf','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hdf5','114','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hidapi','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hinnant-date','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hiredis','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hoedown','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hpx','748','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/html5lib','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/htmlcxx','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/http-parser','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hug','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hurry-filesize','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hyphen','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/id3lib','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ignition-math2','42','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ignition-transport','50','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/iksemel','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/imageloader','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/imlib','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/imlib2','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/imlib2_loaders','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/iniparser','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ipaddress','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/irrlicht','94','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/isodate','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/itcl','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/jai','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/jai-imageio','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/jansson','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/jbig2dec','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/jbig2enc','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/jbigkit','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/jcal','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/jreen','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/json-glib','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/jsoncpp','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/kaadbg','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/kashmir','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lablgtk','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lame','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lapack','188','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lapack-atlas','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lastfmlib','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lensfun','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/leptonica','69','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/levmar','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lgi','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libAfterImage','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libaacs','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libaaio','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libaal','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libabw','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libalkimia','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libantlr3c','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libaosd','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libasr','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libass','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libast','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libaudclient','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libavc1394','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libb64','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libbde','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libbdplus','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libbluray','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libbonobo','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libbonoboui','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libbsd','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libburn','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libc++','54','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcacard','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcangjie','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libccd','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcdaudio','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcdr','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcec','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libchewing','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcitadel','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libclaw','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcli','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcmis','104','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcoap','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcompizconfig','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libconfig','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcpuid','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcryptui','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcsv','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcue','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcuefile','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdaemon','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdbh2','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdbi','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdbi-drivers','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdbus-c++','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdbusmenu','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdc1394','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdca','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdesktop-agnostic','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdivecomputer','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdmapsharing','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdmtx','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdockapp','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdsk','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdv','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdvbcsa','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdvbpsi','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdvdcss','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdynamite','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libe-book','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libearth','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libeatmydata','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libebml','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libechonest','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libedit','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libee','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libeot','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libesedb','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libesmtp','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libestr','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libetonyek','160','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libetpan','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libev','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libevhtp','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libevt','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libevtx','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libewf','35','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libexif-gtk','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libexttextcat','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfaketime','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfastjson','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfdk-aac','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libffado','94','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfilezilla','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfilteraudio','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfixposix','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfm','47','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfm-extra','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfprint','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfreehand','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfsntfs','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libftdi','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libftdi1','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfvde','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfwnt','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfwsi','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libg15','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libg15render','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libg3d','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgadu','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgaiagraphics','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgaminggear','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgcrypt15','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgda','138','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgdiplus','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgee','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgee1','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgexiv2','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgig','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgit2','71','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgit2-glib','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgksu','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libglademm','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libglpng','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgltf','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgme','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnome','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnomecanvas','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnomecanvasmm','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnomecups','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnomemm','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnomeprint','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnomeprintui','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnomeui','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnomeuimm','61','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgraphqlparser','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgringotts','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgsasl','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgtop','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgusb','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libharu','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libhid','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libhoard','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libhtp','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libicns','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libidn2','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libiec61883','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libilbc','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libinfinity','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libinput','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libinstpatch','39','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libiptcdata','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libircclient','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libisoburn','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libisofs','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libixion','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libixp','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libjson-rpc-cpp','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libkate','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libkface','39','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libkgeomap4','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libkml','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libkqueue','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libkvkontakte','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblangtag','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblightgrep','71','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblinebreak','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblnk','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblo','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblogging','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblqr','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblrdf','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblscp','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmaa','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmatroska','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmediainfo','165','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmemcached','137','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmicrohttpd','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmikmod','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmimedir','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmimic','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libminizip','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmirage','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmms','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmodbus','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmodplug','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmp3splt','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmp4v2','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmpd','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmpdclient','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmpeg2','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmrss','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmseed','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmsiecf','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmspack','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmspub','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmusicbrainz','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmusicbrainz3','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmwaw','177','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmygpo-qt','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libnet','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libnfs','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libnice','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libnids','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libnova','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libnxml','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liboauth','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libofa','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libofx','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liboglappth','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libolecf','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liboop','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libopenraw','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libopenshot','136','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libopenshot-audio','49','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libopusenc','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liborcus','154','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liborigin','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libosinfo','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libosmo-dsp','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libotf','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libotr','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libowfat','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libp11','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpagemaker','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpano13','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpaper','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpar2','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpeas','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpff','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpgf','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpng-legacy12','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpqxx','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpseudo','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpst','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpwquality','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libqcow','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libqinfinity','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libqsqlpsql','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libquicktime','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libquvi','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libquvi-scripts','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libqxt','80','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libr3','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librapi2','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librcd','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libreadline-java','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libregf','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librelp','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librep','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libreplaygain','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librevisa','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librfm5','42','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librra','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librsync','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librtfcomp','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libscca','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libscrypt','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsearpc','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libseccomp','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libserialport','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libshout','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsidplay2','70','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsidplayfp','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsieve','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsigc++-legacy12','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsigrok','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsigrokdecode','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsigscan','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libskk','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libslack','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsmbios','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsmdev','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsmf','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsmi','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsmraw','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsodium','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libspectrum','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libstatgrab','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libstrophe','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsuinput','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsunpinyin','42','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsynce','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtar','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtecla','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtelnet','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtermkey','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libticables2','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libticalcs2','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libticonv','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtifiles2','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtomcrypt','53','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtommath','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtorrent','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtorrent-rasterbar','209','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtsm','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtubo0','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtunepimp','46','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtxc_dxtn','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libu2f-host','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libuchardet','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libucil','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libunibreak','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libunicap','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libunicapgtk','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libunique','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libunique3','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libunwind','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libupnp','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liburcu','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libuv','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvdpau-va-gl','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libversion','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvhdi','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvidcap','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libviper','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvirt','213','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvirt-glib','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvirt-python','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvmdk','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvmime','348','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvorbisidec','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvshadow','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvslvm','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvterm','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libwacom','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libwebp','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libwebsocketpp','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libwebsockets','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libwfut','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libwhisker2','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libwnck3','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libwps','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libx86emu','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libxfcegui4','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libxkbcommon','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libxmi','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libxml++','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libxmp','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libxnvctrl','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libyubikey','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libzdb','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libzen','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libzookeeper','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libzrtpcpp','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lilv','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/linphone-srtp','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/live555','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lksctp-tools','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lockdev','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/log4c','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/log4cplus','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/log4cpp','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/log4shib','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/logilab-astng','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/logilab-common','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lua-MessagePack','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lua-cmsgpack','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lua-filesystem','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lua-lpeg','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lua-md5','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lua-mpack','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lua-zlib','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/luacrypto','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/luadbi','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/luaevent','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/luaexpat','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/luasec','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/luasocket','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lunatic-python','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lvtk','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lxqt-build-tools','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/manubulon-snmp-plugins','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mariadb_client','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mathgl','76','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/matplotlib','44','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mbedtls','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mdds','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/meanwhile','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/menu-cache','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mercator','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mini18n','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/miniupnpc','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mongo-c-driver','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mongo-cxx-driver','194','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mowitz','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mpir','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/msgpack-c','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/msx264','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/muParser','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mujs','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/munkres','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/musicbrainzngs','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mxml','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mygpoclient','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mysql++','46','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mysql-connector-c++','111','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mysql-connector-python','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mysqltcl','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/nacl','68','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/netcdf','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/nextaw','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/nini','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/nltk','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/node-xoauth2','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/npth','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/nspluginwrapper','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ntl','474','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/nx-libs','185','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/oath-toolkit','64','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ocaml-batteries','42','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ocaml-bisect','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ocaml-camomile','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ocaml-findlib','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ocaml-ounit','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ode','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ogre','620','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ois','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/openbabel','170','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/opencv','1000','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/opendht','60','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/opensaml','83','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ordereddict','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/p4api','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pangox-compat','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pcre2','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pexpect','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pgplot','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pgtcl-ng','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/php-imagick','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/php-memcache','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/php-memcached','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/php-mssql','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/php-oci8','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/php-pdo_dblib','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/php-pgsql','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/php-redis','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/php-ssh2','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/php-stomp','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/physfs','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pidgin-mra','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pies','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pies2overrides','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pjproject-ring','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pjsip','60','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/platform','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/plib','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/plplot','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/plumbum','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/plyr','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/png++','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/poco','390','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/podofo','65','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/polarssl','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/polyclipping','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/poppler-qt5','107','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/process-cpp','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/progressbar','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/properties-cpp','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/protobuf-c','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/psqlodbc','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pthsem','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ptlib','45','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ptypes','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pugixml','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/purple-plugin_pack','49','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pyPEG2','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pyacoustid','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pycparser','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pyechonest','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pyftpdlib','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pyjf3','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pykdtree','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pylast','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pylibpcap','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pymdstat','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pymediainfo','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pyqode.core','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pysendfile','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pyte','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python-efl','286','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python-gnupg','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python-librtmp','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python-ntlm','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python-xlrd','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python3-PyQt5','1017','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python3-PyYAML','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python3-dvdvideo','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python3-pillow','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qmltermwidget','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qoauth','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qt-assistant-compat','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qt3','531','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qt5','13259','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qt5-webkit','2823','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qtermwidget','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qtmimetypes','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/quazip','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/quesoglc','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qutepart','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qwt','67','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qwt5','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qwtplot3d','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qwtpolar','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rabbitmq-c','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rapidjson','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/raptor','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rarfile','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rep-gtk','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/reportlab','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rest','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/restbed','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rhino','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ring-daemon','164','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ring-lrc','124','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rlog','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rocksdb','608','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rpyc','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rrdtool','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rtorrent-python','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rudiments','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rxtx','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/sdformat','87','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/secp256k1','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/serd','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/setproctitle','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/shairplay','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/simbody','544','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/skstream','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/slib','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/slv2','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/smmap','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/smmap2','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/soil','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/sord','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/soundtouch','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/sparsehash','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/sqlpp11','46','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/sqlpp11-connector-mysql','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/sqlpp11-connector-sqlite3','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/squish','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/sratom','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/stfl','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/stk','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/suil','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/swfdec','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/t4k_common','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tDOM','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tamu_anova','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tbb','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tcl-inotify','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tclap','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tcllib','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tff','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tinyxml','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tinyxml2','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tkimg','53','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tls','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tlsh','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tmdb3','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tolua++','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/toro','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/totem-pl-parser','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/trader','35','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tre','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/trilinos','514','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tsocks','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ucl','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ucommon','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/unarr','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/unibilium','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/unixODBC','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/upscaledb','257','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/uptime','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/urdfdom','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/urdfdom_headers','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/uriparser','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/utfcpp','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/uthash','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/uuid','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/varconf','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/vdpau-video','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/vid.stab','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/vigra','309','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/virglrenderer','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/vsqlite++','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/vte3','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/vte3-ng','42','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wayland','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wcslib','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/web.py','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/webkit2gtk','7899','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/webkitgtk','4833','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/webkitgtk3','6365','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wfmath','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wjelement','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ws4py','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wv','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wvstreams','39','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wxGTK','116','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wxGTK3','287','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wxPython','316','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wxPython3','727','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wxSQLite3','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wxX11','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wxsvg','55','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xalan','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xalan-c','148','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xapian-bindings','49','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xawplus','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xblas','79','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xcb-util-xrm','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xerces','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xerces-c','97','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xf86-input-libinput','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xforms','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xine-plugin','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xlt','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xml-security-c','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xmlgen','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xmlrpc-c','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xmlsec','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xmltooling','71','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xmpppy','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xulrunner','2152','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xylib','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/yajl','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/yaml-cpp','68','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/yaml-cpp0.3','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/yaz','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zbase32','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zc.lockfile','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zdaemon','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zeromq','35','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zfec','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zita-alsa-pcmi','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zita-convolver','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zita-resampler','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zope.component','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zope.configuration','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zope.event','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zope.hookable','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zope.i18nmessageid','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zope.location','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zope.proxy','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zope.schema','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zope.security','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zsi','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zziplib','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/Colormake','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/KeePass','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/KeePassHttp','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/QCMA','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/abduco','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/abgx360','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/abgx360gui','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ardesia','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ascii','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/asr-manpages','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/asterixInspector','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/bbrun','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/bchunk','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/bdf2psf','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/bdfresize','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/beep','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/biosdisk','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/birthday','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/bodr','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/boncuk','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/boxes','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/brewtarget','88','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/bsdmainutils','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/bsdsfv','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ccze','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/cflow','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/chemical-mime-data','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/chntpw','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ciso','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/cksfv','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/clawsker','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/conkyforecast','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/cups-windows','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/cwiid','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/dbview','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/dcc','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/deb2tgz','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/discount','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/dmg2img','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/doclifter','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/dos2unix','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/drg2sbg','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/dvtm','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/dwdiff','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/esniper','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fbcat','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fbdump','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fcitx','56','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fcitx-configtool','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fcitx-mozc','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fcitx-qt5','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fcrackzip','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/figlet','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/figlet-contributed','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/figlet-international','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/flashrom','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/floating-IPS','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fribid','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/funny-manpages','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/g15composer','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/g15daemon','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/g15stats','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/gcolor2','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/gcp','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/glogg','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/gnome-mime-data','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/gourmet','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/gr-iqbal','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/grabc','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/gramps','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/grc','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/gxmessage','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/hashkill','62','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/heimdall','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/heyu','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/hoorex','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/html2ps','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ibus','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ibus-anthy','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ibus-chewing','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ibus-hangul','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ibus-pinyin','50','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ibus-qt','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ibus-skk','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ibus-sunpinyin','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ibus-unikey','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/igal2','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/iscan-firmware','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/iscan-proprietary-drivers','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/jmri','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/kasumi','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/kcm-fcitx','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/kde_cdemu','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/keychain','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/klibc','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/krename','39','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/kronometer','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/lbdb','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/libydpdict','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/linux_logo','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/lppf','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ltunify','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/mdic','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/megatunix','49','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/moedict-desktop.app','63','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/moreutils','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/mosquitto','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/myclippings','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/nfoview','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/nixnote','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/no-more-secrets','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/nvclock','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/opencc','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ophcrack','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/owfs','45','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/paperkey','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/par','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/par2cmdline','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/par2cmdline-tbb','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/pinfo','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/po4a','42','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/portecle','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/projectM','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/protobuf','35','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/protobuf3','131','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/qtkeychain','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/qtspell','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ramdefrag','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/rarcrack','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/readesm','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/recode','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/rlwrap','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/rsibreak','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/rsnapshot','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/samdump2','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/sbo-templates','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/scim-chewing','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/sdcv','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/seahorse-plugins','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/sigrok-firmware-fx2lafw','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/slop','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/snow','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/solunar','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/spotlighter','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ssss','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/stardict','90','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/stardict-tools','35','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/subsurface','521','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/sutils','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/tal','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/tegaki-zinnia-japanese','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/texscythe','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/toilet','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/torrentinfo','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/txt2tags','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/uni2ascii','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/utimer','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/vdpauinfo','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/wcd','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/weather','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/wkhtmltopdf','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/wmbday','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xapian-omega','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xbanish','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xca','55','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xcape','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xclip','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xdelta','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xdelta3','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xvnkb','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xwinwrap','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/yapet','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/yara','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ydpdict','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ykpers','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/yubikey-personalization-gui','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/zinnia','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/zyGrib','176','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/AtomicParsley','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/DivFix++','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/ExMplayer','68','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/Gem','219','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/JSampler','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/MP3Diags','81','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/MediathekView','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/Mopidy','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/MuseScore','648','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/OpenLP','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/OpenPHT','2003','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/QMPlay2','120','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/Yolo','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/a2jmidid','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/aces-dev','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/aces_container','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/acetoneiso','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/aegisub','278','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/anyremote','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/apetag','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/audiopreview','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/avidemux','475','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/bdtools','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/beets','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/bino','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/bombono-dvd','296','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/cantata','131','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/castnow','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/ccxstream','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/cfourcc','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/cinelerra','257','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/clamz','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/coriander','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/cowbell','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/cuetools','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/devedeng','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/dirac','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/dsmidiwifi','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/dssi','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/dvbsnoop','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/dvd-slideshow','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/dvdauthor','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/dvdbackup','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/dvdrip','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/dvdstyler','103','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/dvgrab','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/emoc','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/exaile','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/farsight2','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/farstream','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/fdkaac','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/ffmpeg','255','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/ffmpeg2theora','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/filebot','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/flashplayer-plugin','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/flowblade','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/flvtool2','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/freshplayerplugin','46','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gaupol','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gecko-mediaplayer','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/get_iplayer','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gigedit','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gimp-gap','213','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/glyr','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gmidimonitor','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gmtp','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gnash','569','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gnome-mplayer','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gnupod','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/google-talkplugin','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gpac','106','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gpodder','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gst-libav','222','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gst-plugins-bad','127','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gst-plugins-ugly','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gst0-ffmpeg','105','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gst0-plugins-bad','102','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gst0-plugins-ugly','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gtick','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gtk-recordmydesktop','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gtkpod','125','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/guvcview','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/hexter','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/huludesktop','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/iat','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/imagination','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/ivtv-utils','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/jlscp','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/kaffeine','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/kaffeine-mozilla','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/kdenlive','116','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/l-smash','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/lightspark','273','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/linuxsampler','39','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/lircaile','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/lives','153','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/lsdvd','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/lwks','49','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/makemkv','40','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mediainfo','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mediatomb','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mimms','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/minidlna','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/minitube','62','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/miro','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mjpegtools','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mkclean','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mkvalidator','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mkvtoolnix','327','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mlt','74','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mozplugger','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mp3fm','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mpgtx','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mplayer-codecs64','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mpv','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mtpfs','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/munt','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/musique','60','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mythplugins','98','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mythtv','542','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/nightingale','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/nrg2iso','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/nted','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/obs-studio','86','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/oggconvert','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/oggvideotools','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/openshot','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/opera-ffmpeg-codecs','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/parole','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/pepperflash-plugin','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/picard','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/picard-plugins','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/playerctl','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/plex-home-theater','1295','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/plexmediaserver','39','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/pngnq-s9','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/podcastparser','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/popcorntime','58','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/pragha','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/pyradio','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/qsampler','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/radiotray','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/ripit','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/ripperX','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/rosa-media-player','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/rtmpdump','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/schroedinger','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/serviio','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/sfnt2woff','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/shotcut','149','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/simpleburn','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/sinthgunt','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/smpeg','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/smplayer','70','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/smtube','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/spek','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/spotify','46','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/stills2dv','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/subtitlecomposer','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/subtitleeditor','610','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/subtitleripper','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/tabu_audio_player','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/telepathy-farstream','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/telepathy-glib','76','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/telepathy-qt','157','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/tivodecode','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/tomahawk','282','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/totem','64','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/tovid','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/transcode','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/tstools','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/tvheadend','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/tvtime','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/ucview','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vcdimager','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/videocut','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vitunes-mplayer','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vivaldi-codecs-ffmpeg-extra','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vkeybd','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vlc','388','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vlsub','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vobcopy','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/w_scan','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/whistle','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/winff','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/wxcam','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/x264','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/x265','145','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/xawtv','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/xjadeo','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/xmltv','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/xmms-crossfade','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/xvid4conf','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/xvidcore','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/xvst','115','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/yarock','90','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/youtube-viewer','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/zvbi','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/3proxy','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ApacheDirectoryStudio','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/BitchX','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/CherryMusic','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ClusterSSH','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/DarTui','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/DenyHosts','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/EarthReader-Web','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/Electrum','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/Electrum-DASH','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/FireWorks','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/Flootty','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/Free-Cinema','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/GeoIP','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/GhostInTheMail','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/LOIC','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/MEGAsync','127','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/NetworkManager-openconnect','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/NetworkManager-openvpn','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/NetworkManager-pptp','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/NetworkManager-vpnc','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/PAC','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/Pafy','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/PaleMoon','1604','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/PyRSS2Gen','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/PySocks','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/Pyrit','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/QuiteRSS','80','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/RSSOwl','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/RYMTracks','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/Signal-Desktop','45','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/SocksiPy','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/SoulseekQt','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/WireGuard','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/aMule','89','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/aggregate','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/aiccu','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/aircrack-ng','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/airpwn','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/aldryn-client','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/amap','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/amavisd-new','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/aoetools','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/argus','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/argus-clients','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/aria2','213','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/arm','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/arno-iptables-firewall','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/arora','45','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/arp-scan','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/arping','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/arpwatch','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/asciinema','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/asterisk','99','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/atftp','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/authbind','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/autossh','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/avahi','35','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/awscli','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/awstats','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/axel','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/b43-firmware','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/b43-fwcutter','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/barnyard2','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/base','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bip','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bird','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bitcoin','607','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bitlbee','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bitlbee-steam','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bmon','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bottle','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/boxcryptor','40','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bozohttpd','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/broadcom-sta','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bsflite','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/btpd','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/btsync','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bwm-ng','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/c-toxcore','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cacti','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cacti-spine','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cadaver','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/canto','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cclive','45','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ccnet','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cgterm','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cherokee','51','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/choqok','71','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/chromium','17046','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/chrony','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cicb','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ckermit','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/clamav-unofficial-sigs','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/claws-mail','121','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/clipgrab','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/clive','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cmdiag','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cmst','44','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/comgt','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/connman','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/corebird','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/corkscrew','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/courier-authlib','48','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/courier-imap','74','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/courier-unicode','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cowpatty','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cryptcat-unix','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/csync','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ctorrent','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cyrus-imapd','42','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dansguardian','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/darkhttpd','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/darkstat','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/davmail','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dclib','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ddclient','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/deis','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/deluge','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dhcping','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dianara','42','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dillo','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/discord','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dkimproxy','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dnscrypt-proxy','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dnscrypt-wrapper','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dnstop','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dnstracer','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/docsis','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dogecoin','262','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dokuwiki','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dovecot','131','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dovecot-pigeonhole','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/driftnet','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dropbear','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dropbox','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dsniff','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dukto','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dwb','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/easyrsa','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/efax-gtk','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/eiskaltdcpp','297','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/elemental-ircd','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/elinks','45','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/emacs-w3m','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/emailrelay','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/emesene','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/enet','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/etherape','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/etherpad-lite','49','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/etherpoke','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ettercap','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ettercap-NG','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/exim','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/fail2ban','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/fcgiwrap','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/fdm','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ferm','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/filezilla','191','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/firehol','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/firewalld','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/flexget','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/flvstreamer','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/fping','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/fping3','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/fping6','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/fqterm','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/freeradius-server','70','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/freerapid','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/freerdp','48','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/frostwire','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ftp-cloudfs','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/fwbuilder','293','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/fwknop','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gajim','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ganglia','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ganglia-web','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/geoipupdate','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/glassfish','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/glusterfs','160','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gmvault','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gns3','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/google-cli','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/googlecl','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/googler','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gophernicus','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gq','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/greenbone-security-assistant','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/grsync','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gtk-gnutella','152','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gtmess','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gtorrentviewer','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/guacamole-server','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gui-ufw','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/hamachi','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/haproxy','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/havp','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/heimdal','754','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/hiawatha','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/hipchat','51','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/hipchat2','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/hostapd','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/hostsblock','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/hping3','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/httpie','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/httping','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/httptunnel','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/httrack','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/hydra','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/hylafax','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/hylafax+','42','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/icecat','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/icinga2','295','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/identicurse','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ifstatus','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ii','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ike','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/imapfilter','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/imapproxy','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/insync','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/iodine','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ip2location-c','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ipcalc','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/iperf','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/iperf3','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ipscan','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ipv6calc','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ipvsadm','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/irssi_otr','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/isync','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/jabberd2','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/jboss-as','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/jetty','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/jigdo','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/jitsi','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kadu','390','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kcm_ufw','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kea','421','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/keepalived','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/keybase-client','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kfilebox','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kismet','56','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kivy-garden','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kmess','51','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/knemo','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/knock','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kobby','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/konversation','77','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/krb5','66','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kvirc','209','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kwakd','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ldapvi','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/leafnode','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/leechcraft','178','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/libcapi20','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/libdnet','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/libident','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/libmaxminddb','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/libnatpmp','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/licq','149','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/liferea','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/light-firefox','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/lighttpd','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/lighttpd2','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/limnoria','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/linphone','60','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/linuxdcpp','105','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/livestreamer','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/livestreamer-curses','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/lldpd','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/luakit','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/lurch','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/macchanger','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/maildrop','58','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mailman','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/malheur','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/masscan','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mdk3','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/megatools','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/memcached','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/memcached-init','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mew','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/midori','50','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/milter-greylist','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/miredo','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mk-livestatus','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mldonkey','151','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_auth_kerb','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_bw','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_chroot','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_dnssd','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_evasive','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_fcgid','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_geoip2','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_ruid2','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_wsgi','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_wsgi-metrics','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/modsecurity-apache','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/monkey','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mosaic-ck','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mosh','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/movgrab','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mpop','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mps','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mps-youtube','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mrtg','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/msmtp','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/msn-proxy','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mumble','62','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/munge','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/murmur','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/museek+','133','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mysecureshell','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nagios','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nagiosgraph','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/naim','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nbd','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nbtscan','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nbwmon','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ncrack','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/netcat','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/netdrive-connector','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nethogs','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/netperf','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/netsniff-ng','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/netstat-nat','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/netsurf','56','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/newsbeuter','40','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nextcloud-client','101','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nfdump','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nghttp2','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nginx','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ngircd','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ngrep','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/niceshaper','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nicotine+','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nikto','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nload','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nmh','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/noip2','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nrpe','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nsca','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nsd','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nss-mdns','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ntop','70','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ntpclient','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nxclient','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nzbget','57','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/offlineimap','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/oidentd','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/onioncat','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openconnect','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/opendchub','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/opendkim','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openfire','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openfortivpn','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openl2tp','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openldap-server','125','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openntpd','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openrdate','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openresolv','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/opensmtpd','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openssh-krb5','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openvas-cli','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openvas-libraries','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openvas-manager','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openvas-scanner','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openvswitch-utils','45','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/opera','39','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/opera-developer','39','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/opera-legacy','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/otter','170','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/owncloud-client','101','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/owncloud-server','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/p0f','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/packETH','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/packit','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/palemoon','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pdns-recursor','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pdnsd','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pebrot','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pentadactyl','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/periscope','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pflogsumm','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pgbouncer','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pgl','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/phodav','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/phpmyadmin','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-authorization-blocker','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-birthday-reminder','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-extprefs','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-gfire','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-musictracker','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-nudge','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-opensteamworks','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-otr','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-privacy-please','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-sipe','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-skypeweb','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-sound','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-toobars','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-toolbar-shrink','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-visnotes','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-whatsapp','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pirate-get','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/plasma-nm-openconnect','78','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/plowshare','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/plowshare-modules','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/policyd','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/policyd2','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/polipo','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/postfix','35','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/postgrey','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pound','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pptp','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pptpd','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/privoxy','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/profanity','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/prosody','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/proxychains','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/proxymini','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/proxytunnel','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/psad','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/psi','216','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ptokax','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pubnub-curses','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pure-ftpd','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/purple-facebook','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/putty','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pyNeighborhood','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pydio-agent','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pydio-sync','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pysub-dl','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/python-onedrive','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pyzor','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/qTox','123','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/qbittorrent','134','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/qbittorrent-qt5','277','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/qtmib','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/quagga','64','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/quassel','117','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/qupzilla','290','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/quvi','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/r2e','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/r8168','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/radicale','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/radiusplugin','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/radvd','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/razor-agents','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rclone','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/reaver','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/redir','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rejik','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rekonq','49','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/remmina','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/retroshare','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rhapsody','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/roger-router','84','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rssh','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rtl8188eu','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rtorrent','58','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rubygem-showterm','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sabnzbd','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/scapy','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/seafile-client','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/seafile-server','45','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/seahub','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/secpanel','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/serialmail','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sftp-cloudfs','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/shibboleth-sp','156','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/shorewall','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/shorewall-core','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/shorewall6','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sic','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sickrage','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/siege','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sieve-connect','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/silc-toolkit','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sipcalc','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sipp','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sks-keyserver','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/skylable-sx','78','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/skypeforlinux','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/slack','42','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/slimjet','65','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/slowhttptest','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/slurm','125','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/smb4k','51','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/smcroute','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/smstools3','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sniffjoke','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/snmpsim','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/snmptt','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/snort','55','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/socat','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/social-engineer-toolkit','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/spamassassin','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/spambayes','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/spawn-fcgi','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/speedometer','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/speedtest-cli','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sphinx','59','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/spice','54','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/spice-gtk','66','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sqlmap','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sqlninja','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sqlrelay','65','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/squid','286','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/squidGuard','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/squidanalyzer','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sshblock','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sshfp','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sshfs-fuse','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sshguard','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sshpass','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sshuttle','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sslh','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ssllabs-scan','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sslscan','120','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sslstrip','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sstp-client','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ssvnc','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/strongswan','54','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/surf','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/surfraw','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/suricata','58','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/swaks','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sylpheed','52','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/syncthing','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/synergy','81','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/t50','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tahoe-lafs','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tcpflow','78','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tcptunnel','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/telegram','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/telegram-cli','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/thrift','167','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/thttpd','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tightvnc','50','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tinc','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tinyproxy','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tixati','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tmate','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tnftp','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tntnet','106','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tor','74','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tor-browser','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tornado','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/torsocks','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/toxcore','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tramp','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/transgui','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/translate','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/transmission','87','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/transmission-remote-gtk','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tribler','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/trojita','266','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ttcp','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ttdnsd','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tucan','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tunctl','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/turbolift','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/twine','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/twitter','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/uTox','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ucspi-ssl','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ucspi-tcp','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/udns','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/udpcast','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/udpxy','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ufw','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/uget','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/uget-chrome-wrapper','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/unbound','56','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/unicornscan','52','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/urlgrabber','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/urlview','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/urlwatch','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/uudeview','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/uwsgi','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vacuum-im','204','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/valknut','53','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/varnish','42','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vblade','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vcspull','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vde2','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/verlihub','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/verm','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/viber','74','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vimb','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vivaldi','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vnstat','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vpnc','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vrrpd','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vuze','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/w3af','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/w3m','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wakeonlan','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wavemon','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/weathercli','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/webalizer','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/weboob','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/weechat','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/weetwit','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/weighmail','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wifi-radar','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wifite','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wildfly','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wire','73','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wireshark','738','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wmget','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wmnd','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wmnotify','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wol','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wput','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wrk','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wvdial','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/x11vnc','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/x2goclient','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/x2goserver','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/xdman','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/xinetd','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/xl2tpd','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/xnetload','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/xombrero','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/xtables-addons','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/yadifa','97','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/yandex-browser-beta','56','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/yoda','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/you-get','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/youtube-dl','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/youtube-dl-gui','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/youtube-dl-server','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/youtube-upload','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/yturl','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/zabbix_agentd','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/zabbix_proxy','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/zabbix_server','40','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/zmap','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/znc','82','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/zsync','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/CherryTree','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/JabRef','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/LanguageTool','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/LibreOffice','12867','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/MasterPDFEditor','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/NME','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/ProjectLibre','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/TaskCoach','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/Ted','93','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/abiword','648','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/abook','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/aigaion','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/aiksaurus','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/alot','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/antiword','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/apvlv','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/asymptote','114','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/basket','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/beaver','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/bibutils','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/bookbinder','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/briss','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/broffice.org','115','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/calcurse','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/calibre','60','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/catdoc','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/ccal','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/color-theme','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/convertlit','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/coolreader','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/cups-pdf','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/devtodo','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/dictd','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/diffpdf','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/ding','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/djvusmooth','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/docx2txt','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/elyxer','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/epdfview','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/eqonomize','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/etmtk','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/evince','95','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/fbpdf','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/fbreader','114','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/flowkeeper','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/focuswriter','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/fop','44','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/freemind','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/full-pack','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/ganttproject','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/gbgoffice','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/gcal','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/gnokii','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/gnucash','215','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/gnumeric','205','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/gocr','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/goldendict','96','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/grisbi','48','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/gummi','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hamster','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hebcal','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hevea','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/homebank','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hunspell-de','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hunspell-en','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hunspell-es','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hunspell-id','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hunspell-pl','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hyphen-id','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/impressive','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/jstock','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/juffed','52','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/kate-latex-plugin','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/kbbi-qt','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/kbgoffice','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/kbibtex','60','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/kchmviewer','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/kchmviewer-qt','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/keepassx','60','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/keepassxc','98','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/keepnote','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/kmymoney','331','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/lcal','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/leafpad','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/libreoffice','117','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/libreoffice-helppack','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/libreoffice-langpack','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/lout','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/lyx','369','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mairix','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/marave','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mathtex','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mdbtools','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mdp','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mined','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/minidjvu','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mobiperl','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/moneydance','51','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mtp2-fonts','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mu','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/multivalent-tool-pdf','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mupdf','59','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/muttprint','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/myrulib','86','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mytetra','39','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mythes','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mythes-id','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/notmuch','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/nts','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/odt2txt','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/onlyoffice-desktopeditors','122','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/openoffice-langpack','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/openoffice.org','69','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/org-mode','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/osmo','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pandoc','170','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/paps','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pcal','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pdf2djvu','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pdfchain','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pdfjam','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pdfpc','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pdfshuffler','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pdfstudio11','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pdfstudio12','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pdftk','93','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/ps2eps','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pstoedit','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pstotext','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pyspread','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/python-gcalcli','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/qpdfview','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/qute','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/rednotebook','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/referencer','42','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/remind','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/sc-im','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/scribus','448','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/siag','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/sigil','370','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/smoffice2016','138','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/sofastats','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/t-prot','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/task','65','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/taskd','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/taskjuggler','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/tasksh','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/teapot','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/tellico','143','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/texlive','541','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/texlive-docs','416','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/texlive-extra','112','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/texlive-fonts','203','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/texmacs','285','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/texmaker','245','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/texstudio','258','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/text2pdf','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/texworks','49','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/timetrap','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/tnote','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/tpp','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/traduisons','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/treesheets','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/trelby','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/tudu','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/unoconv','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/verbiste','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/vlna','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/vym','51','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/wammu','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/watson','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/wink','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/wps-office','120','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/wyrd','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/xmind','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/xournal','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/xpad','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/xreader','70','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/xtrkcad','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/zathura','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/zathura-cb','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/zathura-djvu','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/zathura-pdf-mupdf','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/zathura-ps','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/zim','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/Gtk2-GladeXML','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/Net-SSLeay','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/cpanminus','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/libpurple-kwallet-plugin','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/libwww-perl','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/mod_perl','55','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Algorithm-Diff','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Alien-SDL','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-AnyEvent','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-AnyEvent-HTTP','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-AnyEvent-I3','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-AppConfig','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Archive-Extract','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Archive-Zip','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Astro-SunTime','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Audio-FLAC-Header','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Authen-SASL','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-BerkeleyDB','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Bit-Vector','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CDDB-get','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CGI','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CPAN-Changes','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CPAN-Perl-Releases','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CPANPLUS','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CPANPLUS-Dist-Build','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CPANPLUS-Dist-Slackware','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CSS-Squish','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Cache-FastMmap','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Canary-Stability','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Capture-Tiny','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Carp-Clan','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-Data-Inheritable','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-Factory-Util','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-Gomor','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-Inspector','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-Load','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-Method-Modifiers','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-MethodMaker','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-ReturnValue','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-Singleton','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Compress-Bzip2','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Config-File','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Config-Find','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Config-IniFiles','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Config-Simple','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Convert-ASN1','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Convert-BinHex','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Convert-Color','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Convert-TNEF','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Convert-UU','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Convert-UUlib','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-Blowfish','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-Blowfish_PP','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-CAST5','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-CBC','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-DES','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-IDEA','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-OpenSSL-Bignum','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-OpenSSL-DSA','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-OpenSSL-EC','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-OpenSSL-ECDSA','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-OpenSSL-RSA','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-OpenSSL-Random','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-Rijndael','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Cstools','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Curses','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Curses-UI','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DBD-Pg','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DBD-SQLite','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DBIx-Simple','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Danga-Socket','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Data-Compare','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Data-OptList','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Data-Random','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Data-UUID','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Date-Calc','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Date-Manip','35','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DateTime','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DateTime-Format-Builder','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DateTime-Format-ISO8601','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DateTime-Format-Strptime','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DateTime-Locale','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DateTime-TimeZone','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Devel-CheckLib','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Devel-GlobalDestruction','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Devel-PatchPerl','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Devel-StackTrace','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Devel-Symdump','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Device-SerialPort','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Digest-GOST','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Digest-MD4','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Dist-CheckConflicts','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-EV','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Email-Date-Format','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Encode-Detect','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Encode-EUCJPASCII','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Encode-HanExtra','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Encode-ISO2022','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Encode-JISX0213','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Exception-Class','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Expect','66','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Exporter-Tiny','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-ExtUtils-Config','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-ExtUtils-Helpers','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-ExtUtils-Install','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-ExtUtils-InstallPaths','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-ExtUtils-ModuleMaker','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-ExtUtils-XSBuilder','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-FCGI','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Copy-Recursive','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Find-Rule','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Finder','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-HomeDir','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-ReadBackwards','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Remove','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Save-Home','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-ShareDir','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-ShareDir-Install','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Slurp','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Slurp-Tiny','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Tail','66','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Which','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-pushd','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Font-AFM','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-GD','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Geo-Coordinates-OSGB','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Geo-Coordinates-UTM','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Geo-IP','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Geography-Countries','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Glib-Object-Introspection','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Gnome2-Vte','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Gtk2-Ex-Simple-List','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Gtk2-Notify','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Gtk2-Unique','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-HTML-TableExtract','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-HTML-Tree','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-HTTP-Cache-Transparent','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-HTTP-Proxy','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-All','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-CaptureOutput','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-HTML','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-Interface','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-Multiplex','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-SessionData','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-Socket-INET6','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-Socket-Multicast','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-Socket-SSL','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-Socket-Socks','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-Socket-Timeout','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-String','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-Stty','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-Tty','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-stringy','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IP-Country','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IPC-DirQueue','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IPC-Run','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IPC-Run3','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IPC-System-Simple','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IPTables-ChainMgr','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IPTables-Parse','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Image-Info','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Image-Size','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Import-Into','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Inline','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Inline-C','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-JSON','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-JSON-XS','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-LWP-Protocol-https','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Lingua-EN-Numbers-Ordinate','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Lingua-Preferred','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-List-MoreUtils','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-List-UtilsBy','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Locale-Maketext-Lexicon','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-LockFile-Simple','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Log-Dispatch','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Log-Log4perl','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Log-Message','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Log-Message-Simple','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MIME-Charset','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MIME-Lite','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MIME-tools','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MP3-Info','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Mail-DKIM','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Mail-DomainKeys','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Mail-SPF','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Mail-SPF-Query','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MailTools','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-Base85','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-BigInt','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-BigInt-GMP','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-Complex_C','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-Complex_C-L','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-Complex_C-Q','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-Decimal64','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-GMPf','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-GMPq','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-GMPz','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-LongDouble','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-MPC','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-MPFR','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-Prime-Util-GMP','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-Round','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Module-Build','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Module-Build-Tiny','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Module-Implementation','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Module-Install','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Module-Manifest','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Module-Pluggable','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Module-Runtime','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Module-ScanDeps','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Module-Versions-Report','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Mojolicious','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Moo','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Mozilla-CA','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MusicBrainz-DiscID','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-ARP','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-CIDR','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-CIDR-Lite','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-DNS-Native','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-DNS-Resolver-Programmable','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-DNS-SEC','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Daemon','71','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-IPv4Addr','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-IPv6Addr','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Ident','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Jabber','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-LDAP','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-LibIDN','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Libdnet','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Netmask','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Patricia','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Pcap','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-RawIP','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-SNMP','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Server','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Telnet','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-UPnP','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Write','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-XMPP','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-NetAddr-IP','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-NetPacket','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Number-Bites-Human','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Number-Compare','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Object-Accessor','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Ogg-Vorbis-Header-PurePerl','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-PAR','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-PAR-Dist','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-PHP-Serialization','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Package-Constants','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Package-DeprecationManager','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Package-Stash','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Package-Stash-XS','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Params-Classify','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Params-Util','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Params-Validate','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Parse-RecDescent','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Parser-MGC','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Path-Class','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Pegex','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-PerlIO-Layers','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-PerlIO-via-Timeout','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Pod-Coverage','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Probe-Perl','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Proc-Daemon','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Proc-PID-File','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Proc-ProcessTable','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Proc-Simple','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Readonly','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Regexp-Common','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Regexp-IPv6','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Religion-Islam-Quran','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Role-Tiny','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-SDL','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-SNMP-Info','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-SOAP-Lite','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sidef','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Socket-GetAddrinfo','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Socket6','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sort-Naturally','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sort-Versions','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Statistics-Descriptive','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-String-PerlIdentifier','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-String-ShellQuote','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sub-Exporter','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sub-Exporter-Progressive','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sub-Install','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sub-Override','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sub-Quote','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sub-Uplevel','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sys-CPU','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sys-Hostname-Long','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sys-MemInfo','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sys-Mmap','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sys-Syscall','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Task-Weaken','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-TeX-Hyphen','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Template-Toolkit','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Term-Animation','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Term-ProgressBar','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Term-ReadLine-Gnu','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Term-UI','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Deep','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Differences','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-DistManifest','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Exception','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Fatal','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Harness','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Inter','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-LongString','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-MockModule','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-MockObject','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Most','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-NoWarnings','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Number-Delta','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Output','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-PerlTidy','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Pod','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Pod-Coverage','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Requires','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Script','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-SharedFork','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Simple','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Spec','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-TCP','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Trap','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Warnings','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Autoformat','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-CSV','44','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-CSV_XS','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-CharWidth','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Diff','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Glob','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Hyphen','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Iconv','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Password-Pronounceable','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Patch','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Quoted','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Reform','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Tabs+Wrap','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Tabulate','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-WrapI18N','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Tidy','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Tie-IxHash','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Tie-Simple','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Time-Out','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Time-Piece','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Time-modules','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-TimeDate','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Tk-TableMatrix','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Tree-Simple','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Try-Tiny','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Types-Serialiser','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Unicode-LineBreak','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Unicode-Map','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Unicode-Map8','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Unicode-String','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Unicode-UTF8simple','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Unix-Syslog','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-WWW-Curl','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-WebService-MusicBrainz','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-X10','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-X11-Protocol-Other','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-XML-Filter-BufferText','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-XML-LibXSLT','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-XML-SAX-Writer','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-XML-Stream','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-XML-TreePP','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-XML-Writer','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-XML-XPath','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-YAML-LibYAML','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-YAML-Syck','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-YAML-Tiny','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-ZMQ-Constants','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-cairo','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-cgi-minimal','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-class-accessor','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-class-accessor-chained','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-common-sense','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-config-general','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-data-dump','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-data-page','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-data-page-pageset','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-digest-hmac','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-digest-sha1','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-encode-locale','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-event','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-extutils-depends','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-extutils-pkgconfig','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-file-basedir','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-file-desktopentry','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-file-libmagic','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-file-listing','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-file-mimeinfo','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-file-path-expand','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-glib','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-gnome2','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-gnome2-canvas','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-gnome2-gconf','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-gnome2-vfs','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-gnome2-wnck','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-goo-canvas','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-gstreamer','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-gtk2','85','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-gtk2-imageview','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-html-form','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-html-parser','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-html-tagset','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-http-cookies','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-http-daemon','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-http-date','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-http-message','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-http-negotiate','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-http-response-encoding','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-http-server-simple','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-image-bmp','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-inc-latest','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-libintl','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-libnet','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-linux-desktop-files','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-lirc-client','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-local-lib','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-lwp-mediatypes','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-lwp-useragent-cached','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-net-dbus','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-net-dns','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-net-http','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-net-ip','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-palm-pdb','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-pango','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-rename','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-strictures','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-test-warn','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-text-aspell','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-text-unaccent','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-tk','46','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-trayicon','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-tree-dagnode','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-www-mechanize','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-www-robotrules','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-x11-protocol','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-xml-fast','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-xml-libxml','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-xml-parser-lite','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-xml-parser-lite-tree','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-xml-twig','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-yaml','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perlbrew','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/APScheduler','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Automat','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Baker','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Flask','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Flask-Babel','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Flask-Compress','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Flask-Cors','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Flask-Gravatar','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Flask-HTMLmin','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Flask-Login','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Flask-Mail','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Flask-Principal','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Flask-RESTful','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Flask-SQLAlchemy','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Flask-Security','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Flask-WTF','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Logbook','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Mako','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Markdown','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/MarkupSafe','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Parsley','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Paver','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PyAudio','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PyColorizer','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PyDispatcher','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PyGithub','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PyGreSQL','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PyInstaller','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PyRIC','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PySDL2','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PyStemmer','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PyXB','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Pygments','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Pyro','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Pyro4','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PythonQt','170','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/RBTools','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/SOAPpy','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Scrapy','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Unidecode','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/WTForms','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/XlsxWriter','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/affine','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/alabaster','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/aniso8601','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/apsw','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/argcomplete','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/argh','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/args','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/artifacts','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/asciimatics','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/asn1crypto','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/astroid','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/attrs','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/autopep8','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/babel','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/babelfish','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/backports-abc','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/bencode','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/binplist','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/bitarray','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/bleach','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/blessings','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/blinker','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/boto','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/boto3','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/bottle-mysql','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/bsddb3','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/buildbot','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/buildbot-slave','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cairocffi','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cddb-py','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cfgparse','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/characteristic','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cheroot','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/chump','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/click','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/click-default-group','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/click-plugins','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/clint','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cocos2d','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/colorama','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/colorclass','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/colored','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/constantly','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/construct','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cov-core','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/coverage','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cryptography','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cssselect','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cssutils','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/curtsies','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cx_Freeze','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/darts.util.lru','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/dbus-python3','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/decorator','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/dfdatetime','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/dfvfs','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/dfwinreg','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/dhcpy6d','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/distorm','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/dnspython','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/docopt','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/dotty','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/dpkt','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/drawille','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/dulwich','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/dxfwrite','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/easygui','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ecdsa','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/egenix-mx-base','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/entrypoints','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/enzyme','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/etk.docking','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/euca2ools','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/extras','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/fabric','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/fasteners','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/feedgenerator','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/feedparser','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/filechunkio','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/fixtures','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/flake8','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/flask-paginate','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/flask-restplus','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/foolscap','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/frosted','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/fudge','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/funcsigs','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/functools-lru-cache','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/functools32','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/git-fame','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/githubpy','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/gmpy','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/gnome-python','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/gnome-python-desktop','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/google-api-python-client','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/google-apputils','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/gst-python','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/gst-python3','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/gst0-python','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/guess-language','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/guessit','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/gunicorn','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/hachoir-core','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/hachoir-metadata','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/hachoir-parser','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/halberd','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/helpy','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/hg-git','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/hgsubversion','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/hpack','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/html2text','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/htmlmin','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/httplib2','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/hyperlink','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/i3-py','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/idna','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/imagesize','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/incremental','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/invoke','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ipaddr-py','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ipython','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ipython_genutils','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/isort','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/itsdangerous','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/jedi','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/jellyfish','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/jmespath','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/json-py','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/jsonschema','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/kaptan','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/linecache2','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/llfuse','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/lockfile','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/lxml','48','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/mando','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/mechanize','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/minidb','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/mistune','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/mnemonic','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/mock','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/monotonic','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/monty','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/msgpack-python','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/munch','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ndg_httpsclient','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/netifaces','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/node-semver','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/npyscreen','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/numexpr','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/numpydoc','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/nxt-python','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/obspy','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/p4python','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/packaging','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pandocfilters','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/paramiko','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/parsel','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/passlib','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/path.py','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pathlib','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pattern','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pbr','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pefile','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pelican','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pendulum','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/phply','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pickleshare','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pilkit','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pillowfight','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pip','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pipdeptree','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/piprot','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pipstat','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pkginfo','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/planes','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/plaso','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pluggy','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pluginbase','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ply','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/polib','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/portend','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/poster','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/progressive','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/prompt_toolkit','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ps_mem','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/psutil','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ptpython','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ptyprocess','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/purl','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/py','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/py-cpuinfo','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/py3cairo','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/py3sensors','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/py4j','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyOpenSSL','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyPdf','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pySmartDL','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyaes','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyasn1','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyasn1-modules','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pybloomfiltermmap','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pycassa','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pycdio','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pychart','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pychecker','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pychm','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pycld2','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pycodestyle','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pycrypto','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pycryptopp','45','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pycxx','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pydblite','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pydf','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pydot','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyewmh','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyfeed','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyfiglet','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyflakes','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyfltk','60','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pygame','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyglet','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pygobject3-python3','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pygoocanvas','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pygpgme','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pygrametl','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pygsl','36','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pygtksourceview','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pygtkspell','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyicu','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyinotify','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyjwt','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyliblo','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pylirc','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pymongo','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pymysql','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pynzb','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyorbit','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyparsing','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyperclip','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pypoppler','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pypy','49','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pypy3','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyqode.cobol','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyqode.qt','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyquery','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyresample','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyrfc3339','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyrsistent','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pysed','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyserial','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyside','525','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pysmssend','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pysnmp','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pystatsd','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pysvn','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pytables','42','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pytest','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pytest-cov','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pytest-runner','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-Levenshtein','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-MonthDelta','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-Safe','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-augeas','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-authres','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-autobahn','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-axolotl','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-axolotl-curve25519','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-bigfloat','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-blake2','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-cached-property','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-certifi','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-chardet','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-cheetah','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-clamd','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-cluster','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-cmdln','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-configargparse','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-d2to1','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-daemon','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-dateutil','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-demjson','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-discid','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-distro','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-distutils-extra','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-django','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-django-legacy','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-django-tagging','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-djvulibre','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-docker','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-docker-pycreds','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-dockerpty','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-editor','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-elementtree','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-elib.intl','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-esmre','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-evdev','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-faulthandler','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-fonttools','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-fusepy','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-future','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-gammu','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-gattlib','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-gflags','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-ipcalc','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-jsonrpclib','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-kerberos','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-keybinder','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-keyczar','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-keyring','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-keyutils','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-klein','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-lazy-object-proxy','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-ldap','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-lhafile','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-libarchive-c','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-libnacl','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-libsass','109','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-libtmux','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-ly','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-m2r','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-magic','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-magick','306','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-mccabe','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-memcached','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-mimeparse','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-morbid','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-mpd','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-musicbrainz2','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-nbxmpp','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-neovim','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-netaddr','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-notify2','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-oauth','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-oauth2','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-oauth2client','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-oauthlib','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-openid','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-orbited','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-parse','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-parsedatetime','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-patch','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-pbkdf2','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-pcapy','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-pdfminer','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-pmw','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-poppler-qt4','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-pydns','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-pyparted','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-pypolicyd-spf','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-pyspf','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-qrcode','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-requestbuilder','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-requests','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-requests-kerberos','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-rubymarshal','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-scandir','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-sh','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-simpy','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-slacklog','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-slip','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-stomper','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-stsci.distutils','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-suds','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-swiftclient','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-tabulate','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-tblib','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-texttable','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-tlslite','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-transaction','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-tvrage','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-twisted','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-twitter','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-txaio','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-unicodecsv','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-unicodedata2','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-uri-templates','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-urllib3','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-uuid','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-vatnumber','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-vulndb','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-webencodings','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-wordpress-xmlrpc','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-wrapt','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-x11_hash','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-xcffib','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-xlib','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-xlwt','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-xrandr','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-yenc','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pythondialog','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3','113','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-Cython','77','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-Pygments','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-appdirs','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-augeas','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-dateutil','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-decorator','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-defusedxml','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-dugong','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-future','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ipython','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ipython_genutils','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-jedi','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-jupyter-ipykernel','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-jupyter_client','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-jupyter_core','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-lhafile','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pexpect','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pickleshare','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-prompt_toolkit','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ptyprocess','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pycrypto','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyparsing','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyparted','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pythondialog','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyzmq','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-simplegeneric','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-sip','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-six','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-tornado','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-traitlets','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-wcwidth','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-xlib','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pytsk','48','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pytz','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pytzdata','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyudev','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyusb','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyutil','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pywebkitgtk','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyxdg','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyzmq','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/queuelib','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ramlfications','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/rebulk','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/regex','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/relatorio','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/requests-futures','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/requests-oauthlib','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/requests-toolbelt','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/requirements-detector','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/rope','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/rsa','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/rst2ansi','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/s3cmd','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/s3transfer','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/scikit-learn','129','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/secretstorage','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/selenium','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/service_identity','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/setuptools-scm','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sge-pygame','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/shutil_get_terminal_size','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/shutil_which','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/shutilwhich','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/simplegeneric','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/simplejson','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/singledispatch','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/six','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sk1libs','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/snowballstemmer','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/snuggs','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/soundcloud','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/speaklater','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sphinxcontrib-plantuml','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sphinxcontrib-websupport','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sql','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sqlalchemy-migrate','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sqlparse','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/statsmodels','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/stem','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/stopit','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/subvertpy','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/suds-jurko','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/tagpy','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/tempita','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/tempora','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/termcolor','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/terminado','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/terminaltables','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/testpath','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/testscenarios','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/testtools','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/threadpool','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/thunarx-python','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/timehash','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/tinycss','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/tornado_systemd','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/tox','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/tqdm','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/traceback2','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/traitlets','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/trezor','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/trollius','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/tweepy','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/txi2p','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/txrequests','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/txsocksx','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/txtorcon','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/typing','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/tzlocal','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/unrardll','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/vcversioner','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/veryprettytable','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/vinetto','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/virtualenv','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/virtualenv-clone','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/vobject','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/w3lib','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/waitress','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/wcwidth','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/webassets','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/websocket-client','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/werkzeug','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/winpdb','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/wsgiref','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/xe','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/xopen','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/xtermcolor','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/yara-python','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/zxcvbn-python','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/chronic','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/gem2tgz','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/heroku-client','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/ncurses-ruby','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/passenger','205','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rbenv','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/ruby-build','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/ruby-faraday','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/ruby-gist','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/ruby-haste','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/ruby-json','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/ruby-multipart-post','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/ruby-native-package-installer','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-asciidoctor','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-atk','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-bundler','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-cairo','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-coderay','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-cool.io','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-fastercsv','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-fluentd','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-gdk_pixbuf2','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-glib2','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-gosu','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-gtk2','57','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-highline','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-hpricot','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-http_parser.rb','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-lolcat','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-method_source','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-msgpack','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-mustache','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-paint','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-pango','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-pkg-config','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-pry','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-rack','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-rdiscount','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-ronn','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-sass','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-sigdump','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-slop3','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-string-scrub','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-terminal-table','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-thread_safe','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-trollop','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-tzinfo','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-tzinfo-data','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-yajl-ruby','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygems-update','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/sequel','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/sqlite3','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/AnonymousPro','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/Attic','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/BasiliskII','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/CNS11643-font','80','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/Eterm','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/FontAwesome','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/GentiumPlus','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/I-Nex','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/Iosevka','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/MySQL-Sandbox','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/PySixel','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/TLP','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/TermRecord','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/Type1_to_OTF','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/USBdev','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ZoneMinder','149','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/a2tools','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ack','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/acpi','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/acpi_call','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/acpitool','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/adobe-source-code-pro-font','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/adobe-source-han-sans-fonts','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/adobe-source-sans-pro-font','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/adobe-source-serif-pro-font','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/aespipe','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/afdko','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/afio','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/afpfs-ng','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/afuse','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/agedu','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/aha','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/aide','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/alien','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/amanda','92','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/anacron','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/andromeda','189','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ansible','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/anything-sync-daemon','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/apache-activemq','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/apache-cassandra','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/apache-tomcat','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/apachetop','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/apcupsd','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/aqemu','67','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/arc','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/argyllcms','67','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/arj','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/artwiz-aleczapka','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/artwiz-cursor','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/asbt','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/aterm','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/atool','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/atop','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/audit','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/augeas','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/autoarchive','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/autocutsel','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/autojump','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/avfs','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/backasa','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bacula','52','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bacula-client','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bareos','50','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bareos-client','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/barman','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bcache-tools','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/betty','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bicon','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/biew','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bin2iso','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bindfs','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bit-babbler','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/blake2','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bleachbit','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/blktrace','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/blocks','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bochs','71','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bogofilter','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bonnie++','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bootchart','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/borgbackup','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/brasero','110','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/btrbk','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bubblewrap','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bulk_extractor','73','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/burp','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/butterfly','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/byobu','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/c-icap','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cabextract','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cantarell-fonts','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/caprice32','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/capstone','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cbmbasic','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cbmfs','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ccd2iso','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ccid','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ccrypt','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cdemu-client','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cdemu-daemon','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cdrkit','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cdw','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cdwrite','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cfengine','49','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/chkrootkit','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/chrpath','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ciopfs','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/clamav','264','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/clamsmtp','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/clamtk','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cldr-emoji-annotation','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cndrvcups-capt','61','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cndrvcups-common','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cntlm','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/collectd','48','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/colord','40','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/colord-gtk','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/commons-daemon','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/conky','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/containerd','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/convmv','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cool-retro-term','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/coolkey','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/courier-prime','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cpmtools','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cpu-g','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cpuid','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cpuid2cpuflags','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cpulimit','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/crashplan','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/crikey','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/crmsh','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cronie','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/crunch','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/culmus','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cups-bjnp','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/curlftpfs','68','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cutecom','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cve-check-tool','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cwtex-q-fonts-TTFs','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/d-feet','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/d52','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/daa2iso','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/daemonize','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dahdi-complete','51','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dar','69','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dash','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dateutils','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/davfs2','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ddg','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ddrescue-gui','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/debootstrap','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ded','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dehydrated','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/detox','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dfc','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dfu-util','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dieharder','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/diit','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dina-font','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/disk-manager','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/diskscan','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/disksearch','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/docbook-xml','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/docker','79','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/docker-compose','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/docker-machine','35','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/docker-proxy','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dos33fsprogs','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dosbox','55','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dosemu','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/drbd-utils','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/driconf','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ds4drv','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dstat','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dtach','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dtrx','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dump','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/duplicity','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dvdisaster','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dymo-cups-drivers','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dynamips','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/eaglemode','60','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ecm-tools','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ecofont-ttf','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/efitools','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/eibd','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/elasticsearch','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/eltclsh','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/em','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/emelfm2','29','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/encfs','49','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/entr','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/epson-inkjet-printer-escpr','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/etckeeper','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/eventlog','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/evilvte','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/exfat-utils','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ext4magic','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/extract-xiso','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/extundelete','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/f2fs_tools','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/f3','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/facter','158','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fakeroot','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fasd','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fbterm','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fcron','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fdupes','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/file-roller','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/findcmd','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/findpkg','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fio','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fira-fonts','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/firejail','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/firetools','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/firmware-gobi-2000','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fish','53','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/flSynclient','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/flashcards','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/flexbackup','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/font-manager','44','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/foomatic-db','47','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/foomatic-db-engine','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/foremost','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/forkstat','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fpm2','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fr','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/franny','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/freedup','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/freefont','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/freeswitch','646','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fs-uae','165','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fs-uae-arcade','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fs-uae-launcher','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fsarchiver','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fslint','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fsv','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fsviewer','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fsvs','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fswatch','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fuse-emulator-utils','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fuse-exfat','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fuseiso','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fwupd','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fwupdate','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fxload','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fzf','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gammu','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gcdemu','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gconf-editor','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gdmap','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/geckodriver','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gencfs','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/genext2fs','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gentoo','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/geoclue','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/geoclue2','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gigolo','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gimp-help','1889','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gitolite','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gkrellm-volume','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gksu','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/glances','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/glyptodon','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gnome-commander','64','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gnome-disk-utility','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gnome-terminal','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gnome-vfs','39','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gnomint','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gnu-unifont','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/goaccess','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gohu-font','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/google-droid-fonts','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gpart','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gphotofs','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/graphterm','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/graveman','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gsmartcontrol','52','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gtk-vnc','67','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gtklp','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/guake','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/guix','133','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hack-font-ttf','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hal-flash','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hardinfo','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hashID','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hashcat','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/haveged','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hdapsd','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hddtemp','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hebe','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/heirloom-doctools','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/heirloom-pax','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/heirloom-sh','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hercules','146','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/herculesstudio','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hh','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hime','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hntool','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hplip-plugin','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hungrycat','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hwinfo','287','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hwloc','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/i2c-tools','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/i8kutils','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ibam','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/icinga-web','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/icingaweb2','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/idle3-tools','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ifuse','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/image-analyzer','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/imwheel','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/incron','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/influxdb','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/info2man','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/innotop','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/intel-microcode','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/inxi','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ioping','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/iotop','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ipmitool','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/irqbalance','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/isdct','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/isight-firmware-tools','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/isomaster','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/isomd5sum','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/iucode_tool','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/jdiskreport','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/jenkins','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/jmtpfs','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/john','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/jq','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/jsawk','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/jshon','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kafka','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kbfs','51','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kc','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kcm_touchpad','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kegs','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kexec-tools','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/keybase','47','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/keylaunch','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kitchen-sync','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kmscon','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/komparator4','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/krusader','73','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ksh-openbsd','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ktsuss','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kvpm','72','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kyotocabinet','61','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/laptop-mode-tools','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lbench','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lbrate','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lbzip2','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/letsencrypt','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lhasa','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/libnatspec','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lilyterm','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/linkchecker','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/linux-libertine-fonts','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/linuxconsoletools','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lirc','35','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lnav','44','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/localepurge','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/loggedfs','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/logkeys','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/logpp','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/logtool','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lomoco','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lshw','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lsyncd','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/luckybackup','35','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxdm','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxinput','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxrandr','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxsession','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxsplit','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxtask','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxterminal','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lynis','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lz4','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lziprecover','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lzop','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/macemu','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/macutils','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/maintboot','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/makefile2graph','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/makepasswd','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/makepp','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/makeself','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/man-db','52','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/man-pages-es','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/man-pages-fr','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/man-pages-ja','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/man-pages-ru','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mariadb-plugin-saslauthd','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mbootpack','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mcrypt','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/md5deep','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mdf2iso','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mdocml','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/memtest86','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/memuse','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mhddfs','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/microcode_ctl','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/microemulator','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/midisport-firmware','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mjpg-streamer','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mksh','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mktorrent','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mlterm','49','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/modules','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mongo-tools','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mongodb','4108','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/monit','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/monitoring-plugins','52','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/monitorix','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/motion','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mpich','193','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mrxvt','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ms-sys','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/msitools','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/msr-tools','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mtftar','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mucommander','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/multibootusb','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/multicorn','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/multipath-tools','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/multitail','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/munin-node','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/musl','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mysql-utilities','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nagios-plugins','58','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nanum-coding-fonts-ttf','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nanum-fonts-ttf','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nbench','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ncdu','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/netatop','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/netdata','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/newrelic-sysmond','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nilfs-utils','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nix','72','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nmon','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nnn','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nomarch','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/noto-emoji','434','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/noto-fonts','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nsca-ng','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nss-pam-ldapd','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nss_ldap','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/numactl','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/numlockx','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nut','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nvidia-driver','63','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nvidia-firmware','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nvidia-kernel','59','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nvidia-legacy304-driver','46','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nvidia-legacy304-kernel','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nvidia-legacy340-driver','61','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nvidia-legacy340-kernel','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nvme-cli','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ohsnap','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/oldschool-pc-fonts','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/onerng','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/opam','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/open-vm-tools','98','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/openct','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/openmpi','478','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/openrc','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/openrc-services','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/opensc','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/openscap','92','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/openstego','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/oracle-instantclient-basic','37','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/oracle-instantclient-devel','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/oracle-xe','114','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/osinfo-db','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/osinfo-db-tools','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ossec-agent','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ossec-local','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ossec-server','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ostree','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/otf-hermit','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/otf2bdf','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/overpass-desktop-fonts','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/overpass-webfonts','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/p7zip','83','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/packagekit','31','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/parallel','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/partclone','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/partimage','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/passwdqc','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/password-store','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/passwordsafe','199','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/patchelf','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/patool','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pax','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/paxctl','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pbzip2','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pce','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pcmanfm','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pcsc-lite','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pcsc-perl','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pcsc-tools','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pdfgrep','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pdksh','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pdsh','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/percona-toolkit','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/percona-xtrabackup','90','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pev','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pgadmin3','254','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pgbadger','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pgsanity','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/phoronix-test-suite','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/phpsysinfo','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/physlock','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/picocom','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pigz','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pipemeter','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/plan9port','245','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pmount','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/polyglotman','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pommed-light','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/porg','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/posixovl','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/postgresql','141','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/powerline-status','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/powernowd','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/prelink','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/preload','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/procinfo-ng','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/profile-cleaner','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/profile-sync-daemon','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/progress','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pstate-frequency','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ptop','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pv','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pwgen','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pwsafe','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pyarchey','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pycp','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pymux','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qControlCenter','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qdirstat','49','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qemu','239','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qemu-guest-agent','42','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qingy','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qingy_0.3_themepack','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qps','28','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qstardict','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qterminal','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qtfm','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qtgzmanager','25','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/quotatool','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qxkb','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rabbitmq-server','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ranger','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rar','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rar2fs','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rdiff-backup','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rdup','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/read-edid','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/recoverdm','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/recoverjpeg','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/redis','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/redis-py','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/refind','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/regionset','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/reiser4progs','32','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rename','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/repeat','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/reptyr','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rfdump','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rhash','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rkhunter','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rmw','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rng-tools','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rover','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rox-filer','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/roxterm','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rsyslog','26','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rtirq','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/run-one','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/runc','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/runit','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rxvt-unicode','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/s3ql','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/safecopy','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sakura','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/salt','17','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/samhain','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/samsung-mfp-drivers','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sandy','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sbbdep','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sbotools','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sboui','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sbsigntools','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/scalpel','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/scanmem','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/schedtool','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/scrypt','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sdl-jstest','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sdltrs','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/seahorse','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sentimental-skk','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ser2net','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/set_rlimits','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/setconf','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/shake','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/signify','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slack-utils','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slackchlog','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slackroll','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slackware-grub2-theme','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slackyd','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sleepd','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sleuthkit','38','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slim','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slpkg','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/smart','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/smbldap-tools','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/smem','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/softhsm','30','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/solarize-12x29-psf','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/spacefm','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/speedcontrol','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/spice-vdagent','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/spkg','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/spl-solaris','42','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/splix','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sqlcipher','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sqlite2','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/squidclamav','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/srm','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ssdeep','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/st','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/steamos-xpad','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/stm32flash','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/stress','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/suckless-tools','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sudosh2','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/suhosin','6','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/supervisor','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/swatch','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/swun','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/symbola-font-ttf','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sysdig','139','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/syslog-ng','84','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/t1utils','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tabbed','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tamsyn-font','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/targetcli-fb','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tarsnap','21','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tc-play','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/termenu','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/terminator','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/terminology','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/termite','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/termsaver','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/testdisk','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/the_silver_searcher','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/thermald','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/thinkfan','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tiger','34','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tilda','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tilix','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tini','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tinyterm','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tinyvm','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tkdvd','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tmuxp','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tokyocabinet','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tokyotyrant','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tomb','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tp_smapi','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tpc','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tpm','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/trachet','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tracker','75','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/trackfs','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/trash-cli','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/trashy','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/trmaid','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/trrntzip','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ts','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-aller','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-arphic-uming','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-bitstream-vera','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-caladea','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-carlito','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-coda','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-console-fonts','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-crosscorefonts','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-dark-courier','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-inconsolata','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-monaco','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-mononoki','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-open-sans','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-paratype-pt-fonts','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-roboto','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-ubuntu-font-family','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-zekton','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttmkfdir','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttop','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tty2gif','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttygif','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttyrec','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/twapake','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/twin','20','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/u3-tool','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/uae','43','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/udevil','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/udftools','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/udiskie','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ufiformat','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/uhd','240','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/uif2iso','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/un-fonts-core','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unac','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unace','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unar','27','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unetbootin','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unhide','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unicode-character-database','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unison','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/univga-font','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unp','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unrar','10','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unrtf','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unshield','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/untex','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/uptimed','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/upx','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/usbmon','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/usbredir','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/usbview','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/usermin','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vagrant','33','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vbindiff','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vcp','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/veracrypt','89','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vhba-module','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vice','143','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vifm','24','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vinterm','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/virt-manager','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/virt-viewer','14','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/virtualbox','784','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/virtualbox-addons','158','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/virtualbox-extension-pack','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/virtualbox-kernel','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/virtualbox-kernel-addons','18','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vlock','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/volatility','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vtcol','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wacomtablet','44','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wangfonts','60','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/watch-fs','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wdfs','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wdiff','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/webcore-fonts','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/webmin','45','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/whowatch','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wiimms-iso-tools','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/winusb','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wipe','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wmconfig','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wmgtemp','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/worker','124','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wqy-microhei-font-ttf','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wterm','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/x86_energy_perf_policy','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xarchiver','11','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xautolock','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xautomation','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xbattmon','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xbindkeys','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xboxdrv-linux','142','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xbrightness','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xcdroast','15','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xchm','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xdiskusage','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xf86-video-qxl','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xfburn','12','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xfe','41','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xfsudo','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xinput_calibrator','9','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xjobs','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xnp2','46','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xonsh','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xosd','7','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xrdp','22','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xrestop','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xstow','13','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xtrs','3','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xwiimote','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/yakuake','16','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/yash','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/yeahconsole','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/yelp','19','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/yelp-tools','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/yelp-xsl','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/z','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zarfy','5','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zerofree','1','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zfs-on-linux','212','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zookeeper','4','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zopfli','2','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zpaq','8','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zstd','23','7000','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zuluCrypt','107','7000','~');
commit;
