begin transaction;
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/3D-ICE','69','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/AlphaPlot','118','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/AstroImageJ','8','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/CAFS_divergence','3','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/CAPS_coevolution','4','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/DRAMPower','17','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/DRAMSim2','6','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/DRAMSys','89','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/DRAMsim3','11','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/Data2FCS','1','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/DendroPy','4','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/EMBASSY','90','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/EMBOSS','225','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/FCSalyzer','2','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/FastTree','4','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/FigTree','2','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/FreeFem','207','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/Gabedit','23','30260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/Gblocks','1','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/GeoGebra','24','29735.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/Gridcoin-Research','248','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/HMMER','42','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/IGV','56','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ITK','255','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/Isabelle','325','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/Kst','120','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/MetaPhlAn2','14','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/NWChem','980','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/OpenFOAM','5414','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/OpenMM','160','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/OpenVSP','1248','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/PhyML','79','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/PyAMG','17','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/R','298','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/RAxML','35','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/RepeatMasker','1100','30260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/STAR','37','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/SU2','170','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/SeqMonk','54','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/Shark','30','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/SimEng','28','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ViennaRNA','378','27856','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/WEVOTE','3','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/WolframEngine','1273','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/WordNet','35','30260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ZeGrapher','43','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/abntex','29','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/acl2','272','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/aenig4','3','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/aeskulap','32','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/aline','1','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/aliview','2','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/amd','13','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/amplicon','2','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/antiprism','46','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/aris','3','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/arka','4','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/armadillo','6','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/arpack','13','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/arpack-ng','21','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/artha','7','30260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/avl','8','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/avogadroapp','12','29954.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/avogadrolibs','197','28084.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bcftools','18','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bedGraphToBigWig','1','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bedtools','25','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/biblesync','3','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bibletime','137','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bibus','6','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/biopython','16','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/boinc','111','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/booktab','104','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bowtie','27','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bowtie2','79','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bowtie2-legacy','73','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bpp-core','36','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bpp-phyl','137','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bpp-phyl-omics','8','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bpp-popgen','12','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bpp-seq','37','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bpp-seq-omics','23','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bpp1.9-numcalc','10','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bpp1.9-phyl','49','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bpp1.9-seq','14','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bpp1.9-utils','9','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bppsuite','10','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/btf','11','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bwa','3','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cadabra2','144','29212.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/calc','23','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/calcoo','2','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/camd','13','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cap3','1','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cblas','6','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ccolamd','12','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cdo','136','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/celestia','64','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/chartgeany','73','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/chemtool','7','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cholmod','27','28727.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cistrome-CEAS','2','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cistrome-MACS','1','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cistrome-conductGO','2','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cistrome-extra','1','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cistrome-mdseqpos','6','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/clark-ugene','19','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cld2','14','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/clustalo','22','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/clustalw','9','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/clustalx','21','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/colamd','12','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/colin','15','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/convertall','2','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/copasi','73','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cryptominisat','32','29266','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/csparse','15','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cufflinks','5','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cutadapt','6','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cvc4','296','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cxsparse','26','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/dakota','659','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/datamash','20','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/denemo','61','28585.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/diamond','39','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/dimod','159','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ds9','254','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/dwave-cloud-client','2','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/dwave-greedy','9','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/dwave-hybrid','52','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/dwave-inspector','2','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/dwave-neal','11','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/dwave-networkx','2','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/dwave-ocean-sdk','257','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/dwave-preprocessing','21','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/dwave-system','2','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/dwave-tabu','9','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/dwavebinarycsp','2','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/easy_spice','6','31738.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ed-v6','1','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/edsim51di','2','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/elk','75','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/engauge','156','35130.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ent','1','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ephem','2','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/eprover','21','31994.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/equalx','13','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ess','6','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/exonerate','258','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/extrema','117','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/fann','3','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/fastqc','5','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/fet','173','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/fiji','141','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/finchtv','4','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/fityk','34','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/flann','123','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/flexbar','26','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/free42','16','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/g3data','4','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gaiasky','74','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/galculator','9','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gaw3','6','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gchemutils','88','29696.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gcompris-qt','80','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gdstk','6','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/geda-gaf','45','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gelemental','17','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gem5','2421','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/geneconv','4','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/genius','34','29690.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/genometools','54','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/genpak','4','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/geomview','44','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gephi','82','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gerbv','26','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/getdp','29','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/getfem','133','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ggobi','31','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ghemical','17','31056.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/glpk','9','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/glvis','19','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gmsh','508','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gnucap','28','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gp2fasta','3','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gplates','668','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gpredict','18','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/grace','15','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/grb','2','28718.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gresistor','2','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gretl','155','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gri','54','29448','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gromacs','197','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/grpn','2','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gspiceui','22','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gtkwave','19','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gtypist','6','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gwyddion','71','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/h5utils','3','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/harminv','5','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/hdfview','59','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/hp-prime','35','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/hyphy','56','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/iTest','14','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/io_lib','17','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/iqtree','4','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/irsim','5','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/isoelectric-point','3','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/itex2mml','3','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/jalview','15','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/jellyfish_k-mer','10','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/jpicedt','4','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/kalign','5','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/karma','11','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/kissat','4','31408','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/kjv','2','28718.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/klavaro','9','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/klu','13','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/knime','75','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/kraken','3','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ktikz','34','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/kurso','10','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/labplot2','176','28763.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ladr','28','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/laghos','6','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/lalnview','3','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/lammps','686','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/lapack95','64','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/lapacke','42','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/latex2html','8','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ldl','12','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/libghemical','14','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/locarna','58','30607.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/louvain-community','5','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/lpsolve','10','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/mafft','13','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/magic','17','29448','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/mathics','106','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/mathjax2','67','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/mathomatic','4','27969','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/matio','14','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/maxima','151','29436.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/meep','41','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/megax','92','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/meme-db-gomo','209','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/meme-db-motif','32','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/meme-db-tgene','847','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/meme-suite','40','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/mendeleydesktop','33','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/metalwalls','26','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/metis','4','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/mfem','65','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/minorminer','50','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/mkDSSP','17','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/molden','55','28767.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/molequeue','19','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/mpb','15','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/mrbayes','19','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/mumsa','1','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/muscle','5','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/muscle5','13','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ncbi-blast','29','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ncbi-blast+','1840','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ncbi-edirect','7','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ncbi-rmblastn','9','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/nco','33','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ncview','7','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ngspice','269','29213.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/nootka','21','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/novocraft','17','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/octave','532','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/openboard','156','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pal2nal','2','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/paml','25','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pamlX','12','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/papi','27','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pari','353','29954.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pari-elldata','158','29696.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pari-galdata','1','29696.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pari-galpol','96','29696.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pari-nflistdata','9','29696.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pari-seadata','44','29696.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/parmetis','9','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/patsy','2','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pcalc','2','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pcb','33','31056.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/penaltymodel','1','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/perlprimer','1','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pianobooster','15','31056.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/plasmidomics','2','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/plus42','19','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pomoxis','1','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/prank-msa','8','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/primer3','6','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pspp','114','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pulseview','91','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pyCRAC','38','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pysam','160','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/python2-kiwisolver','14','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/python2-scipy','456','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/python3-ase','14','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/python3-scipy','192','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/qalculate-gtk','35','29735.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/qcl','6','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/qhull','9','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/qrupdate','15','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/qtdmm','19','27129','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/qucs','61','31291.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/qucs-s','133','28811.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/qucsator','184','31291.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ramulator','58','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/rbio','12','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/reduce-algebra','704','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/root','2244','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/rpy2','4','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/sage','2416','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/sage-bin','2297','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/samtools','51','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/samtools-legacy','5','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/scidavis','118','27947.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/scilab','541','31994.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/scmutils','16','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/seaview','5','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/seqkit','7','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/sequence-analysis','2','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/sigrok-cli','6','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/sim','1','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/siril','29','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/smath-studio','3','29212.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/snpEff','29','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/solfege','13','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/spades','136','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/speedcrunch','24','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/spiceopus','8','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/spidey','3','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/spqr','19','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/squizz','7','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/staden','90','28681','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/stellarium','278','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/stellarsolver','101','30582.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/stringtie','9','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/suitesparse','800','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/suitesparseconfig','11','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/sundials','13','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/sword','33','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/sword-data-kjv','3','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/t_coffee','16','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/tabix-legacy','2','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/tiemu','32','27834.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/tilem','5','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/tilem2','10','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/topdraw','1','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/tophat','95','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/treebest-ensembl','4','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/treerecs','108','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/treeviewx','12','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/trfind','4','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/trimmomatic','2','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/tuxmath','14','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/tuxtype2','18','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ucsc-blat','1','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ugene','1017','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/umfpack','19','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/units','4','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/vCAPS_coevolution','14','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/vcftools','20','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/velocyto','8','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/verilator','162','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/verilog','216','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/veusz','84','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/voro++','7','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/vul','1','29954.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/wcalc','5','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/wehi-cellsymphony','1','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/wehi-deid','1','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/wehi-hackit','2','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/wise','14','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/wmcalc','3','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/wxMaxima','46','29436.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/x48','63','27477.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/x48ng','59','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/xcircuit','9','28428.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/xephem','38','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/xflr5','100','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/xfoil','5','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/xiphos','20','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/xoscope','5','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/xschem','29','29809.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/xsimd','149','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/yices2','31','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/z3','379','31994.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/zotero','93','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/zotero-gtk2','37','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/Synkron','8','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/blugon','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/easystroke','11','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/eflite','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/espeakup','2','28763.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/flite','31','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/fonts-opendyslexic','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/jpilot','9','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/mag','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/magnus','1','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/mbrola','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/mbrola-voices','219','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/orca','25','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/pastebinit','2','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/picospeaker','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/ruvim','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/sct','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/shorturl','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/svkbd','3','27834.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/svox','15','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/termbin','2','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/unclutter','2','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/unclutter-xfixes','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/wgetpaste','1','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/xdotool','3','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/xsct','1','29690.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/xsel','5','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/xwit','1','28084.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/xzoom','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/ydotool','2','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/DPF-Plugins','223','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/FAudio','3','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/JDSP4Linux','95','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/Luppp','5','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/MP3Diags','37','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/PAmix','3','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/SAM','2','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/SongRec','108','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/SuperCollider','147','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/TiMidity++','21','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/Ymuse','457','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/a2jmidid','3','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/aacgain','33','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/aacplusenc','2','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/abcde','1','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/acousticbrainz-client','2','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/acousticbrainz-gui','5','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/acousticbrainz-music-extractor','6','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/acxi','1','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/aeolus','3','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/aften','4','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/aj-snapshot','3','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/alsa-plugin-jack','5','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/alsa-tools','48','30260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/alsacap','2','28452.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/alsaequal','1','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/alsamixergui','6','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/alsamodularsynth','43','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/amSynth','11','35130.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/amrnb','6','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/amrwb','4','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/apulse','3','29301','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ardour','669','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ario','19','29167.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/asap','30','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/aseqview','7','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/asunder','6','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/atunes','14','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/audacious-aac','6','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/audacious-jack','5','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/audacity','283','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/audioconvert','3','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/azr3','7','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/bitwig-studio','226','27947.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/bristol','19','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/butt','10','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/calf','31','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/calf-ladspa','18','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/callaudiod','3','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/caps','4','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/cardinal','409','28767.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/carla','108','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/cava','6','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/cd-discid','1','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/celt','6','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/celt051','7','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/chuck','63','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/clap','2','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/clementine','296','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/clockchimes','2','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/clthreads','1','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/clxclient','1','28879.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/cmt','2','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/cmus','7','31738.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/connie','2','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/coppwr','65','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/crip','1','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/cubeb','17','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/darkice','8','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/dcaenc','11','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/dcd','1','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/deadbeef','90','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/distrho-ports','57','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/dopamine-bin','102','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/drumkv1','13','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/drumstick','26','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/easymp3gain','6','27918.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/eawpats','14','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ecasound','42','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/exhale','4','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/eyeD3','3','28044.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ezstream','8','29602.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/fIcy','2','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/faac','8','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/faad2','9','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/fabla','4','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/famitracker-qt','10','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/faust','13','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/fdk-aac','41','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ff8dls','18','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/flac-opt','19','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/flac123','3','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/flacon','38','30162.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/fluid-soundfont','72','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/fluidsynth-dssi','6','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/fmit','12','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/fmmidi','3','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/foo-yc20','58','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/freepats','13','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ghostess','4','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gimmix','5','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gmpc','20','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gmpc-plugins','62','28305.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gmusicbrowser','4','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gnomad2','6','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gnome-music','11','29457.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gnome-shell-extension-sound-output-device','1','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gnump3d','2','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gogglesmm','63','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gqradio','7','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/grip','5','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gtklick','1','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/guitarix','132','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gusersoundfont','12','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gvolwheel','9','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/herrie','2','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/horgand','9','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/hydrogen','51','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/icecast','11','29602.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ices','10','29602.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/id3','2','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/id3tool','3','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/id3v2','2','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/invada_studio_plugins_lv2','3','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/iripdb','1','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jaaa','2','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jack','17','28156.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jack-keyboard','3','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jack-rack','4','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jack-tools','5','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jack1','29','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jack_capture','3','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jack_delay','1','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jackmeter','5','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jalv','8','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jamin','14','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jamulus','24','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/japa','2','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jkmeter','2','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jnoise','2','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jvmetro','2','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/klick','6','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/kradio','50','30636.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ladspa-bs2b','5','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ladspa_sdk','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/lash','15','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/libbs2b','6','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/libebur128','3','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/libfreeaptx','2','28727.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/lingot','7','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/linuxsampler','66','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/listener','2','28452.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/lmms','108','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/lsmi','2','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/lv2','13','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/lv2-mda-metapiano','4','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/lv2dynparam1','6','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/lv2file','2','29272.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/lxmusic','8','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mac','7','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/match-vamp-plugin','3','28945.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mda-lv2','6','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/meterbridge','5','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mhwaveedit','8','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/microenc','1','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mididings','34','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/midillo','16','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/milkytracker','12','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/millennium-plugins','2','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/miniAudicle','14','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/minimodem','5','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mixxx','475','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mkcue','3','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mktoc','2','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mp3blaster','6','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mp3check','4','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mp3gain','1','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mp3splt','3','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mp3splt-gtk','6','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mp3val','2','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mp3wrap','3','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mpc','3','29167.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mpd','46','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mpdscribble','6','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mppenc','2','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mt-daapd','6','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/multimidicast','1','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/muse','170','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/musepack-tools','9','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/musikcube','160','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/nas','31','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ncmpc','17','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ncmpcpp','109','29167.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ncspot','217','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/nekobee','2','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ngjackspa','3','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/nnls-chroma','4','28945.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/noisetorch-bin','3','29954.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/non','11','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ntk','17','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ocenaudio-bin','32','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ocp','127','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ogmtools','5','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/opencore-amr','10','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/openvino-plugins-ai-audacity','349','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/opustags','3','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/padthv1','13','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/paman','7','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/paprefs','5','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pasystray','4','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/patchage','13','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pavucontrol-qt','10','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pavumeter','5','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pd','32','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pd_mrpeach','3','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/phat','11','30582.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pianobar','2','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pipewire-native-jack','29','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pithos','2','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/plug','12','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pmidi','3','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pnmixer','3','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/podget','1','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pogo','2','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ponymix','3','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/portmidi','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/puddletag','6','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pulseaudio-ctl','2','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pulsemixer','1','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pyalsaaudio','6','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/python-audiotools','28','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/python3-pyalsaaudio','3','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qastools','39','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qjackctl','13','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qm-vamp-plugins','7','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qmidiarp','11','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qmidinet','4','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qmidiroute','7','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qmmp','66','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qmmp-plugin-pack','7','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qmmp-plugin-pack-qt5','10','28609','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qmmp-qt5','107','28700.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qpwgraph','7','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qsynth','9','29527.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qtractor','399','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/quodlibet','11','29149.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qxgedit','11','29212.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rack','12','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rakarrack','19','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/reaper','24','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rebmp','24','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/retrovol','3','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rev-plugins','1','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rezound','33','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rhythmbox','61','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rosegarden','124','29194.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rplay','5','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rubberband','17','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rubyripper','2','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rumor','6','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/samplv1','15','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/sayonara-player','136','29101.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/sbagen','2','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/schismtracker','12','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/seq24','14','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/setBfree','4','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/sfarkxtc','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/sfc','6','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/shntool','5','29212.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/shorten','4','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/sidplayfp','5','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/siren','4','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/snd','45','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/sndio','4','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/sonata','3','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/sonic-visualiser','120','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/sonobus','236','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/sooperlooper','27','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/soundconverter','5','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/soundkonverter','40','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/soxr','6','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/specimen','7','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/spectmorph','111','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/stops','1','29527.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/streamripper','5','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/streamtuner','16','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/stretchplayer','4','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/sunvox','12','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/swami','8','30582.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/swh-plugins','14','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/swh-plugins-lv2','4','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/synthv1','13','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/tagtool','6','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/tap_plugins','6','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/tapiir','5','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/tauonmb','10','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/transcribe','4','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/tta','3','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/tuner','6','28811.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/tuxguitar','38','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/twolame','5','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/unsf','2','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/vamp-plugin-sdk','18','28945.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/vcf_plugins','2','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/vimpc','15','29167.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/vmpk','10','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/vocoder','2','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/vocoder-ladspa','1','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/volctl','1','28315.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/volti','3','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/volume.app','1','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/volumeicon','4','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/vorbisgain','4','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/waon','3','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/wavbreaker','4','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/whipper','4','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/whysynth','7','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/wildmidi','3','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/wireplumber','9','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/wmusic','2','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/wmusic-xmms','1','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xcfa','11','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xforge','1','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmms-ctl','2','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmms-jack','8','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmms-ladspa','1','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmms-mp4','9','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmms-pulse','5','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmms-scrobbler','7','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmms-shn','6','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmms-wavpack','7','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmms-wma','4','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmms2','18','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmp','3','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xsynth-dssi','6','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xwax','2','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/yoshimi','45','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/zita-ajbridge','1','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/zita-dpl1','2','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/zita-rev1','2','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/zoom-edit-n-share','25','30607.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/zynaddsubfx','49','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/zynjacku','6','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('business/beancount','2','27714.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('business/binance-desktop','130','27714.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('business/bisq','102','27714.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('business/exercism_repack','4','29194.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('business/ledger','49','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('business/maltego','54','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('business/reckon','2','27714.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('business/skrooge','130','27714.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('business/stansoft','22','27714.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/2bwm','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/9menu','1','31056.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/Alphacube-GTK','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/AlsaMixer.app','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/AppStream','21','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/Ater-MPlayer-skin','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/BlockstreamGreen','17','26426.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ClamAV-GUI','17','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ClipboardViewer','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ComixCursors','24','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/LX-Colors-Revival','390','29690.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/Matcha-gtk-theme','36','31985.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/NsCDE','125','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/Pulse-Glass','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/PyPanel','2','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/Qogir-icon-theme','55','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/SiriKali','30','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/Surfn','54','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/SwayNotificationCenter','7','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/SysTray-X','29','29149.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/Tela-icon-theme','18','29194.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/TextSnatcher','6','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/UltimateDailyWallpaper','7','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/Vanilla-DMZ','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/XDecorations','5','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/Xfce-Theme-Manager','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/adwaita-qt','16','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/aero-mouse-theme','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/afterstep','60','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/alacarte','11','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/alltray','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ambiance-radiance-flat-colors-gtk-theme','22','30582.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/amiwm','9','29212.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/anki','118','31033.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ansiweather','12','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/applet-window-buttons','19','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/arandr','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/arc-firefox-theme','8','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/arc-icon-theme','13','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/arc-theme','24','30582.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/areao43-icon-theme','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/artix-backgrounds','5','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/autokey-gtk','3','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/autorandr','1','29527.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/aweather','9','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/awesome','7','29457.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/awesome-appmenu','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/awf','12','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/azote','4','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/baobab','9','28945.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/bashmount','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/bashrun','1','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/bashrun2','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/battery-notify','2','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/bemenu','3','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/bibata-cursor-theme','35','26869','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/birdtray','11','28692.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/blit','2','29457.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/bmpanel2','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/boston-icon-theme','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/bottles','5','30607.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/bspwm','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/bubblemon-dockapp','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/budgie-desktop','21','29272.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/budgie-desktop-view','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/budgie-screensaver','5','30162.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/buku','3','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/bukubrow','35','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/bumblebee-status','4','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/buttonmaker','17','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/byzanz','22','29246.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/caffeine-ng','1','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cairo-clock','9','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cairo-dock','13','31056.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cairo-dock-plugins','26','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/caja','29','35130.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/calcure','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/calls','14','28945.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/candle','14','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/catfish','4','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cbatticon','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ccsm','3','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cde','428','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cellwriter','4','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cheese','19','30162.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cinnamon-desktop','9','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/clearlooks-classic-themes','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/clearlooks-phenix-theme','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/clipit','7','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/clipmenu','1','28705.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cmatrix','4','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cnslock','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/compiz-boxmenu','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/compton','10','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/conky-manager','5','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/copyq','20','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/crystalcursors','14','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ctwm','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cutefish-calculator','5','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cutefish-core','30','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cutefish-dock','7','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cutefish-filemanager','13','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cutefish-gtk-themes','5','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cutefish-icons','528','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cutefish-launcher','7','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cutefish-screenlocker','6','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cutefish-settings','11','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cutefish-statusbar','12','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cutefish-terminal','8','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cutefish-wallpapers','4','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cwm','1','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cwm-openbsd','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/dapper','5','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/dbgl','39','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/dbgl-legacy','10','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/devilspie','7','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/devilspie2','2','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/disper','1','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/dmenu','2','28705.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/docfetcher','37','29735.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/doctest','12','28305.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/dolphin-megasync','10','29272.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/domterm','48','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/dpscreenocr','114','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/draco','65','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/dunst','3','28315.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/durden','3','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/dwl','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/dwm','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/dwmblocks','2','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/dzen2','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/e16','62','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/e16.theme.pack','9','29293.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/echinus','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ede','77','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ee','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/emerald','10','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/emerald-themes','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/emwm','3','29293.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/engrampa','14','28763.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/enlightenment','169','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/evilwm','2','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/faba-icon-theme','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/fbautostart','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/fbmenugen','1','28763.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/fbpanel','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/fbrowse-tray','2','29735.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/fdpowermon','1','29696.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/fehlstart','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/find-cursor','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/flat-remix','567','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/flatpak','20','30636.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/flatpak-builder','8','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/flatwoken-icon-theme','10','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/florence','8','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/foot','7','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/fusion-icon','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/fuzzel','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/fvwm-crystal','9','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/fvwm3','22','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/fvwm95','10','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gammastep','19','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gdm','14','29272.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gfi','2','29822.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gis-weather','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gkrellm-themes','12','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/glabels','25','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gmrun','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gmusicbrowser-art','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-backgrounds','22','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-boxes','25','28305.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-browser-connector','2','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-calculator','23','28945.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-calendar','14','28945.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-characters','7','28945.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-clocks','12','30162.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-connections','11','28305.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-contacts','10','30162.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-control-center','44','28945.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-font-viewer','4','30162.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-icon-theme','57','28763.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-icon-theme-stock','7','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-maps','8','28945.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-menus','18','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-photos','19','28945.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-screenshot','5','28945.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-session','6','30162.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-settings-daemon','12','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-shell','21','29272.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-shell-extension-appindicator','3','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-shell-extension-arc-menu','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-shell-extension-dash-to-panel','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-shell-extensions','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-tour','77','28945.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-tweaks','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-weather','3','28945.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gntp-send','7','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gopsuinfo','8','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gr1d','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/grim','2','27768.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/grits','7','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gromit-mpx','3','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/grun','9','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gsimplecal','9','29436.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtk-engines','13','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtk-engines-unico','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtk-theme-Bluebird','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtk-xfce-engine','9','29602.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtk1-industrial-engine','11','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtk1-theme-switch','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtkhash','20','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtklock','3','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtklock-playerctl-module','2','29293.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtklock-powerbar-module','2','29293.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtklock-userinfo-module','2','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gxkb','5','27834.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/happy-hacking-gnu','3','27864','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/herbe','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/herbstluftwm','27','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/hhg-gui','45','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/hikari','8','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/hsetroot','1','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/human-gtk-theme','7','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/hushboard','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/hyfetch','4','29301','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/hyprland-bin','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/i3','10','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/i3-easyfocus','2','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/i3-gaps','11','28705.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/i3blocks','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/i3ipc-glib','18','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/i3lock','9','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/i3minator','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/i3pystatus','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/i3situation','2','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/i3status','4','29457.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/icewm','125','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/icewm-themes','18','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/idesk','36','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ideviceinstaller','4','29272.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/idevicerestore','5','30607.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/imgur-screenshot','2','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/insync-thunar','1','28763.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ion','19','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/j4-dmenu-desktop','8','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/jgmenu','5','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/joystickwake','2','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/jwm','7','28811.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/kanshi','4','31408','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/kanyremote','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/kbdd','8','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/kde44-oxygen-molecule','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/kdocker','8','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/keynav','2','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/kfaenza-icon-theme','49','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/kolorcontrol','1','29602.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/kupfer','8','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/kwalletcli','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/labwc','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/leftwm','36','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lemonbar','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lemonbar-xft','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lfm','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/libqt6xdg','18','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/libqtxdg','12','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/light','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lightdm-slick-greeter','9','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/liquidshell','28','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lumina','886','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/luna-wallpapers','25','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lxappearance','5','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lxappearance-obconf','10','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lxde-common','5','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lxhotkey','6','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lxlauncher','7','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lxmed','1','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lxmenu-data','4','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lxpanel','22','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lxqt-menu-data','3','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lxqt-panel','81','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lxqt-runner','9','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lxqt-themes','11','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/macopix','24','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/maitreya','65','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/marco','16','29457.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mate-control-center','53','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mate-desktop','11','35130.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mate-desktop-environment','1','35130.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mate-media','7','28681','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mate-menus','7','28477.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mate-panel','30','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mate-power-manager','75','28681','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mate-screensaver','10','35130.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mate-session-manager','9','29338.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mate-settings-daemon','19','35130.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mate-system-monitor','57','29338.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mate-terminal','9','29527.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mate-utils','139','28681','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/materia-theme','16','30582.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/matrixgl','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mcwm','2','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/meliae-icon-theme','10','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/menulibre','6','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/menumaker','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/menutray','2','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mh5000','11','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mhsetroot','7','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mimi','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mint-l-icons','98','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mint-l-theme','61','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mint-themes','67','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mint-x-icons','22','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mint-y-icons','90','27910.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/moka-icon-theme','57','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mozo','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mplayer-skins','9','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mugshot','3','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/murrine','11','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/murrine-themes','6','30582.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/musca','2','28705.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mutter','56','29272.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mwm-utils','2','29293.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/myGtkMenu','2','29735.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/naps2','23','28305.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nautilus','22','30162.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ncpamixer','5','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/netwmpager','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/neutral','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/neutrality-cursor-theme','2','35130.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nitrogen','7','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nitrokey-app','46','29436.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/noice','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nordic-gtk-theme','10','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/notification-daemon','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/notion','12','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/numix-blue-theme','2','30582.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/numix-folders','9','29194.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/numix-icon-theme','13','29194.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/numix-icon-theme-circle','7','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/numix-icon-theme-square','7','29194.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/numix-ocean-theme','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/numix-pack','130','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/numix-sx-theme','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nwg-bar','453','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nwg-clipman','3','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nwg-displays','2','29212.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nwg-dock','461','30162.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nwg-drawer','459','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nwg-hello','3','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nwg-icon-picker','2','27768.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nwg-launchers','15','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nwg-look','452','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nwg-menu','453','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nwg-panel','4','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nwg-readme-browser','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nwg-shell','3','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nwg-shell-config','2','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nwg-shell-wallpapers','6','28712.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nwg-wrapper','2','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/obapps','1','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/obbrowser','2','29735.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/obconf','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/obkey','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/obmenu','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/obmenu-generator','1','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/obsession','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/obshutdown','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/oomox','133','29293.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/openbox','24','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/openbox-menu','2','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/openbox-themes','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/openbox-themes-extra','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/orage','11','29822.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/paginator','2','29822.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/paper-icon-theme','47','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/papirus-icon-theme','79','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/parcellite','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/pcb2gcodeGUI','27','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/pclock','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/peksystray','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/pekwm','18','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/perwindowlayoutd','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/pfetch','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/picom','10','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/pidgin-guifications','7','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/pidgin-libnotify','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/pidgin-window_merge','10','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/pinentry-dmenu','4','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/plank','41','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/plasma5-themes-breath','13','35130.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/plasma5-themes-breath-migration','5','35130.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/polar-cursor-themes','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/polybar','33','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/progman','4','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/protozoa-cursor-themes','15','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ptwit','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/py3status','4','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/pyful','2','27477.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/python-specbar','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/python3-autotiling','1','35130.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/pytyle','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/q4wine','37','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/qlipper','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/qmmp-skins','5','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/qt5ct','25','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/qt6ct','14','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/qtile','6','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/qtile-extras','5','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/qtpass','17','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/quickswitch-i3','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ratpoison','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/razercfg','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/razorqt','61','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/rclone-browser','19','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/recoll','68','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/redshift','8','28452.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/river','26','29272.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/rodent','37','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/rofi','7','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/rofi-calc','11','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/rofi-emoji','14','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/rofi-lbonn-wayland','7','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/rofi-pass','2','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/root-tail','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/rootactions_servicemenu','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/rsl','41','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/rss-glx','15','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/rss-guard','144','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/s1kls','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/salmon','2','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/screenfetch','1','29954.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/screenkey','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/sddm-theme-artix','1','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/sdorfehs','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/seafile-client','50','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/sfwbar','4','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/sgsearch','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/simplenote','87','27804','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/skb','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/skippy-xd','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/skulpture','7','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/sl-fortune','1','28727.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/slack-wallpapers','141','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/slackware-xdg-menu','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/slackware-xdm-theme','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/slbar','2','28763.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/slock','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/slstatus','1','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/slurp','2','27768.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/smaragd','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/somebar','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/spectrwm','7','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/stalonetray','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/standard-notes-bin','110','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/stoken','15','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/subtle','10','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/sun','2','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/superkey-launch','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/suru-plus-icon-theme','666','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/sushi','9','29822.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/sway','9','27768.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/swaybg','2','27768.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/swayidle','2','27768.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/swaylock','3','27768.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/sxhkd','2','29916','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/sync-engine','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/synce-kpm','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/thunar-archive-plugin','8','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/thunar-custom-actions','1','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/thunar-media-tags-plugin','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/thunar-megasync','8','29272.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/thunar-sendto-clamtk','1','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/thunar-vcs-plugin','9','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/tint2','28','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/todoman','4','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/trayer-srg','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/tty-clock','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ufetch','2','31738.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/unsaver','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/uwm','10','28754.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/vala-panel-appmenu','8','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/variety','4','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/vbox-runner','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/vdesk','3','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/velox','4','28692.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/vertex-maia-icon-themes','17','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/vim-qt','27','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/vinagre','15','28305.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/vtwm','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wally','19','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/waybar','35','31738.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wayfire','28','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wbar','11','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wbg','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wcm','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wdisplays','3','31033.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/weston','13','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wf-config','11','28305.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wf-shell','12','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wl-mirror','2','29457.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wlr-randr','3','27768.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wlsunset','3','29293.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmCalClock','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmMoonClock','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmSun','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmSunMoon','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmakerconf','12','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmalauncher','3','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmamixer','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmbackground','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmbattery','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmbiff','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmblob','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmbutton','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmcellauto','5','29301','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmcliphist','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmclock','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmclockmon','5','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmcpufreq','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmcpuwatch','3','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmctrl','4','29822.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmcube','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmdate','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmdl','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmdrawer','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmfire','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmfishtime','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmii','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmimagedock','3','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wminfo','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmix','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmmand','2','28585.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmmixer','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmmon','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmname','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmnet','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmphoto+','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmpinboard','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmpower','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmrandwalk','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmshutdown','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmslideshow','1','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmsm.app','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmsorsen','2','29527.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmstickynotes','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmsupermon','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmswallow','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmsysmon','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmsystemtray','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmtime','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmtimer','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmtop','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmudmount','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmuptime','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmweather','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmweather+','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmwebcam','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmwifi','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmxss','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wofi','5','47951.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wofi-pass','1','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/workrave','55','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wtype','3','47951.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/x-tile','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/x2x','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xapp-symbolic-icons','2','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xdg-desktop-portal-gtk','8','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xdgmenumaker','2','27804','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xdm-slackware-theme','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xdo','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xearth','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce-hkmon','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-alsa-plugin','4','30607.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-battery-plugin','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-cpufreq-plugin','11','29602.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-cpugraph-plugin','9','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-datetime-plugin','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-dict','7','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-diskperf-plugin','7','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-docklike-plugin','8','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-eyes-plugin','7','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-fsguard-plugin','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-genmon-plugin','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-i3-workspaces-plugin','15','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-mailwatch-plugin','5','28585.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-mount-plugin','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-mpc-plugin','6','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-netload-plugin','7','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-notes-plugin','8','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-places-plugin','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-sensors-plugin','13','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-smartbookmark-plugin','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-timer-plugin','7','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-verve-plugin','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-volumed-pulse','5','29602.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-wavelan-plugin','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-windowck-plugin','8','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-xkb-plugin','7','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfdashboard','22','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfwm4-themes','14','29602.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xkblayout-state','2','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xlunch','2','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xmenu','2','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xmms-skins-almond','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xmms-skins-ater','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xmms-status-plugin','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xmobar','26','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xmonad','24','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xmouseless','2','29954.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xob','3','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xonclock','9','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xpenguins','6','28763.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xpenguins-xtrathemes','2','28763.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xpra','165','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xsecurelock','6','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xsession','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xsession-xinitrc','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xseticon','2','30162.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xss-lock','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xtitle','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xtrascreenhacks','8','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xtrlock','2','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xvkbd','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xwallpaper','6','29954.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xwarppointer','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xxkb','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/yad','16','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/yambar','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/yaru-icon-theme','61','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/zenity','14','29457.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/zuki-themes','2','30582.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/4th','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/9pro','1','27947.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Archi','85','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Arduino','1273','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Bear','34','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/CImg','6','31994.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/CVSTools','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Catch','1','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Catch2','14','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ChezScheme','202','29194.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Coin','100','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Fennel','3','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/FreeBASIC','34','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/GHDL','88','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/HeidiSQL','20','28467.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Hex2bin','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ICTester','14','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/J-Link','22','30352.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/JAForth','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Kiro','140','28044.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/LLgen','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/MinForth','6','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/MySQL-python','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/OpenCL-CLHPP','1','35130.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/OpenHantek6022','15','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/OpenJDK11','787','29436.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/OpenJDK17','748','29436.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/OpenJDK21','890','29436.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ParadisEO','23','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/PhpStorm','1232','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Pivy','117','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/PythonToolkit','3','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/QStlink','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/QtPy','3','29194.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/REDasm','172','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/SQLAlchemy','25','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/SQLiteStudio','245','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ShellCheck-bin','6','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/SmallBASIC','65','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/SoQt','21','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Sphinx','9','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/StarPU','63','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/SystemC','24','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/THE','17','28718.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/UASM','3','30260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/VisUAL','6','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Xdialog','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/XiMpLe','3','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/a68g','13','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/abseil-cpp','40','27854.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/acc','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/acme','3','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/acme2k','18','29457.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/acpica','10','29293.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/actionlint','21','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ada-2022-rm','3','30582.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/adms','14','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/aflex','8','30162.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/aflplusplus','83','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/alembic','3','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/alex','9','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/alire','87','29457.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/amazon-corretto','82','28315.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/android-studio','578','29167.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/android-studio-for-platform','596','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/android-tools','52','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/antigravity','161','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/antlr2','17','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/apache-ant','11','27129','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/apache-ivy','3','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/apache-jmeter','35','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/apache-log4cxx','27','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/apache-maven','6','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/arcan','28','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/arduino','64','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/arduino-cli','10','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/argagg','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/argouml','8','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/arm-binutils','36','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/arm-gcc','267','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/arm-gdb','122','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/as31','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/asar','11','28718.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/asdf','7','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/asem51','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/asl','64','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/asm6809','10','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/aspnetcore-runtime-10.0','41','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/aspnetcore-runtime-7.0','56','29345.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/aspnetcore-runtime-8.0','37','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/aspnetcore-runtime-9.0','40','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/astyle','4','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/async-profiler','1','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/atasm','3','28084.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/atom-amd64','129','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/autogen','55','28811.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/avarice','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/avr-binutils','78','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/avr-gcc','787','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/avr-gdb','118','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/avr8-burn-o-mat','2','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/avr8-gnu-toolchain','30','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/avra','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/avrdude','17','28305.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/aws-cdk','9','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ayacc','11','30162.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/azuredatastudio','122','29735.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bacon','539','31738.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bakefile','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bas','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bas55','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bascat','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bashdb','7','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bats-core','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bazel','463','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bbcsdl','46','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bbe','5','28945.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bcc','60','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bcpp','4','29301','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/beautysh','2','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/beav','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bed','20','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/berkeley-abc','84','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bfg','5','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bigloo','110','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/binwalk','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/blassic','8','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bless','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/blocksruntime','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bluefish','29','30582.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bmake','43','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/boost-di','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/boron','3','28945.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bpftrace','62','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bpython','2','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bruno','105','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bsdiff','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/buf','69','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/build2','270','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bun-bin','37','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bvi','4','31994.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bviplus','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bw-atari8-tools','5','31408','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bwbasic','13','28609','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/byacc','5','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bzr','42','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cJSON','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/c_count','4','27864','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/camlp4','92','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/camlp5','69','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cargo-auditable','34','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cargo-c','234','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cargo-vendor-filterer','15','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/casperjs','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/castxml','16','30162.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cbmc','529','28093.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cc-tool','36','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cc65','16','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ccl','29','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cf-cli','9','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cgdb','8','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cgit','14','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/chameleon','38','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/chatgpt_sh','2','28609','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/checkbashisms','1','28811.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/chibi-scheme','19','28767.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/chicken','299','29345.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ciforth','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cim','13','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/claude-code','12','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/clion','1015','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/clips','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cloc','5','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/clojure','5','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cmake-opt','300','27947.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cmocka','6','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/coccinelle','73','28794.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cocotb','29','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/codeblocks','150','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/codelite','680','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/codespell','4','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/codex','78','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/colm','51','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/colordiff','2','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/complexity','13','28811.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/composer','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/config-file-validator','13','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/corrosion','3','30607.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cosmocc','266','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cpan2tgz','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cppcheck','45','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cppcms','45','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cppdb','7','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cppunit','35','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cproc','2','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cproto','4','29301','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cream','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/creduce','167','28763.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/critcl','12','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/crosstool-ng','20','29149.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/crystal-lang','55','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/csmith','12','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cudatoolkit','769','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cudatoolkit_12','455','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cudatoolkit_12.6','369','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cudatoolkit_13','284','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cunit','14','29822.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cursor','194','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cutter','107','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cvsd','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cvsps','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cxref','7','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cxxtest','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cxxtools','84','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/d-tools','77','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dart-sass','21','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dart-sdk','147','29167.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dasht','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dasm','2','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/datagrip','477','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dbeaver-ce','79','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ddisasm','2081','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dejagnu','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/deno','593','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/deno-bin','51','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/devhelp','13','29822.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/device-tree-compiler','10','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dfsimage','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dfu-programmer','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dhex','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/diakonos','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/diffoscope','7','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/difftastic','80','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/diffuse','4','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/digikey-kicad-library','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dis','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dis6502','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dkms','3','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dmd','41','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dotnet-runtime-10.0','30','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dotnet-runtime-3.1','27','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dotnet-runtime-5.0','27','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dotnet-runtime-6.0','27','28811.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dotnet-runtime-7.0','42','29345.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dotnet-runtime-8.0','27','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dotnet-runtime-9.0','29','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dotnet-sdk-10.0','111','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dotnet-sdk-3.1','58','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dotnet-sdk-5.0','68','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dotnet-sdk-6.0','96','28811.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dotnet-sdk-7.0','180','29345.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dotnet-sdk-8.0','116','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dotnet-sdk-9.0','114','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dpkg','26','31994.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dreampie','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dte','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dune','21','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/eagle','48','28093.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ebe','34','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ecl','131','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/eclipse-cpp','120','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/eclipse-java','100','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/eclipse-jee','208','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/eclipse-php','181','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/eclipse-rcp','122','30607.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/edb-debugger','53','29293.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/edwood','9','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/egypt','1','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/eisl','7','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/electron','97','27910.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/electron-bin','94','31595.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/elixir','62','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/emsdk','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/energia','79','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/enki','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/envision','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/envytools','36','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/eovim','4','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/epydoc','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/erlang-otp','132','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/es-shell','10','27804','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/evemu','11','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ex-vi','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/extra-cmake-modules-opt','3','28564.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/f2c','3','28794.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/fasm','3','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/flawfinder','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/fly','13','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/focal','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/fortitude-bin','5','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/fortls','3','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/fossil','45','31994.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/fpc','218','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/freetds','25','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/frobtads','19','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ftjam','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ftnchek','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/g3log','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gambas3','735','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gambit','83','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gauche','222','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gcc14','3217','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gcc5','767','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gcli','5','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gdb-dashboard','1','29293.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/geany','42','29211.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/geany-plugins','67','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/geany-themes','2','28315.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gecode','121','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gede','24','29177.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gedit','33','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gedit-plugins','13','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/generatorrunner','7','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gengetopt','43','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gerbil','2283','29246.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gf','10','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gforth','28','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ghex','12','29822.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ghi','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ghidra','176','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gisto','40','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/git-cola','8','28754.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/git-crypt','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/git-extras','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/git-lfs','5','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/git-tools','1','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/github-cli','17','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/github-desktop','106','28711','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gitify-bin','96','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gitlab-cli','14','29177.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gitnuro','44','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gitstats','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/glm','14','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/global','15','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gn','45','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gnats','10','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gnatstudio-bin','473','27804','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gnome-builder','158','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gnome-common','3','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gnome-system-monitor','22','28945.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gnu-apl','56','29246.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gnulib','26','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gnuradio','650','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gnustep-base','29','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gnustep-make','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gob2','3','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/godot','934','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/goland','564','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/golang-googlecode-gonet','2','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/golang-googlecode-gotools','38','29954.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/golangci-lint','59','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/google-cloud-cli','165','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/google-go-lang','539','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gopls','55','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/got','25','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gplcver','7','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gprbuild','139','29457.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gprolog','21','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gpsim','34','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gputils','84','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gr-osmosdr','24','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gradle','83','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/grails','65','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gsoap','56','29293.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gtest','15','30231.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gtkdialog','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gtkextra','13','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/guile-colorized','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/guile1.8','63','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/guile2.0','663','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/guile2.2','232','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/guitar','27','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gum','5','30607.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/guzzle_sphinx_theme','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/hadolint-bin','19','31994.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/halibut','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/happy','14','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/hare','19','30582.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/harec','2','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/haxe-bin','16','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/helix','319','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/henplus','2','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/heroku-cli','128','29177.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/hexcurse','6','30162.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/hexe','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/hexedit','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/hexer','2','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/hexinator','92','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/hexnet','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/highlight','38','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/hipSYCL','12','29448','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/hoppscotch','24','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/howl','15','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/hte','13','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/hub','17','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/hugo','19','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/icdiff','2','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/icecream-sundae','6','28044.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/icemon','7','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/icon','26','29194.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/icoutils','23','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/idea','1191','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/idlex','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ieee-pilot','4','29602.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ignition-math','11','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/inform','6','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ioncube-loader','2','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ispc','874','31994.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jam','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/janet','12','28322.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jasspa','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/javacc','1','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/javafx-scenebuilder','116','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jdk','72','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jdk11','99','48046.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jdk19','84','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jdk23','97','28084.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jeex','13','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jenv','2','28315.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jpm','1','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jsmin','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jujutsu','278','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/julia','78','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/julius','9','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter-ipykernel','2','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter-ipywidgets','2','28428.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter-nbclassic','20','31985.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter-nbclient','2','27910.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter-nbconvert','3','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter-nbformat','2','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter-notebook','49','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter-notebook_shim','2','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter-qtconsole','2','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter-widgetsnbextension','1','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter_client','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter_console','2','28305.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter_core','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter_events','2','28044.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter_packaging','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter_server','4','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter_server_terminals','3','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyterlab','33','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyterlab_pygments','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyterlab_server','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kForth-64','6','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kaaedit','6','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kakoune','33','28084.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kawa.bin','3','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kconfig-frontends','7','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kcov','13','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kdesvn','41','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kdiff3','57','29293.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kelbt','7','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kicad','1754','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kicad-doc','54','30149.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kicad-footprints','91','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kicad-packages3D','1054','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kicad-symbols','18','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kicad-templates','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kiwi','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kodespel','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/komodo-edit','89','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/komodo-ide','99','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kotlin','42','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/latrace','9','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/lazarus','339','27918.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/lazygit','43','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/lcov','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ldc','169','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ldns','105','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/le','40','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/lean-elan','43','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/leiningen','2','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/leo','17','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/lepton-eda','48','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/leveldb','14','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/lfe','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/libb2','12','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/libcutl','11','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/liberica-jdk-bin','83','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/libnvidia-container','38','29046.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/libtree','1','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/libxbgi','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/lighttable','196','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/likepython','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/liquibase','20','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/litehtml','12','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/llvm-opt','3089','31994.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/lm-studio-bin','299','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/logisim','2','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/love','36','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/love-legacy072','17','28811.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/love-legacy080','20','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ltrace','19','29149.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/lua','3','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/lua-language-server','77','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/lua51','2','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/luajit','7','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/luarocks','3','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/lwtools','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mads','6','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/maude','219','29448','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mawk','6','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mcu8051ide','4','28811.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/md2roff','2','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mdbook','46','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/medit','19','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/meld','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/meld3','5','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/memchan','5','28681','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/menhir','11','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/metakit','10','28093.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mg-openbsd','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mg-troglobit','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/micro','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/minikube','31','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/minimal-basic','7','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/minipro','2','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mit-scheme','105','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mkchroot','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mle','6','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mlr','16','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mlton','466','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mm-common','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mmtf-cpp','2','29954.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mockpp','30','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mold','338','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mongodb-compass','112','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mongodb-shell','106','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mono','1544','29212.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mono-addins','24','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mono-basic','23','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mosml','13','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mphidflash','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mrustc','263','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/msbasic2ascii','2','29246.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/msc-generator','188','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mspgcc-legacy','307','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mstch','14','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mtasc','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mtd-utils','52','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/muon','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mutagen','5','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/myrddin','12','29457.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mysql-workbench','2996','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nant','9','29212.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nchexedit','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ne','15','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/neatvi','2','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nedit','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/neko','8','29527.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/neovim','224','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/neovim-qt','24','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/neovim-remote','2','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/netbeans','169','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/newlib','115','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/newlisp','9','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nextvi','3','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nexus','82','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nickle','8','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nim','308','27918.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ninja-ide','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nodeenv','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nodejs','1188','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nodejs18','909','27932.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nodejs22','2121','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nodejs24-bin','60','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nose','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/notepad++','3','29139.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/notepadnext','56','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/notepadqq','44','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/noweb','4','29194.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/npm2tgz','1','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/numpy-legacy','150','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nvidia-container-toolkit','25','28700.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nwjs','74','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/obcpl','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/obnc','9','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/obnc-libext','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/obsidian','132','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ocaml','163','29513.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ocaml-num','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ocamlbuild','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/octant','43','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/omake','33','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/omniORB','52','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/omniORBpy','20','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/oneDPL','7','29293.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/oneTBB','256','28084.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/onetrueawk','3','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/oni','103','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/opencomal','6','28428.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/opencvs','8','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/openjdk7','1564','29345.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/openjdk8','992','29101.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/openmodelica','689','28692.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/openocd','29','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/opentofu','119','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/openvr','8','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ophis','6','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/osc','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/oscar64','33','29436.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/osmo-sdr','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/p4','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/p4d','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/p4v','67','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pacoteco','2','29222','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/page','21','31994.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pahole','7','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pandas','414','29602.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pasdoc','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/patchutils','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pcb2gcode','1743','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pcbasic','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pcc','9','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pd-psql','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/peg','2','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/perf','25','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pforth','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pgmodeler','257','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/phantomjs','9','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/php-apcu','8','28711','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/php-xdebug','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/php82','191','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/php84','197','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/phpDocumentor','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pingendo','40','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pinner','3','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pip2tgz','1','27477.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pkgconf','33','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pkgdiff','1','30607.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pli','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pnpm','8','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pocl','50','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/poedit','53','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/polyml','37','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/portaudio','10','28452.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/positron','331','29182.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/postman','85','30607.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pp','2','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pre-commit','3','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/premake','3','29457.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/premake5','8','29457.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/processing','80','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/protobuf3','158','27854.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/protoc-gen-go','12','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/protoc-gen-go-grpc','9','26869','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pstack-expect-gdb','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pudb','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pulsar','200','31738.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pycharm','597','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pyclewn','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pycscope','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pyenchant','2','29272.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pylint','4','26426.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pyside-tools','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/python-sphinx-lv2-theme','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/python2-cycler','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/python2-numpy','82','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/python3-cycler','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/python3-hylang','3','30582.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/python3-hyrule','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/python3-matplotlib','87','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/python3-numpy','98','28824.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/python3-numpy-legacy','83','28936.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/python3-pyqt-distutils','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/python3-spyder-kernels','2','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pyvim','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/qb64','4','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/qb64pe','27','28067.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/qbe','2','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/qbs','123','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/qbsolv','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/qconf','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/qt-creator','1140','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/qt-creator-llvm','2769','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/quickjs','35','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/quickjs-ng','25','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/quilt','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/racket','588','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/radare2','126','29696.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ragel','53','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rapidsvn','35','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rarian','17','27129','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ratfor','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rc','2','29457.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rebar','4','26859.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rebar3','16','27477.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/reckless','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/recutils','35','28585.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/redo','5','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/reflex','5','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/regexxer','9','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/regina-rexx','43','28718.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/remake','11','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/reuse-tool','3','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rgbds','12','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rider','984','28253.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rizin','125','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rmac','8','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/robomongo','11','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/robotframework','12','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rocmtoolkit_6','2405','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rocmtoolkit_7','3014','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/roswell','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rr','53','29194.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rstudio-desktop','186','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rstudio-desktop-legacy','87','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rtags','62','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rtaudio','15','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rubymine','514','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ruff','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rust-opt','145','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rust16','103','28758.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rustup','132','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/s51dude','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/s9fes','7','30636.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sam','7','29457.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/samurai','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sasm','15','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sauce','19','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sbcl','1135','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sbt','14','29267.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/scala','10','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/scala3','78','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/scheme48','17','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/schroot','51','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/scikit-build','2','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/scite','37','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sdcc','235','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/se','5','27804','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/setedit','75','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/setuptools-git','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/shc','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/shed','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/shiboken','16','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/shiboken2','25','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/simavr','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/simulavr','90','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/slibtool','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sloccount','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/smalltalk','55','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/smartgit','68','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/smartsvn','42','28044.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/snack','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/snappy','6','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sonarqube','96','29436.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/souffle','202','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/source-highlight','33','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sourcegit','41','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/spice-protocol','2','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/spin','8','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/splint','41','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/spyder','44','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sqlcl','31','30582.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sqldeveloper','158','30582.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sqlitebrowser','263','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sqliteman','15','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/squeak-vm','14','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/squirrel-sql','23','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/src-vcs','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/srcML','177','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/srecord','29','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/stagit','2','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/staruml','66','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/stgit','7','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/stklos','26','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/stlink','6','30260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/stm32cubeide','536','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/stm32cubeprog','172','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/stubb','2','29301','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sublime_merge','7','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sublime_text','18','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/svn-all-fast-export','5','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/swfmill','12','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/swi-prolog','132','31994.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sysprof','16','29822.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/systemtap','57','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tcc','11','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tcludp','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tclvfs','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tea','16','29101.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/teensy_loader_cli','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/terraform','162','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/terraform-ls','32','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/texi2html','12','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/textadept','31','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tfenv','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tflint','62','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/thread','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tig','6','29301','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tiled','109','29149.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tinyscheme','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tkcon','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tkcvs','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tkdiff','1','29167.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tkdnd','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tklib','5','28681','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tkman','2','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tktable','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tofuenv','1','28718.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tora','171','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/trac','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tracy','11','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/trf','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ttk-themes','7','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tup','67','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tweak','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/txr','28','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/txt2man','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/typescript','35','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/u-boot-tools','37','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ucblogo','11','29167.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ucpp','13','29735.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/udis86','15','29101.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/uemacs','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/uftrace','15','28609','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/uncrustify','29','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/unicorn','86','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/unifdef','1','27778.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/unittest-cpp','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/universal-ctags','51','29149.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/unmake','2','30607.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/unreliablefs','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/uprog2','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/urjtag','18','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/urn','10','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/uxd','2','32580.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/uz80as','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/vala-language-server','9','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/valabind','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/valgrind','97','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/valkyrie','10','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/vasm','23','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/vc-intrinsics','8','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/velero','33','30607.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/viewvc','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/vif','145','28585.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/vile','55','29527.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/vis','8','29457.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/vispatch','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/voc','25','29457.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/vscode-bin','114','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/vscode-insiders-updater','1','27969','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/vscodium','105','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/vstudio','88','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/watchman','47','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/webstorm','502','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/whisper.cpp','28','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/witsy','264','29149.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/wla-dx','20','28718.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/wxFormBuilder','38','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/wxHexEditor','20','28305.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xa','3','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xasm','2','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xatmas2','2','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xkeys-sdk','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xmake','58','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xmlcopyeditor','31','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xmlstarlet','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xnedit','16','27129','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xsp','18','29212.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xtruss','7','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xvi','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xvile','56','29527.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xxd-standalone','2','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xxdiff','11','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/yabasic','7','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/yaegi','30','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/yarn','3','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/yosys','127','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/z88dk','266','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zeal','28','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zed-editor-bin','89','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zig','84','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zile','26','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zizmor','203','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zmac','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zmakebas','1','28609','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zooey','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zope.interface','4','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zope.sqlalchemy','3','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zope.testing','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zulu-openjdk11','85','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zulu-openjdk17','87','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zulu-openjdk21','87','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zulu-openjdk25','89','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zulu-openjdk6','57','29735.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zulu-openjdk7','60','29735.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zulu-openjdk8','74','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zxbasic','161','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zxbasicstudio','39','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zxtext2p','1','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/0ad','1259','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/0ad-data','766','31985.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/1oom','18','31994.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/2048-in-terminal','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/3D-Fasteroids','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/4digits','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/7kaa','64','31994.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/BeastieWorker','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/BeastieWorker-levels','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/BrogueCE','5','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/Chatbot-Eliza','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ClassiCube','8','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/CorsixTH','15','31994.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/FlightGear','899','29954.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/FreeSerf','12','27969','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/KAdventure','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/KoboDeluxe','10','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/LucasChess','214','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/Mindustry','31','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/OpenJK','56','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/OpenRA','31','29212.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/OpenXcom','73','29167.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/PrismLauncher','167','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/PrismLauncher-Unlocked','226','29301','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/Pyfa','10','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/QuickNES-Core','3','27932.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/RMG','508','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/RetroArch','63','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/SLADE','154','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/Scorched3D','149','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/Sokoban','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/SpaceCadetPinball','8','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/SpaceZero','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/Tempus-Irae','69','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/UrbanTerror','392','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/VASSAL','12','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ZEsarUX','36','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/abe','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/abuse','20','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ace','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/acidwarp','2','29690.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/adl','7','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/adom','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/advancemame','133','27804','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/advancescan','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ags','51','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/aisleriot','24','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/aklabeth','6','27834.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/alephone','117','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/alienarena','309','27834.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/alienwave','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/an','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/angband','45','29212.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/antares','52','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/antimicrox','31','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ardentryst','9','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/armagetronad','88','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/arnold-cpc','7','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/arx-libertatis','60','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/asc','141','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/asciijump','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/asciisec','5','31994.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/assaultcube','47','29735.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/assaultcube-reloaded','41','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/asteroidsinfinity','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/astromenace','53','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/atanks','8','27834.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/atari++','23','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/atari800','8','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/atari800_roms','2','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/atari_8bit_utils','4','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/attract','23','28832.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/augustus-game','44','28727.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/barrage','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bastet','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/beetle-psx-libretro','87','35411.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/berusky','12','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/berusky-data','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/berusky2','22','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/berusky2-data','207','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bitfighter','69','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/blasphemer','9','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/blobby2','13','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/blobwars','38','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bloodfrontier','133','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bluemoon','37','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bluez-sixaxis','18','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/blupimania','20','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bomberclone','7','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/boosteroid','42','29293.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/boswars','145','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/braincurses','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/brainparty','22','29602.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/brickout','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/briquolo','15','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/brutalchess','10','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bsd-tetris','2','29690.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bstone','25','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/burgerspace','5','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bzflag','110','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/caesar3','9','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/caph','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cataclysmdda','71','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cbonsai','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ceferino','8','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cgames','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cgmadness','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cgoban','9','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/chessx','84','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/chexquest','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/chexquest3','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/chocolate-doom','27','28705.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/chroma','8','31172.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/chromium-bsu','10','29602.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/circuslinux','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ckmame','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/clksignal','113','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/clksignal-roms','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/clonekeen','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/colem','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/commandergenius','113','29602.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/connectagram','7','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/conquest','37','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cosmic_assault','10','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cosmosmash','5','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cowsay','1','29809.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cpat','4','28754.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/crack-attack','15','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/crawl','311','28692.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/crimson','9','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/crispy-doom','26','27969','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/crossfire-client','6','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/crrcsim','42','28452.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cryptokwot','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cse2','19','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cubosphere','148','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/curseofwar','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cutechess','57','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cutemaze','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cuyo','14','29602.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cyphesis','40','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/d1x-rebirth','123','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/d2x-rebirth','135','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/darkplaces','24','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ddnet','84','28315.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dealers-choice','19','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/defendguin','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/desmume','86','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/devilutionx','142','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dfarc','18','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dgen','24','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dhewm3','44','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dice','10','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/digger','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dmagnetic','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dolphin-emu','383','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/domination','12','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/domino-chain','431','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/doom_shareware_data','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/doomretro','16','31172.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/doomsday','172','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/doomseeker','75','28705.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dopewars','16','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dreamchess','7','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dreamchess-music','33','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ds-models','54','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/duckstation','535','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dunelegacy','54','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dungeon','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dustrac','63','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dwarffortress','14','29101.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/easyrpg-player','67','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ecwolf','18','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/edgar','96','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/eduke32','188','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/eduke32_hires_pack','338','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/eduke32_shareware_data','7','47771.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/egoboo','142','29293.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/einstein','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/empire','10','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/emulationstation-de','92','31408','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/endgame-singularity','4','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/endless-sky','231','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/enigma','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/enigma-marbles','99','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/epsxe','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/etlegacy','71','31738.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/etlegacy-data','101','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/eureka','24','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/extremetuxracer','28','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/exult','108','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ezquake','25','28681','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/f1spirit','14','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fallout1-ce','12','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fallout2-ce','14','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fbneo-libretro','25733','27932.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fceux','70','28681','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ffgo','2','28044.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fgo','2','29954.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fheroes2','68','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fifengine','212','29735.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fillets-ng','13','28811.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fillets-ng-data','69','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/firestorm','79','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fizmo','8','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/flare','26','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/flare-game','68','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/foobillardplus','21','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fortune-ASR','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fortune-dharma','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fortune-dune','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fortune-farscape','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fortune-futurama','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fortune-game_of_thrones','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fortune-noobfarm','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fortune-slackware','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fortune-starwars','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fortune_doctor_who','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fortune_firefly','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/freeblocks','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/freeciv','144','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/freedink','50','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/freedoom','24','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/freedroid','7','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/freedroidrpg','115','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/freeminer','155','31738.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/freeorion','501','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/frogatto','148','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/frotz','5','31408','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/frozen-bubble','14','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fs-uae','59','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fs-uae-arcade','23','29457.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fs-uae-launcher','25','27947.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fs2open','106','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fuse-emulator','16','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/galaxyv2','19','29448','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/game_1945','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/garden','7','28428.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gargoyle','54','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gbrainy','17','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gemrb','45','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ggmud','18','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gl-117','8','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/glbsp','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/glestae','102','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/glob2','237','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gltron','9','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/glulxe','1','29527.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/glxosd','14','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gnome-chess','9','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gnome-klotski','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gnome-mahjongg','13','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gnome-mines','9','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gnonograms','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gnubg','34','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gnugo','11','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gnurobbo','4','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gnuski','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/golly','24','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/goonies','10','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gottet','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/greed','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gtetrinet','12','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gtkballs','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gtklife','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gzdoom','124','28084.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gzdoom-legacy','53','27834.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/hack-of-life','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/hatari','65','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/hatari_tos_roms','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/haxima-nazghul','11','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/hedgewars','136','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/heretic_shareware_data','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/heroes','18','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/heroic-bin','102','28727.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/hex-a-hop','9','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/hexalate','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/hexen_demo_data','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/hexglass','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/higan','45','30582.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/hlsdk','9','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/hnefatafl','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ho1','8','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/homm3_shareware_data','51','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/iagno','23','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/icebreaker','2','30582.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ifm','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/img2xterm','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/innoextract','18','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/instead','6','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/invasion3d','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ioquake3','27','27804','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/iortcw','71','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jag','17','31172.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/javacpc','18','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jezzball-kazzmir','3','28428.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jfsw','13','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jfsw_demo_data','8','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jfsw_hires_pack','34','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jfsw_registered_data','77','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jfsw_twin_dragon','19','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jfsw_wanton_destruction','17','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jg-bsnes','9','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jg-cega','9','28700.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jg-gambatte','4','28700.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jg-genplus','10','28700.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jg-geolith','7','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jg-jollycv','2','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jg-mednafen','102','28700.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jg-melonds','18','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jg-mgba','6','28700.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jg-nestopia','10','29301','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jg-picodrive','45','28700.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jg-prosystem','2','28700.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jg-sameboy','2','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jg-vecx','2','28700.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jin','7','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jollygood','4','28700.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/joy2key','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/julius-game','22','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jzintv','23','31056.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/kardsgt','84','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/kildclient','9','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/kmuddy','69','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/koules','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/larn','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/lbreakout2','12','28763.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/lbreakouthd','13','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/lgogdownloader','22','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/lightyears','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/lincity-ng','55','29696.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/liquidwar6','46','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/lmarbles','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/lmpc','33','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/loemu','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ltris','9','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/luanti','140','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/lutris','8','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/m64py','3','28428.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/madbomber','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/maelstrom','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mame','2323','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mame2000-libretro','29','27932.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/manaplus','98','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mangband','11','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/marathon-data','17','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/marathon-eternal-data','200','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/marathon-evil-data','31','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/marathon-infinity-data','30','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/marathon-phoenix-data','145','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/marathon-red-data','23','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/marathon-rubicon-data','86','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/marathon2-data','28','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mari0','2','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mars','46','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/meandmyshadow','23','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mednafen','162','27477.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mednaffe','5','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/megaglest','507','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/megamario','25','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/melonds','55','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mesen-x','99','29212.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mfe','2','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mgba','59','31408','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/micropolis','27','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/minecraft','2','28727.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/minigalaxy','3','29822.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mininim','49','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/miscom','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mog','19','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/moon-buggy','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/moria','3','30260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mrboom','9','28681','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mrrescue','5','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mupen64plus','68','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mupen64plus-extraplugins','31','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mupen64plus-input-raphnetraw','2','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mupen64plus-nightly','68','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mupen64plus-rsp-parallel','8','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mupen64plus-video-angrylionplus','9','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mupen64plus-video-gliden64','32','27804','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mupen64plus-video-parallel','10','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/myman','121','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nFlood','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nSnake','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/naev','111','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nblood','84','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nbsdgames','2','28754.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ncursesoflife','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nestopia','23','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/neverball','84','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nevernoid','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nexuiz','215','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ninvaders','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/njam','6','28763.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nlarn','3','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/noteye','26','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/notpacman','2','28811.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nottetris2','2','28811.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/npush','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nsudoku','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nsuds','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nuvie','33','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nxengine-evo','97','28609','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/o2em','2','27834.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/oblige','11','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/oblige-legacy4','8','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/odamex','160','27932.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/onscripter','4','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/oolite','77','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/open-adventure','4','31056.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/open-invaders','8','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openarena','104','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openclonk','187','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/opendune','13','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openjazz','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openlierox','206','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openmortal','69','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openmsx','209','27947.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openmsx-catapult','43','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openmsx-debugger','19','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openmw','354','28692.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openrct2','234','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/opensonic','7','28428.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openspades','41','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/opensurge','20','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openttd','246','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/opentyrian','11','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openyahtzee','7','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ostrichriders','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ovcc','6','29436.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pacman','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pacman-arena','13','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pangzero','3','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/paranoia','1','29690.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pasang-emas','11','31056.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pathways-into-darkness','19','28585.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pcgen','99','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pcsx-rearmed','18','28700.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pcsx2','506','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pcsxr','26','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/peg-e','5','31056.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/penguin-command','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pengupop','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pentagram','45','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pgn-extract','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pinball','92','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pingus','151','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pioneer','322','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pipewalker','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/planetblupi','49','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/playonlinux','5','30607.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pokerth','205','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pom1','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/powerslave_demo_data','10','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ppsspp','250','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/prboom','13','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/prboom-plus','14','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/principia','49','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/protontricks','2','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/protonup','2','29809.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/puNES','52','29809.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/puzzles','32','30545.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pybik','133','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pychess','12','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pysolfc','31','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pysolfc-extra-cardsets','25','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/qmc2','196','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/qtads','27','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/quackle','60','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/quake3_shareware_data','20','27804','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/quake_shareware_data','7','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/quakeforge','107','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/quakespasm','7','26426.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/quarry','10','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/qxw','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/qzdl','13','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/qzdoom','68','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/redeclipse','399','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/rejoystick','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/residualvm','34','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/returntotheroots','26','27969','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/retux','17','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/rezerwar','3','29602.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/rigel-engine','53','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/rlvm','37','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/roadfighter','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/robotfindskitten','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/roccat-tools','35','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/rocksndiamonds','18','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/rogue','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/roll','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/rolldice','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/rott','42','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/rottexpr','66','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/rpcs3','989','28832.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/sameboy','7','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/sauerbraten','336','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/scid','11','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/scid_vs_pc','40','29822.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/scrabbleai','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/scummvm','624','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/sdlpop','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/shattered-pixel-dungeon','63','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/simplesok','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/simsu','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/singularity','46','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/skulltag','13','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/sl','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/smashbattle','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/smc','92','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/sms_sdl','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/snes9x','78','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/snes9x2010','36','27932.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/sokowiz','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/solarus','49','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/solarus-quest-editor','61','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/solarwolf','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/space-invaders','3','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/speed-dreams','276','28692.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/spring','310','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/srb2','92','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/starfighter','23','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/starlanes','1','27804','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/steam','2','29457.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/steamtinkerlaunch','3','27834.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/stella','116','29690.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/stepmania','141','31738.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/stockfish','77','28315.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/stormbaancoureur','4','27834.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/stratagus','35','29457.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/stuntrally','671','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/sumeria','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/sunshine','64660','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/supermariowar','31','29167.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/supertux','198','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/supertuxkart','337','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/surgescript','4','31972.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/syobonaction','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/taipan','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/talkfilters','10','28754.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tanglet','17','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tbftss','32','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/teeworlds','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tetrinet','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tetrinetx','2','26869','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tint','2','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tintin','7','31291.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tkgames','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tome','251','29457.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tome-ah','51','31056.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tome-sx','12','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tong','18','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/torcs','265','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/trackballs','18','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/trigger-rally','68','27129','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/triplane','4','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/triplea','89','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/trs80gp','4','28705.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tt','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tty-solitaire','1','28794.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tuxfootball','26','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tuxkart','9','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tworld','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/typespeed','7','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ufoai','398','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/uligo','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ultimatestunts','78','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/unigine-superposition-benchmark','598','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/unknown-horizons','100','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/unnethack','13','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/uqm','13','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/uqm-megamod','209','28945.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/uqm_3domusic','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/uqm_remixes','100','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/uqm_voice','50','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ut2004','16','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ut99','196','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vbam','35','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vcmi','316','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vectoroids','3','27804','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vegastrike','392','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vera','8','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vice-libretro','63','28700.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vitaquake2-libretro','6','27932.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vitetris','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vkBasalt','11','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vkQuake','20','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vms-empire','1','28794.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/voxelands','65','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/wadext','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/wargus','28','29457.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/warsow','167','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/warzone2100','967','29822.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/waterCloset','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/wesnoth','487','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/widelands','637','27918.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/wolf4sdl','9','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/wolfmame','2688','28727.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/wordwarvi','6','28452.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/worldofpadman','297','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/wyrmsun','252','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/x16emu','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xarchon','8','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xbill','4','29690.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xconq','5','28052.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xcowsay','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xcpc','25','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xemu','121','27804','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xlennart','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xmahjongg','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xmoto','75','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xonotic','334','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xpilot-ng','19','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xroar','11','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xroar-roms','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xskat','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xspacewarp','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xtetris','2','29690.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xu4','40','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xu4-recreated','7','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xye','8','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/yabause','29','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/yae','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/yahtzee','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/yamagi-quake2','17','29301','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/yar','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/yarntown','9','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/yetris','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zapm','1','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zaz','16','29301','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zbom','85','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zdbsp','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zdoom','40','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zelda-roth-se','7','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zelda-xd2-mercuris-chess','17','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zennode','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/znake','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zod-engine','35','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zoom','13','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zork','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zork-mit','21','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zsdx','12','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zseal','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zsxd','6','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ztools','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ztrack','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/Fiona','72','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/GMT','118','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/Geohash','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/OTB','450','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/OTB-legacy','675','28692.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/OWSLib','7','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/SFCGAL','301','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/SFCGAL-legacy','101','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/TauDEM','159','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/cligj','1','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/eccodes','130','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/foxtrotgps','7','31056.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/gdal','327','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/geographiclib-python','1','28044.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/geojson','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/geopandas','8','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/geopy','3','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/geos','64','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/gj2ascii','2','28692.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/gmapcatcher','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/google-earth','81','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/google-earth-legacy','83','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/gpsbabel','18','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/gpsd','77','31056.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/gpxsee','68','29696.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/grass','195','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/josm','4','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/laszip','5','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/libLAS','29','31408','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/libecwj2','38','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/libgeotiff','6','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/librasterlite','16','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/librttopo','17','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/libspatialindex','18','28428.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/libspatialite','50','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/mapnik','1257','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/merkaartor','69','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/navit','30','31056.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/opencpn','137','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/opencpn-plugin-oesenc','15','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/opengrads','22','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/openorienteering-mapper','374','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/osgEarth','366','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/osm-gps-map','21','31056.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/osm2pgrouting','8','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/osm2pgsql','41','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/osm2pgsql-legacy','39','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/ossim','189','31408','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/pdal','173','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/pgrouting','50','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/pgsql-ogr-fdw','3','29436.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/pktools','40','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/pointcloud','6','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/polyline','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/postgis','72','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/proj','56','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/proj-data','194','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/python-mapnik','282','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/python-pygeos','13','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/python3-basemap','101','31408','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/python3-cartopy','21','29517.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/python3-cftime','20','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/python3-netcdf4-python','40','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/python3-pyproj','36','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/python3-pyshp','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/python3-shapely','16','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/python3-xarray','8','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/pytrainer','3','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/qgis','2006','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/qmapshack','246','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/rasterio','88','28692.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/readosm','5','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/routino','6','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/saga-gis','159','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/shapelib','7','28754.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/spatialite-tools','17','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/survex','26','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/tangogps','7','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/therion','55','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/viking','20','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/whitebox-tools','121','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/xplanet','13','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/ART','161','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/BOSL2','2','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/Blender','1881','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/CairoSVG','2','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/FSPViewer','3','26426.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/FotoKilof','2','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/FreeCAD','1874','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/GPS','8','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/GraphicsMagick','42','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/HotShots','18','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/LDView','27','28794.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/LDraw_data','153','28794.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/MaterialX','208','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/OpenCASCADE','730','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/OpenShadingLanguage','102','29167.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/OpenSubdiv','46','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/Pixie','93','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/PureRef','10','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/PyMOL','41','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/Shellpic','2','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/SweetHome3D','58','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/VariCAD','84','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/XnViewMP','73','29293.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/aaphoto','5','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/abcm2ps','3','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/advancecomp','5','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/aewan','4','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/alembic-framework','51','29954.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/amide','86','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/apitrace','70','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/asciiquarium','1','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/autotrace','17','26584.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/azpainter','8','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/baires','4','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/barcode','23','27834.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/bdfedit','1','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/birdfont','69','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/blender','260','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/blockbench','97','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/brlcad','674','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/cam','7','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/catimg','3','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/chafa','15','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/converseen','11','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/ctpv','1','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/darktable','145','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/dblatex','11','47949.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/dcraw','5','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/deskew','2','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/dia','76','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/djview4','13','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/drawio','100','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/dxftoqet','42','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/embree','294','27864','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/enblend-enfuse','99','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/entangle','18','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/eog','31','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/eog-plugins','8','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/evolus-pencil','3','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/exact-image','27','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/exif','5','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/exiftags','2','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/farbfeld','1','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fbgrab','1','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fbida','3','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fbpic','2','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fbv','2','28428.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/feh','2','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fig2sxd','4','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fim','99','28452.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/findimagedupes','2','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/flam3','7','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/flameshot','341','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/flpsed','5','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/focus-stack','9','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fontforge','42','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fotowall','29','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/frescobaldi','9','29194.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fritzing','123','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fstl','7','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fyre','6','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/g3dviewer','6','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gallery','3','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gbdfed','5','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gcolor2','9','29735.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gcolor3','4','27804','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gdk-pixbuf-psd','11','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gifsicle','8','27129','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gimagereader','18','28044.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gimp-feca_hdr-plugin','1','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gimp-lqr-plugin','7','29602.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gimp-plugin-bimp','5','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gimp-plugin-export-layers','4','28811.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gimp-wideangle-plugin','2','29293.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gliv','18','31056.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/glmark2','21','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gmic','553','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gnofract4d','14','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gnome-themes-extras','129','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/goocanvas','9','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/goocanvas2','10','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gource','16','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gpaint','5','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gpick','24','29212.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gpicview','6','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gpscorrelate','2','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gqview','9','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/grafx2','13','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/graph-easy','8','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/graphviz','98','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gscan2pdf','3','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gthumb','43','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gtkam','8','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/heif-gimp-plugin','8','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/hp2xx','4','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/hugin','140','28811.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/icc_profiles','2','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/icc_profiles_from_eci','11','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/img2pdf','2','27129','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/imgcurses','2','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/imgmin','5','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/imgp','1','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/implot','40','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/imv','4','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/inkscape','563','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/inkscape-gtk2','295','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/ipe','17','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/jUploadr','3','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/jhead','1','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/jp2-pixbuf-loader','3','28084.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/jp2a','5','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/jpeg2ps','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/jpegoptim','4','29301','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/kColorPicker','4','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/kImageAnnotator','27','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/kgeotag','11','31408','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/kim4','1','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/klayout','2039','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/kphotoalbum','69','27932.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/ksnip','31','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/kuickshow','11','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/lcdtest','2','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/lddot','1','28084.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/ldglite','2','28794.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/lensfun03','4','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/leocad','29','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/leocad_pieces','34','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/libfpx','10','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/libplacebo','8','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/librecad','187','27947.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/libresprite','119','27129','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/lilypond','122','29194.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/luminance-hdr','47','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/luxcorerender','79','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/lximage-qt','17','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/maim','8','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/makehuman','6','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mangohud','15','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mapcrafter','21','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mcomix','4','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/meh','1','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/meshlab','347','27947.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/metapixel','2','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mftrace','2','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mirage','2','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mitsuba2','269','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mozjpeg','8','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mtpaint','6','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mypaint','28','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mypaint-brushes2','4','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/nomacs','30','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/nsxiv','2','29212.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/nvidia-cg-toolkit','29','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/nvidia-texture-tools','14','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/openclipart','358','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/opencollada','180','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/opencolorio','237','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/openimageio','170','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/openscad','183','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/openscad-gearbox','1','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/openscad-mcad','1','28945.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/openscad-threads','1','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/opentoonz','397','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/optipng','2','28609','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/oyranos','18','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/paraview','3138','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pcx-pixbuf-loader','3','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pd-gears','1','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pdf2png','1','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pdf2svg','3','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pdfsyntax','2','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/peek','6','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pencil2d','35','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pencil3','117','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/phatch','6','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pho','2','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/photoqt','29','28832.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/phototonic','21','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/plotutils','33','27722.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/png2ico','2','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pngcheck','7','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pngcrush','10','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pngquant','3','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/posterazor','3','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/potrace','7','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/povray','106','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pqiv','5','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pr0ntools','2','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/psftools','7','29345.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pyformex','10','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pygraphviz','4','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/python3-gds3xtrude','1','32580.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/qcad','620','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/qcomicbook','18','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/qcustomplot','26','29293.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/qelectrotech','199','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/qimgv','39','29602.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/qiv','2','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/ralcgm','4','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/raw-thumbnailer','4','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/rawtherapee','131','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/rawtherapee-haldclut','106','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/realesrgan-ncnn-vulkan','72','28305.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/renderdoc','284','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/resynthesizer','8','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/ristretto','6','28093.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/scantailor-advanced','99','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/scour','2','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/screencloud','19','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/screengrab','18','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/scrot','3','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/shalbum','1','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/shotwell','46','27918.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/shutter','5','28811.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/simple-scan','14','28945.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/sng','3','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/stltools','1','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/svp','2','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/swappy','4','28705.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/sxiv','2','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/synfig','174','28604.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/synfigstudio','175','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/tclblt','45','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/teighafileconverter','31','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/tesseract','752','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/textext','1','28711','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/tgif-QPL','8','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/tikzit','14','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/tintii','30','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/tkpng','3','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/triangulizor','1','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/trimage','1','28609','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/ttfautohint','16','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/ttyload','2','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/tuxpaint','72','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/tuxpaint-stamps','45','28754.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/ueberzug','2','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/ueberzugpp','15','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/ufraw','21','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/uniconvertor','5','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/unpaper','3','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/urbanlightscape','30','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/veles','69','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/viewnior','4','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/vimiv-qt','3','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/vips','36','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/vuescan','35','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/wavelet-denoise','2','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/weasis-bin','63','31994.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/webp-pixbuf-loader','5','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/white_dune','119','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/whyteboard','2','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/wine-nine-standalone','4','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xbmbrowser','1','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xcalib','1','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xcf-pixbuf-loader','12','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xcur2png','4','29436.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xdaliclock','4','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xdot','1','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xfishtank','3','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/ximaging','3','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xli','3','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xloadimage','2','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xmedcon','14','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xnap','1','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xpe','51','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xviewer','39','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xzgv','2','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/yacreader','169','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/yafaray','30','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/yafaray-blender','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/yagf','13','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/yed','15','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/yvonne','3','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/zbar','53','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/zgrviewer','3','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/zgv','2','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/7plus','1','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/CubicSDR','30','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/DroidStar','12','28084.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/QtSoundModem','8','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/QtTermTCP','8','27932.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/SDRPlay','1','30045.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/SoapyFobosSDR','4','28565.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/SoapyRTLSDR','3','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/SoapyRemote','6','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/SoapySDR','31','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/SoapySDRPlay','4','31738.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/Xastir','25','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/aldo','5','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/antscope2','39','28767.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/aprx','4','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/aptdec','2','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/ax25-apps','6','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/ax25-tools','10','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/baudline','2','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/chirp','5','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/cm256cc','2','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/codec2','7','29272.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/cqrlog','27','27918.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/csdr','3','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/cwdaemon','3','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/cwstation','4','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/dab_lib','5','28315.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/demorse','2','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/direwolf','8','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/dsdcc','4','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/dxcc','2','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/ebook2cw','2','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/ebook2cwgui','5','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/fbb','13','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/fldigi','46','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/flmsg','15','28305.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/flrig','30','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/freedv','58','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/ggmorse','4','28052.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/glrpt','4','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/gqrx-sdr','29','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/gr-gsm','68','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/gridloc','1','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/gridtracker','26','31056.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/gridtracker2','99','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/grig','5','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/gsmlib','15','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/hackrf','8','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/hamclock','9','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/hamlib','22','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/hamlib4','61','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/inmarsatc','6','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/inspectrum','7','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/js8call','97','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/jtdx','66','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/jtdxhamlib','77','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/klog','107','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/libax25','4','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/libfobos','3','28689.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/libfobos-sdr-agile','3','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/libosmocore','73','29968.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/libsigmf','13','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/linpsk','11','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/liquid-dsp','14','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/lpcnetfreedv','21','29272.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/lpsk31','1','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/mbelib','2','26426.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/multimon-ng','3','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/nec2c','3','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/psk31lx','4','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/qlog','117','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/qrq','2','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/qsstv','56','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/qt-dab','41','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/redsea','7','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/rtl-sdr','4','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/rtl_433','12','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/satdump','237','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/sdrangel','737','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/sdrplay_apiService','1','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/sdrpp','52','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/serialDV','5','28718.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/sgp4','5','27864','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/soundmodem','10','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/spectools','4','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/svxlink','39','31738.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/svxlink-sounds-en_US-heather-16k','10','28763.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/tlf','7','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/tqsl','67','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/tucnak','21','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/twhamqth','3','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/unixcw','8','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/wsjtz','105','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/wxapt','4','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/xanalyser','4','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/xdemorse','8','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/xdx','6','27129','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/xfhell','4','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/xgridloc','4','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/xlog','11','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/xnec2c','14','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/xpsk31','4','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/xwxapt','5','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/yagiuda','6','27864','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/cabal-install','11','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/ghc','1858','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/ghcup','6','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-Crypto','37','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-Diff','6','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-FindBin','4','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-GLURaw','7','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-GLUT','22','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-Glob','8','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-HTTP','19','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-HUnit','6','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-HsYAML','38','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-JuicyPixels','197','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-ObjectName','3','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-OneTuple','6','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-OpenGL','92','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-OpenGLRaw','233','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-QuickCheck','35','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-SHA','34','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-ShellCheck','334','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-StateVar','4','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-Tensor','9','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-X11','47','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-X11-xft','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-aeson','120','28114.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-aeson-compat','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-aeson-pretty','9','26397.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-annotated-wl-pprint','4','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-ansi-terminal','8','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-ansi-wl-pprint','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-appar','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-asn1-encoding','8','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-asn1-parse','4','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-asn1-types','7','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-assoc','4','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-async','6','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-attoparsec','49','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-attoparsec-iso8601','7','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-auto-update','4','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-base-compat','19','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-base-compat-batteries','19','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-base-orphans','8','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-base-prelude','7','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-base16-bytestring','3','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-base64-bytestring','6','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-basement','65','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-bifunctors','22','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-bindings-uname','4','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-bitarray','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-blaze-builder','8','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-blaze-html','28','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-blaze-markup','15','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-bmp','10','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-byteable','4','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-byteorder','3','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-bytestring-builder','3','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cabal-doctest','6','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-call-stack','4','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-case-insensitive','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cereal','21','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cgi','8','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cipher-aes','8','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cipher-des','7','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cipher-rc4','4','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-citeproc','122','28767.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-clock','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cmark','25','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cmark-gfm','31','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cmdargs','27','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-colour','12','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-commonmark','76','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-commonmark-extensions','37','28767.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-commonmark-pandoc','7','26818.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-comonad','9','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-composition','3','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-conduit','25','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-conduit-extra','13','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-connection','6','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-constraints','11','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-contravariant','6','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cookie','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-crypto-cipher-types','6','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-crypto-numbers','7','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-crypto-pubkey','8','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-crypto-pubkey-types','15','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-crypto-random','6','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cryptohash','12','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cryptohash-sha256','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cryptonite','114','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cryptonite-conduit','6','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-curl','15','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-data-default','3','29448','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-data-default-class','4','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-data-default-instances-base','3','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-data-default-instances-containers','3','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-data-default-instances-dlist','3','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-data-default-instances-old-locale','4','29448','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-data-fix','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-data-ordlist','4','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-dataenc','11','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-deepseq-generics','3','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-digest','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-distributive','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-dlist','4','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-doclayout','11','28767.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-doctemplates','22','26818.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-double-conversion','9','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-easy-file','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-echo','4','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-ed25519','7','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-edit-distance','8','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-either','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-emojis','30','28767.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-enclosed-exceptions','4','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-env-locale','4','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-erf','4','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-errors','6','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-extensible-exceptions','3','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-extra','10','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-fail','3','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-fast-logger','6','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-fgl','20','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-file-embed','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-filelock','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-filemanip','6','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-fixed','4','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-foundation','63','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-free','32','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-fsnotify','6','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-generic-deriving','41','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-ghc-paths','4','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-gitrev','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-gloss','15','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-gloss-rendering','11','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-graphviz','155','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-haddock-library','21','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-half','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hashable','8','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-highlighting-kate','132','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hinotify','6','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hostname','3','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hourglass','18','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hpack','57','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hslua','4','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hslua-aeson','5','26818.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hslua-classes','6','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hslua-core','10','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hslua-marshalling','7','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hslua-module-path','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hslua-module-system','6','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hslua-module-text','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hslua-module-version','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hslua-objectorientation','7','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hslua-packaging','9','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hsp','25','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hspec','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hspec-core','28','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hspec-discover','6','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hspec-expectations','6','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hspec-smallcheck','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-html','7','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-http-client','23','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-http-client-tls','7','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-http-conduit','8','26763.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-http-types','10','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hxt','100','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hxt-charproperties','19','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hxt-regex-xmlschema','12','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hxt-unicode','7','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-idna','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-indexed-traversable','10','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-indexed-traversable-instances','5','28767.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-infer-license','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-integer-logarithms','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-iproute','14','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-ipynb','14','26818.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-iwlib','4','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-jira-wiki-markup','25','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-json','12','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-lcs','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-lexer','35','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-libyaml','18','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-lifted-async','7','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-lifted-base','6','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-logict','6','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-lpeg','8','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-lua','27','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-lukko','6','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-memory','15','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-microlens','6','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-microlens-mtl','6','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-microlens-th','10','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-mime-types','13','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-mintty','3','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-mmap','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-mmorph','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-monad-control','4','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-monad-logger','10','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-monad-loops','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-monads-tf','8','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-mono-traversable','44','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-multipart','6','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-nats','2','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-network','22','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-network-bsd','6','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-network-uri','11','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-old-locale','4','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-old-time','9','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-open-browser','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-optparse-applicative','14','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-pandoc-lua-marshal','22','26818.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-pandoc-types','58','27201.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-parsec-numbers','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-parser-combinators','6','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-path','14','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-path-io','9','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-path-pieces','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-pem','4','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-pipes','11','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-polyparse','23','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-pretty-show','12','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-primitive','20','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-profunctors','17','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-project-template','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-publicsuffixlist','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-punycode','6','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-quickcheck-io','8','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-random','10','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-regex-applicative','7','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-regex-applicative-text','4','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-regex-base','7','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-regex-compat','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-regex-compat-tdfa','4','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-regex-pcre','8','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-regex-pcre-builtin','31','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-regex-posix','8','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-regex-tdfa','50','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-resolv','30','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-resource-pool','8','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-resourcet','7','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-retry','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-rio','24','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-safe','6','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-sandi','11','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-scientific','12','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-securemem','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-semialign','12','28767.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-semigroupoids','22','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-semigroups','3','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-setenv','9','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-setlocale','4','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-shelly','24','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-silently','4','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-skylighting','31','27830.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-skylighting-core','70','26818.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-smallcheck','13','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-socks','12','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-split','4','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-splitmix','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-stm-chans','6','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-store-core','6','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-streaming-commons','14','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-strict','12','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-stringprep','11','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-stringsearch','13','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-syb','8','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-system-fileio','7','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-system-filepath','10','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-tagged','6','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-tagsoup','27','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-tar','13','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-temporary','4','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-test-framework','11','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-test-framework-hunit','5','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-test-framework-quickcheck2','5','28945.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-texmath','153','26818.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-text-conversions','4','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-text-icu','30','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-text-metrics','13','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-text-short','8','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-tf-random','7','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-th-abstraction','12','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-th-compat','7','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-th-expand-syns','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-th-lift','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-th-lift-instances','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-th-orphans','14','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-th-reify-many','4','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-th-utilities','13','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-these','8','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-time-compat','10','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-time-locale-compat','4','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-tls','66','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-transformers-base','4','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-transformers-compat','6','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-type-equality','3','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-typed-process','6','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-unicode-collation','37','28767.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-unicode-transforms','49','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-uniplate','17','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-unix-compat','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-unix-time','7','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-unliftio','19','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-unliftio-core','4','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-unordered-containers','28','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-utf8-string','7','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-uuid-types','7','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-vector','121','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-vector-algorithms','31','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-void','4','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-witherable','12','28767.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-wl-pprint-text','6','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-x509','16','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-x509-store','6','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-x509-system','5','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-x509-validation','6','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-xml','10','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-xml-conduit','30','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-xml-types','12','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-yaml','17','26818.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-zeromq4-haskell','14','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-zip-archive','11','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-zlib','11','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/hscolour','16','27364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/xmonad-contrib','232','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/2geom','29','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Atlas-C++','12','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/BeautifulSoup','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Botan','73','31985.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Botan3','202','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/CCfits','13','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/CEGUI','38','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/CEGUI0.7','77','31408','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/CGAL','34','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/CGAL5','68','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/CLI11','12','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/CoinMP','162','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/CondConfigParser','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ConfigParser','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Crypt-SSLeay','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/DevIL','6','29513.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ETL','3','30280.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/FormEncode','1','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/FreeImage','34','31408','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/GLee','3','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/GeographicLib','21','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Geraldo','1','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/GitPython','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/GtkD','211','35130.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/HawkNL','1','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Hypodermic','18','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/IMAPClient','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Impacket','8','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Ipopt','34','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/JSBSim','28','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/KDSingleApplication','14','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Kivy','71','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/LuaBitOp','2','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/LucenePlusPlus','71','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/MDI_Library','10','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/MuseScore-Libraries','2512','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Mustache','17','29082.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/MyGUI','73','29194.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ORBit2','15','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/OpenBLAS','275','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/OpenCSG','3','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/OpenSceneGraph','563','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/OptiX-Headers','2','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/PDCurses','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/PDFlib-Lite','28','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/PLUMED','103','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/PrettyTable','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/PyODE','1','27834.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/PyOpenGL','18','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/PySoundCard','2','28452.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/PyXML','7','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Pykka','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/QDarkStyle','4','29194.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/QR-Code-generator','3','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/QUSB','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/QtUsb','7','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/RxCpp','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SDL2_Pango','12','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SDL2_sound','11','29194.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SDL3','45','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SDL3_ttf','7','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SDL_Pango','9','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SDL_audiolib','11','29311.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SDL_gfx','10','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SDL_kitchensink','4','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SFGUI','15','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SFML','13','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SOPE','46','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SQLObject','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SimGear','108','28692.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SlowAES','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Smarty','1','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/VTK','1742','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/VitaMTP','12','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/WebOb','1','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/activemq-cpp','69','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/adns','4','28598.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/adodb','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/afflib','26','28315.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/agar','35','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/agg','19','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ahven','9','30162.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/alglib','64','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/allegro','32','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/allegro4','26','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/alure','4','27778.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/aml','2','28698.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ant-contrib','2','27129','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/antlr4','38','29602.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/apiextractor','9','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/appstream-glib','15','28763.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/aqbanking','110','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/argtable','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/aribb24','11','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/aribb25','11','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/asio','45','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/aspnetcore-runtime-3.1','36','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/aspnetcore-runtime-5.0','36','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/aspnetcore-runtime-6.0','36','28811.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/assimp','115','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/async','1','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/aubio','8','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/audiere','13','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/audioread','3','27864','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/aunit','9','28428.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/avbin','3','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/avr-libc','73','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/awscrt','77','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ayatana-ido','6','28763.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/backports-ssl-match-hostname','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/bamf','19','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/barry','44','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/batinfo','1','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/bcdoc','1','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/bctoolbox','5','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/bitsery','1','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/bitstream','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/blas','8','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/blis','26','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/bluez-alsa','19','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/boolstuff','7','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/botocore','49','30162.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/bsddialog','3','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/buddy','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/bullet','153','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/bwidget','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/bzrtp','7','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/c++-gtk-utils','15','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/c-blosc','14','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cal3d','10','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/camlpdf','16','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/canfigger','3','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/capnproto','47','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cchardet','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ccrtp','8','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cdk','37','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cereal','68','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ceres-solver','260','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cffi','5','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cgicc','17','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cglm','3','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cheapglk','2','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/chromaprint','14','30035.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/clalsadrv','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cln','49','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cloog','13','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/clutter','29','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/clutter-gst','10','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/clutter-gtk','8','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cmark','4','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/coeurl','4','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cogl','88','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/collada-dom','83','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/compface','3','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/compiz-bcop','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/compiz-plugins-experimental','32','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/compiz-plugins-extra','20','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/compiz-plugins-main','24','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/compizconfig-python','7','29527.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/confuse','5','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/console_bridge','7','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cpp-httplib','3','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cpp-jwt','2','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cpp-netlib','11','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cpprestsdk','131','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cppzmq','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/crossguid','3','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ctemplate','18','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cuneiform','43','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/curlpp','4','29602.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/curses_ex','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cutefish-kwin-plugins','5','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cutefish-qt-plugins','8','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cyrus-sasl-xoauth2','7','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cyrus-timezones','13','28754.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/daq','26','29212.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/date','5','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/dav1d','20','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/dbcsr','115','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/dbus-cpp','26','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/dcadec','3','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/dcmtk','189','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/dietlibc','7','27932.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/digimend-kernel-drivers','4','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/dlib','45','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/dmlc-core','12','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/docopt.cpp','8','29903.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/double-conversion','3','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/dpdk','312','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/dropbox-python','5','29515.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/duktape','8','29246.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/dumb','7','28428.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/eb','16','28718.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/edelib','24','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/eduvpn-common','17','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/efl','183','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ell','7','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/enca','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/eris','10','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/evernote-python','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/exempi','33','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/exiftool','39','28811.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fast_float','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/faun','2','27804','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fcft','3','27768.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fcgi','13','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fcitx5-rime','6','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fcl','92','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ffms2','15','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/field3d','78','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fifechan','8','28428.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fishui','8','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/flatbuffers','31','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/flatzebra','8','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/flexiblas','668','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/flickcurl','12','31056.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/flint','56','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/florist','70','28428.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fltk','18','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fmodapi','3','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fmt','50','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/folks','12','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fox-toolkit','55','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fox-toolkit-devel','107','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fpconst','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/freealut','7','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/freexl','7','28681','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fstrcmp','7','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fstrm','14','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ftgl','18','31056.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ftplib','1','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/function2','12','29272.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/futures','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fuzzylite','21','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fzssh','11','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gallerize','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gamemode','5','30162.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ganv','9','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gavl','40','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gcab','8','28763.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gconfmm','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gdata','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gdl','10','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/genshi','3','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/geocode-glib','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/geocode-glib2','5','27918.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/geshi','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gevent','76','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gf2x','15','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gfbgraph','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gflags','6','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ghc_filesystem','24','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/giblib','23','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/girara','4','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gismo','124','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gitdb','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gitdb2','1','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gl2ps','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/glfw3','10','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/glktermw','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/glog','8','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gloox','67','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/glw','9','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gmtk','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gnatcoll-bindings','76','29335.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gnatcoll-core','119','29809.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gnatcoll-db','242','27910.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gnet','8','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gnome-autoar','6','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gnome-desktop','12','30162.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gnome-media','19','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gnome-sharp','10','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gnome-vfsmm','17','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/goffice','37','29696.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/goffice0.8','34','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gom','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/google-fruit','8','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/googlebenchmark','22','30231.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gperftools','30','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/granite','10','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/greenlet','5','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/grilo','16','28754.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/grilo-plugins','9','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/grpc','622','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/grx','19','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gsm','4','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gsnmp','7','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gsound','4','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gspell','13','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gss-ntlmssp','15','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gssdp','6','27918.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gst-editing-services','26','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gstreamermm','40','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtk-aurora-engine','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtk-fortran','9','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtk-layer-shell','9','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtk-session-lock','7','29293.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtk-sharp','27','29212.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtk-sharp3','9','29212.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtk3-nocsd','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtkada','833','29194.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtkdatabox','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtkglarea','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtkglext','19','28681','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtkglextmm','25','31056.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtkhtml','27','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtkimageview','11','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtkmm-utils','18','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtksourceview','11','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtksourceview4','13','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtksourceview5','18','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtkspell3','9','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gts','17','29513.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/guichan','13','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gumbo-parser','4','29301','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gupnp','8','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gupnp-av','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gupnp-dlna','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/guppy','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gwenhywfar','48','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hdf','18','29048.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hdf5','130','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hidapi','4','29436.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/highway','59','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hiredict','2','27477.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hiredis','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hoedown','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hoel','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hptt','54','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/html5lib','3','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/htmlcxx','8','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/htslib','20','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/http-parser','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hurry-filesize','1','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hyperscan','193','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hypre','32','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/iddawc','5','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ignition-math2','14','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/iir1','6','28692.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/iksemel','9','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/imageloader','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/imbe_vocoder','2','27772.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/imgui','7','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/imlib','11','27778.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/imlib2','78','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/imlib2_loaders','4','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/iniparser','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ipaddress','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/irrlicht','35','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/irrlichtmt','14','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/itcl','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/jbig2dec','14','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/jbig2enc','14','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/jbigkit','3','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/jcal','11','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/jogamp','248','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/jreen','17','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/json-parser','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/jsoncpp','8','31738.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/jsonrpc-glib','8','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/kaadbg','1','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/kaidan','24','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/kashmir','1','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/kddockwidgets','57','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/keybinder','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/kidletime6','28','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/kirigami-addons','5','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/kquickimageeditor','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/kwindowsystem6','24','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lablgtk','22','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lapack','49','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/layer-shell-qt6','11','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ldacBT','3','29436.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/leptonica','261','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/levmar','3','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lgi','2','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lib3ds','9','29194.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lib3mf','43','32580.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libADLMIDI','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libAfterImage','10','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libEMF','7','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libQuotient','27','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libROM','12','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libXCFun','52','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libaacs','8','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libaaio','11','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libaal','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libabw','15','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libadwaita','16','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libaec','3','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libagentcrypt','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libantlr3c','5','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libaosd','3','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libappimage','8','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libaribcaption','4','29301','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libart-lgpl','5','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libasr','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libass','4','25460.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libaudclient','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libavc1394','5','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libavif','12','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libayatana-appindicator','8','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libayatana-appindicator-glib','4','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libayatana-indicator','3','28763.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libb64','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libbass','2','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libbassmidi','1','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libbde','18','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libbdplus','7','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libbonobo','22','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libbonoboui','88','30582.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libbpf','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libbpg','73','29293.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libbroadvoice','9','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libbsd','19','29167.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libbtbb','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcacard','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcaes','9','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcangjie','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcapsimage','3','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcbor','4','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libccd','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcdaudio','7','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcdr','27','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcec','13','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libchamplain','10','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libchardet','4','30035.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libchewing','7','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcifpp','31','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcint','13','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcitadel','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libclaw','9','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcli','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcmis','55','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcoap','66','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcompizconfig','11','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libconfig','4','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcpuid','16','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcreg','16','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcryptui','13','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcsv','3','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libctl','7','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcuefile','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcurl-gnutls','26','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcutefish','22','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdaemon','8','38338.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdart','88','28692.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdatachannel','26','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdazzle','18','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdbh2','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdbi','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdbi-drivers','10','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdbus-c++','14','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdbusmenu-lxqt','18','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdc1394','9','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdca','10','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libde265','18','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdex','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdispatch','10','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdisplay-info','4','29272.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdivecomputer','7','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdmapsharing','7','28305.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdockapp','9','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdraco','85','31408','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdsk','7','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdv','8','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdvbcsa','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdvbpsi','7','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdvdcss','4','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdwarf','17','31994.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdxfrw','10','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdynamite','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libe-book','30','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libearth','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libeatmydata','3','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libebml','3','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libeditline','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libee','7','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libei','6','28754.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libeot','8','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libepubgen','14','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libesedb','18','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libesmtp','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libestr','4','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libetonyek','67','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libetpan','26','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libev','5','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libevhtp','6','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libevt','20','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libevtx','21','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libewf','28','28374.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libexe','16','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libexif-gtk','9','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libexttextcat','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfabric','106','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfaketime','2','26426.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfastjson','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfdk-aac','33','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libffado','216','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfido2','18','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfilezilla','25','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfilteraudio','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfishsound','7','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfixposix','13','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfm','39','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfm-extra','12','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfm-qt','30','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfm-qt6','46','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfprint','14','29293.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfreehand','19','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfsapfs','20','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfsext','17','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfsfat','17','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfshfs','18','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfsntfs','20','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfsxfs','18','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libftdi','9','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libftdi1','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfvde','22','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfwnt','10','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfwsi','13','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfxt','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libg15','5','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libg15render','6','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libg3d','36','31056.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libg7221','9','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgadu','7','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgaminggear','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgbinder','3','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgdata','14','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgdiplus','28','29212.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgdsii','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgedit-amtk','7','29457.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgedit-gtksourceview','15','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgee1','6','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgig','27','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgit2','62','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgit2-glib','19','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgksu','11','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libglademm','17','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libglibutil','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libglpng','1','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgme','7','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnome','17','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnome-games-support','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnomecanvas','10','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnomecanvasmm','23','30582.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnomecups','7','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnomekbd','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnomemm','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnomeprint','15','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnomeprintui','12','30582.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnomeui','27','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnomeuimm','26','30582.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgpr','85','29690.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgrapheme','6','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgraphqlparser','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgringotts','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgrss','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgsasl','28','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgta','3','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgusb','5','29293.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgweather','19','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgweather4','20','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgxps','7','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libhandy','26','28945.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libharu','9','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libheif','39','28876','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libhid','7','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libhoard','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libhtp','14','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libical4-opt','51','28864','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libicns','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libiec61883','4','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libiio','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libilbc','5','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libindi','127','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libindi-drivers','66','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libindi-libraries','88','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libinklevel','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libinstpatch','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libiptcdata','7','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libircclient','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libirecovery','6','28796.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libisoburn','7','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libisofs','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libixion','38','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libixp','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libjcat','8','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libjodycode','3','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libjson-rpc-cpp','7','31738.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libjuice','3','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libjwt','14','29822.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libjxl','99','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libkate','16','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libkiwix','14','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libkml','21','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libkqueue','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libks','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libkscreen6','150','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblangtag','26','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblastfm-qt5','10','29345.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblc3','4','29301','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblcf','36','28770','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libldm','15','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libliftoff','3','26426.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblightgrep','29','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblinebreak','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblnk','17','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblo','8','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libloc','31','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblogging','15','29954.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblqr','6','29602.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblrdf','9','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblscp','7','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libluksde','18','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblxi','5','28305.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblxqt','14','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblxrad','20','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblzf','6','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmaa','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmatekbd','10','28364.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmatemixer','7','27592.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmateweather','40','28477.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmatheval','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmatroska','6','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmd','6','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmediaart','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmediainfo','84','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmemcached','124','28093.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmicrohttpd','10','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmigdb','1','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmikmod','7','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmimedir','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmimic','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libminizip','7','28681','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmirage','12','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmms','4','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmobi','8','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmodbus','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmodi','22','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmodplug','7','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmodsecurity','109','29167.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmp3splt','7','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmp4v2','18','31738.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmpd','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmpdclient','4','29167.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmpeg2','9','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmrss','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmseed','10','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmsiecf','16','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmspack','5','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmspub','11','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmsym','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmusicbrainz','8','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmusicbrainz3','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmusicbrainz5','9','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmwaw','70','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmygpo-qt','13','31985.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmysofa','5','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libnbcompat','18','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libnet','7','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libnfc','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libnfs','19','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libnids','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libnodave','3','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libnova','15','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libnsbmp','2','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libnsgif','2','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libnumbertext','16','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libnxml','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liboauth','7','28754.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libofa','8','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libofx','7','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liboglappth','7','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libolecf','17','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libomemo-c','3','31985.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liboop','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libopenglrecorder','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libopenmpt','49','28452.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libopenraw','35','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libopenshot','141','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libopenshot-audio','48','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libopensmtpd','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liborcus','58','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liborigin','8','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liborocos-kdl','16','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libosinfo','19','28428.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libosmium','3','29438.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libosmo-dsp','13','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libotf','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libotr','9','27910.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libowfat','11','27932.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libp11','5','35130.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpagemaker','9','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpano13','7','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpaper','37','27834.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpar2','9','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpeas','11','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpff','24','28763.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpgf','14','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libphdi','19','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpng-legacy12','1','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpoly','13','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libportal','9','28428.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpqxx','12','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libprelude','54','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpst','60','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libqb','20','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libqcow','21','28315.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libqtpas','151','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libquicktime','26','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libqxp','12','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libqxt','35','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libr3','11','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librapi2','8','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librapidcheck','6','31291.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libraqm','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librcd','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libreadline-java','4','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libregf','17','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librelp','7','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librepfunc','4','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libreplaygain','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librevisa','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librfm5','36','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librhtv','19','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librime','47','29167.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librist','16','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librra','11','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librsync','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librtfcomp','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsbsms','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libscca','16','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libscfg','3','29419.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libscrypt','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsearpc','8','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libserialport','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libshout','6','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsidplay','8','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsidplay2','96','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsidplayfp','12','27734.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsieve','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsigrok','36','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsigrokdecode','13','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsigscan','15','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsixel','7','31738.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libskk','18','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libslack','3','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libslirp','4','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsmbios','21','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsmdev','12','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsmf','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsmi','20','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsmraw','17','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsoup3','12','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libspectrum','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libspf2','10','30636.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libspiro','15','29735.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libspng','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsrs2','11','28754.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsrtp','4','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsrtp2','8','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libstaroffice','29','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libstatgrab','13','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libstrophe','15','31033.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsuinput','3','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsvm','11','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsynce','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsysstat','6','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtar','12','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtecla','11','29448','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtelnet','13','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtermkey','2','28681','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libticables2','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libticalcs2','7','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libticonv','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtifiles2','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtins','9','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtomcrypt','16','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtommath','4','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtorrent','36','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtorrent-rasterbar','97','31738.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtorrent2','113','27947.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtpms','34','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtraceevent','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtracefs','3','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtubo0','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libu2f-host','18','29436.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libuchardet','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libucil','13','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libucl','13','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libudfread','12','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libunibreak','4','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libunicap','9','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libunicapgtk','14','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libuninameslist','19','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libunique','10','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libunique3','9','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libupnp','8','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libupnp-legacy','9','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liburcu','46','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvdpau-va-gl','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvdwxc','7','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libversion','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvhdi','20','28315.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvidcap','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvirt','100','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvirt-glib','22','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvirt-python','7','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvmdk','20','28315.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvmime','56','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvorbisidec','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvsapm','15','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvsgpt','16','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvshadow','16','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvslvm','16','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvterm','2','29272.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libwbxml','4','27864','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libwebsocketpp','4','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libwebsockets','17','27864','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libwfut','22','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libwhisker2','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libwpe','3','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libwps','32','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libwrc','22','28763.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libx86emu','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libxc','89','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libxdg-basedir','11','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libxfcegui4','10','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libxls','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libxlsxwriter','8','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libxmi','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libxml++','9','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libxml++3','17','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libxml++5','8','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libxmlb','11','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libxmlbird','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libxmp','7','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libxnvctrl','9','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libxsmm','70','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libyang','11','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libyubikey','4','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libyuv','10','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libzdb','11','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libzen','15','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libzia','14','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libzim','10','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libzmf','11','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libzookeeper','10','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libzrtpcpp','7','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lilv','4','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/live555','9','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lksctp-tools','7','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lockdev','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/log4c','10','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/log4cplus','30','28727.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/log4cpp','10','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/log4net','3','29212.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/log4shib','11','29101.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/logilab-astng','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/logilab-common','1','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lua-cjson','1','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lua-filesystem','1','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lua-lpeg','1','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lua-luv','3','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lua-md5','2','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lua-mpack','2','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lua-posix','15','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lua-readline','1','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lua-std._debug','1','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lua-std.normalize','1','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lua-zlib','2','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/luacrypto','10','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/luadbi','2','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/luaevent','2','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/luaexpat','2','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lualdap','2','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/luasec','4','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/luasocket','5','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/luaunbound','2','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lunasvg','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lunatic-python','2','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lvtk','21','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lxqt-build-tools','1','28093.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lxqt-build-tools-qt6','2','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lxqt-notificationd','10','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lxqt-qt6plugin','8','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lxqt-qtplugin','4','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lxqt-sudo','7','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/manubulon-snmp-plugins','1','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mathgl','70','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mbedtls','7','28267.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/md4c','4','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mdds','8','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/meanwhile','9','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/med','39','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/menu-cache','10','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mercator','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/microsoft-gsl','2','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mimalloc','6','31738.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mini18n','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/miniupnpc','16','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mlt-bindings','20','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/moddb','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mongo-c-driver','16','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mongo-cxx-driver','40','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mowitz','10','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mpir','24','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/msgpack-c','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/msgpack-c-c','1','28452.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/msgpack-c-cpp','4','29954.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mtxclient','88','27918.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/muParser','8','27947.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mujs','9','29177.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/munkres','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/muparserx','9','27993.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/musicbrainzngs','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mxml','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mygpoclient','1','28093.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mysql++','68','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mysqltcl','3','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/nDPI','27','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/nacl','52','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/nanodbc','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/nanosvg','3','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/netcdf','37','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/netcdf-fortran','25','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/nextaw','8','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/nghttp3','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ngtcp2','11','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/nini','2','29212.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/nlohmann_json','2','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/nlopt','9','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/nng','15','29167.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/node-xoauth2','1','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/notcurses','31','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/nspluginwrapper','3','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ntl','74','28633.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/nv-codec-headers','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/nvidia-vaapi-driver','3','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/nx-libs','469','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/oath-toolkit','68','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ocaml-batteries','49','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ocaml-camomile','14','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ocaml-findlib','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ode','33','27834.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ogre','271','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ois','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/olm','12','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/onevpl','7','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/openapv','5','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/openbabel','95','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/opencl-amd','128','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/opendht','22','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/openimagedenoise','43','27864','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/opensaml','51','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/openvdb','1118','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/openvino','1873','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/openvkl','167','26675.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/orcania','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ordereddict','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/osgQt','14','28692.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ospray','75','28601','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ospray_studio','78','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/p4api','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/p8-platform','4','31056.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pam_usb','1','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pandarallel','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pangox-compat','11','28681','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/parsec','66','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pcl','1208','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pdf4tcl','1','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pexpect','1','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pgtcl-ng','4','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/php-imagick','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/php-memcache','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/php-memcached','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/php-oci8','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/php-pdo_dblib','35','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/php-pgsql','78','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/php-redis','8','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/php-ssh2','5','30162.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/php-stomp','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/physfs','3','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pidgin-mra','1','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pies','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pies2overrides','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pigpio','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pjsip','73','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pkcs11-helper','9','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/plasma-wayland-protocols-opt','2','29297.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/plastid','80','28727','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/plib','17','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/plog','7','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/plplot','18','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/png++','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/poco','470','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/podcastparser','2','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/podofo','79','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/polkit-qt6-1','9','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/polyclipping','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/portsmf','10','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/posixsignalmanager','4','30936.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/process-cpp','16','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/progressbar','1','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/properties-cpp','3','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/protobuf-c','21','28725.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/protozero','13','29438.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/psqlodbc','9','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ptex','7','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pthsem','7','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ptypes','11','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pugixml','6','28832.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pyPEG2','3','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pyacoustid','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pycparser','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pyftpdlib','3','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pyjf3','1','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pylast','2','28044.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pylibpcap','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pymediainfo','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pyqode.core','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pysendfile','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pystring','3','29167.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pytaglib','5','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pyte','1','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python-efl','146','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python-gnupg','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python-librtmp','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python-meld3','1','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python-ntlm','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python-xlrd','2','29272.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python2-PyYAML','8','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python2-matplotlib','67','29602.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python2-reportlab','8','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python2-ruamel.yaml','2','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python3-CherryPy','2','27969','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python3-PyQtWebEngine','18','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python3-contourpy','17','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python3-dvdvideo','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python3-gbinder','7','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python3-imapclient','1','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python3-plumbum','2','27864','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python3-pymdstat','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python3-rpyc','2','27864','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python3-ruamel.yaml','2','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python3-ruamel.yaml.clib','10','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python3-zc.lockfile','3','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pytorch','2345','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qgnomeplatform','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qml-material','9','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qmltermwidget','9','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qoauth','6','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qt-assistant-compat','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qt-avif-image-plugin','14','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qt-heif-image-plugin','3','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qt-installer-script','1','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qt-jpegxl-image-plugin','4','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qt3','619','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qt4','445','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qt5-pbfimageplugin','6','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qt5-postgresql','62','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qt5-styleplugins','11','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qt5gtk2','7','29272.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qt6','13522','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qt6gtk2','11364','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qt6keychain','10','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qtermwidget','9','28093.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qtmimetypes','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/quazip-qt4','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/quesoglc','8','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qutepart','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qwt','40','31408','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qxmpp','62','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rabbitmq-c','5','27404.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/range-v3','93','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rapidjson','47','31291.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rapidyaml','11','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/raptor','8','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rarfile','2','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rasqal','12','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/raylib','20','28084.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/re2','9','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/redland','15','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/remglk','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rest','6','27918.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rest-legacy','8','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/restbed','16','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rhino','2','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rhonabwy','4','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rkcommon','20','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/robin-map','2','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rocksdb','154','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rrdtool','21','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rtmidi','4','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rtrlib','4','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rudiments','26','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rxtx','21','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/scafacos','239','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/scalapack','303','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/sdformat','31','29735.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/seatd','3','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/secp256k1','14','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/serd','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/shairplay','16','28305.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/simbody','179','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/simde','5','29448','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/simdjson','9','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/simdutf','15','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/simpleini','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/skalibs','9','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/skstream','7','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/slib','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/slv2','1','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/smmap','1','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/smmap2','1','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/smpeg2','7','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/soci','17','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/sofia-sip','36','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/soil','3','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/solid6','40','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/sord','3','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/soundtouch','14','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/spandsp','25','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/sparsehash','35','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/spawn','17','30162.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/spdlog','18','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/spfft','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/spglib','6','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/spread-sheet-widget','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/sqliteodbc','9','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/sqlpp11','17','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/squish','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/sratom','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/srt','16','29735.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/stfl','6','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/stk','11','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/stormlib','4','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/suil','7','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/superlu','10','30607.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/swfdec','14','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/t4k_common','9','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tDOM','9','29301','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ta-lib','60','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tamu_anova','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tbb','11','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tblis','63','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tcc4tcl','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tcl-inotify','3','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tcl-promise','1','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tcl-tls','8','30636.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tclap','13','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tclcurl-fa','5','28428.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tcllib','11','28681','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tclsoap','2','28681','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tclws','2','28811.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tclxml','5','28767.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/td_lib','26','27864','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/template-glib','8','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tepl','12','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/termpaint','22','30936.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tff','13','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tinygltf','29','31408','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tinyxml','2','29735.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tinyxml2','5','27129','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tkimg','63','28681','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tksvg','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tllist','1','27768.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tlsh','7','30607.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tmdb3','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tolua++','2','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tomcat-native','6','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/toml11','4','30235.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tomlplusplus','14','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/toro','1','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/totem-pl-parser','7','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tox-extension-messages','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/toxext','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/trader','12','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tre','16','30260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tree-sitter','5','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tree-sitter-yaml','3','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/trilinos','750','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tslib','8','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tsocks','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tuiwidgets','19','27969','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tvision','10','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ucl','10','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ucommon','14','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/udunits','8','28718.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/uhttpmock','7','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ulfius','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/unarr','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/unibilium','5','29527.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/unixODBC','26','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/uptime','1','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/urdfdom','9','29735.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/urdfdom_headers','1','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/uriparser','4','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/usrsctp','9','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/utfcpp','1','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/uthash','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/uuid','8','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/vali','4','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/valijson','2','28681','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/varconf','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/vcglib','9','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/vdk','12','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/vigra','30','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/virglrenderer','9','28428.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/virtualpg','6','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/vmaf','12','29246.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/volk','30','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/vsqlite++','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/vst3sdk','44','28609','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/vte2','17','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/waylandpp','52','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wcslib','55','30582.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/web.py','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/webkit2gtk','6098','29822.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/webkit2gtk4.1','4652','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/webrtc-audio-processing','8','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wfmath','7','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wimlib','9','29212.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wjelement','3','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wlroots','6','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wpebackend-fdo','5','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ws4py','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wslay','16','28754.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wv','8','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wvstreams','21','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wxGTK3','209','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wxPython','298','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wxPython3','348','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wxPython4','344','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wxWidgets','146','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wxX11','17','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wxpdfdoc','38','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wxsvg','32','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xalan-c','200','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xapian-bindings','54','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xapp','10','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xawplus','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xblas','29','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xbyak','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xcb-util-xrm','4','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xerces','4','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xerces-c','30','29048.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xevd','9','29301','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xeve','13','29301','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xforms','20','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xgboost','59','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xine-plugin','5','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xlt','13','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xml-security-c','12','29101.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xmlada','42','29457.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xmlgen','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xmlrpc-c','41','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xmlsec','18','28084.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xmltooling','38','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xmpppy','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xview','136','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xylib','19','28811.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/yajl','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/yaml-cpp','53','29167.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/yaml-cpp0.3','8','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/yaml-cpp0.6','40','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/yascreen','2','26426.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/yaz','17','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/yder','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ytnef','12','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zeromq','17','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zimg','24','28452.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zint','12','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zita-alsa-pcmi','4','29194.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zita-convolver','1','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zita-resampler','2','26399.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zlib-ng','14','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zmusic','18','30582.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zope.component','3','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zope.configuration','3','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zope.event','3','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zope.hookable','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zope.i18nmessageid','3','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zope.location','3','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zope.proxy','3','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zope.schema','3','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zope.security','3','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zsi','2','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zstd-cmake','12','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zxing-cpp','20','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zycore-c','3','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zydis','6','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zziplib','7','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/KeePass','20','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/KeePassHttp','2','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/OSCAR','47','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/QCMA','12','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/SecLists','521','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/abduco','2','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/altermime','2','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/anymeal','23','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ardesia','10','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ascii','1','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/asr-manpages','1','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/asterixInspector','11','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/autofirma','25','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/balena-etcher-bin','99','27918.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/bbrun','1','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/bchunk','2','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/bdf2psf','1','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/bdfresize','3','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/bee-host','1','27477.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/beep','2','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/binclock','1','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/biosdisk','1','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/birthday','2','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/bitwarden-desktop','98','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/bkt','1','28700.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/bodr','3','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/boncuk','10','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/boxes','5','28727.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/brewtarget','43','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/bsdsfv','1','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ccze','4','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/cflow','9','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/cheat','27','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/chemical-mime-data','5','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/chm2pdf','1','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/chntpw','2','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ciso','2','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/cksfv','1','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/clawsker','1','28763.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/cliphist','5','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/countryfetch','4','28084.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/cp437','1','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/cups-pk-helper','3','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/cwiid','7','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/dbview','2','29735.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/dcc','10','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/deb2tgz','1','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/deroff','3','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/dex','2','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/discount','4','27947.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/dmg2img','1','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/doclifter','2','27918.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/dos2unix','9','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/drg2sbg','3','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/dtmflib','6','26464.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/duhdraw','2','30162.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/dvtm','2','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/dwdiff','2','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ekphos','112','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/esniper','3','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/exploitdb','496','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fbcat','2','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fbdump','6','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fcitx-mozc','129','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fcitx5','80','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fcitx5-chewing','5','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fcitx5-chinese-addons','28','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fcitx5-configtool','24','29194.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fcitx5-gtk','7','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fcitx5-mcbopomofo','18','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fcitx5-mozc','126','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fcitx5-qt','15','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fcitx5-table-extra','22','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fcitx5-table-other','9','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fcrackzip','4','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/feedbackd','6','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/figlet','2','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/figlet-contributed','9','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/figlet-international','16','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/flashrom','5','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/flips','4','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/font-unscii','46','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/funny-manpages','1','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/g15composer','4','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/g15daemon','8','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/g15stats','6','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/g810-led','5','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/gcp','1','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ghostpcl','118','29301','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/git-store-meta','1','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/glogg','14','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/gnome-mime-data','7','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/gnome-user-docs','279','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/go-md2man','7','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/goldencheetah','329','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/gourmet','25','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/gprename','2','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/gr-iqbal','11','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/grabc','1','28452.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/gramps','32','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/grc','1','26996.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/gxmessage','4','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/heimdall','9','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/heyu','5','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/hoorex','2','28692.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/html-xml-utils','7','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/html2ps','1','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/hxtools','16','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ibus-chewing','5','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ibus-qt','10','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ibus-rime','2','27129','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ibus-skk','17','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ibus-table-chinese','20','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/iscan-firmware','1','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/iscan-proprietary-drivers','2','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/jmri','90','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/kasumi','7','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/kde_cdemu','5','27834.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/keychain','1','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/klibc','13','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/kronometer','12','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/langdrill','2','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/lastpass-cli','4','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/lbdb','2','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/libime','40','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/libydpdict','6','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/linux_logo','1','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/lppf','1','29602.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ltunify','2','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/mdic','5','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/megatunix','34','31056.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/modprobed-db','2','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/moedict-desktop.app','112','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/moodbar','8','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/moreutils','2','28705.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/mosquitto','62','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/myclippings','1','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/nfoview','1','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/no-more-secrets','1','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/nordpass','82','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/nuclei','286','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ollama','277','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/open-simh','1604','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ophcrack','13','31408','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/owfs','24','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/pam-u2f','12','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/paperkey','6','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/par','1','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/par2cmdline','258','27918.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/par2cmdline-turbo','20','28727.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/pc','2','30925.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/pinfo','19','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/pmw','11','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/po4a','58','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/pokemon-colorscripts','13','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/portecle','3','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/projectM','78','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/python3-random-username','1','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/qprint','2','29301','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/qtspell','6','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ramdefrag','4','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/rarcrack','1','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/rbutil','30','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/recode','60','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/rlwrap','10','27969','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/rmlint','7','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/rsibreak','11','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/rsnapshot','3','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/samdump2','1','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/sdcv','4','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/seahorse-plugins','19','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/sgrep','1','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/shh','1','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/sigrok-firmware-fx2lafw','3','29293.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/simh-classic','105','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/slop','9','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/smu','2','29246.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/snow','2','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/solunar','2','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/spotlighter','5','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ssss','1','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/stardict','94','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/stardict-tools','73','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/subsurface','542','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/sunwait','1','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/sutils','1','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/swege','1','27969','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/tal','2','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/tarotplane','3','29809.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/tegaki-zinnia-japanese','16','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/tidyp','9','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/tinytldr','2','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/toilet','4','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/torrentinfo','1','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/twa','1','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/txt2regex','2','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/txt2tags','2','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/uni2ascii','4','29735.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/utimer','4','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/vdpauinfo','2','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/vttest','4','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/wcd','3','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/weather','9','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/with-readline','3','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/wkhtmltopdf','16','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/wkhtmltox','47','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/wl-clipboard','3','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/wmbday','1','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xapian-omega','12','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xbanish','2','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xca','59','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xcape','2','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xcb-imdkit','4','28585.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xclip','6','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xdelta','7','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xdelta3','26','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xhidecursor','1','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xvnkb','2','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xwinwrap','2','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/yapet','45','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/yara','22','28315.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ydpdict','9','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ykpers','5','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/yubikey-manager','3','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/yubikey-manager-qt','11','27804','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/yubikey-personalization-gui','12','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/yubioath-desktop','87','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/zinnia','8','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/AtomicParsley','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/AviSynthPlus','52','29101.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/DivFix++','3','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/FFaudioConverter','5','28093.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/Gem','263','31056.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/HandBrake','529','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/JSampler','5','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/LBRY','153','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/MediathekView','78','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/Mopidy','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/MuseScore','725','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/MuseScore-Legacy','410','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/QMPlay2','55','31994.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/RenameMyTVSeries','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/SimpleSysexxer','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/UxPlay','6','28305.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/abcMIDI','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/aces-core','51','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/aces_container','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/acetoneiso','11','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/aegisub','63','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/amarok','250','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/anyremote','9','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/aom','245','29048.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/apetag','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/audio-recorder','7','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/audiopreview','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/avidemux','298','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/bdtools','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/beets','5','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/bino','14','31291.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/bombono-dvd','259','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/cantata','87','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/castget','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/castnow','2','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/ccextractor','33','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/ccxstream','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/celluloid','7','28832.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/cfourcc','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/cinelerra','610','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/clamz','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/coriander','8','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/cowbell','7','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/ctl','17','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/cuetools','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/davs2','18','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/devedeng','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/dirac','22','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/droidcam','2','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/dsmidiwifi','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/dssi','7','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/dvblast','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/dvd-slideshow','1','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/dvdbackup','9','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/dvdrip','6','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/dvdstyler','46','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/dvgrab','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/easyeffects','56','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/emoc','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/exaile','22','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/farsight2','21','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/fdkaac','9','29212.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/ffmpeg2theora','3','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/ffmpegfs','32','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/filebot','38','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/filebot-legacy','35','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/flowblade','21','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/flvtool2','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/freetube','49','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gaupol','4','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/get_iplayer','1','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gigedit','29','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gimp-gap','243','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/glyr','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gmidimonitor','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gmtp','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gnash','157','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gnome-mplayer','8','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gnome-video-effects','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gnupod','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/goom','8','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gpac','77','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gst-plugins-bad-nonfree','18','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gst-plugins-ugly','8','28687.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gst0-ffmpeg','41','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gst0-plugins-base','49','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gst0-plugins-good','41','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gst0-plugins-ugly','10','28198.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gstreamer-vaapi','17','29167.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gstreamer0','30','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gtick','8','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gtk-recordmydesktop','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/guvcview','6','27969','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gxine','12','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/haruna','13','29149.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/hexter','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/huludesktop','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/iat','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/imagination','17','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/inputstream.adaptive','51','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/intel-gmmlib','17','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/intel-media-driver','191','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/intel-media-sdk','129','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/iptvnator','97','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/ivtv-utils','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/jellyfin-bin','77','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/jellyfin-media-player','17','31738.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/jlscp','2','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/k9copy-reloaded','43','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/kaffeine','19','30607.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/kaffeine-mozilla','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/kodi','666','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/kvazaar','17','28609','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/l-smash','5','28763.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/lightspark','94','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/lives','178','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/lsdvd','3','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/makemkv','64','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mediainfo','29','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mimms','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/minidlna','10','28428.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/minitube','81','29177.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mirage2iso','3','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mjpegtools','21','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mkvtoolnix','215','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mozplugger','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mp3fm','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mp3fs','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mpgtx','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mplayer-codecs64','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mpv','16','29177.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mtn','3','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mtpfs','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/munt','17','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/musique','72','29177.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/musixmatch','80','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/nrg2iso','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/nted','20','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/obs-studio','232','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/obs-studio-legacy','195','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/oggconvert','2','29345.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/oggvideotools','6','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/olive','37','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/opencaster','11','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/openh264','11','30162.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/openshot','104','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/opera-ffmpeg-codecs','2','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/parole','14','28093.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/picard','12','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/picard-plugins','3','30162.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/pipe-viewer','2','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/pitivi','16','29602.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/playerctl','7','27866.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/plexmediaserver','76','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/pngnq-s9','3','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/pocketsphinx','22','30162.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/popcorntime','331','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/pragha','13','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/pyradio','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/qdvdauthor','122','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/qsampler','10','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/radiotray','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/rav1e','179','31056.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/ripit','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/ripperX','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/rnnoise','7','28692.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/rosa-media-player','17','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/rpi-imager','14','28093.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/rtmpdump','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/rygel','26','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/saflashplayer','7','29177.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/schroedinger','15','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/serviio','10','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/sfnt2woff','2','26426.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/shine','11','26426.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/shotcut','77','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/sickchill','39','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/sinthgunt','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/smpeg','8','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/smplayer','45','27804','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/smtube','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/sonivox','8','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/spek','21','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/spot','208','27524.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/spotify','104','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/srt-to-vtt-cl','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/stills2dv','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/strawberry','286','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/stream2chromecast','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/stremio','14','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/subtitleeditor','232','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/subtitleripper','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/svt-av1','44','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/svt-hevc','15','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/svt-vp9','14','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/tabu_audio_player','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/telepathy-farstream','7','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/telepathy-glib','33','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/telepathy-qt','57','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/tivodecode','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/totem','27','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/tovid','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/tragtor','1','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/transcode','32','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/tsduck','283','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/tstools','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/tvheadend','33','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/tvtime','12','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/uavs3d','6','26426.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/uavs3e','8','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/ucview','14','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vapoursynth','39','31056.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vapoursynth-mvtools','67','31056.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vcdimager','7','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vgmplay','4','29448','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/videocut','6','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/videomass','7','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vitunes-mplayer','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vkeybd','2','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vlc','184','28834.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vlsub','1','30607.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vo-amrwbenc','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vobcopy','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vobsub2srt','3','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vokoscreenNG','61','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vokoscreenNG-QT5','29','28585.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vvdec','26','28093.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vvenc','51','26426.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/w_scan','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/w_scan2','4','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/w_scan_cpp','22','29212.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/webvfx','10','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/whistle','1','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/winff','12','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/wxcam','20','31994.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/x264','21','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/x265','177','28534.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/xavs2','7','26426.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/xawtv','7','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/xjadeo','5','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/xmltv','9','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/xmms-crossfade','8','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/xmms-cue','5','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/xvid4conf','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/xvidcore','4','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/xvst','42','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/yarock','56','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/youtube-music','80','28585.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/ytdlp-gui','8','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/zvbi','11','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/2ping','1','28681','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/3proxy','7','31074.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/9pfuse','4','29457.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/AdGuardHome','19','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ApacheDirectoryStudio','58','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ClusterSSH','8','28763.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/DenyHosts','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/DownTube','8','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/Electrum','12','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/FireWorks','3','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/GeoIP','28','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/GhostInTheMail','7','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/GlobalProtect-openconnect','13','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/LOIC','2','29212.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/MEGAcmd','120','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/MEGAsync','142','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/NetworkManager-fortisslvpn','25','29177.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/NetworkManager-l2tp','26','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/NetworkManager-openconnect','10','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/NetworkManager-pptp','8','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/NetworkManager-ssh','17','27969','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/NetworkManager-sstp','9','27947.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/NetworkManager-vpnc','10','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/PAC','2','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/Pafy','3','29101.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/PySocks','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/Pyrit','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/Quaternion','11','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/QuiteRSS','61','26426.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/SoulseekQt','31','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/SpiderOakONE','27','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/UDR','14','27477.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/WhatWeb','18','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/XD','15','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/aMule','75','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/aardvark-dns','19','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/abdownloadmanager-bin','64','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/acme.sh','3','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/addrwatch','6','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/aerc','19','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/aggregate','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/aiccu','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/aircrack-ng','18','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/aldryn-client','2','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/amap','4','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/amavisd-new','4','28945.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/amfora-bin','3','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/amneziawg-go','7','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/amneziawg-kernel','8','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/amneziawg-tools','3','27918.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ani-cli','1','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/anydesk','27','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/aoetools','2','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/apt-mirror','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/argus','7','28093.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/argus-clients','28','26426.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/aria2','65','25460.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/armcord','99','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/arno-iptables-firewall','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/arp-scan','7','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/arping','4','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/arpwatch','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/asciinema','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/asterisk','216','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/asuka','34','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/atftp','6','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/atheme','33','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/authbind','2','28084.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/autossh','5','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/avahi','41','28305.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/awscli','58','30162.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/awscli-v2','1273','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/awstats','2','28093.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/axel','5','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/b43-firmware','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/b43-fwcutter','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/badwolf','2','30607.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/baikal','2','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/barnyard2','15','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/barrier','34','28305.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/basilisk-bin','68','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/beegfs','363','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/betterbird-bin','92','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bip','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bitchx','16','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bitcoin','200','29212.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bitcoin-knots','604','29436.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bitlbee','4','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bitlbee-discord','10','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bitlbee-steam','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bluejeans','99','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bmon','8','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bollux','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bombadillo','11','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bottle','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bozohttpd','3','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bpfmon','2','26426.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/brave-browser','110','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/broadcom-wl','6','29149.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/browsh','15','29301','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bsflite','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bsky-desktop-bin','59','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/btpd','5','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bwm-ng','7','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/c-toxcore','5','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cacti','32','26426.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cacti-spine','6','26426.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cadaver','11','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/caddy','55','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/canto','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/canto-next','2','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/castor','83','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/catgirl','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ccnet','13','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ceb-tools','3','28681','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cgterm','3','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/chawan','33','27918.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cherokee','34','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/choqok','48','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/chrony','5','30607.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cicb','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cieid','8','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cinny-desktop','7','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cisco-secure-client-vpn','78','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ckermit','6','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/clamav-unofficial-sigs','1','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/claws-mail','61','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/clipgrab','13','31994.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/clive','3','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cloudflared-bin','10','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/clusterit','8','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cmdiag','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cmst','20','31033.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cni-plugins','47','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/comgt','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/conkeror','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/connman','21','31033.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/copyparty','5','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/corkscrew','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/coturn','14','27910.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/courier-authlib','69','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/courier-imap','90','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/courier-unicode','8','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cowpatty','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cryptcat-unix','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/csync','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ctorrent','8','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/curl-quic','26','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cyrus-imapd','37','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dante','47','31738.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/darkhttpd','2','28305.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/darkstat','7','29149.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/davmail','3','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ddclient','6','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ddgr','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/deltachat','96','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/deltachat-desktop','96','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/deluge','28','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/demagnetize-rs-bin','4','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/deskflow','57','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dhcping','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dianara','28','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dillo','16','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dino','77','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dirb','4','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/discord','98','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dkimproxy','3','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dma','2','29194.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dmitry','5','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dnscrypt-proxy','6','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dnscrypt-wrapper','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dnsmap','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dnsproxy-bin','7','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dnstop','4','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dnstracer','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/docsis','6','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dog','16','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dokuwiki','12','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dooble','103','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dothost','1','28945.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dovecot-pgsql','23','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dovecot-pigeonhole','20','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/drawterm','6','27947.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/driftnet','8','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dropbear','16','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dropbox','59','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dsniff','11','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dstp','12','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dublin-traceroute','5','31738.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dukto','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/e2guardian','14','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/easyrsa','1','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/edbrowse','3','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/eduvpn','3','27864','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/efax','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/efax-gtk','13','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/eiskaltdcpp','83','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ejabberd','69','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ejabberd-bin','12','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/electrs','163','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/element-desktop','639','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/element-desktop-bin','100','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/elinks','52','29809.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/emailrelay','67','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/enet','4','27497','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/epiphany','38','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/et','185','28718.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/etherape','14','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/etherpad-lite','63','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/etherpoke','2','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ethersrv-linux','1','29194.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ettercap','10','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/exim','12','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/fail2ban','9','30582.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/fcgiwrap','5','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/fdm','4','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/fdns','4','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/feather','259','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/felinks','46','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ferdium','103','28811.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ferm','2','31291.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ffuf','14','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/filezilla','158','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/firehol','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/firewalld','19','29101.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/floorp-bin','83','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/flvstreamer','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/fping','4','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/fqterm','18','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/franz','101','28718.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/freedownloadmanager','55','28945.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/freeradius-server','104','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/freerapid','5','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/frostwire','8','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/frr','96','28718.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ftp-cloudfs','1','35130.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/fujinet-pc','30','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/fwknop','12','28315.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gajim','11','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gallery-dl','12','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ganglia','17','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ganglia-web','4','29457.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gem','3','29954.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gemget','8','29954.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gemget-bin','2','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/geoipupdate','5','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/geomyidae','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/git-lfs-transfer','20','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/glance','16','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/glewlwyd','14','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/glusterfs','176','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gmi100','2','29954.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gmid','4','29457.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gnirehtet-bin','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gnome-bluetooth','8','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gnome-remote-desktop','5','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gnome-shell-extension-gsconnect','5','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gnome-user-share','4','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gns3','12','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/go-ipfs','13','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/go-sendxmpp','49','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gobuster','16','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gofiche','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/goimapnotify','44','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/googlecl','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/googler','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gophernicus','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gotty','3','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gpodder','6','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/grafana','308','29172.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/grepcidr3','2','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/grpcurl','8','29954.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/grsync','5','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gtk-gnutella','184','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gtorrentviewer','5','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/guacamole-client','3','27129','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gui-ufw','6','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gutenberg','2','27932.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/hakuneko','86','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/hamachi','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/haproxy','22','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/haskell-network-multicast','4','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/havp','6','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/heimdal','78','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/helium-browser','112','28237','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/hiawatha','9','35130.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/holehe','5','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/host-sflow','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/hostapd','5','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/hostsblock','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/hping3','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/httpie','4','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/httping','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/httptunnel','4','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/httrack','13','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/hydra','9','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/hylafax+','53','31738.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/i2pd','57','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/icbm','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/icecat','72','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/icinga2','331','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/identicurse','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ifplugd','5','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/igmpproxy','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ii','1','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/imapfilter','2','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/inadyn','13','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/insync','85','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/iodine','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ip2location-c','10','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ipcalc','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ipcalc-ng','3','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/iperf','9','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/iperf3','8','29436.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ipfs-pack','6','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/iptv-analyzer','12','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ipv6calc','10','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ipvsadm','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ipxnet','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/irc.c','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/irc2dc','5','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ircII','9','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ircd-hybrid','11','29448','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/irssi-icb','5','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/irssi-python','12','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/irssi_otr','7','27910.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/isync','4','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/iwd','10','27804','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/jabberd2','21','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/jboss-as','37','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/jigdo','10','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/jitsi','75','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/jitsi-meet-desktop','92','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/js185','23','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/k9s','118','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kdrive','86','31033.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kea','402','28727.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/keama','67','29167.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/keepalived','15','35130.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kirc','1','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kismet','428','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kivy-garden','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/knock','6','29457.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kstart','4','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kubectl','47','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kubectl-bin','18','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kvirc','151','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kwakd','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/lagrange','18','27864','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/landrun','9','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/lchat','1','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ldapvi','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/leafnode','6','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/libdnet','8','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/libident','6','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/libiscsi','13','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/libmaxminddb','5','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/libnatpmp','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/librespeed-cli','2','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/libressl','28','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/librewolf','96','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/libteam','14','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/libtls','21','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/libvarlink','4','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/libvma','39','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/liferea','19','29822.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/lighttpd','26','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/limnoria','5','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/linksys-tftp','2','28093.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/linphone-desktop','374','28305.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/linssid','11','31408','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/lizardfs','208','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/lldpd','21','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/llhttp','2','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/lorawan-stack-cli','17','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/luakit','5','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/lurch','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/lxi-tools','11','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/macchanger','3','29735.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/maildrop','55','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mailman','20','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mailspring','95','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/malheur','6','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/masscan','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mattermost-bin','157','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mattermost-desktop','90','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mblaze','3','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mdk3','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mdk4','6','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/megatools','5','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/memcached','6','28093.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/memcached-init','1','28093.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mew','8','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/microsoft-edge','185','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/milter-greylist','9','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/miniupnpd','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mitmproxy-bin','35','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mk-livestatus','36','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mktorrent','2','28681','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mkvserver_mk2','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_auth_gssapi','13','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_auth_kerb','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_bw','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_chroot','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_dnssd','4','28305.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_evasive','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_fcgid','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_geoip2','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_ruid2','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_wsgi','6','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_wsgi-metrics','1','29696.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/modemu2k','4','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/modsecurity-apache','16','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/monkey','5','26517','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/moonlight-qt','35','29457.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mosaic-ck','15','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mosh','27','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/movgrab','4','26426.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mpop','5','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mps','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mps-youtube','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mptcpd','9','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mrtg','11','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/msmtp','4','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mstflint','124','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mullvad-browser','83','30389.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mullvadvpn-app','76','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mumble','111','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mumble-server','22','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/munge','18','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mysecureshell','5','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nagios','18','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nagiosgraph','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/naim','15','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nbd','17','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nbtscan','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nbwmon','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nchat','463','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ncp','1','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ncrack','34','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/neomutt','28','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nessus','38','27864','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/netavark','47','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/netcat','6','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/netdiscover','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/netdrive-connector','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nethogs','3','29527.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/netperf','8','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/netqmail','27','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/netsniff-ng','33','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/netstat-nat','4','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/netsurf','71','29345.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/newsboat','207','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/newsraft','4','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nextcloud-desktop','150','28681','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nextcloud-server','137','29167.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nfdump','8','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nginx','17','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ngircd','7','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ngrep','4','28711','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ngrok','11','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nheko','85','27918.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/niceshaper','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nicotine+','16','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nikto','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nload','5','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nm-tray','8','29167.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nmh','9','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/node_exporter','18','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/noip-duc','27','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/noip2','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nordvpn','39','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nordvpn-gui','14','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/noss','4','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nrpe','6','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nsca','5','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nsd','11','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nss-mdns','6','28305.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nss-tls','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ntpclient','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nullidentd','2','29602.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nweb','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nxclient','4','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nyxt','101','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nzbget','24','28811.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/obfs4proxy','7','28763.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ocserv','10','28713.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/offlineimap','3','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/oidentd','9','29602.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/oinkmaster','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/onedrive','12','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/onioncat','4','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/onionshare','3','27969','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/onkyo-eiscp','2','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/onyx','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ookla-speedtest','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/open-iscsi','7','26869','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/open-isns','3','26869','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openconnect','9','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openconnect-sso','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/opendchub','7','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/opendkim','32','29167.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/opendmarc','16','30636.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openfire','11','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openfortivpn','9','29177.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openntpd','8','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openresty','39','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/opensm','24','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/opensmtpd','23','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/opensmtpd-extras','18','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/opensmtpd-filter-dkimsign','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openvswitch-utils','29','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/opera','97','31787.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/otter','69','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/owncloud-client','73','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/owncloud-server','25','26425.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/p0f','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/packETH','8','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/packit','9','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/paho-mqtt-c','11','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/palemoon','1156','27918.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/palemoon-bin','55','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/passt','49','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pdns','108','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pdns-recursor','111','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pebrot','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/periscope','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/persepolis','4','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pexip-infinity-connect','80','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pflogsumm','1','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pfqueue','7','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pgbouncer','9','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pgpool-II','25','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/phodav','7','27918.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/phpmyadmin','19','28044.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/phppgadmin','3','26426.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/piavpn-app','75','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-authorization-blocker','5','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-birthday-reminder','6','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-extprefs','6','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-gfire','16','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-musictracker','7','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-nudge','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-opensteamworks','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-otr','7','27910.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-privacy-please','5','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-sipe','10','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-sound','5','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-toobars','5','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-toolbar-shrink','4','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-visnotes','4','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-whatsapp','6','31408','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pirate-get','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/plasma-nm-openconnect','60','28945.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/plowshare','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/plowshare-modules','1','28767.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pmacct','15','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/policyd','1','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/policyd2','1','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/polipo','3','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/postfix-pgsql','16','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/postgrey','1','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pounce','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pound','7','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pptp','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pptpd','4','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/prettyping','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/privoxy','12','29194.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/profanity','29','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/prosody','4','29177.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/prosody-mod-block-strangers','1','29177.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/prosody-mod-cloud-notify','2','31994.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/prosody-mod-filter-chatstates','2','29177.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/prosody-mod-http-upload','1','29177.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/prosody-mod-throttle-presence','1','29177.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/prosody-mod-turncredentials','1','29177.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/prosody-mod-vcard-muc','2','31994.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/protonmail-bridge','83','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/protonmail-import-export-app','62','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/proxychains','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/proxymini','1','32580.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/proxytunnel','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/psad','22','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/psi','165','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/psi-plus','175','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ptokax','6','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pubnub-curses','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pulse-sms','84','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pure-ftpd','15','27918.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/purple-facebook','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/purple-gowhatsapp','4','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/purple-plugin-pack','27','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/purple-teams','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/purple-telegram','20','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/putty','18','29194.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pyNeighborhood','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pydio-agent','8','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pydio-sync','8','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pysub-dl','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/python-onedrive','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/python3-CherryMusic','4','29101.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/python3-PyRSS2Gen','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pyzor','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/qTox','55','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/qbittorrent','81','27947.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/qodem','11','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/qtmib','13','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/quagga','58','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/quark','2','28585.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/quassel','96','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/qutebrowser','6','27947.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/qutebrowser-bin','252','28767.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/r8168','11','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/radicale','3','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/radiusplugin','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/radvd','4','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rclone','31','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rclone-src','103','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rdma-core','39','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/reaver','7','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/redir','5','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rejik','1','29272.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/remmina','12','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/remotebox','4','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/repo','1','28692.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/republicanywhere','71','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/resilio-sync','9','27864','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/retroshare','333','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rhapsody','5','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/roundcubemail','10','29194.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rspamd','80','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rss2email','2','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rssh','4','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rtl8188eu','7','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rtl8188fu','11','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rtl8812bu','23','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rtl8821ce','20','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rtorrent','23','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rubygem-showterm','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rustdesk','31','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rustdesk-opt','570','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/s','4','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sabnzbd','4','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/scapy','9','28428.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/scrcpy','7','29177.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/seadrive-fuse','9','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/seafile','14','29690.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/seafile-server','16','30582.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/seahub','36','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/secpanel','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/senpai','13','28142.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/serialmail','9','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/session','87','29954.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sfeed','3','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sflowtool','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sftp-cloudfs','2','35130.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/shadowsocks-libev','11','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/shadowsocks-rust','146','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/shibboleth-sp','68','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/shorewall','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/shorewall-core','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/shorewall6','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sic','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/siege','10','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sieve-connect','1','31291.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/signal-desktop','96','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/silc-toolkit','24','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/simplex-desktop','168','30162.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sipcalc','5','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sipp','7','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/skype4pidgin','4','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/slack','64','29301','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/slirp4netns','10','27804','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/slowhttptest','4','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/slurm','121','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/smcroute','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/smstools3','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/snac2','3','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sngrep','6','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sniffjoke','4','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/snmpsim','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/snmptt','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/snort','193','29212.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sntpc','1','29167.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/spamassassin','9','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/spambayes','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/spawn-fcgi','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/speedometer','2','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/speedtest-cli','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/spf-engine','2','29177.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sphinx','31','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/spice','34','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/spice-gtk','26','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sqlmap','7','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sqlninja','1','31291.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sqlrelay','90','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/squid','238','29272.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/squidanalyzer','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/squirrelmail','22','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ssh-chat','5','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sshamble','19','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sshblock','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sshfp','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sshguard','5','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sshpass','3','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sshuttle','3','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sslh','5','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ssllabs-scan','10','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sslscan','56','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ssmtp','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sstp-client','22','27947.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/static-web-server','90','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/strongswan','110','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/surf','2','28705.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/surfraw','5','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/surl','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/swaks','2','28811.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/swift','244','28305.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sylpheed','20','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/syncterm','47','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/syncthing','10','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/syncthingtray-bin','34','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/synochat','92','27834.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/t50','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tailscale','29','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tcpbench','2','29212.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tcpflow','36','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tcptunnel','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tdlib','417','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/teams-for-linux','82','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/teamviewer','104','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/telegram','97','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/teleport-connect','92','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tgpt','19','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tgt','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/thorium-browser','164','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/thrift','54','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/thttpd','4','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tinc','10','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tinyirc','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tinyproxy','5','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tixati','30','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tmate','9','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tnfs-fuse','3','30607.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tnfsd','2','30607.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tnftp','11','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tntnet','35','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tokodon','8','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/toot','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tor','67','27969','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tor-browser','64','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tornado','3','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/torsocks','14','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/toxcore','15','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/transgui','8','27918.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/translate','1','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/translate-shell','2','30607.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/transmission','119','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/transmission-remote-gtk','7','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/trojita','81','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/trurl','3','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ttcp','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ttdnsd','2','27969','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tucan','2','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tunctl','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/turbo-attack','7','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/turbo-scanner','7','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tut','56','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/twine','3','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/twinkle','38','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/twitter','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/u9fs','1','29457.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/uTox','26','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ubertooth','4','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ucarp','10','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ucspi-tcp','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/udns','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/udpbench','2','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/udpcast','4','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/udpxy','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/udsclient3','1','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ufw','4','32580.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/uget','7','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/uget-chrome-wrapper','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/uget-integrator','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/unbound','28','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/unicornscan','34','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/uredir','5','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/urlgrabber','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/urlscan','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/urlview','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/urlwatch','3','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/uudeview','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/uwsgi','24','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/varnish','45','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vblade','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vcspull','2','29602.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vde2','18','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vdirsyncer','2','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/verlihub','18','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vesktop','67','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/viber','102','26426.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vidtuber','3','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vimb','3','30801.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vimfx','1','30636.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vivaldi','118','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vmware-horizon-client','130','29293.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vnstat','7','27969','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vpnc','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vrrpd','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vuze','7','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/w3af','39','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/w3m','12','30035.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wakeonlan','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/warpinator','27','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/waterfox','86','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wavemon','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/webalizer','4','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/webex','258','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/webhook','15','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wechat-universal','100','27947.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/weechat','145','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/weetwit','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wego','12','29246.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/weighmail','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wendzelnntpd','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/whalebird','168','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/whatsie','33','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wifi-radar','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wifite','1','27918.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wifitui','10','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wildfly','78','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/windscribe','29','28763.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wire','87','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wireshark','318','27918.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wmget','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wmnd','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wmnotify','4','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wol','8','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wp-cli','3','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wpa-cute','7','27477.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wput','3','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wrk','6','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wsdd2','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wvdial','7','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/x11spice','12','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/x11vnc','17','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/x2goclient','25','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/x2goserver','8','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/xaric','7','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/xbuffy','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/xdman','15','29149.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/xinetd','12','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/xl2tpd','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/xmrig','46','30607.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/xnetload','1','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/xtables-addons','11','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/yadifa','27','29954.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/yate','145','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/yle-dl','2','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/you-get','2','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/youtube-dl','13','29101.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/youtube-dl-gui','1','29101.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/youtube-dl-server','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/youtube-upload','1','26464.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/yt-dlp','12','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/yt-dlp-bin','13','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ytfzf','1','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/zabbix_agent2','137','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/zabbix_agentd','53','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/zabbix_frontend','43','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/zabbix_java_gateway','95','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/zabbix_proxy','89','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/zabbix_server','104','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/zdns','19','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/zeek','1099','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/zen-browser','88','31497.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/zerotier-one','24','29272.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/zmap','5','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/znc','49','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/zoom-linux','249','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/zsync','5','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/1password','104','26426.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/BibTool','19','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/CherryTree','4','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/FreeOffice','71','31994.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/JabRef','103','28711','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/Joplin','74','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/LaTeXML','238','28763.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/LanguageTool','92','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/LibreOffice','4800','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/Logseq','121','29301','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/MasterPDFEditor','69','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/MasterPDFEditor4','14','28763.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/NME','3','28681','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/OpenAudible-bin','93','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/PortfolioPerformance-bin','29','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/ProjectLibre','7','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/ReText','2','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/SOGo','69','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/Ted','109','27969','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/abiword','253','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/abook','6','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/aigaion','3','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/aiksaurus','8','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/anorack','1','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/antiword','2','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/apvlv','6','29822.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/asymptote','59','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/atril','37','32580.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/basket','19','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/batdoc-bin','3','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/beaver','7','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/bibclean','5','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/bibtex2html','8','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/bibutils','3','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/bookbinder','2','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/briss','4','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/broffice.org','159','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/calcurse','5','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/calibre-bin','134','27947.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/catdoc','3','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/catdvi','2','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/ccal','2','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/convertlit','2','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/coolreader','27','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/cpdf','21','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/crow-translate','20','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/csbooks-bin','88','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/cups-pdf','1','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/devtodo','11','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/dictd','10','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/diction','3','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/diffpdf','7','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/ding','6','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/djvusmooth','1','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/docx2txt','1','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/dspdfviewer','10','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/eid-mw','30','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/elyxer','1','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/enpass','26','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/epdfview','7','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/epr','1','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/epub2txt2','3','30607.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/epy','2','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/etmtk','2','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/evince','59','28945.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/evince-gtk2','35','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/evolution','116','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/evolution-ews','11','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/extractpdfmark','6','31994.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/fbpdf','1','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/fbreader','98','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/featherpad','19','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/flowkeeper','2','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/focuswriter','20','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/focuswriter-qt6','29','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/fop','18','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/freemind','14','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/freeplane','31','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/full-pack','10','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/ganttproject','7','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/gbgoffice','10','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/gcal','25','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/gcalctool-gtk2','15','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/ghostwriter','35','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/glow','7','27804','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/gmdb2','14','28084.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/gnokii','15','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/gnucash','140','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/gnucash-docs','41','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/gnumeric','156','29696.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/gnumeric-gtk2','71','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/gocr','6','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/goldendict','55','29527.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/grisbi','37','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/gummi','6','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hamster','5','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hebcal','8','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/heimer','25','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hevea','10','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/homebank','20','30007','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/htmldoc','12','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hunspell-ancient-greek','9','30607.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hunspell-de','6','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hunspell-en','4','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hunspell-es','9','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hunspell-french','3','29301','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hunspell-gr','8','30607.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hunspell-id','1','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hunspell-pl','3','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hunspell-russian-aot','6','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hunspell-russian-kliukvin','3','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hunspell-russian-lebedev','3','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hunspell-russian-wiktionary','8','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hunspell-sv','3','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/impressive','1','29448','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/ishmael','11','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/ispell-dict-english','36','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/ispell-dict-russian-lebedev','7','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/jfbview','39','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/jpdfbookmarks','3','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/jstock','18','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/kbbi-qt','11','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/kbgoffice','7','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/kbibtex','50','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/kchmviewer-qt','14','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/keepassx','17','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/keepassxc','84','31985.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/keepnote','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/khal','2','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/khard','4','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/kitscenarist','92','27834.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/kiwix-desktop','32','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/kraft','38','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/krop','2','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/kuro','96','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/kuro-legacy','92','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/latex2rtf','3','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/latexdiff','2','29602.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/lcal','2','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/leafpad','5','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/ledger-live','94','28694.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/libreoffice','259','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/libreoffice-helppack','8','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/libreoffice-langpack','11','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/lout','10','28084.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/lowdown','6','31738.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/luametatex','14','31056.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/lumina-calculator','4','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/lumina-pdf','6','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/lyx','158','27918.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mairix','3','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mate-calc','35','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mathtex','2','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mdbtools','8','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mdp','1','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/miktex','429','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mined','14','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/minidjvu','26','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mobiperl','2','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/moneydance','85','28093.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/msee','5','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mtp2-fonts','2','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mu','73','29954.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/multivalent-tool-pdf','2','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mupdf','51','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/muttprint','3','31172.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mytetra','14','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mythes','5','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mythes-id','2','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/navi','27','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/nb','2','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/neoleo','7','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/notes','2','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/notmuch','14','28754.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/novelWriter','35','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/nts','2','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/nuspell','8','29398','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/ocrmypdf','2','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/odt2txt','1','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/omegat','84','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/onlyoffice-desktopeditors','341','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/openoffice-langpack','8','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/openoffice.org','95','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/osmo','8','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/osmo-gtk2','8','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pageEdit','41','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pandoc','1205','28767.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pandoc-bin','104','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/paps','16','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pcal','2','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pdf-bunny','2','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pdf2djvu','13','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pdfarranger','3','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pdfchain','9','32580.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pdfpc','7','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pdfsam-basic','54','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pdfsandwich','1','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pdfshuffler','1','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pdfstudio','74','29345.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pdfstudioviewer','73','29345.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pdftk','3','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/plover','30','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/ps2eps','1','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pstoedit','13','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pstotext','1','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pyspread','5','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/python-gcalcli','2','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/python3-proselint','2','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/python3-xlsx2csv','4','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/qownnotes','89','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/qpdfview','23','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/qtrans','8','30630.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/qute','30','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/re-typeset','6','27834.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/referencer','21','27129','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/remind','10','29149.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/scribus','317','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/semantik','29','29293.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/sent','2','30924.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/siag','17','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/sigil','226','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/sioyek','26','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/smoffice2016','166','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/smoffice2018','176','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/smoffice2021','141','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/smoffice2024','122','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/star-tex','17','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/t-prot','1','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/tapecalc','5','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/task','21','29457.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/taskd','5','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/taskjuggler','8','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/taskopen','1','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/tasksh','3','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/tellico','118','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/texlive-docs','429','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/texlive-extra','942','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/texmacs','165','27477.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/texmaker','76','27477.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/texstudio','159','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/text2pdf','1','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/texworks','49','29194.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/timetrap','2','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/timew','10','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/tnote','1','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/todo.txt-cli','1','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/tpp','2','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/traduisons','1','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/treesheets','6','26869','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/trelby','2','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/trello-cli','15','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/tudu','8','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/turtl','70','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/typobuster','3','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/unaesgcm','3','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/uncsv','1','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/unoconv','2','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/verbiste','14','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/visidata','5','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/vit','2','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/vlna','3','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/vmd','1','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/vym','49','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/vzic','2','28864','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/wammu','3','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/watson','2','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/when','1','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/wink','3','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/wordgrinder','6','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/wordtsar','51','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/wps-office','292','28763.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/wps-office-dicts','76','28671.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/x_x','2','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/xmind','57','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/xournal','8','30582.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/xournalpp','70','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/xpad','5','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/xpdf3','11','27969','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/xreader','16','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/xtrkcad','32','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/zathura','10','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/zathura-cb','4','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/zathura-djvu','4','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/zathura-pdf-mupdf','4','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/zathura-ps','4','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/zettlr','101','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/zim','6','28216.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/Alien-Build','5','29191.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/Alien-Libxml2','39','28059.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/Download-GitLab','1','29191.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/File-chdir','1','29225.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/Gtk2-GladeXML','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/MoarVM','20','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/XML-LibXML','8','31033.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/cpanminus','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/libpurple-kwallet-plugin','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/libwww-perl','26','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/mod_perl','48','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/nqp','24','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Algorithm-Combinatorics','5','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Algorithm-Diff','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Algorithm-Loops','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Alien-SDL','4','27910.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-AnyEvent','15','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-AnyEvent-HTTP','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-AppConfig','2','32580.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Archive-Extract','4','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Archive-Zip','2','27167.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Audio-FLAC-Header','3','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-B-COW','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-B-Hooks-EndOfScope','3','31172.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-B-Hooks-OP-Check','2','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-B-Hooks-OP-PPAddr','3','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-B-Keywords','2','29171.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-B-Lint','3','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-BerkeleyDB','5','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Bit-Vector','6','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CDDB-get','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CGI','6','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CGI-Application','4','29602.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CGI-Fast','3','32580.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CPAN-Changes','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CPAN-Meta-Check','1','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CPAN-Perl-Releases','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CPAN-Requirements-Dynamic','1','28718.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CPANPLUS','24','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CPANPLUS-Dist-Build','5','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CPANPLUS-Dist-Slackware','2','28044.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CSS','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CSS-Squish','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Cache-FastMmap','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Cairo-GObject','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Canary-Stability','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Capture-Tiny','39','27910.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Carp-Clan','1','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-Data-Inheritable','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-Factory-Util','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-Gomor','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-ISA','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-Inspector','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-Load','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-Load-XS','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-Method-Modifiers','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-MethodMaker','7','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-Mix','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-ReturnValue','2','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-Singleton','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-Tiny','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Clone','2','26925.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Clone-PP','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Compress-Bzip2','7','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Config-File','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Config-Find','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Config-Grammar','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Config-IniFiles','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Config-Simple','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Config-Tiny','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Convert-ASN1','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Convert-BinHex','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Convert-Color','3','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Convert-TNEF','1','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Convert-UUlib','3','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Coro','5','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Cpanel-JSON-XS','6','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-Blowfish','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-Blowfish_PP','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-CAST5','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-CBC','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-DES','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-Eksblowfish','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-IDEA','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-OpenSSL-AES','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-OpenSSL-Bignum','3','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-OpenSSL-DSA','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-OpenSSL-EC','4','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-OpenSSL-ECDSA','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-OpenSSL-Guess','1','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-OpenSSL-RSA','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-OpenSSL-Random','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-Rijndael','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CryptX','24','29288','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Cstools','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Curses','17','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Curses-UI','4','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Cwd-Guard','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DBD-Pg','4','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DBD-SQLite','50','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DBIx-Simple','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Danga-Socket','9','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Data-Compare','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Data-Dumper-Concise','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Data-IEEE754','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Data-OptList','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Data-Printer','4','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Data-Random','6','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Data-UUID','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Data-Validate-IP','3','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Date-Calc','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Date-Manip','10','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DateTime','14','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DateTime-Format-Builder','4','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DateTime-Format-DateParse','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DateTime-Format-ISO8601','4','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DateTime-Format-Mail','6','27804','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DateTime-Format-RFC3339','2','27804','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DateTime-Format-Strptime','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DateTime-Locale','16','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DateTime-TimeZone','5','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Devel-CheckCompiler','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Devel-Cycle','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Devel-Declare','5','27910.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Devel-GlobalDestruction','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Devel-NYTProf','99','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Devel-OverloadInfo','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Devel-PartialDump','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Devel-PatchPerl','2','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Devel-StackTrace','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Devel-Symdump','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Devel-Trace','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Device-SerialPort','10','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Digest-GOST','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Digest-SHA','2','35851.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Digest-SHA-PurePerl','3','27910.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Dist-CheckConflicts','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-EV','10','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Email-Abstract','2','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Email-Address-XS','2','26584.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Email-Date-Format','2','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Email-MIME','4','29751.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Email-MIME-ContentType','3','28945.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Email-MIME-Encodings','2','28945.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Email-MessageID','2','28945.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Email-Sender','3','28171.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Email-Simple','3','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Encode-Detect','4','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Encode-EUCJPASCII','3','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Encode-HanExtra','10','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Encode-ISO2022','5','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Encode-JISX0213','4','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Error','2','29246.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Eval-Closure','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Exception-Class','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Expect','76','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Exporter-Lite','1','28052.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Exporter-Tiny','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-ExtUtils-CChecker','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-ExtUtils-Config','2','28727.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-ExtUtils-Helpers','2','27762.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-ExtUtils-Install','4','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-ExtUtils-InstallPaths','2','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-ExtUtils-ModuleMaker','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-ExtUtils-XSBuilder','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-FCGI','4','32580.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Feature-Compat-Try','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Copy-Recursive','3','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Copy-Recursive-Reduced','2','27910.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Download','1','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-FcntlLock','8','32580.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Find-Rule','2','27907.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Finder','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Grep','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-HomeDir','2','28763.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Inplace','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-ReadBackwards','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Remove','6','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Save-Home','2','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Share','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-ShareDir','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-ShareDir-Install','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Slurp','4','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Slurp-Tiny','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Tail','66','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Type','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Which','2','28763.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-pushd','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Filesys-Df','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Font-AFM','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Font-TTF','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-GD','5','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Geo-Coordinates-OSGB','4','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Geo-Coordinates-UTM','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Geo-IP','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Geography-Countries','1','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Getopt-Long-Descriptive','2','27910.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Getopt-Tabular','2','28052.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Git-Sub','2','27910.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Glib-Object-Introspection','6','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Gnome2-Vte','4','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-GooCanvas2','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Gtk2-Ex-Simple-List','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Gtk2-Notify','3','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Gtk2-Unique','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Gtk3','2','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Gtk3-ImageView','1','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Gtk3-SimpleList','2','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Guard','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-HTML-FormatText-WithLinks','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-HTML-FormatText-WithLinks-AndTables','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-HTML-Formatter','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-HTML-HTMLDoc','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-HTML-TableExtract','3','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-HTML-Template','4','29602.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-HTML-Tidy','3','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-HTML-Tree','3','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-HTML-WikiConverter','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-HTML-WikiConverter-Markdown','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-HTTP-Cache-Transparent','1','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-HTTP-CookieJar','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-HTTP-Proxy','2','32580.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Hook-LexWrap','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-All','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-CaptureOutput','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-HTML','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-Interface','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-Multiplex','17','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-SessionData','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-Socket-INET6','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-Socket-Multicast','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-Socket-Socks','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-Socket-Timeout','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-String','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-Stty','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-Tty','15','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-stringy','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IP-Country','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IPC-DirQueue','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IPC-Run','1','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IPC-Run3','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IPC-System-Simple','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IPTables-ChainMgr','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IPTables-Parse','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Image-Info','3','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Image-Sane','3','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Image-Size','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Import-Into','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Inline','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Inline-C','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-JSON','32','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-JSON-MaybeXS','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-JSON-XS','4','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-LWP-Protocol-https','2','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-LWP-Protocol-socks','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Lingua-EN-Fathom','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Lingua-EN-Inflect','4','30712.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Lingua-EN-Numbers-Ordinate','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Lingua-EN-Syllable','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Lingua-Preferred','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Linux-FD','3','31985.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-List-AllUtils','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-List-MoreUtils','8','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-List-MoreUtils-XS','8','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-List-SomeUtils','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-List-UtilsBy','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Locale-Codes','5','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Locale-Maketext-Gettext','5','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Locale-Maketext-Lexicon','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Locale-PO','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-LockFile-Simple','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Log-Dispatch','7','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Log-Log4perl','31','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Log-Message','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Log-Message-Simple','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MIME-Charset','2','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MIME-Lite','3','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MIME-Types','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MIME-tools','2','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MP3-Info','2','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MRO-Compat','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Mail-DKIM','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Mail-DMARC','24','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Mail-DomainKeys','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Mail-SPF','2','29246.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Mail-SPF-Query','2','31291.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MailTools','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-Base85','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-BigInt','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-BigInt-GMP','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-Complex_C','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-Complex_C-L','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-Complex_C-Q','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-Decimal64','9','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-GMPf','4','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-GMPq','4','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-GMPz','5','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-GSL-Linalg-SVD','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-Int128','5','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-Int64','4','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-LongDouble','5','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-MPC','5','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-MPFR','7','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-Prime-Util','11','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-Prime-Util-GMP','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-Round','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MaxMind-DB-Common','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MaxMind-DB-Reader','5','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MaxMind-DB-Reader-XS','4','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Meta-Requirements-Range','2','29448','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Modern-Perl','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Module-Build','37','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Module-Build-Tiny','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Module-Build-XSUtil','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Module-Implementation','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Module-Install','17','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Module-Manifest','2','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Module-Pluggable','5','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Module-Runtime-Conflicts','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Module-ScanDeps','2','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Module-Versions-Report','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Mojolicious','74','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Mojolicious-Plugin-Humane','3','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Mojolicious-Plugin-I18N','3','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MooX-StrictConstructor','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MooX-Types-MooseLike','3','28945.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Moose','70','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MooseX-Traits','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MooseX-Types','4','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MooseX-Types-Structured','6','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Mozilla-CA','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MusicBrainz-DiscID','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Mutex','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-ARP','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-CIDR','2','30573','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-CIDR-Lite','2','27178.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Curl','8','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-DNS-Native','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-DNS-Resolver-Mock','2','29751.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-DNS-Resolver-Programmable','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-DNS-SEC','4','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Daemon','71','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-IDN-Encode','5','28945.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-IPv4Addr','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-IPv6Addr','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Ident','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Jabber','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-LDAP','5','29602.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-LibIDN','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-LibIDN2','3','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Libdnet','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Netmask','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Patricia','1','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Pcap','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-RawIP','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-SNMP','6','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-SSH','2','27910.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Server','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Telnet','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-UPnP','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Works','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Write','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-XMPP','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-NetAddr-IP','14','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-NetPacket','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Number-Bites-Human','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Number-Compare','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Number-Convert-Roman','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Object-Accessor','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Ogg-Vorbis-Header-PurePerl','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-PAR','3','29735.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-PAR-Dist','2','29735.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-PDF-API2','9','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-PDF-Builder','20','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-PPI','21','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-PPIx-QuoteLike','3','27762.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-PPIx-Regexp','10','27762.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-PPIx-Utils','2','27762.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Package-Constants','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Package-DeprecationManager','2','27910.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Package-Stash','7','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Package-Stash-XS','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Params-Classify','3','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Params-Util','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Params-Validate','5','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Params-ValidationCompiler','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Parse-Method-Signatures','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Parse-RecDescent','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Parser-MGC','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Path-Class','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Pegex','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Perl-Critic','30','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Perl-Tidy','48','27762.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-PerlIO-via-Timeout','6','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Pod-Coverage','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Pod-POM','3','29333.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Pod-POM-View-Restructured','2','28754.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Pod-Parser','3','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Pod-Spell','3','27762.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Probe-Perl','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Proc-Daemon','1','32580.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Proc-PID-File','16','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Proc-ProcessTable','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Proc-Simple','16','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Readonly','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Regexp-Common','11','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Regexp-IPv6','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Religion-Islam-Quran','13','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Return-MultiLevel','2','28741.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Role-Tiny','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-SDL','28','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-SNMP-Info','5','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-SOAP-Lite','2','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Scalar-List-Utils','4','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Schedule-Cron','42','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Scope-Guard','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Scope-Upper','9','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Set-IntSpan','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sidef','6','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Socket-GetAddrinfo','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Socket6','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sort-Naturally','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sort-Versions','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Statistics-Descriptive','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-String-Format','2','27762.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-String-PerlIdentifier','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-String-Random','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-String-Scanf','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-String-ShellQuote','1','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-String-Similarity','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sub-Exporter','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sub-Exporter-ForMethods','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sub-Exporter-Progressive','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sub-Identify','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sub-Install','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sub-Name','3','30575.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sub-Override','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sub-Uplevel','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Switch','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Syntax-Highlight-Engine-Kate','24','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Syntax-Keyword-Try','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sys-CPU','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sys-Hostname-Long','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sys-MemInfo','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sys-Mmap','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sys-Syscall','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-System-Command','17','27910.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-System-Sub','3','30575.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Task-Weaken','2','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-TeX-Hyphen','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Term-Animation','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Term-ProgressBar','13','27910.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Term-ReadLine-Gnu','3','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Term-UI','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Bits','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Class','4','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-CleanNamespaces','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Deep','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Differences','3','27910.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-DistManifest','2','26426.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Exception','1','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Exit','1','28741.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Fatal','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-File','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-File-ShareDir','3','28945.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-File-ShareDir-Dist','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Harness','2','28727.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-HexDifferences','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Inter','2','29809.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-LongString','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Memory-Cycle','3','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-MockModule','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-MockObject','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-MockTime','30','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Most','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Needs','3','30636.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-NoWarnings','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Number-Delta','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Object','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Output','3','27910.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-PerlTidy','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Pod','1','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Pod-Coverage','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Requires','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-RequiresInternet','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-SharedFork','43','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Simple','12','28727.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Spec','3','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-SubCalls','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-TCP','12','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-TempDir-Tiny','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Trap','4','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Warnings','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-utf8','3','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Autoformat','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-CSV','15','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-CSV_XS','16','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-CharWidth','1','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Diff','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Glob','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Hyphen','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Iconv','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Password-Pronounceable','2','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Patch','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Quoted','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Reform','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Soundex','2','29602.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Tabs+Wrap','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Tabulate','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Unidecode','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-WrapI18N','1','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Throwable','2','28945.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Tidy','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Tie-IxHash','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Tie-Simple','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Time-Duration','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Time-Out','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Time-ParseDate','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Time-Piece','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Time-modules','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-TimeDate','1','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Tk-TableMatrix','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Tree-Simple','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Try-Tiny','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-TryCatch','5','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Types-Serialiser','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-URI-Simple','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-UUID','15','27910.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Unicode-LineBreak','6','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Unicode-Map','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Unicode-Map8','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Unicode-String','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Unicode-UTF8simple','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Unix-Syslog','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Variable-Magic','5','31172.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-WWW-Curl','4','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-WebService-Gyazo-B','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-WebService-MusicBrainz','2','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-X11-Protocol-Other','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-XML-Catalog','2','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-XML-Filter-BufferText','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-XML-LibXSLT','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-XML-SAX-Writer','2','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-XML-Stream','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-XML-TreeBuilder','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-XML-TreePP','5','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-XML-Writer','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-XML-XPath','5','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-XS-Parse-Keyword','6','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-YAML-LibYAML','6','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-YAML-Syck','24','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-YAML-Tiny','3','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-ZMQ-Constants','2','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-aliased','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-audio-sndfile','6','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-cairo','3','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-cgi-minimal','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-class-accessor','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-class-accessor-chained','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-common-sense','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-config-general','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-crypt-dh-gmp','2','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-data-dump','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-data-messagepack','6','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-data-page','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-data-page-pageset','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-data-section-simple','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-digest-hmac','1','30836.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-digest-sha1','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-encode-locale','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-event','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-extutils-depends','1','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-extutils-pkgconfig','1','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-file-basedir','1','28763.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-file-desktopentry','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-file-libmagic','2','27910.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-file-listing','4','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-file-mimeinfo','2','29735.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-file-path-expand','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-file-slurper','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-glib','9','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-gnome2','9','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-gnome2-canvas','4','30582.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-gnome2-gconf','3','26495.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-gnome2-vfs','4','26516','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-gnome2-wnck','4','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-goo-canvas','4','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-gstreamer','7','26495.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-gtk2','29','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-gtk2-imageview','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-html-form','3','26426.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-html-parser','5','31291.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-html-tagset','2','26925.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-http-cookies','2','31291.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-http-daemon','4','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-http-date','2','29527.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-http-message','4','31291.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-http-negotiate','1','31291.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-http-response-encoding','1','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-http-server-simple','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-http-tiny','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-image-bmp','3','28763.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-inc-latest','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-inc-module-install','18','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-io-uncompress-brotli','23','29822.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-libintl','4','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-linux-desktop-files','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-lirc-client','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-local-lib','5','29602.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-lwp-mediatypes','2','26925.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-lwp-useragent-cached','2','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-namespace-autoclean','3','27910.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-namespace-clean','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-net-dbus','9','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-net-dns','3','31291.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-net-http','2','29527.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-net-ip','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-net-mpd','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-palm-pdb','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-pango','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-parallel-forkmanager','70','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-strictures','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-test-warn','3','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-text-aspell','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-text-unaccent','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-tk','29','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-trayicon','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-tree-dagnode','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-www-mechanize','2','26502.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-www-robotrules','6','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-x11-protocol','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-xml-fast','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-xml-libxml','9','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-xml-parser-lite','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-xml-parser-lite-tree','2','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-xml-twig','3','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-yaml','1','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perlbrew','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/raku-Readline','15','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/rakudo','134','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/txt2html','3','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/zef','22','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/BeautifulSoup4','3','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/CANard','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/CommonMark','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/DVR-Scan','1','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Flask','3','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Flask-Gravatar','1','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Flask-HTMLmin','1','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/GeoIP-Python','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Kallithea','5','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Markdown','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Markups','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Paver','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PyAudio','2','28452.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PyDispatcher','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PyGreSQL','4','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PyMeeus','3','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PyPDF2','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PyPy','2039','31546','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PyPy3','1954','28692.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PyRIC','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PyStemmer','11','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PyXB','4','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Pyro','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Pyro4','3','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PythonQt','56','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/RBTools','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/SOAPpy','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/SQLAlchemy-legacy','8','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/TatSu','3','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Unidecode','3','29101.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/XlsxWriter','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Yapps2','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/acstore','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/affine','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/alabaster','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/apprise','3','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/apsw','8','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/arrow','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/artifacts','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/asciimatics','3','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/asgiref','2','28705.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/astroid','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/async_generator','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/audible','2','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/audible-cli','2','29101.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/autopep8','3','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/babel','20','29602.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/backcall','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/backports-abc','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/backports.lzma','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/bencode','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/binaryornot','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/binplist','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/biplist','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/bitarray','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/blessings','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/blink-qt','8','28052.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/boolean-py','2','28585.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/bottle-mysql','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/breathe','3','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/bsddb3','8','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/buildbot','10','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/buildbot-badges','2','27932.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/buildbot-worker','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/captcha','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cddb-py','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/certbot-dns-cloudflare','2','28315.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cfgparse','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cheetah3','3','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/click','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/click-default-group','2','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/click-legacy7','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/click-log','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/click-plugins','2','28692.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/click-threading','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cloudpickle','2','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cloudscraper','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/colorama','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/colored','2','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/construct','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/contextlib2','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/convertdate','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/coverage','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cppy','2','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/crcmod','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cryptography','23','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/css-parser','3','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cssselect2','1','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cssutils','11','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/curtsies','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cwcwidth','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cx_Freeze','3','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/darts.util.lru','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/dfdatetime','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/dfvfs','5','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/dfwinreg','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/dhcpy6d','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/django-debug-toolbar','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/django-sass','2','29448','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/doit','3','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/dotty','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/dpkt','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/drawille','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/dtfabric','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/dukpy','14','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/dxfwrite','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/easygui','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ebaysdk','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ecdsa','3','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/elasticsearch5','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/email-validator','2','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/enzyme','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/esptool','3','29436.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/euca2ools','8','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/extras','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/fasteners','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/feedgenerator','3','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/feedparser','8','29149.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/filechunkio','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/findsystemfontsfilename','3','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/fixtures','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/flake8','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/freesimplegui','3','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/freetype-py','3','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/frosted','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/fudge','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/funcsigs','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/functools-lru-cache','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/functools32','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/gi-docgen','3','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/git-fame','3','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/githubpy','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/gnome-python','3','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/gnome-python-desktop','7','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/google-api-python-client','13','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/google-images-download','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/gst-python','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/gst0-python','11','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/gunicorn','2','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/hachoir','5','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/hachoir-core','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/hachoir-metadata','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/hachoir-parser','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/halberd','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/hg-git','3','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/hgsubversion','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/hijri-converter','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/holidays','7','29101.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/homebase','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/hpack','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/html2text','3','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/html5-parser','10','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/htmlmin','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/huepy','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/humanize','3','28700.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/i3-layouts','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/i3-py','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/i3expo-ng','2','31033.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/i3ipc','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ics','4','29149.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/imagesize','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/importlib-resources','4','28754.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/insipid-sphinx-theme','22','30260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/instaloader','2','29954.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/internetarchive','2','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ipaddr-py','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ipython','4','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/isort','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/jaraco-packaging','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/jdatetime','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/jdcal','3','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/jellyfish','9','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/jmespath','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/josepy','2','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/json-py','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/kallithea-frontend','15','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/kaptan','1','29272.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/kitchen','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/korean_lunar_calendar','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/license-expression','2','28585.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/linecache2','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/llfuse','18','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/mechanize','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/memory-profiler','2','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/mibuild','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/migen','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/mini-amf','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/minidb','2','29272.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/mock','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/monotonic','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/monty','3','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/more-itertools','2','29272.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/msgpack-python','5','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/munch','2','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/mypy','13','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/mypy_extensions','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/nanobind','14','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ndg_httpsclient','2','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/node-semver','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/npm','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/npyscreen','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/num2words','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/numexpr','8','29428.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/nyx','4','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/objgraph','2','28681','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/odict','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/odoo','301','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ofxparse','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/optional-django','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/p4python','6','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pallets-sphinx-themes','2','28585.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/parsedatetime','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pathlib','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pathtools','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pathtools3','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pattern','29','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pefile','2','28315.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pelican','5','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/phply','5','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pickleshare','2','32805.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pikepdf','22','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pipdeptree','2','28700.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/piprot','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pipstat','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pkginfo','2','28692.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/plaso','10','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pluginbase','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/polib','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/progressive','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ps_mem','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ptpython','3','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ptyprocess','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/py3sensors','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyOpenSSL','2','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pySmartDL','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyaes','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyamf','17','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyatspi','4','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pybind11','5','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pybind11-legacy','4','29757.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyblake2','3','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pybloomfiltermmap','4','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pycassa','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pychecker','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pychm','3','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pycld2','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pycodestyle','1','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pycxx','1','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pydblite','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pydf','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pydot','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyelftools','2','29301','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyewmh','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyfeed','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyfiglet','10','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyflakes','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyfltk','63','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyfuse3','19','28692.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyglet','3','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pygpgme','3','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pygrametl','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pygsl','58','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pygtksourceview','9','30582.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pygtkspell','7','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyjwt','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyliblo','10','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pymongo','9','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pymysql','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyorbit','6','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyotherside','12','31408','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyotp','1','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyperclip','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pypng','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pypoppler','10','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pypy','50','28692.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pypy3','58','28692.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyqode.cobol','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyqode.qt','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyquery','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyrfc3339','3','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyrsistent','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pysass','2','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyscard','4','29149.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyserial','3','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pysha3','6','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyside','208','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyside2','649','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyside2-tools','5','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pysmssend','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pysnmp','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pystatsd','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pysvn','6','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyta-lib','51','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pytest','3','28681','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-PySnooper','2','29272.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-argopt','2','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-axolotl','3','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-axolotl-curve25519','8','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-bitstring','3','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-colour','2','31033.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-daemonize','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-debian','3','29101.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-discid','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-diskcache','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-djvulibre','51','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-emoji','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-faulthandler','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-fido2','3','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-fusepy','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-gflags','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-gntp','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-gssapi','38','31994.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-importlib_metadata','4','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-ipcalc','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-jeepney','2','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-jsonrpclib','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-kerberos','3','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-krb5','61','29345.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-libarchive-c','2','28455.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-libnacl','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-ly','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-magic','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-markdown-math','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-musicbrainz2','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-mysql-replication','2','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-mysqlclient','3','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-nbxmpp','4','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-oauthlib','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-parse','2','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-parsedatetime','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-pbkdf2','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-pcapy','4','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-poppler-qt5','97','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-precis-i18n','1','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-pyparted','3','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-qpageview','3','29122.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-qrcode','2','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-rubymarshal','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-serpent','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-setuptools-git','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-sh','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-slip','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-smartypants','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-stdnum','3','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-tlslite','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-unicodedata2','4','29212.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-uritemplate','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-urwid_readline','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-vdf','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-wand','3','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-webpack','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-xlib','3','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-zipp','4','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-BeautifulSoup4','2','29177.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-Levenshtein','3','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-MarkupSafe','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-MonthDelta','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-Safe','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-asn1crypto','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-atomicwrites','1','28681','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-attrs','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-authres','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-bigfloat','9','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-blessed','2','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-cached-property','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-cheetah','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-clamd','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-cluster','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-configobj','2','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-cryptography','13','30162.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-cssselect','2','28209.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-daemon','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-dateutil','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-demjson','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-distorm','3','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-distutils-extra','2','28705.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-django','22','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-django-legacy','21','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-django-tagging','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-dnspython','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-docopt','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-editor','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-elementtree','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-elib.intl','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-enlighten','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-esmre','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-gammu','5','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-gattlib','17','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-gmpy2','10','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-httplib2','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-invoke','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-keyczar','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-keyutils','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-lazy-object-proxy','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-lhafile','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-lockfile','1','28054.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-lxml','211','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-lz4','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-magick','77','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-mccabe','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-memcached','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-mimeparse','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-morbid','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-mpd','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-mpmath','3','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-neovim','2','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-networkx','6','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-nuitka','20','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-oauth','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-oauth2','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-oauth2client','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-openid','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-paramiko','3','30162.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-passlib','3','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-patch','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pathlib2','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pbr','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pdfrw','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pillow','10','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pkgconfig','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-plucky','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pluggy','2','28681','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pmw','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-prefixed','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-progress','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-psutil','3','28428.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-py','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pyasn1','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pybluez','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pycdio','4','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pycrypto','6','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pycryptodomex','22','27932.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pydns','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pygame','56','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pyicu','38','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pypolicyd-spf','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pyspf','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pytest-runner','2','26426.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pythondialog','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pytz','2','29735.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-regex','9','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-requestbuilder','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-requests-kerberos','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-scandir','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-selectors2','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-setuptools-scm','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-simpy','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-slacklog','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-soupsieve','2','28681','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-stomper','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-suds','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-swiftclient','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-sympy','27','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-tabulate','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-tblib','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-texttable','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-toml','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-transaction','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-tvrage','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-twitter','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-uri-templates','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-uuid','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-vatnumber','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-vulndb','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-wheel','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-whisper','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-wordpress-xmlrpc','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-wrapt','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-x11_hash','12','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-xcffib','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-xlwt','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-xrandr','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-yara','3','28315.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-yenc','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-APScheduler','3','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-BTrees','39','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-Flask-Babel','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-Flask-Basicauth','1','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-Flask-Bcrypt','1','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-Flask-Compress','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-Flask-Cors','3','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-Flask-Dance','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-Flask-Login','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-Flask-Mail','1','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-Flask-Principal','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-Flask-RESTX','4','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-Flask-RESTful','1','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-Flask-SQLAlchemy','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-Flask-Security-Too','3','27969','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-Flask-WTF','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-Flask-httpauth','1','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-MutatorMath','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-PyGithub','15','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-PyMuPDF','198','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-PyPDF2','2','28705.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-WTForms','3','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ZConfig','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ZEO','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ZODB','5','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-aiofiles','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-aiohappyeyeballs','2','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-aiohttp','19','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-aiohttp-oauthlib','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-aiohttp-socks','2','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-aiorpcX','3','28754.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-aiosignal','1','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-aiosmtpd','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-aiostream','1','27932.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-aksharamukha','5','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-alembic','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-amqp','2','29822.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-aniso8601','2','29101.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-annotated-types','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-anyio','4','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-anysqlite','2','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-application','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-arabic-reshaper','2','29149.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-argcomplete','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-argon2-cffi','2','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-argon2-cffi-bindings','9','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-arrow','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-asn1crypto','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-astroid','3','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-asttokens','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-async-lru','2','28705.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-async-timeout','1','28705.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-atomicwrites','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-atpublic','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-attrdict','2','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-attrs','2','28754.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-auditok','3','29767.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-augeas','3','26983.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-authheaders','2','26584.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-authres','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-autobahn','6','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-autocommand','3','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-automat','4','28705.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-babel','19','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-babelfish','3','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-backlash','2','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-backports_tarfile','4','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-base58','1','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-bcrypt','15','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-beaker','1','27477.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-bech32','2','30801.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-beekeeper-alt','2','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-bencode','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-beniget','2','28428.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-bidi','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-billiard','1','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-bitarray','4','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-black','3','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-bleach','3','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-blessed','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-blinker','2','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-booleanOperations','2','30582.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-boto3','8','33876.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-bottle','2','28692.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-brotlicffi','20','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-bsdiff4','2','29293.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-build','1','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-cachecontrol','2','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-cachetools','3','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-cacheyou','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-cairocffi','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-caldav','4','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-calver','3','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-cattrs','3','27969','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-celery','3','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-cffi','6','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-cffsubr','4','28754.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-cfgv','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-characteristic','2','33944.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-cheroot','4','27969','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-cinemagoer','4','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-click-didyoumean','2','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-click-option-group','2','28044.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-click-repl','3','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-cloudflare','2','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-cmake-build-extension','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-cogapp','1','29436.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-colorlog','2','29149.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-comm','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-compreffor','6','29149.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-configargparse','3','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-configobj','2','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-confuse','2','27864','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-constantly','2','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-construct','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-construct-classes','1','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-cookiecutter','3','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-crank','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-croniter','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-cssselect','2','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-cython-opt','141','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-daemon','3','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-dasbus','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-dateutil','1','28551.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-dbus-fast','2','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-dbus-next','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-debugpy','32','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-decorator','4','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-defcon','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-defusedxml','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-deluge-client','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-dep-logic','3','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-deprecated','2','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-deprecation','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-desktop-notify','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-diff-match-patch','2','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-dill','2','29272.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-discovery','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-distlib','5','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-distutils-extra','2','28705.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-django','106','29735.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-dkimpy','2','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-dnspython','3','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-docker','2','28044.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-dockerpty','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-docopt','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-docstring-to-markdown','1','28705.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-dogpile.cache','3','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-dotenv','4','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-dugong','2','32495.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-dulwich','4','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-dunamai','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-editables','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-editorconfig','2','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-elgato-streamdeck','3','29436.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-enlighten','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-entrypoints','1','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-enzyme','2','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-et_xmlfile','1','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-euclid3','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-evdev','4','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-eventlet','2','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-eventlib','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-exceptiongroup','2','27969','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-executing','2','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-exiv2','11','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-expandvars','1','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-falcon','10','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-fanart','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-fasteners','3','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-fastjsonschema','2','29436.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ffmpeg','2','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ffsubsync','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-filelock','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-filetype','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-findpython','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-flake8_annotations','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-flask-paginate','1','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-flit','2','28718.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-flit_core','1','28718.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-flit_scm','2','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-flufl.bounce','3','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-flufl.i18n','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-flufl.lock','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-fontMath','3','47997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-fontParts','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-fontPens','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-fontmake','2','28585.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-fonttools','27','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-formencode','2','28692.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-fqdn','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-fritzconnection','2','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-frozenlist','4','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-fs','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-funcparserlib','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-fypp','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-gTTS','2','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-gast','2','29345.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-gattlib','18','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-gearbox','4','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-gensim','40','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-geventhttpclient','3','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-gitdb2','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-glad','7','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-glances','6','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-glyphslib','7','29149.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-gmpy2','21','29212.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-gnutls','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-google-api-python-client','37','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-google-auth-httplib2','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-google-auth-library-python','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-google-python-api-core','3','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-googleapis-common-protos','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-grpcio','477','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-guessit','2','29272.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-h11','1','29101.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-h2','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-h5py','85','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-hatch-nodejs-version','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-hatch_fancy_pypi_readme','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-hatch_jupyter_builder','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-hatch_vcs','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-hatchling','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-helpdev','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-hishel','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-hsaudiotag3k','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-httpcore','2','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-httplib2','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-httpx','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-hupper','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-hyperframe','4','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-hyperlink','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-icalendar','3','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-icoextract','2','30607.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-id','1','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-identify','1','31408','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ifaddr','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-incremental','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-inflate64','4','29101.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-inflect','4','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-inflection','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-iniconfig','2','28700.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-inotify-simple','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-installer','1','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-intelHex','4','29822.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-intervaltree','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-invoke','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ipaddr','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ipython-pygments-lexers','3','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ipython-sql','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ipython_genutils','1','28681','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-iso3166','3','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-iso639','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-isodate','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-isoduration','2','29149.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-isort','2','29272.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-isounidecode','3','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-itsdangerous','1','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-jaconv','2','28763.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-janus','3','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-jaraco.classes','3','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-jaraco.collections','4','29212.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-jaraco.context','4','30260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-jaraco.functools','4','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-jaraco.text','4','29212.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-jedi','13','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-jinja2-time','2','29149.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-joblib','4','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-json-logger','6','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-json5','3','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-jsonlines','2','28754.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-jsonpatch','2','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-jsonpointer','2','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-jsonschema','3','28585.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-jsonschema-specifications','2','28585.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-keyring','4','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-kiwisolver','15','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-klein','3','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-klepto','2','31985.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-knowit','2','28585.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-kodipydent','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-kombu','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-langcodes','3','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-language-data','11','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-lap','8','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-lazr.config','2','29822.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-lazr.delegates','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-lazy-object-proxy','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-lazy_loader','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ldap','5','29101.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-levenshtein','16','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-lhafile','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-libagent','2','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-libretranslatepy','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-librosa','5','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-libsass','118','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-libtmux','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-libusb1','1','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-libvcs','2','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-llvmlite','9','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-lockfile','1','26983.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-logbook','6','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-loguru','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-lsp-black','2','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-lsp-jsonrpc','4','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-lsp-server','5','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-lxml','101','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-lz4','2','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-mailman','12','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-marisa-trie','16','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-markdown-it-py','3','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-markdown2','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-mat2','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-matplotlib-inline','2','29149.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-matrix-nio','4','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-maturin','57','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-mccabe','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-mdurl','2','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-mediafile','2','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-meson-opt','14','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-mesonpy','3','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-mir_eval','3','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-mistune','6','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-mnemonic','2','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-monotonic','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-more-itertools','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-mpi4py','56','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-mpmath','6','27918.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-msal','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-msrplib','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-multidict','4','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-multivolumefile','1','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-narwhals','4','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-natsort','2','26869','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-neovim','2','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-nest_asyncio','1','29527.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-netaddr','8','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-netifaces','3','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-networkx','7','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-new-rtorrent','1','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-nltk','10','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-nuitka','20','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-numba','24','28793.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-numpydoc','3','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-nxt-python','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-oauth2client','2','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-obspy','52','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-odfdo','9','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-olm','25','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-opencv','325','27786.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-openpyxl','3','27947.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-openstep-plist','10','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ordered-set','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-orjson','13','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-oscrypto','3','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-otr','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-outcome','2','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-overrides','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-packaging-opt','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-paginate','2','29822.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-paginate_sqlalchemy','1','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-paho-mqtt','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pandas','133','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pandocfilters','1','29735.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pantalaimon','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-paramiko','4','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-parso','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-passlib','4','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-paste','2','26426.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pastedeploy','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-patch-ng','1','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pathlib2','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pathspec','2','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-patsy','2','29149.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pdfCropMargins','4','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pdfminer.six','8','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pdfrw','2','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pdm','3','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pdm-backend','3','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pdm-build-locked','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pdm-pep517','3','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-peewee','15','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-persistent','4','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pexpect','1','26869','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pgpy','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pillow-opt','24','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pint','5','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pipx','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pkgconfig','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-platformdirs','1','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-playsound','1','26983.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-plexapi','3','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-plotly','55','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-plucky','1','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pluggy','2','27932.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ply','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pmw','3','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-poetry-core','4','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-poetry-dynamic-versioning','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pooch','4','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-portend','3','28428.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pox','1','31985.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-prefixed','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-presets','2','29194.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-profilehooks','2','29457.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-progress','1','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-prometheus_client','6','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-prompt_toolkit','5','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-prompt_toolkit38','6','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-propcache','5','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-psautohint','5','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-psutil','5','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-psycopg2','7','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-psycopg3','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ptyprocess','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-publicsuffix2','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pubsub','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pulsectl','2','28428.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pulsectl-asyncio','3','29301','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pure_eval','2','29345.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-puremagic','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-putio.py','2','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-py','1','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-py-cpuinfo','3','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-py7zr','3','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyasn1','3','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyasn1-modules','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pybcj','2','29101.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pybluez','3','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pycdio','6','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyclipper','8','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pycrypto','18','32495.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pycryptodome','23','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pycryptodomex','22','27932.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pydantic','4','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pydantic-core','161','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pydbus','3','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pydispatcher','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pydocstyle','2','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pydub','2','29954.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pydyf','2','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyflakes','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pygame','69','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyicu','51','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyinotify','2','29301','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyjwt','3','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pykakasi','8','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pykeepass','5','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pylint','4','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pylint-venv','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyls-spyder','2','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pylyrics','2','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pymupdf_fonts','4','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pynacl','27','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pynma','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pynzb','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyogrio','27','28044.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyosmium','48','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pypandoc','2','28663.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyparted','3','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pypdf','7','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyphen','3','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pypng','3','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyppmd','3','29101.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pypresence','2','26869','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyproject-api','2','28700.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyproject-hooks','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyproject-metadata','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyrsistent','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pysdl2','4','28428.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyspf','2','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pysrt','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pystache','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pysubs2','3','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pytest','4','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pytest-cov','1','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pytest-runner','3','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pytest-timeout','2','29145.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-python-editor','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pythondialog','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pythran','16','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pytoolconfig','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pytube','2','29272.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pytz','3','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyudev','2','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyusb','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyzmq','29','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyzstd','22','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-qbittorrent-api','3','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-qstylizer','2','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-qtawesome','3','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-questionary','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-quimb','11','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-rapidfuzz','101','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-rebulk','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-recurring-ical-events','3','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-referencing','3','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-regex','12','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-reportlab','6','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-repoze.lru','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-requests-cache','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-requests-unixsocket','4','28754.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-resolvelib','3','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-rfc3339_validator','1','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-rfc3986','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-rfc3986_validator','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-rich','3','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-rope','6','28692.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-routes','2','26983.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-rpds-py','11','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-rsa','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-rtree','3','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-sabctools','12','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-schedule','1','29272.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-scikit-build-core','4','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-seaborn','7','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-semantic-version','2','29470.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-semver','2','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-service-identity','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-setproctitle','2','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-setuptools-git-ls-files','1','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-setuptools-git-versioning','3','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-setuptools-opt','9','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-setuptools-rust-opt','3','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-setuptools-scm-opt','4','28705.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-setuptools_autover','1','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-setuptools_scm_git_archive','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-sge','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-sgmllib3k','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-shamir-mnemonic','2','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-shellingham','2','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-shtab','4','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-simple-term-menu','1','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-sipclients3','2','28767.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-sipsimple','159','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-slacklog','1','28705.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-slip10','1','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-slugify','2','28054.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-smart_open','4','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-smmap2','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-sniffio','3','29101.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-socks','3','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-solidpython','6','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-sortedcontainers','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-soundfile','2','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-soupsieve','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-soxr','8','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-sqlobject','2','28692.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-srt','1','30636.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-stack_data','2','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-stagger','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-statsmodels','268','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-stevedore','2','28705.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-stone','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-svglib','2','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-sympy','34','27918.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-sysrsync','1','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-tabulate','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-tasklib','3','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-tekore','3','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-teletype','3','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-tempora','4','28428.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-tenacity','2','31985.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-terminado','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-testpath','2','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-text-unidecode','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-textdistance','1','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-texttable','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-tgext.routes','2','29101.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-threadpoolctl','2','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-three-merge','1','27947.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-timeago','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-tinycss2','2','31994.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-tinydb','2','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-tmdbsimple','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-toml','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-tomli-opt','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-tomli_w','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-tomlkit','3','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-toolz','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-tornado','4','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-tox','3','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-trakit','2','29436.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-transaction','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-treq','2','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-trezor','4','29212.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-trezor-agent','2','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-trio','5','27969','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-trove-classifiers','3','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-tubes','2','28692.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-turbogears2','2','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-tus.py','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-tvdbsimple','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-twisted','20','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-txaio','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-typeguard','3','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-types-python-dateutil','3','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-typing-extensions','2','28705.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-typing-inspection','3','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-typogrify','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ufo2ft','3','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ufoProcessor','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ufolib2','3','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ufonormalizer','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-uharfbuzz','41','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ujson','9','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-unearth','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-unidiff','2','29194.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-unpaddedbase64','2','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-uri_template','4','29101.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-uritools','1','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-url-normalize','3','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-urlobject','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-userpath','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-uth','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-uv_build','53','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-validators','3','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-versioneer','3','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-vharfbuzz','1','29149.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-videohash','1','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-vine','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-virtualenv','4','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-voluptuous','2','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-waitress','2','29822.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-watchdog','2','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-wcwidth','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-weasyprint','3','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-webcolors','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-webencodings','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-webhelpers2','1','26983.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-webob','1','29822.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-webrtcvad','4','29954.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-websocket-client','2','28044.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-websockets','4','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-whatthepatch','4','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-wheel','1','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-whoosh','3','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-wrapt','2','29272.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-wurlitzer','1','28052.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-x-wr-timezone','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-xapp','3','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-xcaplib','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-xcffib','5','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-xdg-base-dirs','2','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-xkeysnail','2','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-xlib','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-xmldiff','2','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-xmltodict','2','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-yara','3','28315.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-yarl','6','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-zdaemon','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-zeroconf','89','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-zodbpickle','5','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-zopfli','10','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-zstandard','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-zxcvbn','2','29246.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pytsk','39','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pytz-deprecation-shim','1','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pytzdata','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyusb','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pywal','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/queuelib','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ramlfications','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/readme_renderer','2','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/recommonmark','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/rectangle-packer','5','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/reedsolo','25','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/relatorio','2','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/rencode','7','30582.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/requests-futures','1','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/requests-oauthlib','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/requests-toolbelt','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/responder','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/rfc6555','2','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/rope','3','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/roundrobin','2','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/rst','1','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/rst-linker','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/rst2ansi','1','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/rst2pdf','3','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ruff-bin','11','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ruffus','4','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/s3cmd','1','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/s3transfer','3','30162.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/schema','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/scikit-learn','111','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/scikit-umfpack','3','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/secretstorage','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/selenium','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/send2trash','2','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/setuptools-scm-git-archive','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/simplejson','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/singledispatch','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sk1libs','15','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/snowballstemmer','3','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/snuggs','2','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/soundcloud','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sounddevice','2','28585.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/speaklater','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sphinx-autobuild','2','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sphinx-last-updated-by-git','2','30260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sphinx-rtd-theme','3','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sphinxcontrib-applehelp','2','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sphinxcontrib-asyncio','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sphinxcontrib-devhelp','2','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sphinxcontrib-fulltoc','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sphinxcontrib-htmlhelp','2','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sphinxcontrib-jquery','2','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sphinxcontrib-jsmath','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sphinxcontrib-programoutput','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sphinxcontrib-qthelp','2','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sphinxcontrib-serializinghtml','2','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sphinxcontrib-websupport','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sql','1','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sqlalchemy-migrate','3','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sqlparse','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/statsmodels','24','29602.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/stem','3','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/stevedore','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/stopit','1','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/subliminal','4','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/subprocess32','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/subuser','2','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/subvertpy','7','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/suds-jurko','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/tagpy','21','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/taskw','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/tempita','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/termcolor','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/terminaltables','1','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/testscenarios','1','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/testtools','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/thonny','5','29167.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/threadpool','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/thunarx-python','4','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/timehash','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/tornado_systemd','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/tqdm','3','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/traceback2','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/traitlets','3','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/trollius','3','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ttfautohint-py','2','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ttkbootstrap','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/tweepy','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ty-bin','9','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/typed_ast','11','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/tzdata','5','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/tzlocal','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/unrardll','2','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/uv-bin','23','29149.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/vanguards','1','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/vcversioner','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/veryprettytable','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/vinetto','1','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/virtualenv','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/virtualenv-clone','2','29272.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/vobject','2','29527.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/wcwidth','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/webassets','1','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/werkzeug','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/werkzeug-legacy1','3','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/winpdb','1','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/workbook','1','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/wsgiref','1','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/xattr','2','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/xe','1','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/xlutils','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/xopen','1','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/xtermcolor','1','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/yapf','4','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/zeep','2','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/3llo','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/chronic','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/gem2tgz','1','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/ncurses-ruby','15','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/passenger','183','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/pastel','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rbenv','2','28832.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/ruby-build','11','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/ruby-native-package-installer','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-activesupport','4','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-ast','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-atk','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-cairo','5','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-cairo-gobject','3','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-coderay','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-concurrent-ruby','4','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-cool.io','5','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-daemons','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-diff-lcs','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-fastercsv','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-file-tail','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-fluentd','3','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-gdk3','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-gdk_pixbuf2','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-gio2','4','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-glib2','7','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-gobject-introspection','4','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-gosu','21','31408','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-gtk2','18','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-gtk3','6','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-highline','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-hpricot','12','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-http_parser.rb','3','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-httparty','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-i18n','3','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-lolcat','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-mail','13','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-manpages','1','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-matrix','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-method_source','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-mime-types','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-mime-types-data','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-mini_mime','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-msgpack','4','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-multi_json','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-multi_xml','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-mustache','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-native-package-installer','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-net-scp','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-net-sftp','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-net-ssh','3','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-optimist','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-paint','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-pango','4','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-parallel','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-parser','38','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-pkg-config','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-pry','3','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-pygments','3','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-rack','3','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-rdiscount','4','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-red-colors','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-ronn','3','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-rspec','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-rspec-core','3','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-rspec-expectations','3','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-rspec-mocks','3','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-rspec-support','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-ruby2ruby','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-ruby_parser','5','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-sass','6','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-sexp_processor','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-sigdump','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-slop3','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-sourcify','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-string-scrub','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-sync','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-term-ansicolor','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-terminal-table','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-thread_safe','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-tins','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-trogdoro-el4r','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-trollop','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-tzinfo','3','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-tzinfo-data','3','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-xiki','5','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-yajl-ruby','3','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygems-update','6','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/sequel','10','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/sequel4','12','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/sqlite3','5','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/tty-color','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/tty-cursor','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/tty-prompt','8','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/tty-reader','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/tty-screen','2','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/wisper','1','29086.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/7zip','24','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/86box','93','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/AnonymousPro','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/Attic','10','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/B-em','27','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/CNS11643-kai-font','73','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/CNS11643-sung-font','74','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/CPU-X','14','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/CanAce','4','28763.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/DisplayCAL','8','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/EpsonTMXCUPS','3','35130.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/FontAwesome','26','27910.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/Fonts-GFS','6','28727.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/Fonts-GFS-Majuscule','1','28727.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/FreeFileSync','16','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/FunkLoad','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/Gentium','4','28084.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/GentiumPlus','3','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/I-Nex','8','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/Iosevka','57','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/Iosevka-aile','49','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/Iosevka-core','26','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/Iosevka-etoile','50','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/Iosevka-slab','60','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/JetBrainsMono','3','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/Lmod','8','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/Microsoft-Fonts','131','30607.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/MySQL-Sandbox','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/OpenSnitch','22','29212.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/PeaZip','19','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/PrintNode','77','28767.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ProFont','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/PySixel','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/SavvyCAN','50','28731.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/Sl','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/Solaar','3','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/TLP','4','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/TermRecord','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/Type1_to_OTF','16','29527.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/UEFITool','10','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/WoeUSB','1','29212.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ZoneMinder','127','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/a2tools','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/acefile','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ack','1','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/acopw-cli','8','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/acpi','2','29696.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/acpi_call','3','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/acpitool','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/adbfs-rootless','2','31408','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/adobe-source-code-pro-font','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/adobe-source-han-sans-fonts','58','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/adobe-source-sans-pro-font','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/adobe-source-serif-pro-font','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/aespipe','5','29301','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/afdko','71','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/afio','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/afpfs-ng','14','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/afuse','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/agedu','5','28084.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/aha','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/aide','7','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/alacritty','69','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/alftools','2','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/alien','2','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/amanda','54','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ancient','20','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/android-file-transfer','17','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/android-udev-rules','1','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/andromeda','29','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ansible','143','35130.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ansible-core','8','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ansifilter','6','29301','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/anything-sync-daemon','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/apache-activemq','25','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/apache-cassandra','22','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/apache-tomcat','11','27129','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/apachetop','6','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/apcupsd','9','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/apg','6','29602.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/apparmor','51','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/apple-emoji','17','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/apple-fonts-Garamond','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/apple-fonts-NY','3','28315.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/apple-fonts-SF','88','28315.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/apptainer','147','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/arc','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/arch-install-scripts','2','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/archivemount','3','28084.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/archmage','2','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/argyllcms','145','29177.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/arj','8','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/arqiver','9','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/artwiz-aleczapka','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/artwiz-cursor','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/asbt','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/asus-fanmode','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/atarisio','5','28084.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/aterm','10','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/atool','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/atop','2','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/audit','35','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/augeas','31','26983.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/autoarchive','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/autocutsel','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/autojump','1','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/avfs','11','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/b2','242','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/babeltrace','18','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/backintime','3','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bacula','36','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bacula-client','23','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/barman','3','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bat','92','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bbf','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bbqsql','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bcache-tools','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/beebem','8','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bees','14','29167.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/betty','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bicon','13','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/biew','4','29735.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bin2iso','2','30582.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bindfs','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/binfmt-support','12','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bit-babbler','11','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bitmapfont2ttf','3','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bitrot','3','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/blake2','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bleachbit','8','31994.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/blktrace','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/blocaled','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/blocks','2','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/blueprint-compiler','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bochs','33','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bogofilter','8','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bonnie++','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bootchart','5','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/borgbackup','41','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/borgmatic','5','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bottom','86','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/brasero','53','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/brave-browser-the-latest','1','28044.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/brightnessctl','1','27768.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/brlaser','5','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/brother-brscan4','2','28305.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/brother-dcp8410-cups','1','28811.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/brother-dcp8410-lpr','2','28811.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/brother-hll2400d-cups','1','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/btop','21','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/btrbk','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bubblewrap','3','27778.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/buildah','25','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bulk_extractor','45','29436.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/burp','7','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/busybox','32','31172.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/butterfly','2','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/byobu','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bzip3','8','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/c-icap','8','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/c-lcrypt','7','30607.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cabextract','3','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/can-utils','12','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cantarell-fonts','9','30636.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/capabilities','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/caprice32','5','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/capstone','12','29527.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/catatonit','10','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cbmbasic','10','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cbmfs','2','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cc1541','3','28052.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ccat','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ccd2cue','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ccd2iso','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ccid','6','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ccrypt','11','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cdemu-client','1','27834.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cdemu-daemon','3','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cdrkit','9','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cdw','8','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cdwrite','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cfengine','44','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cfpeek','5','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/chipsec','12','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/chkrootkit','2','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/chkxex','1','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/chr','15','27969','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/chronograf','32','29177.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/chrpath','3','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ciderpress-tools','6','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ciopfs','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ck4up','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ckb-next','27','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/clamav','110','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/clamsmtp','11','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/clamtk','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/clifm','15','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/clinfo','2','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cluster-glue','26','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cndrvcups-capt','53','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cndrvcups-common','14','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cnijfilter2','67','27864','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cntlm','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/collectd','91','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/colord','20','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/colord-gtk','8','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/colorize','1','28945.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/colortail','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/commons-daemon','3','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/compsize','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/conky','43','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/conmon','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/consolation','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/conspy','5','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/containerd','220','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/convmv','2','27129','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cool-retro-term','6','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/coolkey','9','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/corectrl','66','31985.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/corosync','23','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/courier-prime','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/courier-screenplay','1','26426.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cowsql','18','29149.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cpmtools','4','31408','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cpuid','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cpuid2cpuflags','3','31994.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cpuinfo','2','28585.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cpulimit','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/crc32_simple','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/createrepo_c','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/crelay','1','29436.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/crikey','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/crmsh','7','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cronie','4','28945.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/crunch','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/csh','5','29167.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/csv2sql','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/csvutils','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/culmus','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cups-bjnp','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/curlftpfs','12','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cutecom','5','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cve-check-tool','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cw','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cwtex-q-fonts-TTFs','37','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cyberjack','9','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/d-feet','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/d52','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/daa2iso','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/daemonize','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/daemontools','16','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/daemontools-bin','6','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dar','58','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dateutils','11','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/davfs2','6','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ddcutil','20','27768.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ddg','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ddrescueview','5','27918.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/debootstrap','4','28754.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ded','6','27864','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dell_bios_fan_control','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/depfinder','2','28585.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/detox','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dfc','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dfm','3','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dfu-util','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dget','2','29728','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dieharder','28','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dina-font','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/direvent','5','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/disk-manager','5','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/diskscan','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/disksearch','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dislocker','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/displaylink','4','30459.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/distrobox','4','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/distrobuilder','20','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dlm','5','26426.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/docbook-xml','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/docbook-xml5','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/docbookMathML1mods','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/docker','178','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/docker-buildx','163','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/docker-cli','27','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/docker-compose','42','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/docker-machine','43','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/docker-machine-kvm','35','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/domitian-fonts','1','28727.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dool','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dos33fsprogs','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dosbox','40','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dosbox-dev','52','29194.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dosbox-staging','82','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dosbox-x','215','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dosemu','13','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dosemu-fonts','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/doublecmd-qt5','56','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dracut','9','29212.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/drbd-utils','14','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/driconf','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ds4drv','2','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dse-typewriter-font','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dstat','1','28711','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dtach','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dtrx','3','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dump','8','28811.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dumptorrent','2','28700.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/duperemove','3','29167.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/duplicity','3','28585.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/duply','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dust','41','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dvdisaster','29','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dymo-cups-drivers','5','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dynamips','19','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/eaglemode','53','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/early-ssh','1','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/earlyoom','2','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ecm-tools','1','29735.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ecofont-ttf','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ecryptfs-utils','28','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/edid-decode','10','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/edit','15','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/edk2-aarch64','7','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/edk2-ovmf','6','29735.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/efi-sync','1','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/efiboots','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/efitools','7','26426.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/eibd','11','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/eightyone','8','30607.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/elasticsearch','12','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/eltclsh','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/em','1','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/emelfm2','8','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/encfs','23','27129','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/entr','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/epson-inkjet-printer-escpr','9','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/epson-inkjet-printer-escpr2','16','27834.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/epson-printer-utility','45','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/erofs-utils','11','28767.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/esekeyd','6','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/etckeeper','2','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/etsh','2','28063.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/evdi','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/evdi-kernel','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/eventlog','7','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/evhz','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/evolution-data-server','53','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/evtest','7','29301','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/execline','3','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/exfat-utils','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/exomizer','6','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ext4magic','5','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/extundelete','7','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/eza','64','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/f3','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/facter','61','29167.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fakechroot','13','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fakeroot','9','28044.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fantasque-sans-mono','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/far2l','102','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fasd','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fastfetch','23','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fatrace','2','29301','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fatsort','1','30636.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fbterm','5','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fcron','7','27918.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fd','97','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fdclone','8','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fdupes','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/felix','47','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fence-agents','19','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fff','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ffproxy','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/file-roller','17','28428.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/file-roller-gtk2','15','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/filebeat-oss','29','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fileschanged','11','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/findcmd','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/findpkg','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fio','12','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fira-code-fonts','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fira-fonts','16','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/firejail','8','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/firetools','10','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/firmware-gobi-2000','9','28811.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fish','61','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/flashcards','2','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/flatpak-xdg-utils','3','27947.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/flexbackup','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fnt','1','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/font-manager','26','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-arabic-extra','20','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-arkandis','6','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-arkpandora','1','28727.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-armenian-extra','12','29954.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-atkinson-hyperlegible','4','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-bengali-extra','5','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-cascadia-code','43','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-devanagari-extra','34','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-ethiopic-extra','7','27129','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-farsi-extra','8','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-georgian-extra','3','29954.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-gujarati-extra','5','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-gurmukhi-extra','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-gurmukhi-nonlibre','3','29246.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-indic-archaic','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-indic-ne','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-indic-nonlibre','3','29246.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-indic-other','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-intel','3','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-japanese-extra-I','73','28727.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-japanese-extra-II','76','28727.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-japanese-extra-III','133','28727.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-japanese-extra-IV','46','28727.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-japanese-extra-V','59','28727.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-japanese-extra-VI','82','28727.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-japanese-extra-VII','62','28727.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-japanese-extra-VIII','52','28727.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-kannada-extra','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-khmer-extra','8','31408','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-kopub','51','27932.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-lao-extra','2','31408','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-malayalam-extra','11','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-mgenplus','71','28727.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-mozilla','4','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-mpluscode','2','28718.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-myanmar-extra','6','31408','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-nastaliq','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-oriya-extra','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-redhat','5','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-sinhala-extra','5','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-tamil-bharathi','12','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-tamil-libre','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-tamil-nonlibre','3','29246.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-tamil-tva','9','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-telugu-extra','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-thai-extra','21','29954.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-tibetan-extra','6','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/foo2zjs','8','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/foomatic-db','46','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/foomatic-db-engine','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/foremost','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/forkstat','2','31985.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fountain','8','29301','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fpm2','8','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fprintd','5','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fr','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/franny','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/freedup','6','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/freefont','5','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/freeipmi','55','28811.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/freeswitch','685','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fsarchiver','5','30162.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fscrypt','19','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fslint','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fsv','10','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fsviewer','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fsvs','8','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fswatch','14','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ft2demos','11','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fuse-archive','5','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fuse-emulator-utils','6','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fuse-exfat','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fuse-overlayfs','7','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fuse-zip','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fuseiso','6','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fwupd','121','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fxload','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fzf','25','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fzf-bin','3','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fzy','1','30607.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/game-devices-udev','1','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gamescope','12','29177.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gammu','15','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gcdemu','1','27834.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gcn64tools','5','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gconf-editor','11','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gdmap','5','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gdu','5','28945.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/geckodriver','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gencfs','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/genext2fs','6','28711','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/genimage','5','29436.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gentoo','7','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/geoclue','10','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/geoclue2','7','28305.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gigolo','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gimp-help','1109','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gitolite','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gksu','6','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/glyptodon','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gnome-color-manager','10','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gnome-commander','40','27129','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gnome-disk-utility','10','29149.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gnome-online-accounts','26','29822.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gnome-pkgtool','9','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gnome-terminal','22','29272.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gnome-vfs','22','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gnomint','14','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gnsu','2','27834.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gnu-unifont','21','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gnupg-pkcs11-scd','8','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/go-fonts','3','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/go-mtree','14','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/goaccess','7','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gohu-font','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/google-chrome-the-latest','1','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/google-droid-fonts','6','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/google-fonts','501','27932.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gpart','5','28452.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gphotofs','5','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gpustat','2','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/graphterm','6','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/graveman','7','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/graylog-sidecar','9','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/greetd','18','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/grub2-splashimages','5','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/grub2-theme-mint','1','28052.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/grub_legacy','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gslapt','10','31985.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gsmartcontrol','19','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gsplus','10','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gtk-vnc','12','28305.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gtklp','19','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/guake','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/guefi','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gyre-fonts','5','28727.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hal-flash','10','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hardinfo','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hardinfo2','11','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hashID','1','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hashcat','81','28044.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hdapsd','4','28044.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hddtemp','6','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/heartbeat','29','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/heatshrink','2','29101.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/heirloom-doctools','35','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/heirloom-ed','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/heirloom-pax','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/heirloom-sh','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/helvum','49','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hercules','128','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/herculesstudio','16','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hexcompare','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hexec','7','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hfsprogs','3','29167.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hime','14','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hivex','15','29735.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hntool','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hp-wmi-sensors','3','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hplip-plugin','5','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hstr','6','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hungrycat','2','28700.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hw-probe','1','31738.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hwinfo','412','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hwloc','25','28084.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hyperfine','35','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/i2c-tools','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/i8kutils','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ibam','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ibus-typing-booster','34','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/icingaweb2','17','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/idle3-tools','1','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ifuse','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ima-evm-utils','12','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/image-analyzer','1','28692.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/imwheel','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/incron','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/incus','149','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/incus-ui','38','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/influxdb','14','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/info2man','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ink','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/inotify-info','3','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/intel-microcode','8','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/intelmas','19','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/interceptty','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/intlfonts','32','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ioninja','12','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ioping','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/iotop','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/iotop-c','3','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ipmitool','17','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/irqbalance','11','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/iscan','16','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/iscan-data','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/iselect','5','29690.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/isextract','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/isight-firmware-tools','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/isomaster','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/isomd5sum','2','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/iucode_tool','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/jdiskreport','2','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/jdupes','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/jenkins','43','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/jnv-bin','3','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/john','17','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/joyce','17','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/jpnevulator','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/jq','12','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/jshon','1','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/jstest-gtk','7','28428.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/k3s','28','28428.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kafka','14','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kapacitor','45','29177.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kbfs','100','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kc','3','29167.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kegs','13','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kexec-tools','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/keyd','4','29212.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/keylaunch','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kfc','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kio-fuse','8','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kitty','61','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kiwix-tools','4','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kiwix-tools-bin','16','29954.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/klish','8','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kmscon','11','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ksar','3','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ksh-openbsd','2','29167.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ktls-utils','3','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ktsuss','5','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kubectx','13','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kurinto-fonts','988','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kvantum-qt5','22','26584.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kyotocabinet','29','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/laptop-mode-tools','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lbench','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lbrate','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lcdf-typetools','16','27578.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lddsafe','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/letsencrypt','11','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lf','34','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/libcloudproviders','5','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/libertinus-fonts','5','27932.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/libnatspec','11','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/libratbag','8','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/libre-franklin-fonts','3','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/libsignal-protocol-c','3','29527.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/libtrash','12','28794.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/libwmf-type1-fonts','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lightdm','14','29301','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lightdm-settings','3','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lilyterm','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/limine','6','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/linguistics-pro-fonts','5','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/linkchecker','2','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/linux-libertine-fonts','5','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/linux-vt-setcolors','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/linuxconsoletools','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lirc','30','30636.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/listpkg','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/literata-fonts','8','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lnav','167','28786.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/local-persist','13','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/locust','3','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/loggedfs','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/logiops','30','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/logkeys','9','26502.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/logpp','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/logwatch','3','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/loksh','4','29149.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lolcat','1','29735.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lomoco','5','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lsb-release','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lshw','11','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lssecret','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lsyncd','3','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/luckybackup','14','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lv','4','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lvm2-cluster','17','26426.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lvm2-lockd','17','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxcfs','6','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxdm','8','29293.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxgw-wenkai-fonts','69','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxgw-wenkai-gb-fonts','68','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxgw-wenkai-tc-fonts','42','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxinput','12','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxqt-about','7','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxqt-admin','11','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxqt-archiver','12','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxqt-config','36','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxqt-globalkeys','20','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxqt-openssh-askpass','6','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxqt-policykit','8','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxqt-powermanagement','17','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxqt-session','18','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxrandr','12','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxsession','12','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxsplit','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxtask','13','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxterminal','9','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lynis','2','29149.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lziprecover','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lzop','6','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/macemu','50','30582.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/macutils','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/magiic','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mailfromd','12','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mailutils','90','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/maintboot','1','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/makefile2graph','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/makepasswd','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/makepp','5','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/makeself','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/man-pages-es','8','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/man-pages-fr','15','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/man-pages-ja','23','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/man-pages-ru','12','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mandoc','10','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/marsond','2','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mathML2dtd','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/maxcso','4','29735.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mbootpack','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mbpfan','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mbuffer','9','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mcrypt','7','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/md5deep','8','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mdcat-bin','5','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mdf2iso','4','28452.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mediaextract','3','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/megacli','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/memkind','75','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/memory-editor','18','31738.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/memtest86','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/memtest86+','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/memtester','2','31738.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/memuse','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mergerfs','14','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mhddfs','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/microcode_ctl','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/midisport-firmware','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mingetty','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mksh','11','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mlterm','38','31985.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mmv','14','29690.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/modules','20','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mongo-tools','82','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mongodb','86','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/monit','19','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/monitoring-plugins','45','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/monitorix','2','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/montagu-slab-fonts','3','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/motion','14','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mount-s3','15','30162.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mount-zip','4','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mpack','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mpich','130','26911.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mpollux-digisign-client','6','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mrxvt','9','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ms-sys','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/msitools','10','28811.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/msr-tools','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mtftar','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mtm','2','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mucommander','36','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/multibootusb','7','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/multiload-ng','11','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/multipath-tools','6','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/multitail','3','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/munin-node','45','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/musl','8','29527.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nagios-plugins','50','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nagvis','3','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nano-syntax-highlighting','2','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nanum-coding-fonts-ttf','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nanum-fonts-ttf','12','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nbench','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ncdu','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nemo','23','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nemo-fileroller','3','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nerdctl','34','28278.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/netatop','4','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/netdata','484','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/newLd','38','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/newrelic-sysmond','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nilfs-utils','10','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nix','227','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nkf','2','28063.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nmon','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nmtree','7','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nnn','4','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nologind','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nomarch','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/noto-emoji','65','29301','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/noto-extra-ttf','13','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nrg2cue','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nsca-ng','7','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nss_ldap','8','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ntopng','108','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/numactl','5','28811.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/numlockx','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nut','16','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nvidia-driver','172','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nvidia-fabricmanager','8','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nvidia-firmware','5','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nvidia-kernel','98','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nvidia-legacy340-driver','76','28044.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nvidia-legacy340-kernel','31','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nvidia-legacy390-driver','80','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nvidia-legacy390-kernel','30','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nvidia-legacy470-driver','148','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nvidia-legacy470-kernel','50','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nvidia-legacy580-driver','180','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nvidia-legacy580-kernel','121','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nvtop','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nwipe','8','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ocfs2-tools','22','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ohmyalias','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ohsnap','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/oils','30','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/oksh','4','27864','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/oldschool-pc-fonts','13','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/omnidb-app','82','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/onefetch','86','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/onerng','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/opam','17','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/open-vm-tools','74','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/openGLRefToMan','293','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/openct','7','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/opendoas','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/openmpi','335','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/openrazer-daemon','4','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/openrazer-kernel','6','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/openrc','12','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/openrc-services','8','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/openrc-settingsd','3','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/openrgb','240','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/opensc','23','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/openscap','48','28084.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/openstego','2','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/openzfs','148','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/oracle-instantclient-basic','61','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/oracle-instantclient-devel','1','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/osinfo-db','3','28428.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/osinfo-db-tools','6','28428.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/osquery-bin','39','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ossec-agent','6','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ossec-local','9','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ossec-server','8','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ostree','15','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/otf-hermit','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/otf-ibm-plex','71','28727.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/otf2bdf','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/overpass-desktop-fonts','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/overpass-webfonts','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/oxker-bin','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/p7zip','54','27129','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pacemaker','103','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pam-gnupg','10','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pam_ssh','7','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pam_ssh_agent_auth','16','28833.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/partclone','30','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/partimage','18','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pass-import','2','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pass-otp','1','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/passwdqc','4','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/password-store','1','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/passwordsafe','48','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/patool','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pause','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pax','7','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pax-utils','4','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/paxctl','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pbzip2','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pbzx','2','27834.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pce','8','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pcem','27','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pcem-roms','6','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pcf2bdf','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pcmanfm','9','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pcmanfm-qt','17','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pcp','555','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pcsc-lite','7','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pcsc-perl','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pcsc-tools','3','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pdfgrep','4','28833.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pdksh','8','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pdsh','11','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/peazip','10','28811.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/percol','2','29527.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/percona-toolkit','13','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pev','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pgbadger','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pgsanity','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ph','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/phoronix-test-suite','28','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/phpsysinfo','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/physlock','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/picocom','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pigz','3','29690.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pipemeter','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/piper','3','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pipes.sh','1','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pis','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pixma','71','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pkg','3','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/plan9port','324','29457.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pledge','2','29527.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/plocate','7','30607.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/plymouth','26','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/plymouth-slackware-theme','2','29301','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pmdk','31','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pmount','5','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/podman','133','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/polychromatic','4','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/polyglotman','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pommed-light','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/porg','10','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/posh','9','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/posixovl','5','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/postgresql','90','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/postgresql15','93','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/postgresql16','103','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/postgresql17','185','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/postgresql18','177','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/powerkit','6','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/powerline-fonts','3','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/powerline-status','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/powernowd','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/powerpanel','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/powershell','70','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/prelink','8','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/preload','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/procinfo-ng','5','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/procs','171','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/profile-cleaner','1','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/profile-sync-daemon','2','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/progress','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/prometheus','64','28529.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/psensor','7','27969','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pspg','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pstate-frequency','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/psuinfo','2','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ptop','1','29246.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pucrunch','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pv','7','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pw','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pwgen','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pwman','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pwsafe','9','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pyarchey','2','29246.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pycp','2','28754.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/python-urwid','4','29493.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/python3-docker-compose','3','27864','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/python3-nvidia-ml-py','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pytrash','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qControlCenter','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qdirstat','32','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qemu','640','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qemu-guest-agent','18','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qemu-static','534','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qjoypad','8','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qman','4','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qps','17','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qstardict','11','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qterminal','11','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qtfm','17','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qtgzmanager','12','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qtxdg-tools','6','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/quake-par','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/quivira-font','2','27834.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/quotatool','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qxkb','7','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rEFInd','17','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rabbitmq-server','152','26673.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/radeon-profile','16','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/radeontop','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/raft','19','29149.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ranger','3','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rar','2','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rar2fs','17','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rasdaemon','13','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ratmen','2','28754.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/razerCommander','3','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/razergenie','16','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rdiff-backup','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rdup','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/read-edid','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/readpe','4','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/realtek-bt','3','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rear','10','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/recoverdm','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/recoverjpeg','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/redditsans','5','32580.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/redict','53','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/redis','58','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/redis-py','5','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/refind-bin','3','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/regionset','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/reiser4progs','21','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rename','2','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rename.pl','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/renameutils','17','28833.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/repeat','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/reprepro','8','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/reptyr','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/resource-agents','15','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/restic','12','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rewise','3','30162.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rfdump','6','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rhash','11','31033.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rhvoice','156','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ripgrep','22','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rkhunter','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rmw','4','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rndaddentropy','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rng-tools','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/routersploit','3','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rover','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rox-filer','8','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/roxterm','6','27969','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rsyslog','42','30096.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rtcqs','3','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rtirq','1','28203.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rtkit','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/run-one','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/runc','14','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/runit','14','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/runit-services','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rush','13','30636.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rxvt','7','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/s-tui','2','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/s3fs-fuse','12','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/s3ql','5','28692.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/s6','4','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/s6-linux-init','2','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/s6-rc','4','33119.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/safecopy','4','28452.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sahel-font','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sakura','4','28585.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/saleae-logic','24','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/samhain','12','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/samsung-mfp-drivers','8','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sandbox','22','28452.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sandy','2','28705.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sane-airscan','3','28305.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sanlock','8','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sanoid','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sarasa-gothic','133','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sargon','15','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sbbdep','26','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sbctl','19','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sbo-create','3','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sbo-maintainer-tools','2','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sbokeeper','2','29696.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sbotest','2','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sbotools','4','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sbotools2','3','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sboui','7','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sbozyp','2','29105.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sbpkg','2','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sbsigntools','8','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/scalpel','6','30260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/scangearmp2','9','27864','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/scanmem','11','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/scdoc','2','28187.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/schedtool','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/scrypt','8','27129','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sdate','13','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sdl-jstest','3','28428.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sdl2trs','4','30607.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sdltrs','5','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sdwmbar','1','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/se98-icon-theme','821','28754.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/se98-wm-theme','2','28754.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/seahorse','16','27785','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/secure-delete','2','29954.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sensord','2','27477.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sentimental-skk','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ser2net','5','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/set_rlimits','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/setconf','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sff','2','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/shake','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/signify','3','29167.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/siji-font','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/silk','1','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/skim','49','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/skopeo','38','27918.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slack-osquery','11','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slack-timedate','3','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slack-utils','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slackbootstrap','1','29954.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slackchlog','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slackrepo','3','29436.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slackrepo-hints','22','29436.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slackroll','2','28833.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slackupdr','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slackware-grub2-theme','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slackyd','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slapt-get','9','31985.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slapt-src','7','31985.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slapt-update-service','7','31985.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sleepd','2','28282','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sleuthkit','30','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slim','6','31056.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slkstore','2','27864','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slmenu','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slpkg','5','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/smart','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/smbldap-tools','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/smem','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/smenu','6','28705.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/smokeping','8','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/smpq','3','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/snore','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/socketcand','6','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/socklog','22','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/softhsm','21','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/solarize-12x29-psf','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/spacefm','11','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sparse','6','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/speedcontrol','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/spice-guest-tools','5','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/spice-vdagent','4','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/spkg','24','29194.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/splitjob','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/splix','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/spman','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sqlcipher','34','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sqldiff','37','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/squashfuse','7','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/squidclamav','10','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/srm','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ssdeep','7','29735.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sst','45','29735.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/st','2','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/statifier','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/steam-devices','2','28705.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/steamos-xpad','3','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/steampipe-bin','17','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/steghide','11','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/stegseek','7','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/stm32flash','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/stow','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/stress','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/stress-ng','57','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/stressapptest','10','29272.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/suckless-tools','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sudosh2','8','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sunflower','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/supercat','4','27804','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/supervisor','3','30162.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/swapinzram','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/swatch','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/swtpm','27','29563.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/swun','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/symbola-font-ttf','3','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sysbench','27','29457.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sysdig','160','29212.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/syslog-ng','126','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/system76-io-dkms','1','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/system76-power','30','29207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/systraq','1','29809.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tabbed','1','28705.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tagainijisho','115','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tagtools','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tamsyn-font','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/targetcli-fb','3','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tarlz','2','30587.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tarsnap','18','29809.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tbsm','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tc-play','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/telegraf','75','27969','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/termenu','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/terminalia-fonts','1','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/terminator','5','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/terminology','13','29164.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/termsaver','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/termsyn-font','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/testdisk','15','29293.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tewi-font','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/the_silver_searcher','6','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/thefuck','5','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/thermald','20','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/thin-provisioning-tools','26','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/thinkfan','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ti99sim','21','26426.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ticpp','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tiger','70','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tilda','11','30048.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tilix','46','29452.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/timeshift','13','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tini','2','26444.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tinycdb','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tinyterm','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tio','5','32307.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tkdvd','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tkinfo','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tm','1','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tmpwatch','5','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tmsu-bin','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tmuxp','2','27864','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tokyocabinet','7','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tokyotyrant','5','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tomb','3','31033.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/torrent-file-editor','9','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tp_smapi','3','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tpc','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tpm','2','29735.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tqemu','1','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/trachet','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tracker','12','27580.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tracker-miners','17','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/trackfs','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/trash','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/trash-cli','4','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/trashy','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/triggerhappy','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/trousers','19','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/trrntzip','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/trs80-roms','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ts','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-aller','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-ancient-fonts','16','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-arphic-uming','9','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-babelstone-cjk','25','26426.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-babelstone-latin','1','30162.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-babelstone-tibetan','2','29293.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-bitstream-vera','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-caladea','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-carlito','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-charis-sil','3','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-coda','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-computer-modern','6','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-console-fonts','3','29345.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-crosscorefonts','8','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-dark-courier','1','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-doulos-sil','2','26584.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-genjyuu-gothic','95','28068','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-hanazono','27','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-ibm-plex','94','28727.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-inconsolata','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-ipaex','5','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-lato','4','29448','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-monaco','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-mononoki','11','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-nerd-fonts-symbols','2','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-open-sans','5','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-paratype-pt-astra-fact-fonts','1','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-paratype-pt-fonts','3','27932.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-roboto','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-sakalbharati','1','28093.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-ubuntu-font-family','2','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-x2','14','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-xo-fonts','2','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-zekton','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttmkfdir','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttop','1','29246.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tty2gif','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttygif','1','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttyrec','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tuptime','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/twapake','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/twin','23','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tzleap','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tzupdate','18','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/u3-tool','6','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/uae','45','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ucs-miscfixed-opentype-fonts','2','28428.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/udevil','5','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/udftools','6','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/udiskie','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ufiformat','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/uhd','364','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/uhubctl','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/uif2iso','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/un-fonts-core','8','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unac','11','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unace','5','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unar','10','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unetbootin','16','27129','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unhide','3','28084.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unicode-character-database','31','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unigine-heaven-benchmark','90','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unigine-valley-benchmark','159','28775.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unionfs-fuse','2','28166.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unison','19','27969','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/univga-font','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unp','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unrar','88','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unrar-free','5','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unrtf','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unshield','3','27804','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unshieldv3','3','27804','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/untex','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/upass','2','30260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/update-initrd','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/uptimed','7','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/upx','10','28794.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/usbguard','55','28429.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/usbimager','4','28705.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/usbmon','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/usbredir','3','28428.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/usbrelay','2','29436.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/usbtop','5','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/usbview','5','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/usermin','70','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/v4l2loopback','4','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/v7sh','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vagrant','85','31738.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vagrant-public-key','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/valkey','59','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vbindiff','5','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vcp','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ventoy','7','30561.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/veracrypt','43','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vhba-module','3','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vice','289','28241.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/victor-mono-fonts','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vifm','22','27804','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vim-airline','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vim-airline-themes','3','28044.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vinterm','3','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/virt-manager','8','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/virt-viewer','11','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/virtiofsd','31','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/virtualbox','822','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/virtualbox-addons','129','29293.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/virtualbox-extension-pack','3','29293.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/virtualbox-kernel','8','28615.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/virtualxt','11','29457.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vlock','2','28756.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vmbinstallpkg','1','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vmfs-tools','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vms','1','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/volatility','17','28763.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/volatility3','6','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vtcol','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vwm','6','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wangfonts','111','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/warp-terminal','35','29527.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/watch-fs','2','29301','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/watchdog','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/waydroid','2','27864','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wayland-utils','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/waypipe','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wdfs','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wdiff','15','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/webcore-fonts','5','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/webmin','383','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wezterm','288','30260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/whowatch','6','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wiimms-iso-tools','10','29735.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wine','821','30607.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wine-staging','901','30607.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/winetricks','2','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/winusb','9','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/witr','3','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wmconfig','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wmfsm','6','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wmgtemp','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/worker','61','28818.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wqy-microhei-font-ttf','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wterm','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/x86_energy_perf_policy','6','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xAce','4','28763.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xar','8','30582.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xarchiver','11','29385','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xautolock','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xautomation','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xbattmon','2','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xbindkeys','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xboxdrv-linux','124','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xbrightness','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xcdroast','8','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xchm','15','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xdg-dbus-proxy','3','27778.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xdg-desktop-portal-gnome','5','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xdg-desktop-portal-wlr','3','27768.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xdg-ninja','4','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xdg-utils-cxx','4','27965.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xdiskusage','3','30451.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xen','991','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xen-nox','260','29809.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xf86-video-qxl','8','29273.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xfburn','8','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xfe','34','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xfile','5','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xfs_undelete','2','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xfsudo','1','29457.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xiccd','5','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xidel','6','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xinput_calibrator','9','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xjobs','4','28547.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xldconfig','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xnp2','14','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xonsh','13','28653.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xorgxrdp','17','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xosd','8','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xosview','4','29696.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xpipe','2','29167.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xprintidle','2','28945.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xrdp','59','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xrestop','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xsettingsd','3','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xssstate','1','29167.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xstow','9','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xtrs','3','30607.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xvfb-run','2','27910.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xwiimote','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/yabsm','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/yaft','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/yank','1','29527.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/yash','45','29220.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/yaze-ag','5','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/yeahconsole','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/yelp','9','28881.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/yelp-tools','2','27897.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/yelp-xsl','6','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/yq','29','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/yubico-piv-tool','6','28192.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/z','1','30582.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zarfy','4','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zellij-bin','16','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zfs-auto-snapshot','1','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zim-tools','10','30148.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/znapzend','7','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zookeeper','23','28779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zopfli','2','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zoxide','48','32284.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zpaq','8','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zramen','1','28945.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zsh-completions','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zsh-syntax-highlighting','2','27656.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zuluCrypt','39','28857.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zutty','8','30001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zzzfm','14','27656.9','~');
commit;
