begin transaction;
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/3D-ICE','79','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/AlphaPlot','136','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/CAPS_coevolution','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/DRAMPower','18','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/DRAMSim2','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/DRAMSys','103','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/DRAMsim3','12','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/Data2FCS','1','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/DendroPy','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/EMBASSY','96','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/EMBOSS','238','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/FCSalyzer','2','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/FastTree','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/FigTree','2','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/FreeFem','223','29398.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/Gabedit','26','29349.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/Gblocks','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/GeoGebra','31','29260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/Gridcoin-Research','240','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/HMMER','56','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ITK','259','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/Kst','125','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/NWChem','1802','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/OpenFOAM','5601','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/OpenMM','148','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/OpenVSP','1259','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/PhyML','34','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/PyAMG','19','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/R','335','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/RAxML','38','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/RepeatMasker','1217','29349.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/Shark','32','27019.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/SimEng','29','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/WEVOTE','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/WordNet','39','29349.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ZeGrapher','44','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/abntex','32','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/acl2','323','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/aenig4','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/aeskulap','35','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/aline','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/aliview','2','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/amd','14','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/amplicon','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/amplifx','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/antiprism','50','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/aris','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/arka','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/armadillo','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/arpack','14','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/arpack-ng','23','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/artha','6','29349.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/avl','13','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/avogadroapp','13','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/avogadrolibs','206','28005','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bcftools','19','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bedtools','26','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/biblesync','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bibletime','113','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bibus','6','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/biopython','19','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/boinc','110','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bowtie2-legacy','83','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bpp-core','38','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bpp-phyl','148','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bpp-phyl-omics','9','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bpp-popgen','12','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bpp-seq','38','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bpp-seq-omics','29','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bpp1.9-numcalc','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bpp1.9-phyl','51','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bpp1.9-seq','15','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bpp1.9-utils','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bppsuite','12','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/btf','13','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/bwa','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cadabra2','151','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/calc','27','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/calcoo','1','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/camd','14','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cap3','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cblas','7','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ccolamd','14','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/celestia','68','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/chartgeany','67','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/chemtool','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cholmod','29','29896.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cistrome-CEAS','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cistrome-MACS','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cistrome-conductGO','1','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cistrome-extra','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cistrome-mdseqpos','8','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cld2','17','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/clustalo','25','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/clustalw','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/clustalx','21','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/colamd','14','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/colin','19','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/convertall','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/copasi','42','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cryptominisat','33','28598.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/csparse','18','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cutadapt','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cvc4','305','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/cxsparse','29','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/dakota','743','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/datamash','25','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/diamond','39','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/dimod','184','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ds9','298','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/dwave-cloud-client','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/dwave-greedy','9','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/dwave-hybrid','56','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/dwave-inspector','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/dwave-neal','14','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/dwave-networkx','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/dwave-ocean-sdk','299','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/dwave-preprocessing','24','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/dwave-system','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/dwave-tabu','11','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/dwavebinarycsp','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/easy_spice','6','28171.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ed-v6','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/edsim51di','2','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/elk','87','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/engauge','71','28788.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ent','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ephem','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/eprover','23','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/equalx','14','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ess','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/exonerate','328','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/extrema','103','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/fann','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/fastqc','5','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/fet','179','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/finchtv','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/fityk','36','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/flann','126','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/free42','21','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/g3data','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/galculator','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gaw3','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gchemutils','93','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gcompris-qt','68','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gdstk','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/geda-gaf','48','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gelemental','17','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gem5','2628','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/geneconv','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/genius','37','34729.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/genometools','55','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/genpak','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/geomview','49','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gerbv','28','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/getdp','31','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/getfem','139','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ggobi','33','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ghemical','18','28047','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/glpk','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/glvis','20','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gmsh','546','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gnucap','28','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gp2fasta','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gplates','695','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gpredict','20','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/grace','16','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/grb','3','29355','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gresistor','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gretl','174','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gri','61','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gromacs','165','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/grpn','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gspiceui','22','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gtkwave','22','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gtypist','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/gwyddion','77','27412.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/h5utils','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/harminv','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/hp-prime','28','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/hyphy','66','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/iTest','13','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/io_lib','16','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/irsim','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/isoelectric-point','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/itex2mml','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/jalview','16','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/jellyfish_k-mer','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/jpicedt','6','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/kalign','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/karma','12','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/kissat','4','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/kjv','2','29355','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/klavaro','9','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/klu','16','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/knime','83','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/kraken','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ktikz','35','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/labplot2','194','28747.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ladr','31','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/laghos','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/lalnview','2','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/lammps','523','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/lapack95','74','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/lapacke','50','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/latex2html','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ldl','14','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/libghemical','14','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/louvain-community','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/lpsolve','13','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/mafft','13','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/magic','19','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/mathics','131','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/mathjax2','74','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/mathomatic','4','28552.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/matio','15','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/maxima','154','31218.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/meep','45','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/megax','105','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/meme-db-gomo','223','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/meme-db-motif','38','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/meme-db-tgene','932','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/meme-suite','45','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/mendeleydesktop','42','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/metalwalls','31','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/metis','4','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/mfem','77','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/minorminer','59','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/mkDSSP','19','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/molequeue','19','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/mpb','17','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/mrbayes','26','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/mumsa','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/muscle','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/muscle5','14','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ncbi-blast','30','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ncbi-blast+','1864','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ncbi-rmblastn','13','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/nco','35','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ncview','7','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ngspice','278','28171.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/octave','539','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/openboard','151','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pal2nal','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/paml','29','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pamlX','12','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/papi','29','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pari','47','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pari-elldata','186','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pari-galdata','2','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pari-galpol','134','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pari-nflistdata','12','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pari-seadata','47','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/parmetis','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/patsy','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pcalc','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pcb','37','28047','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/penaltymodel','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/perlprimer','2','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pianobooster','14','28047','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/plasmidomics','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/plus42','22','29535.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pomoxis','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/primer3','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pspp','74','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pulseview','93','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/pysam','182','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/python2-kiwisolver','16','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/python2-scipy','538','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/python3-ase','16','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/python3-scipy','209','29371.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/qalculate-gtk','39','29260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/qcl','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/qhull','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/qrupdate','16','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/qtdmm','18','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/qucs','62','28887.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/qucs-s','144','29131.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/qucsator','204','28887.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ramulator','65','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/rbio','14','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/reduce-algebra','816','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/root','2328','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/rpy2','4','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/sage','11601','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/samtools','14','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/samtools-legacy','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/scidavis','84','31070.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/seaview','4','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/seqkit','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/sequence-analysis','2','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/sigrok-cli','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/sim','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/siril','28','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/smath-studio','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/snpEff','33','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/solfege','15','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/speedcrunch','26','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/spiceopus','9','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/spidey','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/spqr','21','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/squizz','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/staden','104','41948','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/stellarium','298','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/stringtie','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/suitesparse','1045','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/suitesparseconfig','13','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/sundials','15','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/sword','35','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/sword-data-kjv','3','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/t_coffee','18','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/tabix-legacy','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/tiemu','33','32180','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/tilem','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/tilem2','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/tkgate','19','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/topdraw','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/tophat','102','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/treebest-ensembl','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/treerecs','116','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/treeviewx','12','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/trfind','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/trimmomatic','1','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/tuxmath','15','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/tuxtype2','20','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/ucsc-blat','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/umfpack','22','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/units','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/vCAPS_coevolution','14','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/vcftools','21','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/velocyto','9','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/verilator','169','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/verilog','237','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/veusz','88','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/voro++','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/vul','2','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/wcalc','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/wehi-cellsymphony','2','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/wehi-deid','1','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/wehi-hackit','1','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/wise','16','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/wmcalc','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/wxMaxima','49','31218.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/x48','67','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/x48ng','63','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/xcircuit','9','34610.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/xephem','42','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/xflr5','94','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/xfoil','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/xiphos','20','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/xoscope','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/xschem','33','34729.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/xsimd','16','29371.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/yices2','32','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/z3','390','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/zotero','98','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('academic/zotero-gtk2','44','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/Synkron','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/blugon','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/easystroke','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/eflite','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/espeakup','1','28747.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/flite','34','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/fonts-opendyslexic','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/jpilot','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/mag','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/magnus','2','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/mbrola','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/mbrola-voices','233','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/orca','30','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/pastebinit','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/picospeaker','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/ruvim','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/sct','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/shorturl','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/svkbd','3','32180','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/svox','16','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/termbin','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/unclutter','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/unclutter-xfixes','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/wgetpaste','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/xdotool','3','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/xsct','1','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/xsel','6','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/xwit','1','30733.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/xzoom','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('accessibility/ydotool','2','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/DPF-Plugins','270','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/FAudio','4','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/JDSP4Linux','110','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/Luppp','6','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/MP3Diags','39','31209.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/MusicMixer','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/PAmix','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/SAM','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/SongRec','126','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/SuperCollider','154','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/TiMidity++','22','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/Ymuse','668','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/a2jmidid','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/aacgain','36','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/aacplusenc','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/abcde','1','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/acousticbrainz-client','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/acousticbrainz-gui','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/acousticbrainz-music-extractor','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/acxi','1','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/aeolus','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/aften','5','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/aj-snapshot','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/alsa-plugin-jack','5','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/alsa-tools','50','29349.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/alsacap','1','28082.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/alsaequal','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/alsamixergui','7','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/alsamodularsynth','46','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/amSynth','12','28788.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/amrnb','6','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/amrwb','4','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/apulse','3','35185.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ardour','826','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ario','19','31029.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/asap','31','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/aseqview','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/asunder','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/atunes','17','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/audacious-aac','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/audacious-jack','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/audacity','300','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/audioconvert','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/azr3','7','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/bristol','21','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/butt','10','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/calf','32','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/calf-ladspa','20','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/callaudiod','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/caps','5','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/cardinal','458','30722.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/carla','113','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/cava','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/cd-discid','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/celt','6','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/celt051','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/chuck','73','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/clap','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/clementine','308','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/clockchimes','2','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/clthreads','1','32670.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/clxclient','2','28204.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/cmt','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/cmus','7','28171.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/connie','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/coppwr','112','28063.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/crip','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/cubeb','20','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/darkice','8','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/dcaenc','12','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/dcd','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/deadbeef','89','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/distrho-ports','63','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/drumkv1','14','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/drumstick','27','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/dssi-vst','9','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/easymp3gain','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/eawpats','15','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ecasound','43','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/exhale','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/eyeD3','2','34287.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ezstream','8','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/fIcy','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/faac','9','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/faad2','10','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/fabla','5','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/famitracker-qt','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/faust','14','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/fdk-aac','44','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ff8dls','20','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/flac-opt','19','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/flac123','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/flacon','42','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/fluid-soundfont','78','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/fluidsynth-dssi','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/fmit','53','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/fmmidi','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/foo-yc20','74','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/freepats','15','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/fst','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ghostess','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gimmix','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gmpc','22','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gmpc-plugins','53','31383.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gmusicbrowser','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gnomad2','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gnome-music','13','27862.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gnome-shell-extension-sound-output-device','1','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gnump3d','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gogglesmm','64','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gqradio','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/grip','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gtklick','1','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/guitarix','134','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gusersoundfont','12','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/gvolwheel','9','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/herrie','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/horgand','9','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/hydrogen','51','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/icecast','12','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ices','10','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/id3','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/id3tool','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/id3v2','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/invada_studio_plugins_lv2','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/iripdb','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jaaa','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jack','17','29077.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jack-keyboard','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jack-rack','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jack-tools','5','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jack1','30','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jack_capture','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jack_delay','1','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jackmeter','6','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jalv','8','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jamin','22','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jamulus','26','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/japa','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jkmeter','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jnoise','1','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/jvmetro','1','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/klick','6','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/kradio','53','28717.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ladspa-bs2b','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ladspa_sdk','1','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/lash','14','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/libbs2b','6','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/libebur128','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/libfreeaptx','2','29896.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/lingot','7','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/linuxsampler','74','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/listener','2','28082.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/lmms','100','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/lsmi','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/lv2','16','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/lv2-mda-metapiano','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/lv2dynparam1','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/lv2file','1','28788.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/lxmusic','8','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mac','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/match-vamp-plugin','4','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mda-lv2','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/meterbridge','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mhwaveedit','9','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/microenc','1','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mididings','36','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/midillo','17','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/milkytracker','13','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/millennium-plugins','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/miniAudicle','16','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/minimodem','6','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mixxx','467','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mkcue','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mktoc','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mp3blaster','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mp3check','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mp3gain','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mp3splt','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mp3splt-gtk','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mp3val','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mp3wrap','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mpc','4','31029.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mpd','50','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mpdscribble','6','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mppenc','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/mt-daapd','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/multimidicast','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/musepack-tools','10','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/musikcube','146','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/nas','41','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ncmpc','17','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ncmpcpp','107','31029.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ncspot','342','28141.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/nekobee','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/nero-aac','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ngjackspa','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/nnls-chroma','4','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/non','12','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ntk','18','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ocp','137','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ogmtools','5','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/opencore-amr','10','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/opustags','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/padthv1','13','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/paman','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/paprefs','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pasystray','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/patchage','14','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pavucontrol-qt','8','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pavumeter','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pd','35','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pd_mrpeach','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/phat','12','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pianobar','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pipewire-native-jack','34','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pithos','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/plug','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pmidi','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pnmixer','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/podget','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pogo','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/ponymix','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/portmidi','3','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/puddletag','6','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pulseaudio-ctl','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pulsemixer','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/pyalsaaudio','5','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/python-audiotools','33','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/python3-pyalsaaudio','4','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qastools','39','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qjackctl','14','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qm-vamp-plugins','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qmidiarp','11','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qmidinet','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qmidiroute','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qmmp','66','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qmmp-plugin-pack','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qmmp-plugin-pack-qt5','11','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qmmp-qt5','123','26938.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qpwgraph','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qsynth','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qtractor','46','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/quodlibet','13','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/qxgedit','12','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rack','6','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rakarrack','22','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/reaper','26','27283.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rebmp','26','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/retrovol','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rev-plugins','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rezound','35','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rhythmbox','68','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rosegarden','130','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rplay','6','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rubberband','17','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rubyripper','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/rumor','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/samplv1','14','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/sbagen','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/schismtracker','15','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/seq24','15','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/setBfree','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/sfarkxtc','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/sfc','9','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/shntool','5','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/shorten','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/sidplayfp','5','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/siren','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/snd','55','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/sndio','4','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/sonata','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/sonic-visualiser','115','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/sonobus','206','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/sooperlooper','28','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/soundconverter','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/soundkonverter','40','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/soxr','6','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/specimen','7','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/spectmorph','118','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/stops','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/streamripper','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/streamtuner','17','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/stretchplayer','5','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/sunvox','14','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/swami','9','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/swh-plugins','15','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/swh-plugins-lv2','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/synthv1','14','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/tagtool','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/tap_plugins','5','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/tapiir','6','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/tauonmb','11','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/transcribe','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/tta','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/tuner','6','29131.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/tuxguitar','40','30722.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/twolame','5','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/unsf','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/vamp-plugin-sdk','20','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/vcf_plugins','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/vimpc','15','31029.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/vmpk','11','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/vocoder','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/vocoder-ladspa','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/volctl','2','28273','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/volti','1','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/volume.app','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/volumeicon','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/vorbisgain','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/waon','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/wavbreaker','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/whipper','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/whysynth','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/wildmidi','4','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/wireplumber','9','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/wmusic','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/wmusic-xmms','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xcfa','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xforge','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmms-ctl','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmms-jack','9','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmms-ladspa','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmms-mp4','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmms-pulse','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmms-scrobbler','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmms-shn','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmms-wavpack','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmms-wma','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmms2','19','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xmp','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xsynth-dssi','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/xwax','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/yabridge','80','28778','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/yoshimi','47','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/zita-ajbridge','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/zita-dpl1','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/zita-rev1','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/zoom-edit-n-share','27','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/zynaddsubfx','54','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('audio/zynjacku','5','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('business/beancount','2','26398','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('business/exercism_repack','4','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('business/ledger','47','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('business/maltego','59','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('business/reckon','2','26398','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('business/skrooge','139','26398','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('business/stansoft','23','26398','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/2bwm','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/9menu','2','28047','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/Alphacube-GTK','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/AlsaMixer.app','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/AppStream','24','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/Ater-MPlayer-skin','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/BlockstreamGreen','18','27622.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ClamAV-GUI','19','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ClipboardViewer','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ComixCursors','28','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/LX-Colors-Revival','400','34729.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/Matcha-gtk-theme','39','30480.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/NsCDE','144','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/Pulse-Glass','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/PyPanel','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/Qogir-icon-theme','108','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/SiriKali','29','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/Surfn','72','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/SwayNotificationCenter','7','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/SysTray-X','27','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/Tela-icon-theme','20','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/TextSnatcher','6','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/UltimateDailyWallpaper','7','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/Vanilla-DMZ','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/XDecorations','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/Xfce-Theme-Manager','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/adwaita-qt','18','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/aero-mouse-theme','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/afterstep','67','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/alacarte','12','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/alltray','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ambiance-radiance-flat-colors-gtk-theme','25','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/amiwm','8','27722.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ansiweather','14','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/applet-window-buttons','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/arandr','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/arc-firefox-theme','9','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/arc-icon-theme','16','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/arc-theme','27','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/areao43-icon-theme','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/artix-backgrounds','5','28628.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/autokey-gtk','3','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/autorandr','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/aweather','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/awesome','7','27862.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/awesome-appmenu','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/awf','13','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/azote','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/baobab','11','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/bashmount','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/bashrun','1','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/bashrun2','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/battery-notify','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/bemenu','3','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/bibata-cursor-theme','40','28816.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/birdtray','11','31818.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/blit','2','27862.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/bmpanel2','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/boston-icon-theme','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/bottles','5','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/bspwm','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/bubblemon-dockapp','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/budgie-desktop','23','27608.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/budgie-desktop-view','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/budgie-screensaver','6','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/buku','3','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/bukubrow','41','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/bumblebee-status','5','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/buttonmaker','8','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/byzanz','25','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/caffeine-ng','2','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cairo-clock','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cairo-dock','15','28047','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cairo-dock-plugins','29','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/caja','33','28788.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/calcure','2','29896.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/calls','15','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/candle','15','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/catfish','4','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cbatticon','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ccsm','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cde','449','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cellwriter','5','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cheese','22','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cinnamon-desktop','10','29336.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/clearlooks-classic-themes','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/clearlooks-phenix-theme','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/clipit','7','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/clipmenu','1','28651.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cmatrix','4','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cnslock','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/compiz-boxmenu','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/compton','5','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/copyq','22','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/crystalcursors','16','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ctwm','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cutefish-calculator','5','33780','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cutefish-core','30','33780','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cutefish-dock','7','33780','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cutefish-filemanager','13','33780','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cutefish-gtk-themes','5','33780','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cutefish-icons','544','33780','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cutefish-launcher','7','33780','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cutefish-screenlocker','7','33780','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cutefish-settings','12','33780','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cutefish-statusbar','12','33780','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cutefish-terminal','9','33780','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cutefish-wallpapers','5','33780','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cwm','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/cwm-openbsd','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/dapper','6','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/dbgl-legacy','10','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/devilspie','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/devilspie2','2','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/disper','2','28628.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/dmenu','2','28651.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/docfetcher','39','29260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/doctest','12','31383.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/dolphin-megasync','10','27608.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/domterm','51','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/draco','28','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/dunst','4','28273','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/durden','3','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/dwl','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/dwm','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/dwmblocks','2','28066.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/dzen2','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/e16','23','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/e16.theme.pack','7','31239.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/echinus','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ede','76','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ee','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/emerald','11','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/emerald-themes','6','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/emwm','4','31239.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/engrampa','15','28747.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/enlightenment','185','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/evilwm','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/faba-icon-theme','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/fbautostart','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/fbmenugen','2','28747.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/fbpanel','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/fbrowse-tray','1','29260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/fdpowermon','1','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/fehlstart','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/find-cursor','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/flat-remix','576','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/flatpak','22','28717.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/flatpak-builder','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/flatwoken-icon-theme','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/florence','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/foot','6','27561.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/fusion-icon','2','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/fuzzel','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/fvwm-crystal','9','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/fvwm3','22','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/fvwm95','11','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gammastep','20','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gdm','16','27608.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gfi','3','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gis-weather','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gkrellm-themes','13','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/glabels','25','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gmrun','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gmusicbrowser-art','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-backgrounds','19','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-boxes','27','31383.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-browser-connector','2','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-calculator','26','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-calendar','15','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-characters','8','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-clocks','14','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-connections','11','31383.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-contacts','10','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-control-center','51','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-font-viewer','5','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-icon-theme','54','28747.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-icon-theme-stock','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-maps','9','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-menus','20','30628.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-photos','21','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-screenshot','5','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-session','7','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-settings-daemon','13','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-shell','24','27608.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-shell-extension-appindicator','3','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-shell-extension-arc-menu','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-shell-extension-dash-to-panel','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-shell-extensions','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-tour','89','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-tweaks','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gnome-weather','3','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gntp-send','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gopsuinfo','8','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gr1d','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/grim','2','27561.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/grits','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gromit-mpx','3','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/grun','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gsimplecal','9','31218.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtk-engines','14','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtk-engines-unico','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtk-theme-Bluebird','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtk-xfce-engine','9','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtk1-industrial-engine','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtk1-theme-switch','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtkhash','21','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtklock','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtklock-playerctl-module','1','31239.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtklock-powerbar-module','1','31239.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gtklock-userinfo-module','2','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/gxkb','6','32180','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/happy-hacking-gnu','2','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/herbe','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/herbstluftwm','18','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/hikari','9','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/hsetroot','1','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/human-gtk-theme','7','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/hushboard','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/hyfetch','4','35185.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/i3','11','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/i3-easyfocus','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/i3-gaps','11','28651.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/i3blocks','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/i3ipc-glib','18','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/i3lock','10','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/i3minator','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/i3pystatus','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/i3situation','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/i3status','4','27862.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/icewm','24','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/icewm-themes','26','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/idesk','10','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ideviceinstaller','5','27608.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/idevicerestore','6','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/imgur-screenshot','1','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ion','20','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/j4-dmenu-desktop','9','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/jgmenu','5','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/joystickwake','1','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/jwm','7','29131.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/kanshi','3','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/kanyremote','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/kbdd','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/kde44-oxygen-molecule','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/kdocker','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/keynav','2','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/kfaenza-icon-theme','56','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/kolorcontrol','1','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/kupfer','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/kwalletcli','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/labwc','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/leftwm','53','29893.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lemonbar','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lemonbar-xft','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lfm','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/libqtxdg','11','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/light','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lightdm-slick-greeter','9','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/liquidshell','15','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lumina','806','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/luna-wallpapers','24','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lxappearance','6','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lxappearance-obconf','10','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lxde-common','5','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lxhotkey','6','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lxlauncher','9','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lxmed','2','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lxmenu-data','5','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lxpanel','23','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lxqt-menu-data','2','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lxqt-panel','41','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lxqt-runner','7','31070.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/lxqt-themes','12','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/macopix','21','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/maitreya','70','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/marco','16','27862.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mate-control-center','57','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mate-desktop','12','28788.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mate-desktop-environment','1','28788.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mate-media','8','41948','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mate-menus','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mate-panel','34','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mate-power-manager','78','41948','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mate-screensaver','11','28788.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mate-session-manager','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mate-settings-daemon','22','28788.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mate-system-monitor','64','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mate-terminal','9','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mate-utils','146','41948','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/materia-theme','18','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/matrixgl','4','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mcwm','2','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/meliae-icon-theme','10','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/menulibre','7','29245.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/menumaker','3','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/menutray','1','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mh5000','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mhsetroot','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mimi','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mint-l-icons','109','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mint-l-theme','69','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mint-themes','73','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mint-x-icons','26','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mint-y-icons','95','28784.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/moka-icon-theme','70','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mozo','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mplayer-skins','10','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mugshot','3','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/murrine','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/murrine-themes','6','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/musca','2','28651.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mutter','59','27608.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/mwm-utils','2','31239.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/myGtkMenu','2','29260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nautilus','25','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ncpamixer','4','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/netwmpager','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/neutral','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/neutrality-cursor-theme','6','28788.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nitrogen','8','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nitrokey-app','58','31218.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/noice','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nordic-gtk-theme','12','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/notification-daemon','5','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/notion','13','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/numix-blue-theme','2','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/numix-folders','11','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/numix-icon-theme','15','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/numix-icon-theme-circle','8','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/numix-icon-theme-square','7','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/numix-ocean-theme','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/numix-pack','142','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/numix-sx-theme','5','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nwg-bar','665','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nwg-clipman','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nwg-displays','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nwg-dock','690','28494.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nwg-drawer','672','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nwg-hello','3','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nwg-icon-picker','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nwg-launchers','15','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nwg-look','665','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nwg-menu','671','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nwg-panel','6','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nwg-readme-browser','1','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nwg-shell','3','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nwg-shell-config','2','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nwg-shell-wallpapers','6','27561.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/nwg-wrapper','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/obapps','1','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/obbrowser','1','29260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/obconf','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/obkey','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/obmenu','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/obmenu-generator','1','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/obsession','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/obshutdown','6','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/oomox','163','31239.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/openbox','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/openbox-menu','2','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/openbox-themes','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/openbox-themes-extra','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/orage','12','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/paginator','2','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/paper-icon-theme','55','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/papirus-icon-theme','87','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/parcellite','6','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/pcb2gcodeGUI','6','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/pclock','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/peksystray','5','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/pekwm','19','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/perwindowlayoutd','3','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/pfetch','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/picom','11','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/pidgin-guifications','8','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/pidgin-libnotify','5','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/pidgin-window_merge','11','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/pinentry-dmenu','2','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/plank','47','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/plasma5-themes-breath','15','28788.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/plasma5-themes-breath-migration','5','28788.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/polar-cursor-themes','5','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/polybar','37','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/progman','2','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/protozoa-cursor-themes','16','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ptwit','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/py3status','4','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/pyful','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/python-specbar','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/python3-autotiling','2','28788.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/pytyle','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/q4wine','44','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/qlipper','6','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/qmmp-skins','5','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/qt5ct','13','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/qtile','8','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/qtile-extras','5','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/qtpass','16','28628.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/quickswitch-i3','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ratpoison','5','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/razercfg','3','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/razorqt','54','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/rclone-browser','16','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/recoll','70','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/redshift','7','28082.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/rodent','41','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/rofi','8','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/rofi-calc','13','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/rofi-emoji','14','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/rofi-pass','1','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/root-tail','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/rootactions_servicemenu','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/rsl','15','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/rss-glx','16','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/rss-guard','143','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/s1kls','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/salmon','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/screenfetch','2','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/screenkey','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/sddm-theme-artix','2','28628.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/sdorfehs','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/seafile-client','56','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/sfwbar','5','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/sgsearch','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/skb','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/skippy-xd','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/skulpture','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/sl-fortune','1','29896.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/slack-wallpapers','152','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/slackware-xdg-menu','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/slackware-xdm-theme','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/slbar','1','28747.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/slock','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/slstatus','1','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/slurp','3','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/smaragd','4','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/somebar','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/spectrwm','7','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/stalonetray','3','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/stoken','16','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/subtle','12','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/sun','2','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/superkey-launch','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/suru-plus-icon-theme','753','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/sushi','10','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/sway','10','27561.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/swaybg','2','27561.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/swayidle','2','27561.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/swaylock','3','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/sxhkd','2','28585.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/sync-engine','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/synce-kpm','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/thunar-archive-plugin','8','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/thunar-custom-actions','1','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/thunar-media-tags-plugin','5','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/thunar-megasync','8','27608.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/thunar-sendto-clamtk','1','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/thunar-vcs-plugin','10','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/tint2','9','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/todoman','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/trayer-srg','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/tty-clock','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/ufetch','1','28171.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/unsaver','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/uwm','11','30544.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/vala-panel-appmenu','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/variety','5','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/vbox-runner','4','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/vdesk','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/velox','4','31818.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/vertex-maia-icon-themes','19','31209.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/vim-qt','31','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/vinagre','15','31383.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/vtwm','7','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wally','20','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/waybar','36','28171.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wayfire','32','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wbar','11','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wbg','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wcm','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wdisplays','3','25960','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/weston','13','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wf-config','13','31383.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wf-shell','13','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wl-mirror','3','27862.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wlr-randr','2','31838.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wlsunset','3','31239.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmCalClock','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmMoonClock','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmSun','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmSunMoon','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmakerconf','13','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmalauncher','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmamixer','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmbackground','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmbattery','3','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmbiff','6','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmblob','3','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmbutton','3','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmcellauto','5','35185.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmcliphist','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmclock','5','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmclockmon','5','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmcpufreq','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmcpuwatch','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmctrl','5','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmcube','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmdate','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmdl','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmdrawer','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmfire','6','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmfishtime','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmii','5','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmimagedock','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wminfo','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmix','3','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmmand','2','28571.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmmixer','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmmon','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmname','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmnet','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmphoto+','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmpinboard','4','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmpower','5','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmrandwalk','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmshutdown','4','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmslideshow','1','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmsm.app','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmsorsen','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmstickynotes','3','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmsupermon','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmswallow','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmsysmon','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmsystemtray','4','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmtime','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmtimer','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmtop','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmudmount','5','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmuptime','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmweather','4','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmweather+','6','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmwebcam','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmwifi','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wmxss','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wofi','3','29151.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wofi-pass','2','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/workrave','57','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/wtype','2','29151.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/x-tile','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/x2x','5','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xapp-symbolic-icons','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xdg-desktop-portal-gtk','9','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xdgmenumaker','1','30722.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xdm-slackware-theme','4','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xdo','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xearth','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce-hkmon','3','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-alsa-plugin','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-battery-plugin','6','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-cpufreq-plugin','12','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-cpugraph-plugin','11','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-datetime-plugin','5','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-dict','8','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-diskperf-plugin','6','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-docklike-plugin','8','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-eyes-plugin','6','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-fsguard-plugin','6','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-genmon-plugin','6','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-i3-workspaces-plugin','15','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-mailwatch-plugin','6','28571.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-mount-plugin','5','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-mpc-plugin','7','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-netload-plugin','6','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-notes-plugin','8','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-places-plugin','7','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-sensors-plugin','14','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-smartbookmark-plugin','6','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-timer-plugin','7','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-verve-plugin','6','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-volumed-pulse','5','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-wavelan-plugin','6','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-windowck-plugin','9','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfce4-xkb-plugin','7','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfdashboard','25','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xfwm4-themes','17','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xkblayout-state','2','28628.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xlunch','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xmenu','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xmms-skins-almond','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xmms-skins-ater','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xmms-status-plugin','6','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xmobar','26','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xmonad','25','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xmouseless','2','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xob','2','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xonclock','10','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xpenguins','6','28747.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xpenguins-xtrathemes','2','28747.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xpra','166','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xsecurelock','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xsession','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xsession-xinitrc','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xseticon','2','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xss-lock','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xtitle','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xtrascreenhacks','9','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xtrlock','1','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xvkbd','3','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xwallpaper','6','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xwarppointer','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/xxkb','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/yad','17','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/yambar','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/yaru-icon-theme','71','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/zenity','17','27862.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('desktop/zuki-themes','3','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/4th','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/9pro','1','31070.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Arduino','1321','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Bear','57','28583.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/CImg','7','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/CVSTools','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Catch','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Catch2','14','29706.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ChezScheme','215','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Coin','121','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Fennel','4','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/FreeBASIC','29','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/GHDL','102','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/HeidiSQL','18','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Hex2bin','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ICTester','15','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/J-Link','25','28816.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/JAForth','2','28861.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/LLgen','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/MinForth','7','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/MySQL-python','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/OpenCL-CLHPP','2','26907.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/OpenHantek6022','16','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/OpenJDK11','765','31218.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/OpenJDK17','747','31218.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/OpenJDK21','828','31218.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ParadisEO','28','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Pivy','128','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/PythonToolkit','3','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/QStlink','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/QtPy','3','29108.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/SQLAlchemy','28','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/SQLiteStudio','272','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/SmallBASIC','186','33022.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/SoQt','23','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Sphinx','11','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/StarPU','72','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/SystemC','26','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/THE','16','29355','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/UASM','3','29349.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/VisUAL','7','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/Xdialog','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/XiMpLe','3','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/a68g','13','28879.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/abseil-cpp','43','29897.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/acc','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/acme','2','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/acme2k','20','27862.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/acpica','10','31239.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/actionlint','17','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ada-2022-rm','4','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/adms','17','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/aflex','9','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/aflplusplus','80','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/alembic','3','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/alex','9','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/alire','103','27862.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/android-tools','47','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/antlr2','18','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/apache-ant','13','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/apache-ivy','3','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/apache-jmeter','36','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/apache-log4cxx','30','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/apache-maven','6','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/arcan','29','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/arduino','70','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/arduino-cli','9','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/argagg','3','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/argouml','9','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/arm-binutils','41','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/arm-gcc','294','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/arm-gdb','127','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/as31','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/asar','12','29355','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/asdf','8','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/asem51','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/asl','71','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/asm6809','12','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/astyle','5','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/atasm','3','30733.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/autogen','57','29131.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/avarice','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/avr-binutils','87','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/avr-gcc','975','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/avr-gdb','128','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/avr8-burn-o-mat','3','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/avr8-gnu-toolchain','34','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/avra','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/avrdude','17','31383.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/aws-cdk','11','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ayacc','12','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bacon','530','28171.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bakefile','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bas','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bas55','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bascat','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bashdb','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bats-core','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bbcsdl','22','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bbe','4','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bcpp','4','35185.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/beautysh','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/beav','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bed','23','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/berkeley-abc','94','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bfg','5','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bigloo','126','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/binwalk','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/blassic','9','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bless','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/blocksruntime','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bluefish','31','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bmake','44','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/boost-di','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/boron','3','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bpython','3','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bsdiff','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/buf','71','28694.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/build2','273','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bvi','4','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bviplus','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bw-atari8-tools','6','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bwbasic','14','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/byacc','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/bzr','51','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cJSON','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/c_count','3','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/camlp4','99','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/camlp5','70','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cargo-auditable','55','27743','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cargo-c','358','28894.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cargo-vendor-filterer','26','28154.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/casperjs','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/castxml','19','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cc-tool','36','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cc65','18','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ccl','34','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cf-cli','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cgdb','9','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cgit','15','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/chameleon','43','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/chatgpt_sh','2','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/checkbashisms','1','29131.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/chibi-scheme','22','30722.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/chicken','201','27973.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ciforth','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cim','14','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/claude-code','29','28430.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/clips','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cloc','5','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/clojure','7','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cmake-opt','327','31070.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cmocka','7','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/coccinelle','82','28788.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cocotb','33','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/codeblocks','157','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/codelite','714','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/codespell','5','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/colm','54','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/colordiff','1','27230.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/complexity','13','29131.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/composer','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/config-file-validator','15','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/corrosion','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cpan2tgz','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cppcheck','51','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cppcms','48','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cppdb','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cppunit','41','29706.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cproto','4','35185.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cream','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/creduce','170','28747.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/critcl','15','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/crosstool-ng','22','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/crystal-lang','25','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/csmith','12','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cunit','16','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cutter','63','31364.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cvsd','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cvsps','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cxref','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cxxtest','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/cxxtools','93','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/d-tools','92','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dart-sass','38','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dart-sdk','107','31029.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dasht','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dasm','2','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dejagnu','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/devhelp','16','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/device-tree-compiler','12','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dfsimage','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dfu-programmer','4','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dhex','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/diakonos','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/diff-so-fancy','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/diffoscope','5','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/difftastic','93','28047.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/diffuse','4','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/digikey-kicad-library','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dis','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dis6502','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dkms','2','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dmd','45','29284.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dpkg','26','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dreampie','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dte','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/dune','49','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/eagle','57','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ebe','33','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ecl','168','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/edb-debugger','52','31239.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/edwood','9','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/egypt','1','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/eisl','7','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/elixir','59','29522.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/emsdk','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/energia','82','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/enki','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/envision','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/envytools','40','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/eovim','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/epydoc','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/erlang-otp','148','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/es-shell','11','30722.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/evemu','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ex-vi','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/extra-cmake-modules-opt','3','27418','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/f2c','4','28788.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/fasm','2','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/flawfinder','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/focal','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/fortitude-bin','5','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/fortls','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/fossil','56','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/fpc','237','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/freetds','28','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/frobtads','21','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ftjam','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ftnchek','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/g3log','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gambas3','787','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gambit','88','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gauche','248','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gcc14','3647','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gcc5','849','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gcli','6','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gdb-dashboard','1','31239.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/geany','43','27927.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/geany-plugins','66','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/geany-themes','1','28273','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gecode','126','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gede','26','28454.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gedit','39','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gedit-plugins','16','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/generatorrunner','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gengetopt','46','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gerbil','2383','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gf','8','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gforth','32','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ghex','14','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ghi','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gisto','51','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/git-cola','8','30544.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/git-crypt','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/git-extras','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/git-lfs','5','33780','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/git-tools','2','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/github-cli','16','28197.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gitlab-cli','15','28454.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gitstats','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/glm','15','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/global','16','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gn','50','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gnats','12','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gnome-builder','174','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gnome-common','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gnome-system-monitor','25','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gnu-apl','59','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gnulib','29','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gnuradio','571','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gnustep-base','32','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gnustep-make','4','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gob2','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/godot','952','33780','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/golang-googlecode-gonet','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/golang-googlecode-gotools','43','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/golangci-lint','51','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/google-cloud-cli','166','27935.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/google-go-lang','1532','28536.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gopls','42','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/got','26','28717.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gplcver','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gprbuild','159','27862.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gprolog','23','31239.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gpsim','38','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gputils','90','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gr-osmosdr','26','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gradle','77','28745.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/grails','61','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gsoap','67','31239.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gtest','16','27608.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gtkdialog','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/gtkextra','14','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/guile-colorized','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/guile1.8','70','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/guile2.0','681','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/guile2.2','242','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/guitar','29','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/guzzle_sphinx_theme','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/halibut','4','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/happy','23','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/helix','444','28392.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/henplus','3','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/hexcurse','7','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/hexe','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/hexedit','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/hexer','2','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/hexinator','85','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/hexnet','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/highlight','41','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/hipSYCL','12','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/howl','16','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/hte','15','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/hub','14','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/icdiff','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/icecream-sundae','6','34287.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/icemon','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/icon','30','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/icoutils','24','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/idlex','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ieee-pilot','3','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ign-math2','14','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ignition-math','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/inform','7','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ioncube-loader','2','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jam','4','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/janet','14','28818.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jasspa','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/javacc','1','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/javafx-scenebuilder','79','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jdk','82','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jeex','14','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jenv','2','28273','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jpm','2','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jsmin','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jujutsu','352','28814.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/julia','83','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/julius','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter-ipykernel','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter-ipywidgets','2','34610.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter-nbclassic','25','30480.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter-nbclient','2','28784.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter-nbconvert','4','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter-nbformat','2','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter-notebook','35','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter-notebook_shim','2','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter-qtconsole','3','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter-widgetsnbextension','1','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter_client','3','27312.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter_console','2','31383.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter_core','3','27312.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter_events','2','34287.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter_packaging','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter_server','4','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyter_server_terminals','3','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyterlab','23','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyterlab_pygments','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/jupyterlab_server','3','29301.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kForth-32','7','28400.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kaaedit','7','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kakoune','40','30733.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kawa.bin','3','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kconfig-frontends','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kcov','14','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kdesvn','44','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kdiff3','59','31239.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kelbt','8','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kforth','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kicad','1732','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kicad-footprints','104','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kicad-packages3D','1187','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kicad-symbols','23','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kicad-templates','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kiwi','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kodespel','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/komodo-edit','103','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/komodo-ide','115','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kompozer','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/kotlin','38','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/latrace','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/lazarus','364','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/lazygit','35','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/lcov','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ldns','116','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/le','43','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/leiningen','1','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/leo','20','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/lepton-eda','53','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/leveldb','16','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/lfe','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/libb2','13','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/libcutl','13','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/liberica-jdk-bin','94','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/libtree','2','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/libxbgi','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/lighttable','134','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/likepython','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/liquibase','18','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/litehtml','12','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/llvm-opt','3345','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/logisim','3','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/love','40','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/love-legacy072','19','29131.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/love-legacy080','22','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ltrace','19','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/lua','3','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/lua-language-server','96','28808.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/lua51','3','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/luajit','8','28379.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/luarocks','3','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/lwtools','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mads','7','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/maude','236','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mawk','6','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mcu8051ide','5','29131.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/md2roff','2','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mdbook','71','28047.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/medit','21','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/meld','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/meld3','5','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/memchan','4','41948','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/menhir','12','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/metakit','12','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mg-openbsd','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mg-troglobit','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/micro','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/minipro','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mkchroot','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mle','7','27900.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mlr','18','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mm-common','3','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mmtf-cpp','2','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mockpp','31','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mold','333','29315.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mono','1624','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mono-addins','20','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mono-basic','28','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mosml','13','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mphidflash','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/msbasic2ascii','2','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mspgcc-legacy','319','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mstch','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mtasc','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mtd-utils','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/muon','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/mutagen','5','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nant','11','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nchexedit','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ne','18','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/neatvi','3','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nedit','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/neko','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/neovim','249','28032','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/neovim-qt','26','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/neovim-remote','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/netbeans','186','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/newlib','124','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/newlisp','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nextvi','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nexus','88','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nickle','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nim','362','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ninja-ide','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nodeenv','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nodejs','1101','28451.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nodejs18','908','33022.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nodejs22','1819','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nose','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/notepad++','3','29263.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/notepadnext','59','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/notepadqq','46','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/noweb','4','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/npm2tgz','1','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/numpy-legacy','171','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/nwjs','86','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/obcpl','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/obnc','9','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/obnc-libext','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ocaml','158','29706.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ocaml-num','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ocamlbuild','11','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/omake','36','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/omniORB','72','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/omniORBpy','31','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/oneDPL','9','31239.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/oneTBB','363','30733.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/onetrueawk','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/opencomal','7','34610.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/opencvs','16','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/openmodelica','740','31818.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/openocd','33','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/opentofu','112','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/openvr','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ophis','6','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/osc','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/oscar64','36','31218.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/osmo-sdr','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/p4','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/p4d','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pacoteco','1','27820.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/page','23','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pahole','8','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pandas','513','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pasdoc','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/patchutils','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pcb2gcode','172','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pcbasic','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pcc','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pd-psql','1','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/peg','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/perf','27','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pforth','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/phantomjs','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/php-apcu','8','29404.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/php-xdebug','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/php82','193','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/php84','202','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/phpDocumentor','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pingendo','48','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pinner','2','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pip2tgz','1','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pkgconf','13','27704.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pkgdiff','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pli','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pnpm','9','28633.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pocl','53','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/poedit','52','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/polyml','37','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/portaudio','11','27365.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pp','1','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pre-commit','3','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/premake','3','27862.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/premake5','9','27862.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/protobuf3','159','29897.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/protoc-gen-go','12','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/protoc-gen-go-grpc','10','28816.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pstack-expect-gdb','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pudb','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pulsar','177','27910.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pycharm','670','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pyclewn','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pycscope','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pyenchant','2','27608.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pylint','4','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pyside-tools','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/python-sphinx-lv2-theme','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/python2-cycler','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/python2-numpy','86','29706.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/python3-cycler','1','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/python3-hylang','4','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/python3-hyrule','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/python3-matplotlib','96','28585.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/python3-numpy','91','29706.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/python3-numpy-legacy','205','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/python3-pyqt-distutils','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/python3-spyder-kernels','2','27312.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/pyvim','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/qb64','5','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/qb64pe','28','27277.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/qbe','3','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/qbs','126','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/qbsolv','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/qconf','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/qt-creator','1176','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/qt-creator-llvm','2476','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/quickjs','41','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/quickjs-ng','28','27239.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/quilt','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/racket','611','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/radare2','162','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ragel','58','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rapidsvn','36','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rarian','18','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ratfor','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rc','2','27862.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rebar','5','30667.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rebar3','18','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/reckless','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/recutils','33','28571.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/redo','5','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/reflex','5','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/regexxer','9','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/regina-rexx','45','29355','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/remake','13','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/reuse-tool','3','30733.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rgbds','12','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rider','994','27046.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rizin','149','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rmac','8','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/robomongo','12','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/robotframework','11','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/roswell','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rr','59','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rstudio-desktop-legacy','93','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rtags','69','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rtaudio','17','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ruff','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rust-opt','151','29499.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rust16','118','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/rustup','194','27813.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/s51dude','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/s9fes','9','28717.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sam','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/samurai','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sasm','15','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sauce','23','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sbcl','889','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/scala','10','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/scala3','84','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/scheme48','18','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/schroot','55','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/scikit-build','2','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/scite','41','30753.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sdcc','270','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/se','5','30722.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/setedit','72','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/setuptools-git','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/shc','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/shed','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/shiboken','18','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/shiboken2','26','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/simavr','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/simulavr','101','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/slibtool','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sloccount','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/smalltalk','63','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/smartgit','18','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/smartsvn','15','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/smlnj','47','28788.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/snack','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/snappy','7','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/source-highlight','35','29042.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/spice-protocol','3','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/spin','9','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/splint','47','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/spyder','35','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sqlcl','33','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sqldeveloper','173','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sqlitebrowser','291','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sqliteman','17','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/squeak-vm','15','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/squirrel-sql','25','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/src-vcs','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/srcML','185','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/srecord','29','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/stagit','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/stgit','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/stklos','27','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/stlink','6','29349.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/stubb','2','35185.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/svn-all-fast-export','4','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/swfmill','14','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/swi-prolog','135','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/sysprof','18','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/systemtap','68','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tcc','13','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tcludp','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tclvfs','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tea','17','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/teensy_loader_cli','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/terraform','156','28692.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/terraform-ls','26','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/texi2html','14','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/textadept','33','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tfenv','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tflint','66','29069.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/thread','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tig','7','35185.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tiled','109','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tinyscheme','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tkcon','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tkcvs','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tkdiff','2','31029.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tkdnd','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tklib','5','41948','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tkman','2','29799.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tktable','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tofuenv','2','29355','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tora','176','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/trac','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tracy','12','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/trf','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ttk-themes','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tup','82','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/tweak','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/txr','28','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/txt2man','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/u-boot-tools','49','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ucblogo','12','31029.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/ucpp','15','29260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/udis86','16','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/uemacs','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/uftrace','8','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/uncrustify','38','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/unicorn','89','31209.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/unifdef','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/unittest-cpp','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/universal-ctags','52','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/unmake','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/unreliablefs','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/uprog2','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/urjtag','21','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/urn','11','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/uxd','1','31364.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/uz80as','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/vala-language-server','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/valabind','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/valgrind','91','28641.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/valkyrie','9','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/vasm','26','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/vc-intrinsics','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/viewvc','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/vif','157','28571.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/vile','61','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/vis','9','27862.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/vispatch','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/voc','26','27862.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/vscode-insiders-updater','1','28552.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/watchman','46','28059.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/whisper.cpp','31','28166.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/wla-dx','21','29355','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/wxFormBuilder','39','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/wxHexEditor','20','31383.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xa','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xasm','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xatmas2','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xkeys-sdk','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xmake','65','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xmlcopyeditor','33','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xmlstarlet','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xnedit','18','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xsp','21','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xtruss','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xvi','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xvile','61','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xxd-standalone','2','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/xxdiff','13','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/yabasic','7','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/yaegi','28','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/yarn','3','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/yosys','146','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/z88dk','313','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zeal','30','28201.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zig','102','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zile','30','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zizmor','290','28716.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zmac','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zmakebas','2','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zooey','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zope.interface','4','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zope.sqlalchemy','3','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zope.testing','3','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zulu-openjdk11','91','28265.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zulu-openjdk17','91','29975','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zulu-openjdk7','68','29260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zulu-openjdk8','81','28111.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zxbasic','163','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('development/zxtext2p','1','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/0ad','1362','29955.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/0ad-data','841','30480.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/1oom','19','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/2048-in-terminal','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/3D-Fasteroids','7','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/4digits','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/7kaa','68','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/BeastieWorker','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/BeastieWorker-levels','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/BrogueCE','5','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/Chatbot-Eliza','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ClassiCube','8','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/CorsixTH','16','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/FlightGear','995','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/KAdventure','3','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/KoboDeluxe','11','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/OpenJK','72','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/OpenXcom','74','31029.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/PrismLauncher','238','31245.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/PrismLauncher-Unlocked','241','35185.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/Pyfa','11','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/QuickNES-Core','4','33022.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/RetroArch','68','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/SLADE','156','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/Scorched3D','154','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/Sokoban','4','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/SpaceCadetPinball','10','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/SpaceZero','4','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/Tempus-Irae','75','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/UrbanTerror','418','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/VASSAL','14','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ZEsarUX','39','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/abe','6','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/abuse','22','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ace','6','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/acidwarp','3','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/adl','6','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/adom','3','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/advancemame','195','30722.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/advancescan','6','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ags','49','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/aisleriot','26','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/aklabeth','6','32180','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/alephone','117','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/alienarena','350','32180','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/alienwave','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/an','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/angband','48','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/antares','59','33780','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/antimicrox','33','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ardentryst','11','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/armagetronad','92','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/arnold-cpc','7','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/arx-libertatis','63','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/asc','147','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/asciijump','4','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/asciisec','5','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/assaultcube','50','29260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/assaultcube-reloaded','45','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/asteroidsinfinity','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/astromenace','58','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/atanks','9','32180','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/atari++','27','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/atari800','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/atari800_roms','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/atari_8bit_utils','5','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/attract','26','32883.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/augustus-game','50','29896.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/barrage','4','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bastet','6','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/beetle-psx-libretro','77','33022.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/berusky','13','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/berusky-data','3','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/berusky2','24','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/berusky2-data','222','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bitfighter','90','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/blasphemer','11','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/blobby2','14','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/blobwars','41','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bloodfrontier','153','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bluemoon','39','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bluez-sixaxis','20','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/blupimania','21','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bomberclone','7','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/boswars','160','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/braincurses','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/brainparty','24','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/brickout','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/briquolo','15','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/brutalchess','11','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bsd-tetris','2','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bstone','28','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/burgerspace','6','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/bzflag','120','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/caesar3','9','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/caph','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cbonsai','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ceferino','9','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cgames','3','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cgmadness','6','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cgoban','9','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/chessx','83','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/chexquest','6','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/chexquest3','7','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/chocolate-doom','28','28651.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/chroma','8','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/chromium-bsu','10','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/circuslinux','4','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ckmame','5','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/clksignal','135','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/clksignal-roms','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/clonekeen','3','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/colem','3','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/commandergenius','127','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/connectagram','8','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/conquest','41','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cosmic_assault','18','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cosmosmash','5','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cowsay','1','34729.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cpat','3','30544.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/crack-attack','16','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/crawl','307','31818.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/crimson','10','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/crispy-doom','28','28552.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/crossfire-client','6','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/crrcsim','45','28082.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cryptokwot','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cse2','19','33780','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cubosphere','159','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/curseofwar','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cutechess','60','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cutemaze','5','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cuyo','15','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/cyphesis','41','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/d1x-rebirth','132','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/d2x-rebirth','144','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/darkplaces','27','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ddnet','93','28273','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dealers-choice','21','28692.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/defendguin','7','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/desmume','30','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/devilutionx','148','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dfarc','14','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dgen','25','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dhewm3','52','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dice','10','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/digger','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dmagnetic','3','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/domination','13','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/domino-chain','436','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/doom_shareware_data','3','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/doomretro','19','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/doomsday','188','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/doomseeker','74','28651.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dopewars','17','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dreamchess','8','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dreamchess-music','36','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ds-models','60','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dunelegacy','50','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dungeon','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dustrac','42','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/dwarffortress','17','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/easyrpg-player','68','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ecwolf','19','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/edgar','108','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/eduke32','214','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/eduke32_hires_pack','369','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/eduke32_shareware_data','5','28788.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/egoboo','160','31239.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/einstein','4','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/empire','11','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/emulationstation-de','101','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/endgame-singularity','5','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/endless-sky','246','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/enigma','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/enigma-marbles','103','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/epsxe','4','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/etlegacy','74','28171.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/etlegacy-data','108','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/eureka','24','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/extremetuxracer','29','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/exult','117','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ezquake','27','41948','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/f1spirit','16','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fallout1-ce','13','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fallout2-ce','16','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fbneo-libretro','142','33022.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fceux','67','41948','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ffgo','2','34287.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fgo','2','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fheroes2','66','27935.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fifengine','304','29260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fillets-ng','14','29131.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fillets-ng-data','74','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/firestorm','88','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/firestorm-opensim','93','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fizmo','9','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/flare','28','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/flare-game','72','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/foobillardplus','23','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fortune-ASR','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fortune-dharma','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fortune-dune','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fortune-farscape','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fortune-futurama','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fortune-game_of_thrones','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fortune-noobfarm','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fortune-slackware','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fortune-starwars','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fortune_doctor_who','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fortune_firefly','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/freeblocks','4','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/freeciv','148','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/freedink','54','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/freedoom','26','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/freedroid','7','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/freedroidrpg','123','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/freeminer','164','28171.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/freeorion','573','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/frogatto','164','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/frotz','5','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/frozen-bubble','16','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fs-uae','61','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fs-uae-arcade','27','27862.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fs-uae-launcher','28','31070.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fs2open','111','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/fuse-emulator','14','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/game_1945','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/garden','7','34610.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gargoyle','37','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gbrainy','17','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gemrb','48','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ggmud','20','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gl-117','9','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/glbsp','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/glestae','109','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/glob2','251','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gltron','10','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/glulxe','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/glxosd','15','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gnome-chess','10','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gnome-klotski','7','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gnome-mahjongg','14','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gnome-mines','9','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gnonograms','4','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gnubg','41','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gnugo','12','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gnurobbo','4','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gnuski','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/golly','26','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/goonies','11','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gottet','4','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/greed','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gtetrinet','13','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gtkballs','7','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gtklife','5','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gzdoom','108','28766.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/gzdoom-legacy','58','32180','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/hack-of-life','4','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/hatari','68','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/hatari_tos_roms','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/haxima-nazghul','13','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/hedgewars','151','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/heretic_shareware_data','3','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/heroes','20','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/hex-a-hop','10','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/hexalate','3','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/hexen_demo_data','5','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/hexglass','4','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/higan','48','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/hlsdk','10','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/hnefatafl','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ho1','8','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/homm3_shareware_data','55','31209.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/iagno','26','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/icebreaker','2','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ifm','6','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/img2xterm','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/innoextract','19','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/instead','7','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/invasion3d','4','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ioquake3','30','30722.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/iortcw','81','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jag','18','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/javacpc','18','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jezzball-kazzmir','3','34610.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jfsw','15','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jfsw_demo_data','10','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jfsw_hires_pack','37','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jfsw_registered_data','118','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jfsw_twin_dragon','21','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jfsw_wanton_destruction','19','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jg-bsnes','10','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jg-cega','10','26938.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jg-gambatte','4','26938.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jg-genplus','9','26938.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jg-geolith','8','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jg-jollycv','3','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jg-mednafen','121','26938.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jg-mgba','6','26938.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jg-nestopia','10','35185.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jg-picodrive','47','26938.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jg-prosystem','2','26938.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jg-sameboy','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jg-vecx','3','26938.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jin','7','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jollygood','4','26938.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/joy2key','3','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/julius-game','24','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/jzintv','27','28047','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/kardsgt','93','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/kildclient','10','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/kmuddy','42','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/koules','5','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/larn','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/lbreakout2','13','28747.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/lbreakouthd','14','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/lgogdownloader','25','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/lightyears','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/lincity-ng','70','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/liquidwar6','50','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/lmarbles','5','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/lmpc','38','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/loemu','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ltris','9','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/luanti','144','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/lutris','7','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/m64py','3','34610.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/madbomber','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/maelstrom','7','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mame','2495','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mame2000-libretro','30','33022.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/manaplus','106','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mangband','12','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/marathon-data','12','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/marathon-eternal-data','152','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/marathon-evil-data','21','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/marathon-infinity-data','21','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/marathon-phoenix-data','104','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/marathon-red-data','16','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/marathon-rubicon-data','64','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/marathon2-data','20','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mari0','2','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mars','49','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/meandmyshadow','24','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mednafen','108','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mednaffe','5','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/megaglest','538','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/megamario','39','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mesen-x','112','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mfe','1','28628.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mgba','64','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/micropolis','30','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/minecraft','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/minigalaxy','3','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mininim','50','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/miscom','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mog','21','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/moon-buggy','4','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/moria','3','29349.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mrboom','9','41948','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mrrescue','5','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mupen64plus','75','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mupen64plus-extraplugins','35','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mupen64plus-input-raphnetraw','2','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mupen64plus-nightly','75','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mupen64plus-rsp-parallel','9','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mupen64plus-video-angrylionplus','10','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mupen64plus-video-gliden64','33','30722.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/mupen64plus-video-parallel','10','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/myman','142','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nFlood','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nSnake','3','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/naev','118','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nblood','94','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nbsdgames','2','30544.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ncursesoflife','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nestopia','25','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/neverball','105','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nevernoid','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nexuiz','233','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ninvaders','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/njam','6','28747.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nlarn','3','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/noteye','30','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/notpacman','2','29131.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nottetris2','2','29131.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/npush','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nsudoku','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nsuds','4','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nuvie','36','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/nxengine-evo','103','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/o2em','2','32180','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/oblige','12','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/oblige-legacy4','9','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/odamex','165','33022.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/onscripter','3','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/oolite','88','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/open-adventure','5','28047','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/open-invaders','7','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openarena','110','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openclonk','204','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/opendune','13','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openjazz','7','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openlierox','218','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openmortal','77','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openmsx','216','31070.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openmsx-catapult','46','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openmsx-debugger','19','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openmw','344','31818.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openrct2','236','33780','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/opensonic','9','34610.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openspades','44','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/opensurge','21','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openttd','258','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/opentyrian','12','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/openyahtzee','7','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ostrichriders','7','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ovcc','6','31218.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pacman','6','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pacman-arena','15','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pangzero','3','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/paranoia','2','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pasang-emas','11','28047','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pathways-into-darkness','21','28571.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pcsx-rearmed','19','26938.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pcsxr','28','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/peg-e','4','28047','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/penguin-command','5','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pengupop','6','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pentagram','51','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pgn-extract','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pinball','96','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pingus','165','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pioneer','351','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pipewalker','6','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/planetblupi','52','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/playonlinux','5','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pokerth','218','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pom1','4','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/powerslave_demo_data','11','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ppsspp','271','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/prboom','13','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/prboom-plus','15','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/principia','53','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/protontricks','2','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/protonup','2','34729.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/puNES','49','34729.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/puzzles','32','28540.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pybik','182','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pychess','13','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pysolfc','34','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/pysolfc-extra-cardsets','28','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/qmc2','215','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/qtads','28','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/quackle','63','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/quake3_shareware_data','21','30722.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/quake_shareware_data','7','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/quakeforge','117','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/quakespasm','7','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/quarry','10','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/qxw','4','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/qzdl','13','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/qzdoom','75','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/redeclipse','423','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/rejoystick','7','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/residualvm','39','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/retux','19','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/rezerwar','3','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/rigel-engine','57','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/rlvm','40','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/roadfighter','6','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/robotfindskitten','3','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/roccat-tools','36','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/rocksndiamonds','21','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/rogue','5','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/roll','3','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/rolldice','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/rott','51','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/rottexpr','79','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/sameboy','7','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/sauerbraten','389','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/scid','13','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/scid_vs_pc','44','28144.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/scrabbleai','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/scummvm','700','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/sdlpop','4','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/shattered-pixel-dungeon','72','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/simplesok','3','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/simsu','5','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/singularity','49','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/skulltag','14','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/sl','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/smashbattle','7','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/smc','98','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/smokinguns','107','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/sms_sdl','4','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/snes9x','85','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/snes9x2010','44','33022.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/sokowiz','5','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/solarus','57','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/solarus-quest-editor','69','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/solarwolf','7','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/speed-dreams','296','31818.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/spring','312','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/srb2','97','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/starfighter','25','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/starlanes','1','30722.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/steam','2','27862.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/steamtinkerlaunch','3','32180','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/stella','114','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/stepmania','151','28171.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/stockfish','100','28273','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/stormbaancoureur','4','32180','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/stratagus','39','27862.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/stuntrally','307','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/sumeria','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/supermariowar','33','31029.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/supertux','214','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/supertuxkart','345','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/surgescript','4','27986.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/syobonaction','6','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/taipan','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/talkfilters','10','30544.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tanglet','20','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tbftss','35','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/teeworlds','5','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tetrinet','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tetrinetx','3','28816.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tint','2','28628.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tintin','7','28887.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tkgames','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tome','283','27862.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tome-ah','53','28047','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tome-sx','14','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tong','20','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/torcs','280','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/trackballs','19','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/trigger-rally','73','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/triplane','5','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/triplea','94','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/trs80gp','5','28651.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tt','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tty-solitaire','2','28788.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tuxfootball','26','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tuxkart','9','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tuxnes','8','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/tworld','3','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/typespeed','7','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/typhoon_2001','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ufoai','385','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/uligo','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ultimatestunts','82','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/unknown-horizons','110','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/unnethack','14','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/uqm','14','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/uqm-megamod','223','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/uqm_3domusic','7','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/uqm_remixes','106','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/uqm_voice','54','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ut2004','19','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ut99','204','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vbam','34','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vcmi','343','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vectoroids','3','30722.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vegastrike','450','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vera','9','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vice-libretro','69','26938.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vitaquake2-libretro','32','33022.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vitetris','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vkBasalt','11','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/vms-empire','1','28788.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/voxelands','73','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/wadext','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/wargus','31','27862.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/warsow','185','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/warzone2100','1250','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/waterCloset','5','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/wesnoth','518','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/widelands','697','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/wolf4sdl','10','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/wolfmame','2789','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/wordwarvi','7','28082.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/worldofpadman','319','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/wyrmsun','280','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xarchon','8','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xbill','4','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xconq','5','28972.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xcowsay','3','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xcpc','32','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xlennart','4','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xmahjongg','6','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xmoto','92','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xonotic','361','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xpilot-ng','19','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xroar','12','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xroar-roms','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xskat','3','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xspacewarp','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xtetris','3','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xu4','30','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xu4-recreated','8','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/xye','9','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/yabause','31','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/yae','6','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/yahtzee','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/yamagi-quake2','17','35185.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/yar','6','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/yarntown','12','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/yetris','4','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zapm','1','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zaz','17','35185.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zbom','97','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zdbsp','3','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zdoom','41','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zelda-roth-se','11','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zelda-xd2-mercuris-chess','23','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zennode','3','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/znake','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zod-engine','38','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zoom','13','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zork','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zork-mit','21','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zsdx','17','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zseal','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zsnes','6','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/zsxd','8','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ztools','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('games/ztrack','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/Fiona','84','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/GMT','130','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/Geohash','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/OTB','473','31218.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/OTB-legacy','728','31818.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/OWSLib','6','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/SFCGAL','306','27479.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/SFCGAL-legacy','115','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/TauDEM','26','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/cligj','1','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/foxtrotgps','7','28047','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/gdal','339','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/geographiclib-python','2','34287.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/geojson','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/geopandas','7','28215.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/geopy','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/geos','62','29943.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/gj2ascii','2','31818.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/gmapcatcher','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/google-earth-legacy','93','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/gpsbabel','19','29922.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/gpsd','86','28047','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/gpxsee','69','29443.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/grass','204','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/josm','4','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/laszip','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/libLAS','30','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/libecwj2','41','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/libgeotiff','6','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/librasterlite','17','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/librttopo','18','29943.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/libspatialindex','19','34610.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/libspatialite','54','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/mapnik','1310','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/merkaartor','67','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/navit','30','28047','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/opencpn','144','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/opencpn-plugin-oesenc','19','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/opengrads','23','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/openorienteering-mapper','457','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/osgEarth','315','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/osm-gps-map','23','28047','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/osm2pgrouting','8','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/osm2pgsql-legacy','39','33780','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/ossim','204','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/pdal','182','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/pgrouting','52','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/pgsql-ogr-fdw','2','31218.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/pktools','46','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/pointcloud','6','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/polyline','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/postgis','75','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/proj','58','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/proj-data','202','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/python-mapnik','320','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/python-pygeos','15','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/python3-basemap','110','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/python3-cartopy','24','31758.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/python3-cftime','24','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/python3-netcdf4-python','48','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/python3-pyproj','42','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/python3-pyshp','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/python3-shapely','20','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/python3-xarray','10','28096.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/pytrainer','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/qgis','2117','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/rasterio','101','31818.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/readosm','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/saga-gis','169','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/shapelib','8','30544.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/spatialite-tools','19','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/survex','26','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/tangogps','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/therion','57','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/viking','22','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/whitebox-tools','144','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('gis/xplanet','14','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/ART','145','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/BOSL2','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/Blender','539','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/CairoSVG','2','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/FSPViewer','3','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/FotoKilof','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/FreeCAD','1880','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/GPS','9','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/GraphicsMagick','45','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/HotShots','18','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/LDView','29','28788.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/LDraw_data','171','28788.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/MaterialX','188','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/OpenCASCADE','794','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/OpenShadingLanguage','197','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/OpenSubdiv','48','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/Pixie','71','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/PyMOL','46','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/Shellpic','1','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/SweetHome3D','64','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/VariCAD','91','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/XnViewMP','67','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/aaphoto','6','28628.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/abcm2ps','3','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/advancecomp','5','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/aewan','3','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/alembic-framework','56','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/amide','93','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/apitrace','81','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/asciiquarium','1','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/autotrace','18','29914.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/azpainter','8','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/baires','4','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/barcode','23','32180','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/bdfedit','1','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/birdfont','71','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/blockbench','107','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/brlcad','751','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/cam','7','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/catimg','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/chafa','15','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/converseen','11','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/ctpv','1','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/dcraw','7','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/deskew','3','28628.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/dia','85','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/djview4','16','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/dxftoqet','44','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/enblend-enfuse','92','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/entangle','20','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/eog','34','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/eog-plugins','9','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/evolus-pencil','4','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/exact-image','30','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/exif','6','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/exiftags','1','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/farbfeld','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fbgrab','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fbida','3','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fbpic','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fbv','2','34610.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/feh','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fig2sxd','3','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fim','107','28082.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/findimagedupes','1','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/flam3','8','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/flameshot','43','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/flpsed','5','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/focus-stack','10','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fontforge','48','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fotowall','31','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/frescobaldi','13','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fritzing','125','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fstl','7','31209.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/fyre','6','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/g3dviewer','6','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gallery','4','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gbdfed','6','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gcolor2','9','29260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gcolor3','4','30722.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gdk-pixbuf-psd','12','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gifsicle','8','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gimagereader','18','34287.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gimp-feca_hdr-plugin','1','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gimp-lqr-plugin','7','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gimp-plugin-bimp','6','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gimp-plugin-export-layers','4','29131.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gimp-wideangle-plugin','1','31239.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gliv','17','28047','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/glmark2','24','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gmic','445','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gnofract4d','16','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gnome-themes-extras','141','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/goocanvas','10','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/goocanvas2','12','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gource','18','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gpaint','4','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gpick','25','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gpicview','6','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gpscorrelate','3','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gqview','10','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/grafx2','14','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/graph-easy','9','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/graphviz','102','29706.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gscan2pdf','3','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gthumb','49','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/gtkam','9','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/heif-gimp-plugin','7','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/hp2xx','5','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/hugin','149','29131.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/icc_profiles','1','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/icc_profiles_from_eci','12','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/img2pdf','2','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/imgcurses','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/imgmin','6','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/imgp','1','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/implot','44','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/imv','5','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/inkscape','569','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/inkscape-gtk2','312','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/ipe','19','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/jUploadr','3','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/jhead','1','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/jp2-pixbuf-loader','3','30733.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/jp2a','6','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/jpeg2ps','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/jpegoptim','4','35185.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/kColorPicker','4','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/kImageAnnotator','27','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/kgeotag','11','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/kim4','1','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/klayout','2217','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/kphotoalbum','77','33022.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/ksnip','31','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/kuickshow','12','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/lcdtest','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/lddot','1','30733.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/ldglite','2','28788.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/lensfun03','4','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/leocad','31','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/leocad_pieces','37','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/libfpx','11','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/libplacebo','8','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/librecad','194','31070.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/libresprite','122','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/lilypond','141','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/luminance-hdr','51','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/lximage-qt','12','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/maim','7','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/makehuman','6','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mangohud','27','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mapcrafter','22','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mcomix','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/meh','1','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/meshlab','335','31070.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/metapixel','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mftrace','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mirage','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mitsuba2','300','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mozjpeg','9','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mtpaint','7','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mypaint','31','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/mypaint-brushes2','5','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/nomacs','33','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/nsxiv','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/nvidia-cg-toolkit','28','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/nvidia-texture-tools','15','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/openclipart','399','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/opencollada','179','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/opencolorio','231','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/openimageio','169','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/openscad','180','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/openscad-gearbox','1','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/openscad-mcad','1','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/openscad-threads','1','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/opentoonz','418','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/optipng','3','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/oyranos','20','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pcx-pixbuf-loader','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pd-gears','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pdf2png','1','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pdf2svg','3','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pdfsyntax','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/peek','7','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pencil2d','36','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pencil3','128','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/phatch','6','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pho','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/photoqt','31','32883.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/phototonic','21','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/plotutils','33','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/png2ico','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pngcheck','6','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pngcrush','11','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pngquant','3','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/posterazor','3','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/potrace','7','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/povray','114','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pqiv','4','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pr0ntools','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/psftools','7','27973.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pyformex','11','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/pygraphviz','5','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/python3-gds3xtrude','2','31364.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/qcad','651','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/qcomicbook','18','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/qcustomplot','30','31239.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/qelectrotech','200','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/qimgv','39','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/qiv','2','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/ralcgm','4','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/raw-thumbnailer','5','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/rawtherapee','138','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/rawtherapee-haldclut','113','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/realesrgan-ncnn-vulkan','77','31383.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/resynthesizer','9','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/ristretto','6','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/scantailor-advanced','99','28741.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/scour','1','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/screencloud','19','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/screengrab','10','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/scrot','3','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/shalbum','1','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/shotwell','51','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/shutter','6','29131.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/simple-scan','16','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/sng','3','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/stltools','1','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/svp','1','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/swappy','3','28651.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/sxiv','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/synfig','178','27533.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/synfigstudio','166','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/tclblt','52','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/teighafileconverter','32','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/tesseract','97','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/textext','1','29404.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/tgif-QPL','9','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/tikzit','14','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/tintii','34','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/tkpng','4','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/triangulizor','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/trimage','2','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/ttfautohint','16','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/ttyload','1','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/tuxpaint','80','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/tuxpaint-stamps','51','30544.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/ueberzug','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/ueberzugpp','14','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/ufraw','48','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/uniconvertor','6','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/unpaper','4','28628.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/urbanlightscape','32','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/veles','111','28132.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/viewnior','5','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/vimiv-qt','4','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/vips','36','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/vuescan','22','41948','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/wavelet-denoise','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/webp-pixbuf-loader','5','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/white_dune','128','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/whyteboard','2','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/wine-nine-standalone','5','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xbmbrowser','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xcalib','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xcf-pixbuf-loader','13','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xcur2png','4','31218.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xdaliclock','4','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xdot','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xfishtank','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/ximaging','3','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xli','3','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xloadimage','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xmedcon','14','27606.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xnap','1','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xpe','54','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xviewer','44','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/xzgv','3','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/yacreader','167','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/yafaray','33','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/yagf','15','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/yed','15','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/yvonne','3','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/zbar','58','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/zgrviewer','3','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('graphics/zgv','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/7plus','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/CubicSDR','30','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/DroidStar','13','30733.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/QtSoundModem','9','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/QtTermTCP','9','33022.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/SDRPlay','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/SoapyFobosSDR','3','27903.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/SoapyRTLSDR','4','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/SoapyRemote','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/SoapySDR','35','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/SoapySDRPlay','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/Xastir','25','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/aldo','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/antscope2','40','30722.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/aprx','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/aptdec','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/ax25-apps','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/ax25-tools','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/baudline','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/chirp','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/cm256cc','2','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/codec2','7','27608.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/cqrlog','31','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/csdr','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/cwdaemon','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/cwstation','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/dab_lib','4','28273','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/demorse','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/direwolf','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/dsdcc','4','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/dxcc','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/ebook2cw','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/ebook2cwgui','4','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/fbb','13','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/fldigi','46','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/flmsg','16','31383.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/flrig','37','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/freedv','61','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/ggmorse','4','28972.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/glrpt','4','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/gqrx-sdr','28','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/gr-gsm','68','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/gridloc','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/gridtracker','26','28047','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/grig','5','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/gsmlib','17','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/hackrf','7','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/hamclock','10','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/hamlib','24','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/hamlib4','62','29304.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/inmarsatc','5','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/inspectrum','8','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/js8call','114','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/jtdx','72','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/jtdxhamlib','83','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/klog','115','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/libax25','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/libfobos','2','30296.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/libfobos-sdr-agile','2','27903.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/libosmocore','80','29706.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/libsigmf','14','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/linpsk','12','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/liquid-dsp','15','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/lpcnetfreedv','22','27608.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/lpsk31','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/mbelib','2','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/multimon-ng','4','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/nec2c','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/psk31lx','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/qlog','110','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/qrq','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/qsstv','60','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/redsea','7','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/rtl-sdr','4','36494.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/rtl_433','10','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/satdump','242','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/sdrangel','648','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/sdrplay_apiService','2','31209.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/sdrpp','44','28573','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/serialDV','6','29355','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/sgp4','5','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/soundmodem','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/spectools','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/svxlink','42','28171.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/svxlink-sounds-en_US-heather-16k','12','28747.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/tlf','7','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/tqsl','26','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/tucnak','23','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/twhamqth','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/unixcw','9','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/wsjtz','112','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/wxapt','4','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/xanalyser','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/xdemorse','9','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/xdx','7','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/xfhell','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/xgridloc','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/xlog','12','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/xnec2c','17','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/xpsk31','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/xwxapt','5','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ham/yagiuda','6','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/cabal-install','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/ghc','1555','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/ghcup','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-Crypto','42','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-Diff','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-FindBin','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-GLURaw','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-GLUT','23','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-Glob','9','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-HTTP','21','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-HUnit','10','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-HsYAML','39','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-JuicyPixels','197','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-ObjectName','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-OneTuple','6','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-OpenGL','96','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-OpenGLRaw','253','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-QuickCheck','35','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-SHA','30','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-ShellCheck','316','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-StateVar','4','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-Tensor','9','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-X11','49','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-X11-xft','5','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-aeson','120','27274.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-aeson-compat','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-aeson-pretty','9','28401.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-annotated-wl-pprint','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-ansi-terminal','11','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-ansi-wl-pprint','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-appar','7','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-asn1-encoding','9','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-asn1-parse','4','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-asn1-types','12','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-assoc','5','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-async','6','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-attoparsec','52','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-attoparsec-iso8601','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-auto-update','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-base-compat','19','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-base-compat-batteries','19','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-base-orphans','9','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-base-prelude','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-base16-bytestring','5','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-base64-bytestring','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-basement','68','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-bifunctors','23','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-bindings-uname','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-bitarray','6','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-blaze-builder','9','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-blaze-html','28','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-blaze-markup','15','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-bmp','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-byteable','3','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-byteorder','4','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-bytestring-builder','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cabal-doctest','6','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-call-stack','5','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-case-insensitive','5','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cereal','24','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cgi','9','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cipher-aes','9','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cipher-des','7','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cipher-rc4','5','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-citeproc','112','30722.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-clock','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cmark','30','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cmark-gfm','33','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cmdargs','29','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-colour','17','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-commonmark','78','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-commonmark-extensions','37','30722.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-commonmark-pandoc','7','28401.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-comonad','9','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-composition','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-conduit','27','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-conduit-extra','15','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-connection','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-constraints','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-contravariant','6','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cookie','6','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-crypto-cipher-types','7','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-crypto-numbers','9','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-crypto-pubkey','9','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-crypto-pubkey-types','16','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-crypto-random','6','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cryptohash','13','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cryptohash-sha256','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cryptonite','124','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-cryptonite-conduit','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-curl','15','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-data-default','4','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-data-default-class','4','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-data-default-instances-base','3','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-data-default-instances-containers','4','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-data-default-instances-dlist','3','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-data-default-instances-old-locale','3','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-data-fix','5','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-data-ordlist','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-dataenc','12','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-deepseq-generics','4','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-digest','4','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-distributive','5','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-dlist','5','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-doclayout','11','30722.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-doctemplates','22','28401.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-double-conversion','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-easy-file','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-echo','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-ed25519','15','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-edit-distance','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-either','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-emojis','28','30722.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-enclosed-exceptions','4','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-env-locale','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-erf','5','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-errors','6','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-extensible-exceptions','4','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-extra','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-fail','3','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-fast-logger','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-fgl','21','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-file-embed','6','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-filelock','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-filemanip','7','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-fixed','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-foundation','68','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-free','34','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-fsnotify','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-generic-deriving','44','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-ghc-paths','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-gitrev','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-gloss','16','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-gloss-rendering','13','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-graphviz','159','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-haddock-library','21','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-half','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hashable','9','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-highlighting-kate','135','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hinotify','7','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hostname','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hourglass','19','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hpack','59','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hslua','5','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hslua-aeson','5','28401.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hslua-classes','6','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hslua-core','10','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hslua-marshalling','8','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hslua-module-path','6','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hslua-module-system','6','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hslua-module-text','5','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hslua-module-version','5','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hslua-objectorientation','7','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hslua-packaging','10','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hsp','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hspec','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hspec-core','29','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hspec-discover','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hspec-expectations','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hspec-smallcheck','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-html','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-http-client','23','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-http-client-tls','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-http-conduit','10','30722.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-http-types','9','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hxt','141','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hxt-charproperties','32','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hxt-regex-xmlschema','19','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-hxt-unicode','11','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-idna','8','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-indexed-traversable','11','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-indexed-traversable-instances','5','30722.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-infer-license','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-integer-logarithms','5','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-iproute','23','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-ipynb','13','28401.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-iwlib','4','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-jira-wiki-markup','40','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-json','13','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-lcs','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-lexer','54','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-libyaml','22','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-lifted-async','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-lifted-base','6','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-logict','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-lpeg','10','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-lua','30','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-lukko','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-memory','16','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-microlens','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-microlens-mtl','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-microlens-th','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-mime-types','21','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-mintty','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-mmap','5','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-mmorph','5','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-monad-control','5','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-monad-logger','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-monad-loops','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-monads-tf','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-mono-traversable','46','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-multipart','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-nats','2','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-network','22','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-network-bsd','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-network-uri','11','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-old-locale','5','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-old-time','10','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-open-browser','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-optparse-applicative','16','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-pandoc-lua-marshal','21','28401.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-pandoc-types','56','26411.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-parsec-numbers','5','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-parser-combinators','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-path','15','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-path-io','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-path-pieces','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-pem','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-pipes','12','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-polyparse','26','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-pretty-show','19','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-primitive','21','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-profunctors','19','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-project-template','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-publicsuffixlist','10','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-punycode','9','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-quickcheck-io','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-random','10','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-regex-applicative','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-regex-applicative-text','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-regex-base','7','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-regex-compat','4','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-regex-compat-tdfa','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-regex-pcre','13','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-regex-pcre-builtin','35','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-regex-posix','8','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-regex-tdfa','51','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-resolv','31','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-resource-pool','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-resourcet','8','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-retry','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-rio','28','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-safe','7','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-sandi','13','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-scientific','12','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-securemem','4','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-semialign','11','30722.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-semigroupoids','23','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-semigroups','4','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-setenv','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-setlocale','4','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-shelly','25','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-silently','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-skylighting','31','29023.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-skylighting-core','65','28401.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-smallcheck','15','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-socks','13','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-split','5','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-splitmix','6','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-stm-chans','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-store-core','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-streaming-commons','15','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-strict','11','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-stringprep','18','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-stringsearch','15','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-syb','8','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-system-fileio','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-system-filepath','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-tagged','6','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-tagsoup','35','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-tar','15','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-temporary','5','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-test-framework','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-test-framework-hunit','4','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-test-framework-quickcheck2','5','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-texmath','146','28401.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-text-conversions','4','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-text-icu','45','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-text-metrics','14','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-text-short','8','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-tf-random','7','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-th-abstraction','13','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-th-compat','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-th-expand-syns','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-th-lift','6','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-th-lift-instances','5','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-th-orphans','15','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-th-reify-many','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-th-utilities','15','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-these','8','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-time-compat','12','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-time-locale-compat','3','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-tls','68','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-transformers-base','5','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-transformers-compat','6','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-type-equality','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-typed-process','8','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-unicode-collation','42','30722.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-unicode-transforms','49','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-uniplate','18','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-unix-compat','5','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-unix-time','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-unliftio','18','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-unliftio-core','4','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-unordered-containers','30','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-utf8-string','8','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-uuid-types','7','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-vector','128','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-vector-algorithms','33','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-void','3','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-witherable','12','30722.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-wl-pprint-text','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-x509','16','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-x509-store','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-x509-system','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-x509-validation','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-xml','13','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-xml-conduit','31','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-xml-types','13','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-yaml','17','28401.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-zeromq4-haskell','15','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-zip-archive','13','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/haskell-zlib','12','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/hscolour','18','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('haskell/xmonad-contrib','234','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/2geom','30','29601.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Atlas-C++','13','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/BeautifulSoup','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Botan','84','30480.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Botan3','199','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/CCfits','14','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/CEGUI','41','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/CEGUI0.7','83','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/CGAL','38','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/CGAL5','72','27479.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/CLI11','13','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/CoinMP','180','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/CondConfigParser','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ConfigParser','1','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Crypt-SSLeay','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/DevIL','6','29706.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ETL','2','28790.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/FormEncode','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/FreeImage','36','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/GLee','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/GeographicLib','24','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Geraldo','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/GitPython','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/HawkNL','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Hypodermic','19','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/IMAPClient','1','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Impacket','9','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Ipopt','32','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/JSBSim','31','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Kivy','84','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/LuaBitOp','2','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/LucenePlusPlus','76','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/MDI_Library','8','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Mustache','19','29608.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/MyGUI','75','28047.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ORBit2','17','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/OpenBLAS','281','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/OpenCSG','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/OpenSceneGraph','546','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/PDCurses','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/PDFlib-Lite','30','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/PLUMED','115','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/PrettyTable','2','26938.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/PyODE','2','32180','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/PyOpenGL','22','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/PySoundCard','1','28082.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/PyXML','8','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Pykka','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/QDarkStyle','4','29108.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/QR-Code-generator','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/QUSB','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/QtUsb','7','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/RxCpp','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SDL2_Pango','13','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SDL2_sound','10','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SDL3','47','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SDL3_ttf','6','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SDL_Pango','10','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SDL_audiolib','11','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SDL_gfx','10','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SDL_kitchensink','3','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SFGUI','16','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SFML','14','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SOPE','47','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SQLObject','8','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SimGear','103','31818.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/SlowAES','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/Smarty','1','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/VTK','1716','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/VitaMTP','13','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/WebOb','1','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/activemq-cpp','72','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/adns','4','28004.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/adodb','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/afflib','27','28273','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/agar','37','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/agg','20','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ahven','9','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/alglib','68','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/allegro','34','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/allegro4','30','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/alure','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/aml','2','28649.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ant-contrib','1','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/antlr4','38','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/apiextractor','10','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/appstream-glib','18','28747.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/aqbanking','116','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/argtable','5','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/aribb24','12','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/aribb25','12','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/asio','54','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/assimp','127','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/async','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/aubio','8','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/audiere','15','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/audioread','2','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/aunit','10','34610.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/avbin','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/avr-libc','83','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/awscrt','48','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ayatana-ido','7','28747.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/backports-ssl-match-hostname','1','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/bamf','21','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/barry','47','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/batinfo','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/bcdoc','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/bctoolbox','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/bitsery','1','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/bitstream','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/blas','9','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/blis','28','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/bluez-alsa','19','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/boolstuff','8','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/botocore','55','28312.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/bsddialog','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/buddy','7','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/bullet','165','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/bwidget','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/bzrtp','7','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/c++-gtk-utils','16','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/c-blosc','15','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cal3d','11','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/camlpdf','16','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/canfigger','3','28692.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/capnproto','51','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cchardet','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ccrtp','8','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cdk','39','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cereal','74','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ceres-solver','238','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cffi','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cgicc','17','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cglm','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cheapglk','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/chromaprint','15','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/clalsadrv','1','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cln','51','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cloog','14','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/clutter','32','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/clutter-gst','11','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/clutter-gtk','9','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cmark','5','29946','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/coeurl','5','27172.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cogl','94','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/collada-dom','85','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/compface','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/compiz-bcop','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/compiz-plugins-experimental','35','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/compiz-plugins-extra','22','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/compiz-plugins-main','26','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/compizconfig-python','7','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/confuse','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/console_bridge','8','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cpp-httplib','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cpp-jwt','2','33780','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cpp-netlib','13','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cpprestsdk','133','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cppzmq','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/crossguid','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ctemplate','20','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cuneiform','54','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/curlpp','4','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/curses_ex','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cutefish-kwin-plugins','5','33780','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cutefish-qt-plugins','9','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/cyrus-sasl-xoauth2','8','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/daq','28','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/date','5','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/dav1d','17','27838.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/dbcsr','67','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/dbus-cpp','27','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/dcadec','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/dcmtk','194','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/dietlibc','8','33022.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/digimend-kernel-drivers','4','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/dlib','49','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/dmlc-core','14','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/docopt.cpp','9','29386.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/double-conversion','3','28706.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/dpdk','197','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/dropbox-python','5','28723.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/duktape','9','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/dumb','8','34610.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/eb','17','29355','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/edelib','28','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/eduvpn-common','19','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/efl','214','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ell','7','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/enca','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/eris','11','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/evernote-python','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/exempi','33','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/exiftool','45','29131.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fast_float','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/faun','2','30722.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fcft','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fcgi','14','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fcitx5-rime','6','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fcl','107','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ffms2','16','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/field3d','92','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fifechan','8','34610.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fishui','8','33780','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/flatbuffers','34','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/flatzebra','8','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/flexiblas','534','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/flickcurl','11','28047','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/flint','61','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/florist','75','34610.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fltk','19','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fmodapi','4','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fmt','59','29706.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/folks','13','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fox-toolkit','61','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fox-toolkit-devel','112','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fpconst','1','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/freealut','7','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/freexl','7','41948','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fstrcmp','7','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fstrm','15','31209.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ftgl','21','28047','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ftplib','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/function2','14','27608.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/futures','1','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fuzzylite','23','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/fzssh','11','28034.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gallerize','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gamemode','6','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ganv','9','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gavl','29','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gcab','9','28747.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gconfmm','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gdata','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gdl','12','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/genshi','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/geocode-glib','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/geocode-glib2','6','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/geshi','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gevent','90','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gf2x','17','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gfbgraph','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gflags','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ghc_filesystem','24','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/giblib','12','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/girara','5','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gismo','151','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gitdb','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gitdb2','1','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gl2ps','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/glfw3','13','33780','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/glktermw','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/glog','8','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gloox','72','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/glw','10','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gmtk','7','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gnatcoll-bindings','92','28165.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gnatcoll-core','138','34729.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gnatcoll-db','279','28784.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gnet','9','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gnome-autoar','7','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gnome-desktop','14','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gnome-media','20','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gnome-sharp','11','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gnome-vfsmm','18','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/goffice','38','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/goffice0.8','37','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gom','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/google-fruit','9','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/googlebenchmark','22','27608.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gperftools','32','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/granite','12','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/greenlet','6','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/grilo','18','30544.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/grilo-plugins','10','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/grpc','563','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/grx','21','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gsm','5','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gsnmp','7','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gsound','5','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gspell','14','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gss-ntlmssp','15','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gssdp','7','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gst-editing-services','32','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gstreamermm','43','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtk-aurora-engine','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtk-fortran','10','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtk-layer-shell','10','28132.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtk-session-lock','8','31239.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtk-sharp','28','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtk-sharp3','10','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtk3-nocsd','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtkada','913','28143.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtkdatabox','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtkglarea','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtkglext','23','41948','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtkglextmm','25','28047','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtkhtml','29','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtkimageview','11','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtkmm-utils','19','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtksourceview','12','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtksourceview4','15','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtksourceview5','20','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gtkspell3','10','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gts','13','29706.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/guichan','14','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gumbo-parser','5','35185.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gupnp','9','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gupnp-av','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gupnp-dlna','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/guppy','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/gwenhywfar','48','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hdf','19','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hdf5','141','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hidapi','3','31218.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/highway','22','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hiredict','2','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hiredis','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hoedown','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hoel','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hptt','60','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/html5lib','2','29706.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/htmlcxx','7','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/htslib','19','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/http-parser','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hurry-filesize','1','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hyperscan','219','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/hypre','35','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/iddawc','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ignition-math2','16','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/iir1','6','31818.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/iksemel','8','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/imageloader','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/imbe_vocoder','2','27744','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/imgui','8','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/imlib','12','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/imlib2','16','28897','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/imlib2_loaders','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/iniparser','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ipaddress','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/irrlicht','33','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/irrlichtmt','16','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/itcl','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/jbig2dec','15','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/jbig2enc','15','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/jbigkit','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/jcal','13','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/jogamp','293','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/jreen','19','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/json-parser','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/jsoncpp','9','28171.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/jsonrpc-glib','10','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/kaadbg','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/kaidan','25','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/kashmir','1','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/kddockwidgets','57','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/keybinder','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/kirigami-addons','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/kquickimageeditor','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lablgtk','23','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lapack','55','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ldacBT','3','31218.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/leptonica','34','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/levmar','3','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lgi','2','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lib3ds','9','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lib3mf','45','31364.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libADLMIDI','6','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libAfterImage','9','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libEMF','8','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libQuotient','28','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libROM','14','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libXCFun','44','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libaacs','7','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libaaio','12','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libaal','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libabw','17','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libadwaita','16','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libaec','3','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libagentcrypt','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libantlr3c','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libaosd','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libappimage','7','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libaribcaption','5','35185.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libart-lgpl','6','29706.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libasr','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libass','5','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libaudclient','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libavc1394','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libavif','13','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libayatana-appindicator','9','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libayatana-appindicator-glib','4','28124','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libayatana-indicator','4','28747.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libb64','1','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libbass','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libbassmidi','1','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libbde','21','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libbdplus','6','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libbonobo','23','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libbonoboui','21','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libbpf','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libbroadvoice','11','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libbsd','20','31029.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libbtbb','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcacard','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcaes','9','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcangjie','7','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcapsimage','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcbor','5','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libccd','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcdaudio','7','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcdr','31','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcec','13','28784.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libchamplain','11','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libchardet','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libchewing','8','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcifpp','35','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcint','11','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcitadel','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libclaw','10','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcli','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcmis','57','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcoap','73','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcompizconfig','11','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libconfig','4','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcpuid','19','31209.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcreg','17','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcryptui','15','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcsv','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libctl','9','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcuefile','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcurl-gnutls','27','28074','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libcutefish','23','33780','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdaemon','6','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdart','85','31818.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdatachannel','26','33780','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdazzle','20','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdbh2','6','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdbi','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdbi-drivers','11','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdbus-c++','16','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdc1394','9','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdca','12','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libde265','18','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdex','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdispatch','10','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdisplay-info','4','27608.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdivecomputer','7','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdmapsharing','7','31383.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdockapp','9','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdraco','88','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdsk','7','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdv','8','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdvbcsa','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdvbpsi','7','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdvdcss','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdwarf','17','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdxfrw','11','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libdynamite','5','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libe-book','32','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libearth','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libeatmydata','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libebml','4','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libeditline','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libee','8','28628.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libei','7','30544.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libeot','9','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libepubgen','15','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libesedb','20','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libesmtp','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libestr','5','28628.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libetonyek','71','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libetpan','29','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libev','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libevhtp','6','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libevt','22','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libevtx','23','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libewf','31','28046.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libexe','17','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libexif-gtk','10','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libexttextcat','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfaketime','3','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfastjson','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfdk-aac','28','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libffado','216','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfido2','17','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfilezilla','24','28600.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfilteraudio','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfishsound','8','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfixposix','14','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfm','41','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfm-extra','12','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfm-qt','28','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfprint','16','31239.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfreehand','21','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfsapfs','22','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfsext','20','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfsfat','19','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfshfs','21','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfsntfs','22','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfsxfs','19','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libftdi','9','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libftdi1','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfvde','24','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfwnt','11','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfwsi','15','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libfxt','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libg15','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libg15render','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libg3d','36','28047','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libg7221','10','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgadu','7','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgaminggear','4','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgbinder','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgdata','15','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgdiplus','29','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgdsii','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgedit-amtk','8','27862.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgedit-gtksourceview','16','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgee1','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgig','30','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgit2','74','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgit2-glib','22','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgksu','13','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libglademm','17','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libglibutil','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libglpng','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgme','9','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnome','19','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnome-games-support','5','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnomecanvas','12','29706.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnomecanvasmm','22','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnomecups','9','29706.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnomekbd','8','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnomemm','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnomeprint','18','29706.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnomeprintui','12','29706.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnomeui','31','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgnomeuimm','27','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgpr','100','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgrapheme','6','31124.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgraphqlparser','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgringotts','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgrss','7','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgsasl','30','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgta','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgusb','7','31239.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgweather','25','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgweather4','26','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libgxps','8','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libhandy','30','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libharu','9','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libheif','39','28152','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libhid','8','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libhoard','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libhtp','15','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libical4-opt','61','28819.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libicns','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libiec61883','4','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libiio','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libilbc','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libindi','131','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libinklevel','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libinstpatch','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libiptcdata','7','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libircclient','7','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libirecovery','6','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libisoburn','7','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libisofs','7','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libixion','39','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libixp','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libjcat','8','27981.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libjodycode','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libjson-rpc-cpp','8','28171.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libjuice','3','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libjwt','15','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libjxl','68','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libkate','17','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libkiwix','15','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libkml','21','28152','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libkqueue','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libks','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblangtag','28','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblastfm-qt5','10','27973.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblc3','4','35185.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblcf','36','30015.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libldm','16','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libliftoff','3','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblightgrep','32','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblinebreak','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblnk','18','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblo','8','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libloc','35','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblogging','18','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblqr','6','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblrdf','10','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblscp','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libluksde','20','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblxi','5','31383.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblxqt','9','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblxrad','21','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liblzf','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmaa','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmatekbd','10','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmatemixer','8','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmateweather','43','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmatheval','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmatroska','6','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmd','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmediaart','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmediainfo','92','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmemcached','140','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmicrohttpd','11','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmigdb','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmikmod','8','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmimedir','6','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmimic','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libminizip','7','41948','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmirage','13','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmms','5','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmobi','9','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmodbus','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmodi','24','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmodplug','7','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmodsecurity','75','31029.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmp3splt','8','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmp4v2','19','28171.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmpd','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmpdclient','4','31029.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmpeg2','9','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmrss','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmseed','12','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmsiecf','17','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmspack','5','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmspub','12','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmsym','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmusicbrainz','8','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmusicbrainz3','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmusicbrainz5','10','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmwaw','82','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmygpo-qt','13','30480.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libmysofa','5','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libnbcompat','20','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libnet','7','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libnfc','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libnfs','22','29260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libnids','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libnodave','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libnova','16','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libnsbmp','1','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libnsgif','2','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libnumbertext','17','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libnxml','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liboauth','7','30544.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libofa','8','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libofx','8','26398','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liboglappth','7','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libolecf','18','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libomemo-c','6','30480.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liboop','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libopenglrecorder','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libopenmpt','50','28082.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libopenraw','38','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libopenshot','149','28726.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libopenshot-audio','54','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libopensmtpd','1','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liborcus','63','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liborigin','10','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liborocos-kdl','15','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libosinfo','20','34610.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libosmium','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libosmo-dsp','15','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libotf','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libotr','9','28784.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libowfat','12','33022.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libp11','5','28788.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpagemaker','9','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpano13','8','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpaper','39','32180','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpar2','10','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpeas','12','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpff','25','28747.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpgf','16','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libphdi','21','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpng-legacy12','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpoly','15','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libportal','9','34610.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpqxx','12','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libprelude','60','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libpst','74','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libqb','23','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libqcow','21','28273','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libqtpas','140','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libquicktime','29','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libqxp','13','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libqxt','33','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libr3','12','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librapi2','8','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librapidcheck','7','28887.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libraqm','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librcd','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libreadline-java','5','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libregf','17','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librelp','8','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librepfunc','5','29240.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libreplaygain','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librevisa','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librfm5','40','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librhtv','21','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librime','42','31029.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librist','20','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librra','10','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librsync','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/librtfcomp','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsbsms','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libscca','18','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libscfg','2','27169.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libscrypt','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsearpc','9','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libserialport','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libshout','7','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsidplay','8','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsidplay2','96','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsidplayfp','13','28189.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsieve','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsigrok','41','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsigrokdecode','15','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsigscan','15','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsixel','8','28171.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libskk','20','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libslack','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libslirp','4','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsmbios','24','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsmdev','12','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsmf','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsmi','23','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsmraw','18','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsoup3','13','32638.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libspectrum','5','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libspf2','11','28816.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libspiro','16','29260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libspng','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsrs2','10','30544.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsrtp','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsrtp2','8','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libstaroffice','31','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libstatgrab','13','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libstrophe','16','25960','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsuinput','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsvm','10','28577.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsynce','6','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libsysstat','3','27862.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtar','14','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtecla','13','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtelnet','14','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtermkey','2','41948','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libticables2','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libticalcs2','7','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libticonv','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtifiles2','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtins','10','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtomcrypt','17','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtommath','4','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtorrent','39','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtorrent-rasterbar','101','28171.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtorrent2','116','31070.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtpms','35','28748.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtraceevent','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtracefs','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libtubo0','5','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libu2f-host','18','31218.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libuchardet','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libucil','15','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libucl','15','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libudfread','13','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libunibreak','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libunicap','9','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libunicapgtk','16','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libuninameslist','17','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libunique','10','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libunique3','10','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libupnp','9','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libupnp-legacy','9','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/liburcu','19','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvdpau-va-gl','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvdwxc','7','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libversion','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvhdi','21','28273','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvidcap','7','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvirt','109','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvirt-glib','24','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvirt-python','8','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvmdk','20','28273','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvmime','59','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvorbisidec','7','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvsapm','16','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvsgpt','17','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvshadow','16','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvslvm','18','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libvterm','2','27608.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libwbxml','5','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libwebsocketpp','5','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libwebsockets','18','28111.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libwfut','25','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libwhisker2','1','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libwpe','4','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libwps','35','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libwrc','23','28747.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libx86emu','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libxc','87','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libxdg-basedir','13','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libxfcegui4','11','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libxls','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libxlsxwriter','9','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libxmi','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libxml++','9','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libxml++3','19','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libxml++5','9','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libxmlb','12','32375.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libxmlbird','3','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libxmp','8','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libyang','11','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libyubikey','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libyuv','5','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libzdb','13','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libzen','17','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libzia','16','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libzim','12','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libzmf','12','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libzookeeper','10','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/libzrtpcpp','10','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lilv','5','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/live555','9','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lksctp-tools','9','29706.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lockdev','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/log4c','10','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/log4cplus','30','29896.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/log4cpp','13','29706.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/log4net','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/log4shib','11','27948.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/logilab-astng','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/logilab-common','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lua-cjson','2','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lua-filesystem','2','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lua-lpeg','2','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lua-luv','3','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lua-md5','2','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lua-mpack','2','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lua-posix','14','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lua-readline','1','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lua-std._debug','1','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lua-std.normalize','1','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lua-zlib','2','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/luacrypto','10','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/luadbi','2','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/luaevent','2','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/luaexpat','1','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lualdap','2','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/luasec','4','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/luasocket','6','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/luaunbound','1','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lunasvg','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lunatic-python','2','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lvtk','21','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lxqt-build-tools','2','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lxqt-notificationd','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lxqt-qtplugin','5','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/lxqt-sudo','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/manubulon-snmp-plugins','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mathgl','40','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mbedtls','8','27411.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/md4c','4','28706.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mdds','8','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/meanwhile','10','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/med','43','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/menu-cache','11','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mercator','8','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/microsoft-gsl','1','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mimalloc','7','28171.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mini18n','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/miniupnpc','16','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mlt-bindings','23','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/moddb','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mongo-c-driver','18','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mongo-cxx-driver','41','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mowitz','10','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mpir','26','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/msgpack-c','4','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/msgpack-c-c','2','28082.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/msgpack-c-cpp','4','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mtxclient','93','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/muParser','7','31070.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mujs','10','28454.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/munkres','2','33207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/muparserx','10','26905.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/musicbrainzngs','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mxml','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mygpoclient','2','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mysql++','73','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/mysqltcl','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/nDPI','30','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/nacl','84','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/nanodbc','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/nanosvg','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/netcdf','38','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/netcdf-fortran','27','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/nextaw','9','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/nghttp3','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ngtcp2','12','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/nini','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/nlohmann_json','2','33780','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/nlopt','9','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/nng','16','31029.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/node-xoauth2','2','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/notcurses','31','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/nspluginwrapper','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ntl','69','27356','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/nv-codec-headers','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/nx-libs','875','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/oath-toolkit','52','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ocaml-batteries','89','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ocaml-camomile','28','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ocaml-findlib','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ode','33','32180','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ogre','303','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ois','6','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/olm','14','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/openapv','6','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/openbabel','832','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/opendht','162','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/opensaml','54','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/openvino','883','28166.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/orcania','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ordereddict','1','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/osgQt','12','31818.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/p4api','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/p8-platform','5','28047','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pam_usb','2','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pandarallel','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pangox-compat','12','41948','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/parsec','30','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pcl','613','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pdf4tcl','1','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pexpect','1','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pgtcl-ng','4','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/php-imagick','6','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/php-memcache','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/php-memcached','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/php-pdo_dblib','38','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/php-pgsql','74','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/php-redis','8','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/php-ssh2','7','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/php-stomp','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/physfs','3','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pidgin-mra','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pies','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pies2overrides','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pigpio','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pjsip','87','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pkcs11-helper','8','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/plasma-wayland-protocols-opt','2','27438.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/plastid','107','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/plib','59','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/plog','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/plplot','19','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/png++','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/poco','526','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/podcastparser','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/podofo','83','29943.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/polyclipping','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/portsmf','12','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/posixsignalmanager','4','23432.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/process-cpp','18','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/progressbar','1','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/properties-cpp','4','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/protobuf-c','21','29897.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/protozero','14','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/psqlodbc','9','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ptex','7','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pthsem','8','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ptypes','12','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pugixml','7','32883.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pyPEG2','3','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pyacoustid','1','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pycparser','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pyftpdlib','3','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pyjf3','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pylast','2','34287.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pylibpcap','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pymediainfo','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pyqode.core','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pysendfile','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pystring','4','31029.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pytaglib','6','28332.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/pyte','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python-efl','174','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python-gnupg','1','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python-librtmp','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python-meld3','1','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python-ntlm','1','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python-xlrd','3','27608.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python2-PyYAML','9','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python2-matplotlib','75','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python2-reportlab','9','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python2-ruamel.yaml','2','28628.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python3-CherryPy','3','28552.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python3-PyQtWebEngine','18','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python3-contourpy','18','28585.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python3-dvdvideo','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python3-gbinder','8','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python3-imapclient','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python3-plumbum','3','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python3-pymdstat','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python3-rpyc','2','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python3-ruamel.yaml','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python3-ruamel.yaml.clib','12','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/python3-zc.lockfile','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qgnomeplatform','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qml-material','9','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qmltermwidget','10','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qoauth','7','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qt-assistant-compat','6','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qt-avif-image-plugin','12','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qt-heif-image-plugin','3','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qt-installer-script','1','27774.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qt-jpegxl-image-plugin','4','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qt3','685','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qt4','1079','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qt5-pbfimageplugin','6','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qt5-postgresql','64','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qt5-styleplugins','12','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qt5gtk2','7','27608.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qtermwidget','9','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qtmimetypes','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/quazip-qt4','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/quesoglc','8','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qutepart','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qwt','40','29706.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/qxmpp','72','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rabbitmq-c','6','26837.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/range-v3','94','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rapidjson','51','28887.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rapidyaml','13','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/raptor','9','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rarfile','2','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rasqal','13','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/raylib','21','30733.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/re2','9','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/redland','15','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/remglk','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rest','7','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rest-legacy','9','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/restbed','17','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rhino','1','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rhonabwy','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/robin-map','2','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rocksdb','169','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rrdtool','22','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rtmidi','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rtrlib','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rudiments','29','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/rxtx','14','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/scafacos','310','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/scalapack','59','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/sdformat','34','29260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/seatd','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/secp256k1','16','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/serd','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/shairplay','17','31383.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/simbody','176','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/simde','6','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/simdjson','6','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/simdutf','7','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/simpleini','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/skalibs','9','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/skstream','7','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/slib','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/slv2','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/smmap','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/smmap2','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/smpeg2','8','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/soci','18','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/sofia-sip','39','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/soil','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/sord','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/soundtouch','15','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/spandsp','25','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/sparsehash','38','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/spawn','19','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/spdlog','35','29706.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/spfft','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/spglib','6','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/spread-sheet-widget','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/sqliteodbc','10','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/sqlpp11','18','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/squish','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/sratom','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/srt','18','29260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/stfl','7','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/stk','11','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/stormlib','5','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/suil','7','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/superlu','11','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/swfdec','14','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/t4k_common','11','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tDOM','9','35185.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ta-lib','66','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tamu_anova','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tbb','12','28717.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tcc4tcl','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tcl-inotify','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tcl-promise','1','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tcl-tls','21','28717.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tclap','14','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tclcurl-fa','5','34610.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tcllib','13','41948','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tclsoap','2','41948','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tclws','2','29131.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tclxml','6','30722.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/td_lib','27','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/template-glib','8','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tepl','13','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/termpaint','23','23432.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tff','17','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tinygltf','32','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tinyxml','2','29260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tinyxml2','5','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tkimg','64','41948','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tksvg','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tllist','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tlsh','7','29232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tmdb3','1','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tolua++','2','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tomcat-native','5','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/toml11','4','29077.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tomlplusplus','15','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/toro','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/totem-pl-parser','9','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tox-extension-messages','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/toxext','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/trader','13','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tre','17','29349.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tree-sitter','4','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tree-sitter-yaml','2','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/trilinos','458','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tslib','7','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tsocks','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tuiwidgets','20','28552.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/tvision','11','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ucl','11','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ucommon','15','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/udunits','7','28734.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/uhttpmock','7','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ulfius','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/unarr','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/unibilium','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/unixODBC','27','28628.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/uptime','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/urdfdom','9','29260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/urdfdom_headers','2','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/uriparser','4','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/usrsctp','10','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/utfcpp','1','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/uthash','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/uuid','8','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/vali','4','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/valijson','2','41948','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/varconf','6','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/vcglib','11','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/vdk','12','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/vigra','35','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/virglrenderer','10','34610.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/virtualpg','6','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/vmaf','18','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/volk','36','29706.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/vsqlite++','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/vst3sdk','51','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/vte2','18','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/waylandpp','15','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wcslib','16','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/web.py','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/webkit2gtk','4895','31135.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/webkit2gtk4.1','4663','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/webrtc-audio-processing','8','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wfmath','8','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wimlib','9','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wjelement','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wlroots','7','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wpebackend-fdo','5','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ws4py','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wslay','18','30544.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wv','9','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wvstreams','22','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wxGTK3','211','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wxPython','310','29706.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wxPython3','410','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wxPython4','344','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wxWidgets','137','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wxX11','18','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wxpdfdoc','39','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/wxsvg','34','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xalan-c','219','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xapian-bindings','63','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xapp','11','28539.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xawplus','7','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xblas','32','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xbyak','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xcb-util-xrm','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xerces','5','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xerces-c','35','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xforms','21','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xgboost','64','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xine-plugin','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xlt','12','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xml-security-c','12','27948.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xmlada','49','27862.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xmlgen','1','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xmlrpc-c','46','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xmlsec','18','30733.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xmltooling','42','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xmpppy','1','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xview','153','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/xylib','20','29131.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/yajl','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/yaml-cpp','56','31029.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/yaml-cpp0.3','8','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/yaml-cpp0.6','38','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/yascreen','3','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/yaz','18','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/yder','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/ytnef','14','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zeromq','18','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zimg','26','28082.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zint','12','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zita-alsa-pcmi','2','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zita-convolver','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zita-resampler','2','28788.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zlib-ng','15','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zmusic','20','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zope.component','3','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zope.configuration','3','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zope.event','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zope.hookable','3','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zope.i18nmessageid','3','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zope.location','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zope.proxy','3','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zope.schema','3','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zope.security','5','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zsi','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zstd-cmake','15','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zxing-cpp','21','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('libraries/zziplib','7','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/BISP','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/KeePass','22','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/KeePassHttp','1','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/OSCAR','52','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/QCMA','13','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/SecLists','557','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/abduco','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/altermime','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/anymeal','24','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ardesia','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ascii','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/asr-manpages','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/asterixInspector','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/bbrun','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/bchunk','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/bdf2psf','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/bdfresize','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/bee-host','1','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/beep','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/binclock','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/biosdisk','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/birthday','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/bkt','1','26938.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/bodr','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/boncuk','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/boxes','5','29896.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/brewtarget','50','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/bsdsfv','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ccze','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/cflow','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/cheat','23','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/chemical-mime-data','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/chm2pdf','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/chntpw','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ciso','1','28628.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/cksfv','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/clawsker','3','28747.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/cliphist','6','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/cp437','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/cups-pk-helper','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/cwiid','9','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/dbview','1','29260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/dcc','12','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/deb2tgz','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/deroff','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/dex','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/discount','4','31070.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/dmg2img','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/doclifter','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/dos2unix','7','28885.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/drg2sbg','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/dtmflib','7','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/duhdraw','2','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/dvtm','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/dwdiff','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ekphos','190','28780.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/esniper','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/exploitdb','522','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fbcat','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fbdump','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fcitx-mozc','149','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fcitx5','79','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fcitx5-chewing','5','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fcitx5-chinese-addons','28','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fcitx5-configtool','25','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fcitx5-gtk','8','29188.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fcitx5-mcbopomofo','19','27954.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fcitx5-qt','15','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fcitx5-table-extra','22','28305.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fcitx5-table-other','7','27396.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/fcrackzip','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/feedbackd','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/figlet','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/figlet-contributed','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/figlet-international','19','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/flashrom','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/flips','5','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/font-unscii','68','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/funny-manpages','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/g15composer','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/g15daemon','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/g15stats','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/g810-led','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/gcp','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ghostpcl','118','35185.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/git-store-meta','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/glogg','15','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/gnome-mime-data','7','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/gnome-user-docs','332','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/go-md2man','6','27941.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/goldencheetah','344','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/gourmet','28','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/gprename','1','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/gr-iqbal','11','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/grabc','1','28082.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/gramps','42','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/grc','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/gxmessage','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/heimdall','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/heyu','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/hoorex','2','31818.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/html-xml-utils','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/html2ps','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/hxtools','15','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ibus-chewing','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ibus-qt','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ibus-rime','2','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ibus-skk','19','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ibus-table-chinese','23','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/iscan-firmware','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/iscan-proprietary-drivers','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/jmri','96','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/kasumi','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/kde_cdemu','5','34610.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/keychain','1','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/klibc','14','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/kronometer','12','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/langdrill','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/lastpass-cli','4','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/lbdb','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/libime','43','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/libydpdict','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/linux_logo','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/lppf','2','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ltunify','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/mdic','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/megatunix','37','28047','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/modprobed-db','1','28628.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/moedict-desktop.app','126','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/moodbar','8','28628.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/moreutils','2','28651.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/mosquitto','69','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/myclippings','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/nfoview','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/no-more-secrets','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/nuclei','313','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/open-simh','1462','27285.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ophcrack','13','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/owfs','27','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/pam-u2f','7','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/paperkey','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/par','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/par2cmdline','264','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/par2cmdline-turbo','21','29896.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/pc','3','27862.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/pinfo','22','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/pmw','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/po4a','67','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/pokemon-colorscripts','16','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/portecle','3','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/projectM','86','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/python3-random-username','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/qprint','2','35185.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/qtspell','6','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ramdefrag','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/rarcrack','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/rbutil','34','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/recode','65','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/rlwrap','11','28552.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/rmlint','9','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/rsibreak','13','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/rsnapshot','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/samdump2','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/sdcv','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/seahorse-plugins','23','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/sgrep','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/shh','1','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/sigrok-firmware-fx2lafw','3','31239.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/slop','9','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/smu','5','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/snow','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/solunar','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/spotlighter','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ssss','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/stardict','118','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/stardict-tools','83','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/subsurface','73','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/sunwait','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/sutils','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/swege','2','28552.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/tal','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/tarotplane','2','34729.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/tegaki-zinnia-japanese','17','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/tidyp','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/tinytldr','1','29355','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/toilet','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/torrentinfo','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/twa','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/txt2regex','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/txt2tags','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/uni2ascii','4','29260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/utimer','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/vdpauinfo','2','29583.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/vttest','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/wcd','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/weather','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/wkhtmltopdf','17','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/wkhtmltox','51','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/wl-clipboard','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/wmbday','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xapian-omega','13','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xbanish','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xca','67','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xcape','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xcb-imdkit','4','29988.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xclip','5','28628.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xdelta','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xdelta3','30','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xhidecursor','2','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xvnkb','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/xwinwrap','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/yapet','49','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/yara','23','28273','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ydpdict','9','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/ykpers','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/yubikey-manager','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/yubikey-manager-qt','13','30722.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/yubikey-personalization-gui','12','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('misc/zinnia','9','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/AtomicParsley','7','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/AviSynthPlus','62','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/DivFix++','3','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/FFaudioConverter','5','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/Gem','280','28047','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/HandBrake','279','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/JSampler','5','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/Mopidy','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/MuseScore','564','28082.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/MuseScore-Legacy','399','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/QMPlay2','50','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/SimpleSysexxer','4','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/UxPlay','6','31383.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/abcMIDI','1','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/aces-core','54','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/aces_container','3','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/acetoneiso','13','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/aegisub','58','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/amarok','272','28628.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/anyremote','9','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/aom','117','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/apetag','3','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/audio-recorder','7','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/audiopreview','4','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/avidemux','190','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/bdtools','6','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/beets','7','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/bino','14','28887.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/bombono-dvd','278','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/cantata','91','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/castget','3','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/castnow','2','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/ccextractor','34','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/ccxstream','2','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/celluloid','7','32883.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/cfourcc','1','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/cinelerra','473','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/clamz','3','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/coriander','7','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/cowbell','6','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/ctl','16','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/cuetools','5','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/davs2','8','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/devedeng','3','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/dirac','25','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/droidcam','2','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/dsmidiwifi','3','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/dssi','7','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/dvblast','4','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/dvd-slideshow','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/dvdbackup','9','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/dvdrip','6','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/dvdstyler','46','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/dvgrab','6','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/easyeffects','59','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/emoc','3','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/exaile','23','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/farsight2','22','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/fdkaac','9','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/ffmpeg2theora','4','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/ffmpegfs','35','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/filebot','43','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/filebot-legacy','38','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/flowblade','24','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/flvtool2','1','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gaupol','4','28816.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/genpuid','5','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/get_iplayer','1','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gigedit','31','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gimp-gap','269','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/glyr','3','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gmidimonitor','2','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gmtp','4','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gnash','188','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gnome-mplayer','8','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gnome-video-effects','4','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gnupod','3','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/goom','8','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gpac','78','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gst-plugins-bad-nonfree','19','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gst-plugins-ugly','8','28798.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gst0-ffmpeg','54','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gst0-plugins-base','56','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gst0-plugins-good','47','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gst0-plugins-ugly','12','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gstreamer-vaapi','9','31029.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gstreamer0','33','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gtick','9','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gtk-recordmydesktop','4','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/guvcview','6','28552.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/gxine','12','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/haruna','14','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/hexter','7','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/huludesktop','2','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/iat','4','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/imagination','20','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/inputstream.adaptive','61','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/intel-gmmlib','24','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/intel-media-sdk','141','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/ivtv-utils','6','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/jellyfin-media-player','18','28171.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/jlscp','2','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/k9copy-reloaded','45','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/kaffeine','20','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/kaffeine-mozilla','6','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/kodi','659','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/kvazaar','18','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/l-smash','32','28747.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/lightspark','99','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/lives','198','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/lsdvd','3','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/makemkv','66','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mediainfo','35','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mimms','1','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/minidlna','11','34610.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/minitube','86','28454.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mirage2iso','3','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mjpegtools','23','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mkvtoolnix','216','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mozplugger','4','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mp3fm','2','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mp3fs','3','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mpgtx','3','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mplayer-codecs32','26','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mpv','18','28454.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mtn','3','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/mtpfs','5','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/munt','19','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/musique','67','28454.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/nrg2iso','2','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/nted','23','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/obs-studio','248','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/obs-studio-legacy','251','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/oggconvert','1','27973.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/oggvideotools','6','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/olive','37','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/opencaster','11','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/openh264','18','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/openshot','111','28791.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/parole','15','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/picard','13','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/picard-plugins','4','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/pipe-viewer','2','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/pitivi','19','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/playerctl','7','31838.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/plexmediaserver','78','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/pngnq-s9','4','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/pocketsphinx','27','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/popcorntime','355','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/pragha','14','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/pyradio','1','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/qsampler','10','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/radiotray','2','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/rav1e','160','28047','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/ripit','1','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/ripperX','6','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/rnnoise','7','31818.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/rosa-media-player','18','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/rpi-imager','16','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/rtmpdump','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/rygel','28','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/saflashplayer-legacy','8','28454.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/schroedinger','15','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/serviio','11','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/sfnt2woff','1','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/shine','12','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/shotcut','83','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/sickchill','27','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/sinthgunt','2','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/smpeg','9','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/smplayer','48','30722.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/smtube','7','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/sonivox','8','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/spek','22','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/spot','245','28717.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/spotify','93','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/srt-to-vtt-cl','7','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/stills2dv','3','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/strawberry','239','28571.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/stream2chromecast','1','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/stremio','15','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/subtitleeditor','234','33780','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/subtitleripper','2','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/svt-av1','20','28152','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/tabu_audio_player','4','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/telepathy-farstream','8','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/telepathy-glib','38','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/telepathy-qt','59','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/tivodecode','5','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/totem','29','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/tovid','2','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/tragtor','2','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/transcode','35','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/tsduck','299','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/tstools','5','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/tvheadend','39','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/tvtime','12','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/uavs3d','5','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/ucview','15','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vapoursynth','44','28047','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vapoursynth-mvtools','86','28047','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vcdimager','8','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vgmplay','4','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/videocut','6','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/videomass','8','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vitunes-mplayer','1','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vkeybd','2','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vlc','195','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vlsub','1','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vo-amrwbenc','5','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vobcopy','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vobsub2srt','4','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vokoscreenNG','33','28047.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vokoscreenNG-QT5','29','28571.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vvdec','29','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/vvenc','58','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/w_scan','5','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/w_scan2','5','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/w_scan_cpp','24','27302','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/webvfx','11','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/whistle','1','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/winff','11','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/wxcam','20','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/x264','22','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/x265','41','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/xawtv','8','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/xjadeo','6','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/xmltv','9','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/xmms-crossfade','8','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/xmms-cue','4','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/xvid4conf','4','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/xvidcore','5','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/xvst','41','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/yarock','60','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('multimedia/zvbi','13','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/2ping','1','41948','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/3proxy','7','32498.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/9pfuse','4','27862.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/AdGuardHome','12','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ApacheDirectoryStudio','63','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ClusterSSH','13','28747.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/DenyHosts','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/DownTube','10','30462.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/Electrum','13','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/FireWorks','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/GeoIP','32','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/GhostInTheMail','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/GlobalProtect-openconnect','14','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/LOIC','1','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/MEGAcmd','132','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/MEGAsync','145','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/NetworkManager-fortisslvpn','27','28454.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/NetworkManager-l2tp','29','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/NetworkManager-openconnect','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/NetworkManager-pptp','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/NetworkManager-ssh','17','28552.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/NetworkManager-sstp','9','31070.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/NetworkManager-vpnc','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/PAC','2','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/Pafy','3','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/PySocks','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/Pyrit','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/Quaternion','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/QuiteRSS','58','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/UDR','16','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/WhatWeb','24','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/XD','15','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/aMule','72','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/acme.sh','3','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/addrwatch','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/aerc','22','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/aggregate','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/aiccu','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/aircrack-ng','22','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/aldryn-client','2','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/amap','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/amavisd-new','3','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/amfora-bin','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/amneziawg-go','8','28635.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/amneziawg-kernel','8','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/amneziawg-tools','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ani-cli','1','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/aoetools','2','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/apt-mirror','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/argus','9','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/argus-clients','29','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/aria2','70','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/arno-iptables-firewall','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/arp-scan','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/arping','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/arpwatch','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/asciinema','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/asterisk','215','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/asuka','40','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/atftp','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/atheme','44','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/authbind','1','30733.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/autossh','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/avahi','40','31383.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/awscli','82','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/awscli-v2','1308','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/awstats','3','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/axel','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/b43-firmware','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/b43-fwcutter','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/badwolf','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/baikal','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/barnyard2','18','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/barrier','34','31383.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bip','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bitchx','17','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bitcoin','210','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bitcoin-knots','600','31218.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bitlbee','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bitlbee-discord','12','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bitlbee-steam','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bmon','9','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bollux','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bombadillo','11','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bottle','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bozohttpd','2','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bpfmon','1','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/broadcom-wl','6','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bsflite','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/btpd','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/bwm-ng','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/c-toxcore','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cacti','36','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cacti-spine','6','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cadaver','12','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/canto','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/canto-next','2','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/castor','96','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/catgirl','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ccnet','16','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ceb-tools','2','41948','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cgterm','3','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/chawan','42','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cherokee','41','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/choqok','56','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/chrony','6','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cicb','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cieid','9','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ckermit','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/clamav-unofficial-sigs','1','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/claws-mail','61','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/clipgrab','13','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/clive','3','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cloudflared-bin','9','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/clusterit','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cmdiag','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cmst','22','25960','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cni-plugins','43','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/comgt','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/conkeror','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/connman','22','25960','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/copyparty','6','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/corkscrew','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/coturn','15','28784.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/courier-authlib','64','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/courier-imap','85','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/courier-unicode','8','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cowpatty','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cryptcat-unix','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/csync','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ctorrent','9','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/curl-quic','30','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/cyrus-imapd','44','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dante','49','28171.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/darkhttpd','2','31383.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/darkstat','7','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/davmail','3','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ddclient','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ddgr','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/deluge','20','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dhcping','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dianara','29','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dillo','16','32180','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dino','40','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dirb','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dkimproxy','3','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dma','2','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dmitry','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dnscrypt-proxy','6','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dnscrypt-wrapper','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dnsmap','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dnsproxy-bin','5','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dnstop','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dnstracer','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/docsis','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dog','20','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dokuwiki','14','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dooble','112','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dothost','2','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dovecot-pgsql','23','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dovecot-pigeonhole','21','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/drawterm','9','31070.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/driftnet','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dropbear','15','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dropbox','92','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dsniff','12','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dstp','12','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dublin-traceroute','6','28171.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/dukto','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/e2guardian','17','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/easyrsa','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/edbrowse','3','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/eduvpn','4','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/efax','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/efax-gtk','15','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/eiskaltdcpp','89','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ejabberd','76','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/electrs','194','28204.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/elinks','56','34729.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/emailrelay','63','28849.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/enet','3','31239.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/epiphany','41','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/et','188','29355','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/etherape','7','35185.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/etherpad-lite','74','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/etherpoke','2','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ethersrv-linux','2','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ettercap','12','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/exim','11','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/fail2ban','11','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/fcgiwrap','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/fdm','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/fdns','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/felinks','54','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ferm','2','28887.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ffuf','15','29393.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/filezilla','97','27966.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/firehol','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/firewalld','21','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/flvstreamer','2','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/fping','4','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/fqterm','20','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/freedownloadmanager','63','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/freeradius-server','246','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/freerapid','6','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/frostwire','9','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/frr','104','29355','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ftp-cloudfs','1','28788.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/fujinet-pc','35','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/fwknop','12','28273','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gajim','13','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gallery-dl','7','29840.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ganglia','17','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ganglia-web','4','27862.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gem','9','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gemget','10','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gemget-bin','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/geoipupdate','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/geomyidae','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/glance','17','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/glewlwyd','16','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/glusterfs','224','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gmi100','2','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gmid','5','27862.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gnome-bluetooth','9','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gnome-remote-desktop','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gnome-shell-extension-gsconnect','5','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gnome-user-share','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gns3','14','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/go-ipfs','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/go-sendxmpp','35','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gobuster','16','28658','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gofiche','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/goimapnotify','31','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/googlecl','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/googler','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gophernicus','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gotty','4','30329.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gpodder','7','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/grepcidr3','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/grpcurl','9','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/grsync','5','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gtk-gnutella','216','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gtorrentviewer','5','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/guacamole-client','4','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gui-ufw','7','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/gutenberg','3','33022.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/hakuneko','99','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/hamachi','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/haproxy','23','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/haskell-network-multicast','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/havp','6','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/heimdal','88','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/hiawatha','10','28788.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/holehe','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/host-sflow','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/hostapd','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/hostsblock','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/hping3','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/httpie','3','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/httping','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/httptunnel','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/httrack','14','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/hydra','9','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/hylafax+','58','28171.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/i2pd','66','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/icbm','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/icecat','86','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/icinga2','368','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/identicurse','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ifplugd','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/igmpproxy','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ii','1','29019.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/imapfilter','2','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/inadyn','14','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/iodine','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ip2location-c','16','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ipcalc','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ipcalc-ng','2','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/iperf','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/iperf3','8','31218.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ipfs-pack','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/iptv-analyzer','12','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ipv6calc','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ipvsadm','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ipxnet','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/irc.c','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/irc2dc','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ircII','10','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ircd-hybrid','12','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/irssi-icb','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/irssi-python','16','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/irssi_otr','8','28784.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/isync','5','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/iwd','12','30722.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/jabberd2','25','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/jboss-as','39','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/jigdo','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/jitsi','72','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/js185','27','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kasp_updater','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kdrive','91','29866.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kea','426','29896.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/keama','71','31029.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/keepalived','16','28788.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kirc','1','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kismet','437','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kivy-garden','1','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/knock','6','27862.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kstart','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kvirc','186','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/kwakd','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/lagrange','20','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/landrun','9','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/lchat','2','31124.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ldapvi','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/leafnode','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/libdnet','9','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/libident','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/libiscsi','15','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/libmaxminddb','5','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/libnatpmp','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/libressl','34','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/libteam','16','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/libtls','25','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/libvarlink','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/liferea','21','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/lighttpd','28','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/limnoria','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/linksys-tftp','1','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/linssid','12','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/lizardfs','264','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/lldpd','21','29341.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/llhttp','2','27747.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/lorawan-stack-cli','19','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/luakit','5','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/lurch','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/lxi-tools','11','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/macchanger','3','29260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/maildrop','54','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mailman','23','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/malheur','5','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/masscan','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mattermost-desktop','118','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mblaze','3','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mdk3','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mdk4','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/megatools','6','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/memcached','8','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/memcached-init','1','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mew','13','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/milter-greylist','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/miniupnpd','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mk-livestatus','44','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mktorrent','2','41948','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mkvserver_mk2','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_auth_gssapi','16','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_auth_kerb','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_bw','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_chroot','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_dnssd','4','31383.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_evasive','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_fcgid','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_geoip2','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_ruid2','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_wsgi','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mod_wsgi-metrics','2','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/modemu2k','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/modsecurity-apache','17','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/monkey','5','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/moonlight-qt','36','27862.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mosaic-ck','17','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mosh','29','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/movgrab','4','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mpop','5','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mps','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mps-youtube','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mptcpd','9','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mrtg','12','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/msmtp','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mumble','122','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mumble-server','41','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/munge','18','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/mysecureshell','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nagios','20','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nagiosgraph','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/naim','16','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nbd','21','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nbtscan','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nbwmon','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ncp','2','33022.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ncrack','43','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/neomutt','30','28454.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/netcat','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/netdiscover','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/netdrive-connector','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nethogs','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/netperf','9','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/netqmail','35','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/netsniff-ng','36','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/netstat-nat','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/netsurf','80','27973.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/newsboat','215','28080.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/newsraft','5','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nextcloud-server','151','31029.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nfdump','12','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nginx','17','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ngircd','9','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ngrep','4','29404.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ngrok','11','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nheko','89','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/niceshaper','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nicotine+','11','28809.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nikto','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nload','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nm-tray','8','31029.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nmh','34','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/node_exporter','20','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/noip-duc','45','29343.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/noip2','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nordvpn','40','29238.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/noss','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nrpe','5','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nsca','4','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nsd','14','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nss-mdns','6','31383.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nss-tls','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ntpclient','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nullidentd','1','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nweb','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nxclient','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nyxt','128','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/nzbget','26','29131.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/obfs4proxy','13','28747.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ocserv','9','28837','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/offlineimap','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/oidentd','7','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/oinkmaster','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/onedrive','13','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/onioncat','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/onionshare','3','28552.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/onkyo-eiscp','2','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/onyx','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ookla-speedtest','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/open-iscsi','8','28816.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/open-isns','4','28816.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openconnect','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openconnect-sso','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/opendchub','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/opendkim','30','31029.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/opendmarc','47','28717.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openfire','12','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openfortivpn','12','28454.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openntpd','8','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openresty','69','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/opensm','26','28628.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/opensmtpd','21','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/opensmtpd-extras','17','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/opensmtpd-filter-dkimsign','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/openvswitch-utils','29','33780','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/otter','88','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/owncloud-client','141','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/owncloud-server','30','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/p0f','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/packETH','22','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/packit','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/paho-mqtt-c','15','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/palemoon','1469','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/passt','19','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pdns','114','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pdns-recursor','114','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pebrot','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/periscope','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/persepolis','4','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pflogsumm','1','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pfqueue','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pgbouncer','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pgpool-II','25','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/phodav','7','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/phpmyadmin','22','34287.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/phppgadmin','4','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-authorization-blocker','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-birthday-reminder','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-extprefs','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-gfire','18','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-musictracker','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-nudge','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-opensteamworks','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-otr','7','28784.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-privacy-please','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-sipe','12','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-sound','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-toobars','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-toolbar-shrink','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-visnotes','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pidgin-whatsapp','7','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pirate-get','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/plasma-nm-openconnect','62','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/plowshare','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/plowshare-modules','2','30722.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pmacct','18','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/policyd','1','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/policyd2','1','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/polipo','3','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/postfix-pgsql','18','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/postgrey','1','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pounce','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pound','7','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pptp','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pptpd','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/prettyping','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/privoxy','14','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/profanity','34','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/prosody','5','28454.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/prosody-mod-block-strangers','1','28454.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/prosody-mod-cloud-notify','1','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/prosody-mod-filter-chatstates','1','28454.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/prosody-mod-http-upload','1','28454.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/prosody-mod-throttle-presence','1','28454.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/prosody-mod-turncredentials','1','28454.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/prosody-mod-vcard-muc','1','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/proxychains','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/proxymini','1','31364.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/proxytunnel','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/psad','26','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/psi','381','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/psi-plus','391','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ptokax','6','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pubnub-curses','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pure-ftpd','16','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/purple-facebook','13','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/purple-plugin-pack','49','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/purple-teams','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/purple-telegram','52','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/putty','21','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pyNeighborhood','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pysub-dl','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/python-onedrive','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/python3-CherryMusic','4','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/python3-PyRSS2Gen','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/pyzor','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/qTox','144','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/qbittorrent','86','31070.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/qodem','29','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/qtmib','33','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/quagga','51','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/quark','2','28571.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/quassel','223','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/qutebrowser','6','31070.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/r8168','12','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/radicale','4','33780','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/radiusplugin','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/radvd','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rclone-src','86','28711.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rdma-core','44','28628.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/reaver','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/redir','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rejik','2','27608.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/remmina','22','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/remotebox','5','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/repo','1','31818.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/resilio-sync','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/retroshare','354','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rhapsody','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/roundcubemail','13','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rspamd','76','27918.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rss2email','2','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rssh','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rtl8188eu','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rtl8188fu','10','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rtl8812bu','22','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rtl8821ce','19','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rtorrent','24','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rubygem-showterm','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/rustdesk-opt','743','32180','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/s','5','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sabnzbd','4','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/scapy','10','34610.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/scrcpy','8','28454.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/seadrive-fuse','12','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/seafile','15','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/seafile-server','19','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/seahub','57','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/secpanel','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/senpai','13','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/serialmail','12','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sfeed','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sflowtool','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sftp-cloudfs','1','28788.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/shadowsocks-libev','12','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/shadowsocks-rust','232','28649.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/shibboleth-sp','69','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/shorewall','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/shorewall-core','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/shorewall6','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sic','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/siege','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sieve-connect','1','28887.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/silc-toolkit','31','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sipcalc','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sipp','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/skype4pidgin','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/slirp4netns','12','30722.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/slowhttptest','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/slurm','141','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/smcroute','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/smstools3','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/snac2','3','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sngrep','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sniffjoke','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/snmpsim','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/snmptt','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/snort','205','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sntpc','2','31029.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sopcast','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sopcast-player','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/spamassassin','10','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/spambayes','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/spawn-fcgi','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/speedometer','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/speedtest-cli','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/spf-engine','2','28454.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sphinx','36','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/spice','37','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/spice-gtk','29','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sqlmap','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sqlninja','1','28887.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sqlrelay','101','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/squid','253','27608.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/squidanalyzer','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/squirrelmail','26','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ssh-chat','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sshamble','21','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sshblock','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sshfp','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sshguard','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sshpass','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sshuttle','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sslh','6','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ssllabs-scan','11','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sslscan','55','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ssmtp','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sstp-client','24','31070.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/strongswan','108','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/surf','5','28651.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/surfraw','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/surl','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/swaks','1','29131.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/swift','253','31383.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/sylpheed','24','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/syncterm','52','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/syncthing','10','28562.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/synochat','101','32180','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/t50','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tcpbench','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tcpflow','34','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tcptunnel','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tdlib','430','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/teamviewer','111','29395.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tgpt','22','28221.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tgt','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/thrift','60','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/thttpd','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tinc','9','28422','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tinyirc','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tinyproxy','4','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tixati','33','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tmate','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tnfs-fuse','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tnfsd','1','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tnftp','12','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tntnet','38','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tokodon','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/toot','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tor','71','28552.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tor-browser','67','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tornado','3','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/torsocks','15','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/toxcore','18','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/transgui','8','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/translate','1','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/translate-shell','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/transmission','126','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/transmission-remote-gtk','7','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/trojita','97','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/trurl','3','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ttcp','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ttdnsd','2','28552.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tucan','2','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tunctl','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/tut','38','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/twine','3','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/twinkle','40','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/twitter','2','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/u9fs','2','27862.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/uTox','81','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ubertooth','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ucarp','12','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ucspi-tcp','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/udns','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/udpbench','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/udpcast','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/udpxy','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ufw','3','31364.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/uget','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/uget-chrome-wrapper','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/uget-integrator','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/unbound','29','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/unicornscan','39','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/uredir','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/urlgrabber','1','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/urlscan','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/urlview','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/urlwatch','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/uudeview','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/uwsgi','25','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/varnish','53','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vblade','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vcspull','2','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vde2','20','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vdirsyncer','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/verlihub','20','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vidtuber','4','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vimb','3','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vimfx','1','28717.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vmware-horizon-client','28','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vnstat','7','28552.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vpnc','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vrrpd','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/vuze','7','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/w3af','29','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/w3m','13','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wakeonlan','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/warpinator','22','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wavemon','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/webalizer','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/webhook','13','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/weechat','26','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/weetwit','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wego','14','28536.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/weighmail','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wendzelnntpd','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/whatsie','37','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wifi-radar','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wifite','1','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wildfly','84','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wireshark','331','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wmget','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wmnd','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wmnotify','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wol','9','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wp-cli','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wpa-cute','8','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wput','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wrk','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wsdd2','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/wvdial','9','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/x11spice','13','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/x11vnc','18','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/x2goclient','23','28197.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/x2goserver','4','30826.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/xaric','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/xbuffy','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/xinetd','14','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/xl2tpd','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/xmrig','53','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/xnetload','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/xtables-addons','10','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/yadifa','39','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/yate','173','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/yle-dl','2','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/you-get','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/youtube-dl','15','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/youtube-dl-gui','2','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/youtube-dl-server','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/youtube-upload','1','31239.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/yt-dlp','15','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/yt-dlp-bin','2','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/ytfzf','1','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/zabbix_agent2','105','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/zabbix_agentd','74','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/zabbix_frontend','46','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/zabbix_java_gateway','38','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/zabbix_proxy','94','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/zabbix_server','110','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/zdns','20','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/zeek','1124','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/zerotier-one','24','27608.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/zmap','5','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/znc','56','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('network/zsync','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/BibTool','25','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/CherryTree','4','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/FreeOffice','80','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/LaTeXML','484','28747.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/LanguageTool','99','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/LibreOffice','5083','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/Logseq','136','25960','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/NME','3','41948','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/ProjectLibre','9','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/ReText','2','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/SOGo','74','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/Ted','112','28552.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/abiword','259','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/abook','6','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/adobe-reader','70','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/adobe-reader-fontpacks','37','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/aigaion','3','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/aiksaurus','9','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/anorack','1','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/antiword','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/apvlv','7','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/asymptote','64','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/atril','40','31364.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/basket','21','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/beaver','8','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/bibclean','5','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/bibtex2html','10','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/bibutils','3','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/bookbinder','2','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/briss','4','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/broffice.org','159','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/calcurse','5','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/catdoc','3','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/catdvi','2','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/ccal','2','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/convertlit','2','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/coolreader','31','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/cpdf','22','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/crow-translate','22','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/cups-pdf','1','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/devtodo','12','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/dictd','11','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/diction','2','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/diffpdf','7','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/ding','7','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/djvusmooth','2','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/docx2txt','1','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/dspdfviewer','11','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/eid-mw','30','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/elyxer','2','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/epdfview','8','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/epr','1','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/epub2txt2','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/epy','2','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/etmtk','2','28628.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/evince','72','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/evince-gtk2','37','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/evolution','130','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/evolution-ews','12','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/extractpdfmark','5','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/fbpdf','2','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/fbreader','114','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/featherpad','22','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/flowkeeper','2','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/focuswriter','21','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/fop','20','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/freemind','15','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/full-pack','11','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/ganttproject','9','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/gbgoffice','10','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/gcal','28','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/gcalctool-gtk2','17','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/glow','6','30722.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/gmdb2','15','30733.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/gnokii','17','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/gnucash','163','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/gnucash-docs','43','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/gnumeric','175','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/gnumeric-gtk2','81','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/gocr','6','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/goldendict','57','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/grisbi','39','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/gummi','6','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hamster','5','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hebcal','9','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/heimer','27','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hevea','10','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/homebank','21','32638.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/htmldoc','12','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hunspell-ancient-greek','10','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hunspell-de','7','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hunspell-en','5','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hunspell-es','10','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hunspell-french','4','35185.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hunspell-gr','9','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hunspell-id','2','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hunspell-pl','3','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hunspell-russian-aot','7','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hunspell-russian-kliukvin','3','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hunspell-russian-lebedev','4','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hunspell-russian-wiktionary','8','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/hunspell-sv','3','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/impressive','2','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/ishmael','13','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/ispell-dict-english','38','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/ispell-dict-russian-lebedev','7','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/jfbview','43','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/jpdfbookmarks','4','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/jstock','19','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/kbbi-qt','13','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/kbgoffice','7','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/kbibtex','54','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/kchmviewer-qt','15','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/keepassx','19','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/keepassxc','85','30480.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/keepnote','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/khal','2','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/khard','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/kitscenarist','96','32180','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/kiwix-desktop','32','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/kraft','38','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/krop','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/kuro-legacy','102','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/latex2rtf','3','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/latexdiff','1','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/lcal','2','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/leafpad','6','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/lotus123','36','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/lout','12','30733.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/lowdown','7','28171.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/luametatex','13','28047','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/lumina-calculator','4','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/lumina-pdf','6','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/lyx','169','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mairix','3','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mate-calc','38','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mathtex','2','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mdbtools','9','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mdp','2','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mined','15','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/minidjvu','28','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mobiperl','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/msee','4','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mtp2-fonts','2','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mu','76','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/multivalent-tool-pdf','2','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mupdf','56','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/muttprint','3','28047','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mytetra','14','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mythes','5','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/mythes-id','1','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/navi','33','27544.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/nb','2','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/notes','2','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/notmuch','15','30544.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/nts','2','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/nuspell','9','25960','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/ocrmypdf','3','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/odt2txt','1','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/openoffice-langpack','8','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/openoffice.org','106','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/osmo','9','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/osmo-gtk2','8','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pandoc','1181','30722.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/paps','17','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pcal','2','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pdf-bunny','2','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pdf2djvu','13','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pdfarranger','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pdfchain','8','31364.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pdfpc','7','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pdfsandwich','2','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pdfshuffler','1','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pdfstudio','85','27973.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pdfstudioviewer','85','27973.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pdftk','3','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/ps2eps','1','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pstoedit','14','30734.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pstotext','1','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/pyspread','5','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/python-gcalcli','2','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/python3-proselint','2','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/python3-xlsx2csv','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/qownnotes','95','28101.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/qpdfview','26','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/qtrans','9','29452.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/re-typeset','5','32180','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/referencer','22','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/remind','10','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/scribus','327','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/semantik','31','31239.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/sent','1','36142.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/siag','25','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/sigil','69','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/sioyek','31','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/smoffice2016','184','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/smoffice2018','299','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/star-tex','17','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/t-prot','2','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/tapecalc','6','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/task','23','27862.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/taskd','26','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/taskjuggler','15','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/taskopen','2','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/tasksh','8','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/tellico','368','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/texlive-docs','602','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/texlive-extra','1090','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/texmacs','182','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/texmaker','87','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/texstudio','189','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/text2pdf','1','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/texworks','52','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/timetrap','2','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/timew','11','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/tnote','1','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/todo.txt-cli','1','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/tpp','2','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/traduisons','1','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/treesheets','4','28816.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/trelby','2','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/trello-cli','18','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/tudu','8','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/turtl','84','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/typobuster','3','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/unaesgcm','3','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/uncsv','2','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/verbiste','15','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/visidata','6','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/vit','3','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/vlna','3','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/vmd','1','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/vym','51','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/vzic','2','28819.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/wammu','3','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/watson','2','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/when','2','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/wink','3','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/wordgrinder','7','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/wordtsar','57','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/x_x','2','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/xmind','59','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/xournal','9','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/xournalpp','69','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/xpad','5','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/xpdf3','12','28552.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/xreader','17','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/xtrkcad','36','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/zathura','10','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/zathura-cb','3','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/zathura-djvu','3','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/zathura-pdf-mupdf','3','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('office/zathura-ps','3','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/Alien-Build','4','28094.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/Alien-Libxml2','42','25960','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/Download-GitLab','2','28094.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/File-chdir','2','28741.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/Gtk2-GladeXML','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/MoarVM','20','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/XML-LibXML','9','25960','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/cpanminus','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/libpurple-kwallet-plugin','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/libwww-perl','27','27870.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/mod_perl','56','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/nqp','35','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Algorithm-Combinatorics','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Algorithm-Diff','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Algorithm-Loops','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Alien-SDL','4','28784.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-AnyEvent','16','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-AnyEvent-HTTP','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-AppConfig','2','31364.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Archive-Extract','4','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Archive-Zip','2','27726.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Audio-FLAC-Header','3','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-B-COW','2','27489.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-B-Hooks-EndOfScope','4','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-B-Hooks-OP-Check','2','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-B-Hooks-OP-PPAddr','2','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-B-Keywords','2','27657','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-B-Lint','2','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-BerkeleyDB','5','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Bit-Vector','7','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CDDB-get','2','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CGI','6','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CGI-Application','4','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CGI-Fast','3','31364.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CPAN-Changes','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CPAN-Meta-Check','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CPAN-Perl-Releases','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CPAN-Requirements-Dynamic','2','29355','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CPANPLUS','24','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CPANPLUS-Dist-Build','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CPANPLUS-Dist-Slackware','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CSS','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CSS-Squish','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Cache-FastMmap','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Cairo-GObject','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Canary-Stability','1','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Capture-Tiny','39','28784.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Carp-Clan','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-Data-Inheritable','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-Factory-Util','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-Gomor','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-ISA','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-Inspector','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-Load','1','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-Load-XS','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-Method-Modifiers','1','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-MethodMaker','9','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-Mix','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-ReturnValue','2','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-Singleton','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Class-Tiny','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Clone','3','28365.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Clone-PP','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Compress-Bzip2','8','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Config-File','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Config-Find','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Config-Grammar','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Config-IniFiles','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Config-Simple','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Config-Tiny','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Convert-ASN1','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Convert-BinHex','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Convert-Color','3','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Convert-TNEF','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Convert-UUlib','4','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Coro','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Cpanel-JSON-XS','6','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-Blowfish','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-Blowfish_PP','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-CAST5','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-CBC','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-DES','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-Eksblowfish','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-IDEA','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-OpenSSL-AES','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-OpenSSL-Bignum','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-OpenSSL-DSA','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-OpenSSL-EC','4','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-OpenSSL-ECDSA','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-OpenSSL-Guess','1','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-OpenSSL-RSA','3','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-OpenSSL-Random','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Crypt-Rijndael','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-CryptX','30','27911.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Cstools','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Curses','20','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Curses-UI','4','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Cwd-Guard','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DBD-Pg','4','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DBD-SQLite','55','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DBIx-Simple','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Danga-Socket','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Data-Compare','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Data-Dumper-Concise','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Data-IEEE754','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Data-OptList','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Data-Printer','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Data-Random','8','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Data-UUID','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Data-Validate-IP','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Date-Calc','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Date-Manip','11','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DateTime','13','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DateTime-Format-Builder','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DateTime-Format-DateParse','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DateTime-Format-ISO8601','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DateTime-Format-Mail','7','30722.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DateTime-Format-RFC3339','2','30722.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DateTime-Format-Strptime','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DateTime-Locale','20','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-DateTime-TimeZone','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Devel-CheckCompiler','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Devel-Cycle','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Devel-Declare','5','28784.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Devel-GlobalDestruction','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Devel-NYTProf','101','29245.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Devel-OverloadInfo','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Devel-PartialDump','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Devel-PatchPerl','3','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Devel-StackTrace','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Devel-Symdump','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Devel-Trace','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Device-SerialPort','9','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Digest-GOST','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Digest-SHA','3','29583.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Digest-SHA-PurePerl','3','28784.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Dist-CheckConflicts','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-EV','10','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Email-Abstract','2','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Email-Address-XS','2','29914.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Email-Date-Format','2','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Email-MIME','3','29832','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Email-MIME-ContentType','3','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Email-MIME-Encodings','2','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Email-MessageID','2','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Email-Sender','5','31688.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Email-Simple','3','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Encode-Detect','3','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Encode-EUCJPASCII','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Encode-HanExtra','7','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Encode-ISO2022','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Encode-JISX0213','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Error','3','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Eval-Closure','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Exception-Class','3','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Expect','76','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Exporter-Lite','2','28972.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Exporter-Tiny','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-ExtUtils-CChecker','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-ExtUtils-Config','2','29896.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-ExtUtils-Helpers','2','27542.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-ExtUtils-Install','4','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-ExtUtils-InstallPaths','1','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-ExtUtils-ModuleMaker','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-ExtUtils-XSBuilder','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-FCGI','4','31364.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Feature-Compat-Try','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Copy-Recursive','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Copy-Recursive-Reduced','2','28784.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Download','1','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-FcntlLock','5','31364.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Find-Rule','2','28983.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Finder','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Grep','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-HomeDir','2','28747.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Inplace','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-ReadBackwards','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Remove','5','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Save-Home','2','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Share','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-ShareDir','3','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-ShareDir-Install','3','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Slurp','4','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Slurp-Tiny','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Tail','66','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Type','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-Which','2','28747.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-File-pushd','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Filesys-Df','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Font-AFM','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Font-TTF','3','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-GD','6','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Geo-Coordinates-OSGB','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Geo-Coordinates-UTM','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Geo-IP','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Geography-Countries','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Getopt-Long-Descriptive','3','28784.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Getopt-Tabular','2','28972.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Git-Sub','2','28784.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Glib-Object-Introspection','7','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Gnome2-Vte','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-GooCanvas2','1','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Gtk2-Ex-Simple-List','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Gtk2-Notify','3','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Gtk2-Unique','3','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Gtk3','1','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Gtk3-ImageView','1','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Gtk3-SimpleList','2','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Guard','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-HTML-FormatText-WithLinks','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-HTML-FormatText-WithLinks-AndTables','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-HTML-Formatter','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-HTML-HTMLDoc','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-HTML-TableExtract','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-HTML-Template','5','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-HTML-Tidy','3','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-HTML-Tree','3','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-HTML-WikiConverter','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-HTML-WikiConverter-Markdown','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-HTTP-Cache-Transparent','2','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-HTTP-CookieJar','3','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-HTTP-Proxy','2','31364.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Hook-LexWrap','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-All','1','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-CaptureOutput','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-HTML','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-Multiplex','17','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-SessionData','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-Socket-INET6','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-Socket-Multicast','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-Socket-Socks','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-Socket-Timeout','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-String','1','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-Stty','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-Tty','15','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IO-stringy','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IP-Country','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IPC-DirQueue','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IPC-Run','1','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IPC-Run3','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IPC-System-Simple','3','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IPTables-ChainMgr','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-IPTables-Parse','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Image-Info','4','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Image-Sane','3','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Image-Size','2','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Import-Into','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Inline','3','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Inline-C','1','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-JSON','42','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-JSON-MaybeXS','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-JSON-XS','4','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-LWP-Protocol-https','3','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-LWP-Protocol-socks','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Lingua-EN-Fathom','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Lingua-EN-Inflect','4','27771.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Lingua-EN-Numbers-Ordinate','2','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Lingua-EN-Syllable','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Lingua-Preferred','1','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Linux-FD','4','30480.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-List-AllUtils','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-List-MoreUtils','9','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-List-MoreUtils-XS','9','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-List-SomeUtils','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-List-UtilsBy','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Locale-Codes','5','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Locale-Maketext-Gettext','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Locale-Maketext-Lexicon','3','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Locale-PO','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-LockFile-Simple','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Log-Dispatch','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Log-Log4perl','31','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Log-Message','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Log-Message-Simple','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MIME-Charset','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MIME-Lite','2','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MIME-Types','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MIME-tools','3','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MP3-Info','3','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MRO-Compat','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Mail-DKIM','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Mail-DMARC','24','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Mail-DomainKeys','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Mail-SPF','3','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Mail-SPF-Query','2','28887.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MailTools','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-Base85','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-BigInt','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-BigInt-GMP','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-Complex_C','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-Complex_C-L','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-Complex_C-Q','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-Decimal64','13','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-GMPf','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-GMPq','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-GMPz','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-GSL-Linalg-SVD','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-Int64','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-LongDouble','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-MPC','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-MPFR','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-Prime-Util','14','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-Prime-Util-GMP','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Math-Round','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MaxMind-DB-Common','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MaxMind-DB-Reader','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Meta-Requirements-Range','3','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Modern-Perl','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Module-Build','42','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Module-Build-Tiny','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Module-Build-XSUtil','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Module-Implementation','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Module-Install','20','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Module-Manifest','2','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Module-Pluggable','4','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Module-Runtime-Conflicts','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Module-ScanDeps','2','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Module-Versions-Report','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Mojolicious','46','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Mojolicious-Plugin-Humane','2','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Mojolicious-Plugin-I18N','3','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MooX-StrictConstructor','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MooX-Types-MooseLike','3','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Moose','82','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MooseX-Traits','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MooseX-Types','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MooseX-Types-Structured','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Mozilla-CA','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-MusicBrainz-DiscID','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Mutex','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-ARP','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-CIDR','2','28534.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-CIDR-Lite','2','29272.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Curl','8','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-DNS-Native','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-DNS-Resolver-Mock','2','29832','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-DNS-Resolver-Programmable','1','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-DNS-SEC','4','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Daemon','71','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-IDN-Encode','6','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-IPv4Addr','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-IPv6Addr','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Ident','1','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Jabber','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-LDAP','7','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-LibIDN','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-LibIDN2','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Libdnet','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Netmask','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Patricia','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Pcap','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-RawIP','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-SNMP','6','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-SSH','2','28784.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Server','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Telnet','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-UPnP','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-Write','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Net-XMPP','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-NetAddr-IP','15','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-NetPacket','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Number-Bites-Human','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Number-Compare','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Number-Convert-Roman','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Object-Accessor','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Ogg-Vorbis-Header-PurePerl','2','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-PAR','3','29260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-PAR-Dist','3','29260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-PDF-API2','10','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-PDF-Builder','22','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-PPI','25','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-PPIx-QuoteLike','3','27542.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-PPIx-Regexp','12','27542.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-PPIx-Utils','3','27542.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Package-Constants','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Package-DeprecationManager','2','28784.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Package-Stash','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Package-Stash-XS','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Params-Classify','4','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Params-Util','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Params-Validate','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Params-ValidationCompiler','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Parse-Method-Signatures','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Parse-RecDescent','3','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Parser-MGC','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Path-Class','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Pegex','3','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Perl-Critic','34','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Perl-Tidy','68','27542.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-PerlIO-via-Timeout','5','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Pod-Coverage','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Pod-POM','3','27283.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Pod-POM-View-Restructured','2','30544.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Pod-Parser','3','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Pod-Spell','4','27542.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Probe-Perl','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Proc-Daemon','2','31364.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Proc-PID-File','15','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Proc-ProcessTable','3','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Proc-Simple','17','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Readonly','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Regexp-Common','14','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Regexp-IPv6','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Religion-Islam-Quran','16','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Return-MultiLevel','2','30591.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Role-Tiny','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-SDL','32','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-SNMP-Info','5','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-SOAP-Lite','3','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Scalar-List-Utils','5','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Schedule-Cron','43','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Scope-Guard','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Scope-Upper','10','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Set-IntSpan','1','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sidef','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Socket-GetAddrinfo','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Socket6','4','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sort-Naturally','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sort-Versions','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Statistics-Descriptive','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-String-Format','2','27542.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-String-PerlIdentifier','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-String-Random','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-String-Scanf','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-String-ShellQuote','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-String-Similarity','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sub-Exporter','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sub-Exporter-ForMethods','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sub-Exporter-Progressive','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sub-Identify','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sub-Install','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sub-Name','3','28622.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sub-Override','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sub-Uplevel','1','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Switch','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Syntax-Highlight-Engine-Kate','27','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Syntax-Keyword-Try','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sys-CPU','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sys-Hostname-Long','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sys-MemInfo','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sys-Mmap','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Sys-Syscall','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-System-Command','18','28784.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-System-Sub','2','28622.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Task-Weaken','2','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-TeX-Hyphen','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Term-Animation','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Term-ProgressBar','13','28784.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Term-ReadLine-Gnu','4','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Term-UI','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Bits','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Class','4','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-CleanNamespaces','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Deep','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Differences','3','28784.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-DistManifest','2','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Exception','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Exit','2','30591.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Fatal','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-File','3','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-File-ShareDir','3','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-File-ShareDir-Dist','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Harness','2','29896.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-HexDifferences','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Inter','2','34729.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-LongString','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Memory-Cycle','3','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-MockModule','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-MockObject','1','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-MockTime','30','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Most','3','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Needs','4','28717.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-NoWarnings','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Number-Delta','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Object','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Output','3','28784.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-PerlTidy','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Pod','1','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Pod-Coverage','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Requires','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-RequiresInternet','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-SharedFork','42','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Simple','13','29896.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Spec','5','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-SubCalls','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-TCP','12','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-TempDir-Tiny','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Trap','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-Warnings','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Test-utf8','3','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Autoformat','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-CSV','18','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-CSV_XS','19','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-CharWidth','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Diff','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Glob','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Hyphen','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Iconv','2','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Password-Pronounceable','2','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Patch','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Quoted','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Reform','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Soundex','2','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Tabs+Wrap','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Tabulate','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-Unidecode','4','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Text-WrapI18N','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Throwable','2','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Tidy','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Tie-IxHash','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Tie-Simple','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Time-Duration','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Time-Out','1','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Time-ParseDate','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Time-Piece','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Time-modules','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-TimeDate','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Tk-TableMatrix','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Tree-Simple','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Try-Tiny','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-TryCatch','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Types-Serialiser','1','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-URI-Simple','1','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-UUID','15','28784.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Unicode-LineBreak','7','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Unicode-Map','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Unicode-Map8','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Unicode-String','2','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Unicode-UTF8simple','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Unix-Syslog','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-Variable-Magic','5','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-WWW-Curl','4','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-WebService-Gyazo-B','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-WebService-MusicBrainz','2','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-X11-Protocol-Other','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-XML-Catalog','2','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-XML-Filter-BufferText','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-XML-LibXSLT','4','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-XML-SAX-Writer','2','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-XML-Stream','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-XML-TreeBuilder','2','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-XML-TreePP','5','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-XML-Writer','1','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-XML-XPath','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-XS-Parse-Keyword','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-YAML-LibYAML','6','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-YAML-Syck','25','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-YAML-Tiny','3','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-ZMQ-Constants','2','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-aliased','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-audio-sndfile','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-cairo','4','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-cgi-minimal','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-class-accessor','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-class-accessor-chained','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-common-sense','2','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-config-general','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-crypt-dh-gmp','3','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-data-dump','2','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-data-messagepack','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-data-page','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-data-page-pageset','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-data-section-simple','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-digest-hmac','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-digest-sha1','2','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-encode-locale','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-event','3','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-extutils-depends','2','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-extutils-pkgconfig','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-file-basedir','2','28747.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-file-desktopentry','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-file-libmagic','3','28784.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-file-listing','5','29783.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-file-mimeinfo','2','29260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-file-path-expand','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-file-slurper','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-glib','10','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-gnome2','10','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-gnome2-canvas','5','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-gnome2-gconf','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-gnome2-vfs','4','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-gnome2-wnck','4','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-goo-canvas','4','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-gstreamer','7','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-gtk2','31','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-gtk2-imageview','3','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-html-form','3','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-html-parser','6','28887.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-html-tagset','1','28365.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-http-cookies','2','28887.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-http-daemon','4','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-http-date','2','30748.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-http-message','4','28887.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-http-negotiate','1','28887.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-http-response-encoding','2','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-http-server-simple','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-http-tiny','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-image-bmp','3','28747.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-inc-latest','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-inc-module-install','19','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-io-uncompress-brotli','28','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-libintl','5','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-linux-desktop-files','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-lirc-client','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-local-lib','6','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-lwp-mediatypes','2','29783.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-lwp-useragent-cached','2','27870.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-namespace-autoclean','3','28784.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-namespace-clean','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-net-dbus','11','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-net-dns','3','28887.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-net-http','2','29783.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-net-ip','2','28487.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-net-mpd','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-palm-pdb','2','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-pango','5','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-parallel-forkmanager','69','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-strictures','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-test-warn','4','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-text-aspell','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-text-unaccent','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-tk','34','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-trayicon','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-tree-dagnode','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-www-mechanize','2','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-www-robotrules','7','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-x11-protocol','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-xml-fast','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-xml-libxml','9','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-xml-parser-lite','1','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-xml-parser-lite-tree','2','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-xml-twig','3','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perl-yaml','1','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/perlbrew','1','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/raku-Readline','22','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/rakudo','184','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/txt2html','2','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('perl/zef','30','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/BeautifulSoup4','3','29706.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/CANard','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/CommonMark','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/DVR-Scan','2','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Flask','3','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Flask-Gravatar','1','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Flask-HTMLmin','1','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/GeoIP-Python','3','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Kallithea','6','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Markdown','2','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Markups','2','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Paver','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PyAudio','3','28082.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PyDispatcher','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PyGreSQL','4','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PyMeeus','3','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PyPDF2','2','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PyPy','2341','31818.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PyPy3','2115','31818.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PyRIC','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PyStemmer','13','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PyXB','3','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Pyro','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Pyro4','3','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/PythonQt','60','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/RBTools','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/SOAPpy','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/SQLAlchemy-legacy','9','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/TatSu','3','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Unidecode','4','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/XlsxWriter','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/Yapps2','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/acstore','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/affine','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/alabaster','2','33207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/apprise','4','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/apsw','9','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/arrow','2','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/artifacts','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/asciimatics','3','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/asgiref','2','28651.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/astroid','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/async_generator','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/audible','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/audible-cli','3','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/autopep8','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/babel','26','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/backcall','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/backports-abc','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/backports.lzma','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/bencode','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/binaryornot','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/binplist','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/biplist','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/bitarray','3','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/blessings','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/boolean-py','2','28571.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/bottle-mysql','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/breathe','4','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/bsddb3','10','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/buildbot','11','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/buildbot-badges','2','33022.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/buildbot-worker','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/captcha','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cddb-py','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/certbot-dns-cloudflare','2','28273','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cfgparse','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cheetah3','3','29706.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/click','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/click-default-group','2','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/click-legacy7','2','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/click-log','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/click-plugins','2','31818.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/click-threading','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cloudpickle','2','27312.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cloudscraper','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/colorama','1','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/colored','2','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/construct','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/contextlib2','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/convertdate','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/coverage','3','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cppy','3','28401.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/crcmod','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cryptography','37','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/css-parser','3','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cssselect2','2','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cssutils','15','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/curtsies','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cwcwidth','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/cx_Freeze','3','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/darts.util.lru','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/dfdatetime','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/dfvfs','5','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/dfwinreg','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/dhcpy6d','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/django-debug-toolbar','3','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/django-sass','1','29101.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/doit','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/dotty','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/dpkt','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/drawille','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/dtfabric','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/dukpy','15','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/dxfwrite','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/easygui','2','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ebaysdk','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ecdsa','3','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/elasticsearch5','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/email-validator','2','31209.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/enzyme','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/esptool','3','31218.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/euca2ools','7','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/extras','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/fasteners','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/feedgenerator','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/feedparser','9','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/filechunkio','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/findsystemfontsfilename','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/fixtures','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/flake8','2','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/freesimplegui','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/freetype-py','4','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/frosted','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/fudge','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/funcsigs','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/functools-lru-cache','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/functools32','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/gi-docgen','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/git-fame','3','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/githubpy','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/gnome-python','3','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/gnome-python-desktop','8','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/google-api-python-client','15','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/google-images-download','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/gst-python','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/gst0-python','13','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/gunicorn','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/hachoir','7','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/hachoir-core','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/hachoir-metadata','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/hachoir-parser','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/halberd','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/hg-git','3','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/hgsubversion','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/hijri-converter','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/holidays','9','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/homebase','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/hpack','3','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/html2text','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/html5-parser','11','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/htmlmin','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/huepy','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/humanize','2','26938.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/i3-layouts','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/i3-py','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/i3expo-ng','2','25960','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/i3ipc','2','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ics','5','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/imagesize','2','33207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/importlib-resources','4','30544.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/insipid-sphinx-theme','21','29349.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/instaloader','2','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/internetarchive','2','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ipaddr-py','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ipython','5','29301.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/isort','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/jaraco-packaging','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/jdatetime','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/jdcal','2','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/jellyfish','16','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/jmespath','1','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/josepy','2','33780','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/json-py','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/kallithea-frontend','17','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/kaptan','2','27608.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/kitchen','3','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/korean_lunar_calendar','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/license-expression','2','28571.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/linecache2','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/llfuse','23','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/mechanize','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/memory-profiler','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/mibuild','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/migen','3','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/mini-amf','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/minidb','1','27608.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/mock','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/monotonic','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/monty','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/more-itertools','2','27608.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/msgpack-python','7','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/munch','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/mypy','16','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/mypy_extensions','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/nanobind','15','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ndg_httpsclient','2','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/node-semver','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/npm','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/npyscreen','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/num2words','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/numexpr','9','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/nyx','3','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/objgraph','3','41948','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/odict','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/odoo','86','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ofxparse','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/optional-django','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/p4python','6','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pallets-sphinx-themes','1','28571.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/parsedatetime','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pathlib','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pathtools','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pathtools3','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pattern','33','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pefile','3','28273','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pelican','6','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/phply','8','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pickleshare','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pikepdf','23','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pipdeptree','3','26938.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/piprot','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pipstat','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pkginfo','2','31818.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/plaso','12','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pluginbase','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/polib','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/progressive','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ps_mem','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ptpython','3','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ptyprocess','1','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/py3sensors','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyOpenSSL','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pySmartDL','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyaes','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyamf','21','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyatspi','4','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pybind11','5','29371.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pybind11-legacy','5','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyblake2','4','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pybloomfiltermmap','5','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pycassa','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pychecker','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pychm','3','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pycld2','3','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pycodestyle','2','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pycxx','1','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pydblite','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pydf','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pydot','1','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyelftools','2','35185.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyewmh','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyfeed','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyfiglet','4','27608.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyflakes','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyfltk','70','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyfuse3','26','31818.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyglet','4','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pygpgme','3','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pygrametl','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pygsl','68','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pygtksourceview','9','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pygtkspell','8','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyjwt','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyliblo','12','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pymongo','11','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pymysql','3','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyorbit','6','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyotherside','18','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyotp','2','28628.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyperclip','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pypng','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pypoppler','10','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pypy','45','31818.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pypy3','53','31818.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyqode.cobol','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyqode.qt','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyquery','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyrfc3339','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyrsistent','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pysass','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyscard','4','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyserial','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pysha3','3','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyside','234','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyside2','645','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyside2-tools','6','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pysmssend','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pysnmp','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pystatsd','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pysvn','7','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyta-lib','68','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pytest','2','41948','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-PySnooper','1','27608.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-argopt','3','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-axolotl','3','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-axolotl-curve25519','15','28628.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-bitstring','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-colour','2','25960','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-daemonize','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-debian','4','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-discid','1','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-diskcache','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-djvulibre','52','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-emoji','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-faulthandler','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-fido2','3','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-fusepy','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-gflags','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-gntp','2','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-gssapi','43','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-importlib_metadata','4','29706.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-ipcalc','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-jeepney','2','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-jsonrpclib','2','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-kerberos','4','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-krb5','73','27973.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-libarchive-c','2','27607.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-libnacl','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-ly','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-magic','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-markdown-math','2','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-musicbrainz2','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-mysql-replication','2','29499.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-mysqlclient','3','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-nbxmpp','4','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-oauthlib','3','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-parse','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-parsedatetime','2','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-pbkdf2','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-pcapy','4','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-poppler-qt5','104','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-precis-i18n','2','28040.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-pyparted','3','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-qpageview','2','31364.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-qrcode','2','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-rubymarshal','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-serpent','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-setuptools-git','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-sh','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-slip','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-smartypants','1','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-stdnum','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-tlslite','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-unicodedata2','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-uritemplate','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-urwid_readline','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-vdf','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-wand','3','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-webpack','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-xlib','3','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python-zipp','4','29706.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-BeautifulSoup4','2','28454.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-Levenshtein','4','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-MarkupSafe','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-MonthDelta','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-Safe','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-asn1crypto','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-atomicwrites','1','41948','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-attrs','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-authres','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-bigfloat','11','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-blessed','2','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-cached-property','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-cheetah','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-clamd','1','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-cluster','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-configobj','2','28628.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-cryptography','18','27974.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-cssselect','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-daemon','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-dateutil','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-demjson','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-distorm','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-distutils-extra','1','28651.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-django','27','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-django-legacy','23','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-django-tagging','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-dnspython','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-docopt','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-editor','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-elementtree','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-elib.intl','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-enlighten','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-esmre','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-gammu','5','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-gattlib','19','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-gmpy2','13','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-httplib2','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-invoke','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-keyczar','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-keyutils','3','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-lazy-object-proxy','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-lhafile','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-lockfile','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-lxml','167','29044.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-lz4','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-magick','91','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-mccabe','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-memcached','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-mimeparse','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-morbid','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-mpd','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-mpmath','4','31209.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-neovim','2','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-networkx','7','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-nuitka','19','29314.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-oauth','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-oauth2','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-oauth2client','2','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-openid','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-paramiko','3','27974.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-passlib','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-patch','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pathlib2','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pbr','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pdfrw','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pillow','11','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pkgconfig','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-plucky','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pluggy','2','41948','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pmw','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-prefixed','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-progress','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-psutil','3','34610.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-py','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pyasn1','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pybluez','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pycdio','6','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pycrypto','8','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pycryptodomex','28','33022.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pydns','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pygame','65','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pyicu','45','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pypolicyd-spf','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pyspf','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pytest-runner','1','29101.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pythondialog','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-pytz','2','29260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-regex','10','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-requestbuilder','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-requests-kerberos','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-scandir','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-selectors2','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-setuptools-scm','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-simpy','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-slacklog','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-soupsieve','2','41948','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-stomper','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-suds','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-swiftclient','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-sympy','29','31209.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-tabulate','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-tblib','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-texttable','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-toml','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-transaction','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-tvrage','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-twitter','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-uri-templates','1','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-uuid','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-vatnumber','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-vulndb','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-wheel','2','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-whisper','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-wordpress-xmlrpc','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-wrapt','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-x11_hash','21','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-xcffib','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-xlwt','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-xrandr','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-yara','3','28273','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python2-yenc','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-APScheduler','5','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-BTrees','45','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-Flask-Babel','3','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-Flask-Basicauth','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-Flask-Bcrypt','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-Flask-Compress','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-Flask-Cors','4','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-Flask-Dance','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-Flask-Login','1','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-Flask-Mail','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-Flask-Principal','1','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-Flask-RESTX','5','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-Flask-RESTful','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-Flask-SQLAlchemy','3','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-Flask-Security-Too','3','28552.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-Flask-WTF','3','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-Flask-httpauth','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-MutatorMath','3','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-PyGithub','8','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-PyMuPDF','240','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-PyPDF2','3','28651.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-WTForms','3','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ZConfig','3','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ZEO','3','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ZODB','5','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-aiofiles','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-aiohappyeyeballs','1','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-aiohttp','21','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-aiohttp-oauthlib','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-aiohttp-socks','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-aiorpcX','3','30544.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-aiosignal','1','33780','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-aiosmtpd','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-aiostream','2','33022.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-aksharamukha','4','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-alembic','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-amqp','2','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-aniso8601','2','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-annotated-types','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-anyio','4','28824.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-anysqlite','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-application','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-arabic-reshaper','2','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-argcomplete','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-argon2-cffi','3','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-argon2-cffi-bindings','5','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-arrow','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-asn1crypto','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-astroid','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-asttokens','2','30733.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-async-lru','2','28651.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-async-timeout','1','28651.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-atomicwrites','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-atpublic','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-attrdict','3','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-attrs','3','30544.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-auditok','2','29228.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-augeas','3','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-authheaders','2','29914.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-authres','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-autobahn','7','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-autocommand','3','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-automat','4','28651.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-babel','23','29301.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-babelfish','2','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-backlash','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-backports_tarfile','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-base58','1','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-bcrypt','22','29384.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-beaker','2','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-bech32','2','27933.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-beekeeper-alt','2','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-bencode','1','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-beniget','1','34610.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-bidi','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-billiard','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-bitarray','6','29292.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-black','4','27312.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-bleach','2','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-blessed','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-blinker','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-booleanOperations','2','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-boto3','9','30917.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-bottle','2','31818.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-brotlicffi','19','28552.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-bsdiff4','2','31239.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-build','1','29706.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-cachecontrol','1','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-cachetools','3','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-cacheyou','2','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-cairocffi','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-caldav','2','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-calver','3','29706.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-cattrs','2','28552.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-celery','3','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-cffi','10','48097.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-cffsubr','4','30544.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-cfgv','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-characteristic','2','28570.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-cheroot','4','28552.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-cinemagoer','3','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-click-didyoumean','2','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-click-option-group','2','34287.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-click-repl','3','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-cloudflare','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-cmake-build-extension','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-cogapp','1','31218.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-colorlog','2','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-comm','3','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-compreffor','6','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-configargparse','3','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-configobj','2','28628.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-confuse','3','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-constantly','2','28748.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-construct','2','30218.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-construct-classes','1','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-cookiecutter','3','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-crank','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-croniter','2','29355','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-cssselect','3','29706.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-cython-opt','167','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-daemon','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-dasbus','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-dateutil','2','27310.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-dbus-fast','2','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-dbus-next','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-debugpy','42','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-decorator','3','28401.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-defcon','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-defusedxml','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-deluge-client','2','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-dep-logic','2','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-deprecated','2','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-deprecation','1','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-desktop-notify','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-diff-match-patch','2','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-dill','2','27608.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-discovery','2','28585.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-distlib','5','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-distutils-extra','2','28651.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-django','101','29260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-dkimpy','2','31209.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-dnspython','4','31209.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-docker','3','34287.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-dockerpty','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-docopt','1','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-docstring-to-markdown','1','28651.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-dogpile.cache','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-dotenv','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-dugong','2','27771.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-dulwich','4','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-dunamai','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-editables','2','30329.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-editorconfig','1','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-elgato-streamdeck','4','31218.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-enlighten','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-entrypoints','2','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-enzyme','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-et_xmlfile','1','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-euclid3','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-evdev','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-eventlet','3','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-eventlib','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-exceptiongroup','3','28552.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-executing','2','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-exiv2','12','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-expandvars','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-falcon','12','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-fanart','2','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-fasteners','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-fastjsonschema','2','31218.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ffmpeg','2','31209.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ffsubsync','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-filelock','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-filetype','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-findpython','2','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-flake8_annotations','2','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-flask-paginate','1','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-flit','2','29355','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-flit_core','1','29706.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-flit_scm','2','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-flufl.bounce','3','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-flufl.i18n','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-flufl.lock','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-fontMath','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-fontParts','3','33207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-fontPens','1','33207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-fontmake','3','28571.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-fonttools','31','28585.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-formencode','2','31818.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-fqdn','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-fritzconnection','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-frozenlist','6','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-fs','2','33207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-funcparserlib','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-fypp','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-gTTS','2','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-gast','2','27973.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-gattlib','21','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-gearbox','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-gensim','46','27979.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-geventhttpclient','4','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-gitdb2','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-glad','8','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-glances','7','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-glyphslib','8','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-gmpy2','24','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-gnutls','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-google-api-python-client','46','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-google-auth-httplib2','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-google-auth-library-python','3','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-google-python-api-core','3','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-googleapis-common-protos','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-grpcio','516','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-guessit','2','27608.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-h11','2','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-h2','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-h5py','104','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-hatch-nodejs-version','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-hatch_fancy_pypi_readme','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-hatch_jupyter_builder','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-hatch_vcs','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-hatchling','3','30329.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-helpdev','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-hishel','3','31029.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-hsaudiotag3k','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-httpcore','2','28824.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-httplib2','2','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-httpx','3','29301.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-hupper','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-hyperframe','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-hyperlink','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-icalendar','4','26938.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-icoextract','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-id','2','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-identify','2','28210.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ifaddr','2','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-incremental','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-inflate64','4','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-inflect','5','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-inflection','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-iniconfig','2','26938.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-inotify-simple','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-installer','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-intelHex','3','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-intervaltree','3','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-invoke','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ipaddr','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ipython-pygments-lexers','2','28719.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ipython-sql','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ipython_genutils','1','41948','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-iso3166','3','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-iso639','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-isodate','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-isoduration','2','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-isort','2','27608.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-isounidecode','3','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-itsdangerous','1','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-jaconv','2','28747.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-janus','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-jaraco.classes','4','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-jaraco.collections','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-jaraco.context','4','29349.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-jaraco.functools','4','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-jaraco.text','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-jedi','13','29301.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-jinja2-time','1','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-joblib','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-json-logger','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-json5','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-jsonlines','1','30544.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-jsonpatch','2','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-jsonpointer','2','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-jsonschema','4','28571.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-jsonschema-specifications','2','28571.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-keyring','5','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-kiwisolver','16','28585.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-klein','4','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-klepto','2','30480.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-knowit','2','28571.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-kodipydent','2','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-kombu','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-langcodes','4','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-language-data','13','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-lap','10','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-lazr.config','2','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-lazr.delegates','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-lazy-object-proxy','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-lazy_loader','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ldap','5','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-levenshtein','22','35185.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-lhafile','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-libagent','2','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-libretranslatepy','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-librosa','3','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-libsass','138','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-libtmux','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-libusb1','2','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-libvcs','2','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-llvmlite','9','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-lockfile','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-logbook','8','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-loguru','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-lsp-black','2','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-lsp-jsonrpc','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-lsp-server','5','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-lxml','122','30329.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-lz4','2','31209.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-mailman','14','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-marisa-trie','19','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-markdown-it-py','2','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-markdown2','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-mat2','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-matplotlib-inline','2','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-matrix-nio','5','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-maturin','88','29499.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-mccabe','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-mdurl','2','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-mediafile','2','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-meson-opt','16','27779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-mesonpy','2','28675.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-mir_eval','4','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-mistune','4','27926','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-mnemonic','2','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-monotonic','2','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-more-itertools','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-mpi4py','66','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-mpmath','7','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-msal','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-msrplib','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-multidict','5','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-multivolumefile','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-narwhals','5','28486.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-natsort','1','28816.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-neovim','2','31209.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-nest_asyncio','2','33381.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-netaddr','9','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-netifaces','3','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-networkx','9','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-new-rtorrent','1','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-nltk','12','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-nuitka','20','28378.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-numba','29','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-numpydoc','3','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-nxt-python','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-oauth2client','4','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-obspy','35','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-odfdo','5','30329.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-olm','30','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-opencv','346','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-openpyxl','4','31070.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-openstep-plist','11','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ordered-set','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-orjson','19','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-oscrypto','3','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-otr','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-outcome','2','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-overrides','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-packaging-opt','2','28675.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-paginate','2','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-paginate_sqlalchemy','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-paho-mqtt','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pandas','158','28556.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pandocfilters','1','29260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pantalaimon','3','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-paramiko','4','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-parso','3','29301.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-passlib','3','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-paste','2','29101.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pastedeploy','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-patch-ng','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pathlib2','2','48251.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pathspec','2','29706.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-patsy','2','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pdfCropMargins','5','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pdfminer.six','10','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pdfrw','2','28628.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pdm','3','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pdm-backend','2','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pdm-build-locked','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pdm-pep517','4','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-peewee','21','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-persistent','4','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pexpect','1','28816.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pgpy','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pillow-opt','24','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pint','6','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pipx','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pkgconfig','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-platformdirs','2','28669.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-playsound','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-plexapi','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-plotly','36','28486.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-plucky','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pluggy','3','29706.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ply','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pmw','3','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-poetry-core','5','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-poetry-dynamic-versioning','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pooch','4','28669.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-portend','4','34610.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pox','2','30480.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-prefixed','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-presets','2','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-profilehooks','2','27862.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-progress','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-prometheus_client','6','28802.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-prompt_toolkit','5','29301.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-prompt_toolkit38','7','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-propcache','6','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-psautohint','6','33207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-psutil','6','28401.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-psycopg2','8','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-psycopg3','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ptyprocess','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-publicsuffix2','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pubsub','2','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pulsectl','2','34610.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pulsectl-asyncio','4','35185.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pure_eval','2','27973.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-puremagic','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-putio.py','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-py','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-py-cpuinfo','4','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-py7zr','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyasn1','3','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyasn1-modules','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pybcj','2','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pybluez','3','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pycdio','6','28628.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyclipper','10','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pycrypto','21','27771.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pycryptodome','30','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pycryptodomex','28','33022.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pydantic','5','28552.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pydantic-core','295','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pydbus','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pydispatcher','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pydocstyle','3','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pydub','2','28419.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pydyf','2','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyflakes','2','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pygame','88','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyicu','56','27920.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyinotify','2','35185.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyjwt','4','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pykakasi','9','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pykeepass','5','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pylint','4','27312.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pylint-venv','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyls-spyder','2','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pylyrics','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pymupdf_fonts','4','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pynacl','31','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pynma','1','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pynzb','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyogrio','31','34287.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyosmium','52','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyparted','4','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pypdf','4','28669.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyphen','4','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pypng','4','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyppmd','3','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pypresence','1','28816.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyproject-api','2','26938.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyproject-hooks','1','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyproject-metadata','2','29706.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyrsistent','3','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pysdl2','4','34610.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyspf','2','31209.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pysrt','2','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pystache','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pysubs2','3','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pytest','4','28585.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pytest-cov','2','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pytest-runner','3','28585.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pytest-timeout','1','27896.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-python-editor','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pythondialog','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pythran','19','29371.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pytoolconfig','2','28552.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pytube','2','27608.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pytz','3','28675.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyudev','2','33780','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyusb','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyzmq','19','27926','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-pyzstd','23','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-qbittorrent-api','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-qstylizer','1','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-qtawesome','4','29362.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-questionary','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-quimb','12','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-rapidfuzz','116','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-rebulk','2','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-recurring-ical-events','3','28571.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-referencing','3','27312.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-regex','14','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-reportlab','6','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-repoze.lru','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-requests-cache','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-requests-unixsocket','4','30544.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-resolvelib','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-rfc3339_validator','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-rfc3986','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-rfc3986_validator','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-rich','3','29301.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-rope','6','31818.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-routes','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-rpds-py','19','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-rsa','1','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-rtree','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-sabctools','13','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-schedule','1','27608.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-scikit-build-core','4','29371.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-seaborn','3','28585.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-semantic-version','2','29266.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-semver','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-service-identity','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-setproctitle','2','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-setuptools-git-ls-files','2','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-setuptools-git-versioning','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-setuptools-opt','11','28675.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-setuptools-rust-opt','4','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-setuptools-scm-opt','4','29706.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-setuptools_autover','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-setuptools_scm_git_archive','1','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-sge','3','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-sgmllib3k','1','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-shamir-mnemonic','2','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-shellingham','3','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-shtab','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-simple-term-menu','1','27608.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-slacklog','1','28651.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-slip10','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-slugify','2','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-smart_open','5','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-smmap2','2','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-sniffio','4','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-socks','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-solidpython','7','31209.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-sortedcontainers','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-soundfile','2','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-soupsieve','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-soxr','7','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-sqlobject','3','31818.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-srt','1','28717.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-stack_data','2','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-stagger','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-statsmodels','303','28491.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-stevedore','2','28651.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-stone','3','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-svglib','2','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-sympy','39','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-sysrsync','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-tabulate','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-tasklib','2','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-tekore','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-teletype','3','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-tempora','4','34610.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-tenacity','2','30480.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-terminado','3','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-testpath','2','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-text-unidecode','2','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-textdistance','1','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-texttable','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-tgext.routes','2','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-threadpoolctl','2','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-three-merge','2','31070.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-timeago','2','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-tinycss2','2','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-tinydb','1','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-tmdbsimple','2','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-toml','2','33207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-tomli-opt','1','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-tomli_w','1','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-tomlkit','3','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-toolz','3','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-tornado','4','27312.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-tox','4','28585.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-trakit','2','31218.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-transaction','1','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-treq','2','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-trezor','5','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-trezor-agent','2','31209.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-trio','6','28552.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-trove-classifiers','3','29706.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-tubes','3','31818.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-turbogears2','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-tus.py','2','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-tvdbsimple','1','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-twisted','23','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-txaio','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-typeguard','4','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-types-python-dateutil','4','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-typing-extensions','3','29706.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-typing-inspection','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-typogrify','2','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ufo2ft','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ufoProcessor','1','33207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ufolib2','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ufonormalizer','2','33207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-uharfbuzz','43','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-ujson','10','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-unearth','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-unidiff','2','29882.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-unpaddedbase64','2','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-uri_template','5','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-uritools','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-url-normalize','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-urlobject','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-userpath','2','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-uth','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-uv_build','82','29764.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-validators','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-versioneer','4','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-vharfbuzz','2','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-videohash','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-vine','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-virtualenv','5','28585.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-voluptuous','2','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-waitress','2','29128.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-watchdog','3','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-wcwidth','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-weasyprint','3','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-webcolors','2','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-webencodings','2','29706.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-webhelpers2','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-webob','2','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-webrtcvad','4','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-websocket-client','2','34287.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-websockets','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-whatthepatch','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-wheel','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-whoosh','3','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-wrapt','3','27608.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-wurlitzer','1','28972.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-x-wr-timezone','1','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-xapp','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-xcaplib','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-xcffib','5','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-xdg-base-dirs','2','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-xkeysnail','2','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-xlib','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-xmldiff','2','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-xmltodict','2','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-yara','3','28273','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-yarl','7','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-zdaemon','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-zeroconf','104','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-zodbpickle','5','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-zopfli','11','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-zstandard','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/python3-zxcvbn','2','28212.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pytsk','47','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pytz-deprecation-shim','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pytzdata','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pyusb','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/pywal','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/queuelib','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ramlfications','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/readme_renderer','2','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/recommonmark','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/rectangle-packer','5','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/reedsolo','30','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/relatorio','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/rencode','10','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/requests-futures','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/requests-oauthlib','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/requests-toolbelt','2','30776.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/responder','3','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/rfc6555','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/rope','3','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/roundrobin','1','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/rst','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/rst-linker','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/rst2ansi','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/rst2pdf','4','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ruff-bin','12','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ruffus','4','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/s3cmd','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/s3transfer','2','28312.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/schema','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/scikit-learn','312','28484.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/scikit-umfpack','4','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/secretstorage','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/selenium','3','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/send2trash','3','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/setuptools-scm-git-archive','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/simplejson','3','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/singledispatch','1','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sk1libs','14','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/snowballstemmer','3','33207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/snuggs','2','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/soundcloud','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sounddevice','2','28571.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/speaklater','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sphinx-autobuild','2','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sphinx-last-updated-by-git','2','29349.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sphinx-rtd-theme','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sphinxcontrib-applehelp','2','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sphinxcontrib-asyncio','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sphinxcontrib-devhelp','2','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sphinxcontrib-fulltoc','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sphinxcontrib-htmlhelp','2','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sphinxcontrib-jquery','2','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sphinxcontrib-jsmath','2','33207.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sphinxcontrib-programoutput','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sphinxcontrib-qthelp','2','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sphinxcontrib-serializinghtml','3','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sphinxcontrib-websupport','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sql','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sqlalchemy-migrate','3','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/sqlparse','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/statsmodels','29','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/stem','3','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/stevedore','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/stopit','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/subliminal','4','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/subprocess32','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/subvertpy','7','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/suds-jurko','3','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/tagpy','24','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/taskw','2','29355','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/tempita','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/termcolor','2','26938.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/terminaltables','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/testscenarios','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/testtools','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/thonny','7','31029.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/threadpool','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/thunarx-python','4','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/timehash','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/tornado_systemd','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/tqdm','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/traceback2','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/traitlets','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/trollius','3','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ttfautohint-py','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ttkbootstrap','3','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/tweepy','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/ty-bin','10','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/typed_ast','13','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/tzdata','5','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/tzlocal','3','29355','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/unrardll','2','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/uv-bin','23','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/vanguards','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/vcversioner','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/veryprettytable','2','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/vinetto','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/virtualenv','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/virtualenv-clone','2','27608.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/vobject','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/wcwidth','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/webassets','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/werkzeug','3','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/werkzeug-legacy1','3','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/winpdb','2','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/workbook','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/wsgiref','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/xattr','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/xe','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/xlutils','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/xopen','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/xtermcolor','1','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/yapf','5','27312.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('python/zeep','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/3llo','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/chronic','2','26398','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/gem2tgz','1','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/ncurses-ruby','18','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/passenger','189','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/pastel','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rbenv','2','32883.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/ruby-build','4','28161.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/ruby-native-package-installer','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-activesupport','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-ast','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-atk','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-cairo','9','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-cairo-gobject','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-coderay','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-concurrent-ruby','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-cool.io','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-daemons','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-diff-lcs','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-fastercsv','2','26398','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-file-tail','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-fluentd','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-gdk3','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-gdk_pixbuf2','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-gio2','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-glib2','9','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-gobject-introspection','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-gosu','22','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-gtk2','28','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-gtk3','14','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-highline','3','26398','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-hpricot','13','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-http_parser.rb','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-httparty','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-i18n','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-lolcat','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-mail','22','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-manpages','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-matrix','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-method_source','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-mime-types','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-mime-types-data','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-mini_mime','2','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-msgpack','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-multi_json','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-multi_xml','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-mustache','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-native-package-installer','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-net-scp','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-net-sftp','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-net-ssh','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-optimist','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-paint','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-pango','15','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-parallel','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-parser','45','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-pkg-config','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-pry','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-pygments','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-rack','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-rdiscount','5','28628.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-red-colors','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-ronn','3','28628.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-rspec','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-rspec-core','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-rspec-expectations','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-rspec-mocks','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-rspec-support','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-ruby2ruby','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-ruby_parser','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-sass','6','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-sexp_processor','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-sigdump','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-slop3','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-sourcify','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-string-scrub','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-sync','2','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-term-ansicolor','2','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-terminal-table','2','26398','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-thread_safe','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-tins','4','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-trogdoro-el4r','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-trollop','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-tzinfo','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-tzinfo-data','4','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-xiki','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygem-yajl-ruby','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/rubygems-update','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/sequel','12','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/sequel4','15','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/sqlite3','6','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/tty-color','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/tty-cursor','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/tty-prompt','11','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/tty-reader','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/tty-screen','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('ruby/wisper','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/7zip','16','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/86box','106','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/AnonymousPro','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/Attic','11','31209.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/B-em','28','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/CNS11643-kai-font','79','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/CNS11643-sung-font','81','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/CPU-X','14','33780','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/CanAce','4','28747.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/DisplayCAL','9','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/EpsonTMXCUPS','3','28788.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/FontAwesome','27','28784.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/Fonts-GFS','8','29896.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/Fonts-GFS-Majuscule','2','29896.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/FreeFileSync','18','31209.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/FunkLoad','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/Gentium','5','30733.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/GentiumPlus','4','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/I-Nex','8','31209.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/Iosevka','66','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/Iosevka-aile','54','28218.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/Iosevka-core','28','29075.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/Iosevka-etoile','56','29994','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/Iosevka-slab','70','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/JetBrainsMono','3','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/Lmod','10','27941.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/Microsoft-Fonts','88','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/MySQL-Sandbox','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/OpenSnitch','26','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/PeaZip','18','27732.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/PrintNode','89','28788.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ProFont','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/PySixel','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/SavvyCAN','50','29296.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/Sl','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/Solaar','5','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/TLP','3','28737.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/TermRecord','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/Type1_to_OTF','21','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/UEFITool','12','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/WoeUSB','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ZoneMinder','130','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/a2tools','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/acefile','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ack','1','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/acopw-cli','8','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/acpi','3','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/acpi_call','3','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/acpitool','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/adbfs-rootless','2','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/adobe-source-code-pro-font','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/adobe-source-han-sans-fonts','62','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/adobe-source-sans-pro-font','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/adobe-source-serif-pro-font','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/aespipe','5','35185.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/afdko','73','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/afio','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/afpfs-ng','14','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/afuse','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/agedu','6','30733.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/aha','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/aide','6','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/alacritty','91','29970.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/alftools','2','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/alien','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/amanda','59','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ancient','23','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/android-file-transfer','14','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/android-udev-rules','1','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/andromeda','38','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ansible','194','28788.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ansible-core','9','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ansifilter','7','35185.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/anything-sync-daemon','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/apache-activemq','25','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/apache-cassandra','24','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/apache-tomcat','11','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/apachetop','7','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/apcupsd','10','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/apg','6','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/apparmor','48','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/apple-emoji','18','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/apple-fonts-Garamond','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/apple-fonts-NY','3','28273','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/apple-fonts-SF','95','28273','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/apptainer','152','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/arc','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/arch-install-scripts','1','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/archivemount','2','30733.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/archmage','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/argyllcms','186','28454.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/arj','8','31209.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/arqiver','8','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/artwiz-aleczapka','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/artwiz-cursor','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/asbt','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/asus-fanmode','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/atarisio','5','30733.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/aterm','10','28628.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/atool','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/atop','3','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/audit','36','28615.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/augeas','33','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/autoarchive','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/autocutsel','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/autojump','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/avfs','11','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/b2','280','27862.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/babeltrace','20','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/backintime','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bacula','40','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bacula-client','25','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/barman','3','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bat','106','27878.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bbf','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bbqsql','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bcache-tools','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/beebem','8','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bees','15','31029.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/betty','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bicon','15','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/biew','5','29260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bin2iso','2','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bindfs','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/binfmt-support','13','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bit-babbler','11','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bitmapfont2ttf','2','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bitrot','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/blake2','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bleachbit','7','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/blktrace','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/blocaled','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/blocks','2','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/blueprint-compiler','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bochs','38','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bogofilter','9','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bonnie++','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bootchart','4','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/borgbackup','48','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/borgmatic','6','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bottom','117','28839','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/brasero','60','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/brave-browser-the-latest','2','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/brightnessctl','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/brlaser','5','33780','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/brother-brscan4','2','31383.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/brother-dcp8410-cups','1','29131.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/brother-dcp8410-lpr','2','29131.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/btop','21','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/btrbk','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bubblewrap','4','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bulk_extractor','47','31218.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/burp','8','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/busybox','34','28455.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/butterfly','2','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/byobu','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/bzip3','8','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/c-icap','9','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/c-lcrypt','5','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cabextract','4','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/can-utils','14','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cantarell-fonts','24','28717.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/capabilities','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/caprice32','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/capstone','12','27633','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/catatonit','10','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cbmbasic','8','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cbmfs','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cc1541','2','28972.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ccat','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ccd2cue','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ccd2iso','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ccid','6','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ccrypt','10','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cdemu-client','1','34610.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cdemu-daemon','2','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cdrkit','10','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cdw','8','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cdwrite','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cfengine','48','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cfpeek','6','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/chipsec','12','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/chkrootkit','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/chkxex','1','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/chr','15','28552.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/chronograf','33','28454.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/chrpath','4','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ciderpress-tools','6','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ciopfs','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ck4up','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ckb-next','26','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/clamav','158','28620.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/clamsmtp','12','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/clamtk','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/clifm','17','31209.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/clinfo','3','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cluster-glue','28','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cndrvcups-capt','60','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cndrvcups-common','16','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cnijfilter2','73','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cntlm','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/collectd','72','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/colord','23','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/colord-gtk','9','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/colorize','2','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/colortail','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/commons-daemon','3','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/compsize','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/conky','42','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/conmon','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/consolation','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/conspy','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/convmv','1','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cool-retro-term','6','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/coolkey','10','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/corectrl','67','30480.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/corosync','24','28628.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/courier-prime','1','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/courier-screenplay','1','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cowsql','18','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cpmtools','4','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cpuid','5','29583.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cpuid2cpuflags','3','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cpuinfo','3','28571.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cpulimit','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/crc32_simple','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/createrepo_c','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/crelay','1','31218.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/crikey','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/crmsh','8','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cronie','4','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/crunch','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/csh','5','31029.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/csv2sql','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/csvutils','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/culmus','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cups-bjnp','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/curlftpfs','12','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cutecom','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cve-check-tool','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cw','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cwtex-q-fonts-TTFs','40','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/cyberjack','10','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/d-feet','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/d52','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/daa2iso','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/daemonize','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/daemontools','16','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/daemontools-bin','8','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dar','59','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dateutils','11','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/davfs2','7','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ddcutil','22','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ddg','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ddrescueview','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/debootstrap','4','30544.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ded','6','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/depfinder','2','28571.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/detox','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dfc','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dfm','3','28345.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dfu-util','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dget','2','27455.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dieharder','33','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dina-font','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/direvent','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/disk-manager','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/diskscan','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/disksearch','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dislocker','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/displaylink','4','28816.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/distrobuilder','21','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dlm','6','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/docbook-xml','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/docbook-xml5','6','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/docbookMathML1mods','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/domitian-fonts','1','29896.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dool','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dos33fsprogs','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dosbox','41','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dosbox-dev','52','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dosbox-staging','87','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dosbox-x','233','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dosemu','28','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dosemu-fonts','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/doublecmd-qt5','56','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dracut','13','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/drbd-utils','14','33780','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/driconf','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ds4drv','2','33780','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dse-typewriter-font','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dstat','2','29404.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dtach','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dtrx','3','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dump','8','29131.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dumptorrent','2','26938.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/duperemove','2','31029.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/duplicity','3','28571.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/duply','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dust','56','27783.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dvdisaster','32','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dymo-cups-drivers','6','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/dynamips','22','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/eaglemode','63','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/early-ssh','1','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/earlyoom','2','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ecm-tools','2','29260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ecofont-ttf','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ecryptfs-utils','67','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/edid-decode','10','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/edit','17','28748.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/edk2-aarch64','9','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/edk2-ovmf','6','29260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/efi-sync','1','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/efiboots','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/efitools','8','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/eibd','12','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/eightyone','10','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/elasticsearch','13','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/eltclsh','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/em','2','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/emelfm2','8','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/encfs','26','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/entr','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/epson-inkjet-printer-escpr','11','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/epson-inkjet-printer-escpr2','19','32180','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/epson-printer-utility','49','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/erofs-utils','12','30722.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/esekeyd','6','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/etckeeper','2','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/etsh','2','31239.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/evdi','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/evdi-kernel','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/eventlog','8','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/evhz','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/evolution-data-server','59','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/evtest','7','35185.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/execline','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/exfat-utils','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/exomizer','8','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ext4magic','6','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/extundelete','7','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/eza','72','29177.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/f3','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/facter','63','31029.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fakechroot','15','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fakeroot','17','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fantasque-sans-mono','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/far2l','88','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fasd','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fastfetch','28','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fatrace','2','35185.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fatsort','2','28717.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fbterm','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fcron','8','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fd','125','31917.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fdclone','10','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fdupes','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/felix','72','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fence-agents','22','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fff','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ffproxy','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/file-roller','19','34610.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/file-roller-gtk2','16','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fileschanged','11','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/findcmd','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/findpkg','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fio','12','28571.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fira-code-fonts','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fira-fonts','27','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/firejail','10','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/firetools','11','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/firmware-gobi-2000','10','29131.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fish','94','29447.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/flashcards','2','28628.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/flatpak-xdg-utils','3','31070.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/flexbackup','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fnt','2','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/font-manager','29','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-arabic-extra','25','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-arkandis','7','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-arkpandora','1','29896.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-armenian-extra','14','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-atkinson-hyperlegible','4','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-bengali-extra','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-cascadia-code','49','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-devanagari-extra','38','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-ethiopic-extra','8','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-farsi-extra','9','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-georgian-extra','4','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-gujarati-extra','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-gurmukhi-extra','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-gurmukhi-nonlibre','3','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-indic-archaic','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-indic-ne','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-indic-nonlibre','4','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-indic-other','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-intel','3','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-japanese-extra-I','80','29896.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-japanese-extra-II','84','29896.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-japanese-extra-III','146','29896.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-japanese-extra-IV','50','29896.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-japanese-extra-V','65','29896.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-japanese-extra-VI','91','29896.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-japanese-extra-VII','69','29896.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-japanese-extra-VIII','58','29896.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-kannada-extra','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-khmer-extra','7','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-kopub','55','33022.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-lao-extra','2','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-malayalam-extra','12','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-mgenplus','80','29896.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-mozilla','4','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-mpluscode','1','29355','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-myanmar-extra','5','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-nastaliq','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-oriya-extra','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-redhat','5','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-sinhala-extra','6','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-tamil-bharathi','13','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-tamil-libre','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-tamil-nonlibre','4','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-tamil-tva','10','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-telugu-extra','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-thai-extra','25','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fonts-tibetan-extra','9','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/foo2zjs','9','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/foomatic-db','50','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/foomatic-db-engine','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/foremost','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/forkstat','2','30480.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fountain','8','35185.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fpm2','8','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fprintd','7','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fr','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/franny','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/freedup','6','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/freefont','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/freeipmi','64','29131.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/freeswitch','761','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fsarchiver','5','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fscrypt','19','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fslint','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fsv','10','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fsviewer','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fsvs','8','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fswatch','15','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ft2demos','13','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fuse-archive','6','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fuse-emulator-utils','6','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fuse-exfat','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fuse-overlayfs','7','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fuse-zip','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fuseiso','6','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fwupd','94','27779.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fxload','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fzf','14','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/fzy','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/game-devices-udev','2','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gamescope','13','28454.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gammu','17','26853.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gcdemu','2','34610.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gcn64tools','5','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gconf-editor','12','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gdmap','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/geckodriver','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gencfs','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/genext2fs','7','29404.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/genimage','5','31218.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gentoo','8','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/geoclue','10','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/geoclue2','8','31383.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gigolo','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gimp-help','1140','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gitolite','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gksu','6','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/glyptodon','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gnome-color-manager','11','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gnome-commander','43','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gnome-disk-utility','11','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gnome-online-accounts','27','28001.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gnome-pkgtool','9','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gnome-terminal','27','27608.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gnome-vfs','22','27280.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gnomint','18','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gnsu','2','32180','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gnu-unifont','25','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gnupg-pkcs11-scd','9','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/go-fonts','3','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/go-mtree','11','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/goaccess','16','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gohu-font','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/google-chrome-the-latest','1','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/google-droid-fonts','7','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/google-fonts','547','33022.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gpart','9','28082.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gphotofs','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gpustat','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/graphterm','6','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/graveman','7','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/graylog-sidecar','10','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/greetd','27','28132.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/grub2-splashimages','5','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/grub2-theme-mint','1','28972.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/grub4dos','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/grub_legacy','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gslapt','10','30480.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gsmartcontrol','20','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gsplus','11','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gtk-vnc','13','31383.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gtklp','21','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/guake','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/guefi','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/gyre-fonts','5','29896.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hal-flash','12','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hardinfo','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hardinfo2','12','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hashID','1','31209.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hashcat','89','34287.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hdapsd','3','31209.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hddtemp','6','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/heartbeat','31','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/heatshrink','2','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/heirloom-doctools','40','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/heirloom-ed','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/heirloom-pax','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/heirloom-sh','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/helvum','84','28591.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hercules','150','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/herculesstudio','17','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hexcompare','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hexec','8','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hfsprogs','3','31029.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hime','15','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hivex','15','29260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hntool','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hp-wmi-sensors','3','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hplip-plugin','5','28842.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hstr','6','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hungrycat','2','26938.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hw-probe','2','28171.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hwinfo','402','29583.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hwloc','24','30733.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/hyperfine','44','29129.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/i2c-tools','3','29583.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/i8kutils','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ibam','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ibus-typing-booster','39','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/icingaweb2','19','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/idle3-tools','1','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ifuse','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ima-evm-utils','13','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/image-analyzer','2','31818.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/imwheel','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/incron','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/incus','164','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/influxdb','12','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/info2man','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ink','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/inotify-info','3','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/intel-microcode','8','27816.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/intelmas','20','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/interceptty','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/intlfonts','37','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ioninja','12','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ioping','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/iotop','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/iotop-c','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ipmitool','19','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/irqbalance','12','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/iscan','18','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/iscan-data','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/iselect','5','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/isextract','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/isight-firmware-tools','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/isomaster','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/isomd5sum','3','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/iucode_tool','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/jdiskreport','2','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/jdupes','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/john','19','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/joyce','19','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/jpnevulator','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/jq','14','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/jshon','2','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/jstest-gtk','7','34610.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kafka','15','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kbfs','110','30480.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kc','3','31029.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kegs','12','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kexec-tools','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/keyd','4','29215.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/keylaunch','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kfc','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kio-fuse','8','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kitty','69','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kiwix-tools','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kiwix-tools-bin','12','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/klish','8','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kmscon','12','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ksar','3','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ksh-openbsd','2','31029.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ktls-utils','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ktsuss','5','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kurinto-fonts','1084','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kvantum-qt5','25','29914.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/kyotocabinet','35','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/laptop-mode-tools','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lbench','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lbrate','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lcdf-typetools','22','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lddsafe','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/letsencrypt','11','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lf','21','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/libcloudproviders','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/libertinus-fonts','5','33022.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/libnatspec','12','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/libratbag','10','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/libre-franklin-fonts','3','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/libsignal-protocol-c','6','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/libtrash','14','28788.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/libwmf-type1-fonts','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lightdm','14','35185.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lightdm-settings','3','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lilyterm','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/limine','7','28706.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/linguistics-pro-fonts','4','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/linkchecker','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/linux-libertine-fonts','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/linux-vt-setcolors','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/linuxconsoletools','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lirc','32','28717.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/listpkg','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/literata-fonts','8','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lnav','175','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/local-persist','14','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/locust','4','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/loggedfs','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/logiops','30','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/logkeys','23','48278.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/logpp','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/logwatch','3','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/loksh','4','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lolcat','1','29260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lomoco','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lsb-release','1','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lshw','12','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lssecret','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lsyncd','3','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/luckybackup','17','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lv','4','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lvm2-cluster','18','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lvm2-lockd','38','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxcfs','6','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxdm','8','31239.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxgw-wenkai-fonts','75','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxgw-wenkai-gb-fonts','75','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxgw-wenkai-tc-fonts','46','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxinput','13','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxqt-about','6','31070.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxqt-admin','7','27973.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxqt-archiver','9','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxqt-config','23','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxqt-globalkeys','12','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxqt-openssh-askpass','5','31070.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxqt-policykit','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxqt-powermanagement','9','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxqt-session','10','31239.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxrandr','13','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxsession','13','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxsplit','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxtask','14','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lxterminal','10','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lynis','2','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lziprecover','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/lzop','7','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/macemu','54','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/macutils','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/magiic','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mailfromd','12','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mailutils','87','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/maintboot','2','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/makefile2graph','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/makepasswd','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/makepp','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/makeself','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/man-pages-es','9','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/man-pages-fr','16','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/man-pages-ja','24','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/man-pages-ru','12','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mandoc','11','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/marsond','1','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mathML2dtd','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/maxcso','3','29260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mbootpack','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mbpfan','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mbuffer','4','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mcrypt','7','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/md5deep','10','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mdf2iso','4','28082.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mediaextract','3','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/megacli','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/memory-editor','20','28171.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/memtest86','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/memtest86+','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/memtester','1','28171.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/memuse','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mergerfs','14','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mhddfs','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/microcode_ctl','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/midisport-firmware','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mingetty','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mksh','13','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mlterm','36','30480.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mmv','18','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/modules','31','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/monit','20','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/monitoring-plugins','47','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/monitorix','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/montagu-slab-fonts','3','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/motion','14','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mount-zip','5','31209.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mpack','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mpich','140','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mpollux-digisign-client','7','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mrxvt','10','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ms-sys','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/msitools','11','29131.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/msr-tools','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mtftar','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mtm','2','28755.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/mucommander','38','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/multibootusb','8','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/multiload-ng','12','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/multipath-tools','7','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/multitail','3','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/munin-node','44','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/musl','13','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nagios-plugins','51','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nagvis','4','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nano-syntax-highlighting','2','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nanum-coding-fonts-ttf','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nanum-fonts-ttf','13','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nbench','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ncdu','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nemo','26','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nemo-fileroller','3','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/netatop','3','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/netdata','518','27162.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/newLd','43','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/newrelic-sysmond','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nilfs-utils','10','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nix','228','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nkf','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nmon','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nmtree','8','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nnn','5','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nologind','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nomarch','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/noto-emoji','60','35185.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/noto-extra-ttf','14','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nrg2cue','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nsca-ng','8','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nss_ldap','9','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ntopng','105','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/numactl','5','29131.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/numlockx','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nut','18','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nvidia-firmware','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nvidia-legacy340-driver','75','31209.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nvidia-legacy340-kernel','28','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nvidia-legacy390-driver','86','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nvidia-legacy390-kernel','19','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nvtop','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/nwipe','8','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ocfs2-tools','25','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ohmyalias','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ohsnap','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/oils','32','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/oksh','4','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/oldschool-pc-fonts','15','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/onefetch','107','28810.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/onerng','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/opam','17','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/open-vm-tools','79','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/openGLRefToMan','329','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/openct','8','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/opendoas','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/openmpi','337','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/openrazer-daemon','5','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/openrazer-kernel','6','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/openrc','12','27804.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/openrc-services','7','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/openrc-settingsd','4','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/openrgb','253','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/opensc','25','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/openscap','53','30733.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/openstego','1','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/openzfs','145','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/osinfo-db','3','34610.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/osinfo-db-tools','6','34610.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ossec-agent','7','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ossec-local','9','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ossec-server','8','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ostree','17','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/otf-hermit','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/otf-ibm-plex','82','29896.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/otf2bdf','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/overpass-desktop-fonts','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/overpass-webfonts','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/p7zip','51','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pacemaker','125','28628.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pam-gnupg','11','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pam_ssh','9','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pam_ssh_agent_auth','19','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/partclone','32','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/partimage','21','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pass-import','2','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pass-otp','1','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/passwdqc','4','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/password-store','2','28628.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/passwordsafe','51','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/patool','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pause','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pax','7','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pax-utils','4','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/paxctl','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pbzip2','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pbzx','2','32180','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pce','10','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pcem','30','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pcem-roms','6','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pcf2bdf','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pcmanfm','10','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pcmanfm-qt','12','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pcp','899','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pcsc-lite','7','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pcsc-perl','2','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pcsc-tools','3','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pdfgrep','5','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pdksh','8','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pdsh','11','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/percol','2','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/percona-toolkit','13','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pev','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pgbadger','3','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pgsanity','1','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ph','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/phoronix-test-suite','29','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/phpsysinfo','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/physlock','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/picocom','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pigz','3','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pipemeter','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/piper','3','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pipes.sh','1','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pis','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pixma','79','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pkg','3','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/plan9port','353','27862.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/plocate','8','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/plymouth','27','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/plymouth-slackware-theme','1','35185.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pmount','6','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/polychromatic','5','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/polyglotman','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pommed-light','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/porg','11','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/posh','9','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/posixovl','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/postgresql','96','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/postgresql15','100','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/postgresql16','108','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/postgresql17','180','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/postgresql18','179','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/powerkit','7','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/powerline-fonts','2','28628.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/powerline-status','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/powernowd','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/powerpanel','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/prelink','7','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/preload','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/procinfo-ng','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/procs','199','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/profile-cleaner','1','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/profile-sync-daemon','1','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/progress','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/psensor','7','29414.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pspg','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pstate-frequency','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/psuinfo','1','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ptop','2','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pucrunch','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pv','7','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pw','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pwgen','2','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pwman','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pwsafe','10','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pyarchey','1','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pycp','3','30544.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/python-urwid','3','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/python3-docker-compose','3','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/python3-nvidia-ml-py','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/pytrash','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qControlCenter','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qdirstat','32','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qemu','814','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qemu-guest-agent','19','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qjoypad','8','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qman','3','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qps','10','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qstardict','12','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qterminal','11','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qtfm','18','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qtgzmanager','11','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qtxdg-tools','4','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/quake-par','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/quivira-font','1','32180','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/quotatool','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/qxkb','7','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rEFInd','14','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rabbitmq-server','162','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/radeon-profile','16','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/radeontop','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/raft','20','28390','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ranger','3','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rar','2','27114.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rar2fs','18','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rasdaemon','15','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ratmen','1','30544.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/razerCommander','3','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/razergenie','17','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rdiff-backup','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rdup','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/read-edid','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/readpe','3','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/realtek-bt','3','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rear','10','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/recoverdm','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/recoverjpeg','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/redditsans','5','31364.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/redict','54','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/redis','61','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/redis-py','6','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/refind-bin','3','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/regionset','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/reiser4progs','22','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rename','2','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rename.pl','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/renameutils','18','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/repeat','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/reprepro','8','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/reptyr','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/resource-agents','16','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/restic','12','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rewise','3','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rfdump','6','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rhash','13','25960','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rhvoice','175','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ripgrep','27','29980.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rkhunter','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rmw','5','29185','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rndaddentropy','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rng-tools','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/routersploit','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rover','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rox-filer','9','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/roxterm','5','28552.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rsyslog','40','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rtcqs','3','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rtirq','1','28106.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rtkit','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/run-one','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/runit','15','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/runit-services','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rush','41','28717.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/rxvt','8','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/s-tui','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/s3fs-fuse','13','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/s3ql','6','31818.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/s6','5','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/s6-linux-init','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/s6-rc','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/safecopy','4','28082.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sahel-font','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sakura','3','28571.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/samhain','14','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/samsung-mfp-drivers','9','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sandbox','24','28082.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sandy','2','28651.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sane-airscan','3','31383.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sanlock','11','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sanoid','1','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sarasa-gothic','149','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sargon','16','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sbbdep','34','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sbctl','18','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sbo-create','1','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sbo-maintainer-tools','2','28628.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sbokeeper','3','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sbotest','1','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sbotools','4','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sbotools2','4','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sboui','8','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sbozyp','2','28714.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sbpkg','2','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sbsigntools','9','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/scalpel','6','29349.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/scangearmp2','11','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/scanmem','13','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/scdoc','2','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/schedtool','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/scrypt','9','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sdate','14','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sdl-jstest','3','34610.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sdl2trs','5','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sdltrs','5','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sdwmbar','2','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/se98-icon-theme','863','30544.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/se98-wm-theme','2','30544.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/seahorse','18','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/secure-delete','1','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sensord','2','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sentimental-skk','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ser2net','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/set_rlimits','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/setconf','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sff','2','27972.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/shake','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/signify','3','31029.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/siji-font','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/silk','1','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/skim','64','29388.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/skopeo','52','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slack-osquery','11','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slack-timedate','3','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slack-utils','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slackbootstrap','2','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slackchlog','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slackrepo','2','31218.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slackrepo-hints','21','31218.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slackroll','3','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slackupdr','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slackware-grub2-theme','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slackyd','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slapt-get','10','30480.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slapt-src','7','30480.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slapt-update-service','7','30480.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sleepd','1','26838.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sleuthkit','32','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slim','7','28047','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slkstore','1','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slmenu','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/slpkg','3','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/smart','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/smbldap-tools','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/smem','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/smenu','7','28651.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/smokeping','8','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/smpq','2','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/snore','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/socketcand','6','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/socklog','22','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/softhsm','24','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/solarize-12x29-psf','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/spacefm','13','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sparse','6','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/speedcontrol','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/spice-guest-tools','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/spice-vdagent','4','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/spkg','17','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/splitjob','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/splix','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/spman','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sqlcipher','41','26398','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sqldiff','45','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/squashfuse','7','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/squidclamav','8','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/srm','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ssdeep','8','29260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sst','51','29260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/st','2','32930.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/statifier','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/steam-devices','1','28651.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/steamos-xpad','3','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/steghide','13','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/stm32flash','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/stow','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/stress','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/stress-ng','67','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/stressapptest','10','27608.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/suckless-tools','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sudosh2','8','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sunflower','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/supercat','4','30722.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/supervisor','3','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/swapinzram','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/swatch','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/swtpm','34','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/swun','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/symbola-font-ttf','4','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/sysbench','29','27862.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/syslog-ng','137','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/system76-io-dkms','1','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/systraq','1','34729.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tabbed','2','28651.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tagainijisho','134','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tagtools','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tamsyn-font','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/targetcli-fb','3','33780','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tarlz','3','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tarsnap','18','34729.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tbsm','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tc-play','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/telegraf','77','28552.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/termenu','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/terminalia-fonts','2','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/terminator','6','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/terminology','18','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/termsaver','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/termsyn-font','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/testdisk','16','31239.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tewi-font','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/the_silver_searcher','7','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/thefuck','7','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/thermald','22','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/thin-provisioning-tools','29','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/thinkfan','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ti99sim','23','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ticpp','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tiger','73','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tilda','12','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/timeshift','16','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tini','2','28196.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tinycdb','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tinyterm','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tio','5','32921.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tkdvd','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tkinfo','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tm','1','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tmpwatch','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tmuxp','2','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tokyocabinet','9','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tokyotyrant','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tomb','4','25960','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/torrent-file-editor','10','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tp_smapi','3','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tpc','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tpm','1','29260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tqemu','1','28552.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/trachet','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tracker','13','26590.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tracker-miners','17','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/trackfs','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/trash','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/trash-cli','5','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/trashy','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/triggerhappy','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/trousers','19','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/trrntzip','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/trs80-roms','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ts','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-aller','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-ancient-fonts','19','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-arphic-uming','11','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-babelstone-cjk','27','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-babelstone-latin','2','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-babelstone-tibetan','3','31239.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-bitstream-vera','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-caladea','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-carlito','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-charis-sil','3','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-coda','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-computer-modern','7','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-console-fonts','4','27973.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-crosscorefonts','11','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-dark-courier','2','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-doulos-sil','2','29914.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-genjyuu-gothic','101','28228.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-hanazono','29','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-ibm-plex','95','29896.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-inconsolata','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-ipaex','6','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-lato','5','26900.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-monaco','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-mononoki','11','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-nerd-fonts-symbols','3','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-open-sans','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-paratype-pt-astra-fact-fonts','2','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-paratype-pt-fonts','4','33022.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-roboto','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-sakalbharati','2','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-ubuntu-font-family','2','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-x2','17','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-xo-fonts','2','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttf-zekton','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttmkfdir','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttop','2','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tty2gif','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttygif','1','28628.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ttyrec','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tuptime','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/twapake','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/twin','25','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tzleap','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/tzupdate','21','28798.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/u3-tool','6','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/uae','48','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ucs-miscfixed-opentype-fonts','2','27912.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/udevil','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/udftools','7','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/udiskie','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ufiformat','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/uhd','365','29706.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/uhubctl','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/uif2iso','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/un-fonts-core','9','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unac','12','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unace','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unar','10','31209.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unetbootin','18','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unhide','4','30733.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unicode-character-database','33','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unigine-heaven-benchmark','95','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unigine-valley-benchmark','168','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unionfs-fuse','2','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unison','18','28552.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/univga-font','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unp','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unrar','94','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unrar-free','6','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unrtf','5','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unshield','4','30722.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/unshieldv3','3','30722.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/untex','1','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/upass','1','29349.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/update-initrd','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/uptimed','8','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/upx','12','28788.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/usbguard','55','27997.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/usbimager','4','28651.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/usbmon','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/usbredir','4','34610.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/usbrelay','2','31218.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/usbtop','6','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/usbview','6','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/usermin','66','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/v4l2loopback','4','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/v7sh','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vagrant','97','28171.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vagrant-public-key','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/valkey','62','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vbindiff','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vcp','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/ventoy','7','28868.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/veracrypt','47','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vhba-module','3','30624','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vice','305','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/victor-mono-fonts','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vifm','23','30722.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vim-airline','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vim-airline-themes','4','34287.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vinterm','3','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/virt-manager','8','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/virt-viewer','10','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/virtualbox-addons','155','31239.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/virtualbox-extension-pack','1','31239.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/virtualxt','12','27862.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vlock','2','28095.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vmbinstallpkg','2','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vmfs-tools','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vms','2','28454.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/volatility','20','28747.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/volatility3','8','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vtcol','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/vwm','6','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wangfonts','121','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/watch-fs','1','35185.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/watchdog','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/waydroid','2','33481.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wayland-utils','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/waypipe','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wdfs','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wdiff','16','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/webcore-fonts','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/webmin','141','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wezterm','391','29349.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/whowatch','6','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wiimms-iso-tools','11','29260.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wine','451','28268.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wine-staging','473','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/winetricks','2','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/winusb','8','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wmconfig','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wmfsm','6','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wmgtemp','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/worker','61','29284.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wqy-microhei-font-ttf','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/wterm','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/x86_energy_perf_policy','7','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xAce','4','28747.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xar','8','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xarchiver','10','32923.5','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xautolock','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xautomation','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xbattmon','1','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xbindkeys','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xboxdrv-linux','144','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xbrightness','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xcdroast','9','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xchm','14','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xdg-dbus-proxy','3','28232.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xdg-desktop-portal-gnome','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xdg-desktop-portal-wlr','4','28676.1','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xdg-ninja','3','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xdg-utils-cxx','5','28647.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xdiskusage','3','31928.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xen-nox','212','34729.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xf86-video-qxl','7','31041.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xfburn','9','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xfe','36','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xfile','5','28012.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xfs_undelete','1','28820.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xfsudo','1','27862.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xiccd','6','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xidel','6','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xinput_calibrator','10','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xjobs','4','28628.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xldconfig','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xnp2','15','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xonsh','17','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xorgxrdp','17','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xosd','8','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xosview','5','30942.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xpipe','1','31029.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xprintidle','2','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xrdp','58','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xrestop','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xsettingsd','4','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xssstate','1','31029.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xstow','10','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xtrs','4','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xvfb-run','1','28784.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/xwiimote','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/yabsm','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/yaft','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/yank','1','26648.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/yash','45','26498.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/yaze-ag','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/yeahconsole','1','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/yelp','10','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/yelp-tools','3','33780','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/yelp-xsl','5','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/yq','23','28074','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/yubico-piv-tool','5','33733.9','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/z','2','28214.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zarfy','5','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zellij-bin','18','29296.3','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zfs-auto-snapshot','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zim-tools','11','30777.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/znapzend','8','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zookeeper','20','29408.6','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zopfli','3','28998.7','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zoxide','53','31459.8','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zpaq','9','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zramen','1','28630.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zsh-completions','3','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zsh-syntax-highlighting','2','26805.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zuluCrypt','41','31122.2','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zutty','9','31327.4','~');
insert or ignore into buildsecs(itemid,secs,mhzsum,guessflag) values ('system/zzzfm','15','26805.4','~');
commit;
